@@ -3,7 +3,7 @@
  *  Summary:    Player kill tracking
  *  Written by: Darshan Shaligram
  *
- *  Modified for Crawl Reference by $Author: dshaligram $ on $Date: 2007-05-02T16:49:28.298370Z $
+ *  Modified for Crawl Reference by $Author: dshaligram $ on $Date: 2007-10-02T11:02:32.986158Z $
  */
 #include "AppHdr.h"
 
@@ -12,9 +12,10 @@
 #include "chardump.h"
 #include "describe.h"
 #include "mon-util.h"
+#include "monstuff.h"
 #include "files.h"
 #include "itemname.h"
-#include "misc.h"
+#include "place.h"
 #include "travel.h"
 #include "tags.h"
 #include "Kills.h"
@@ -38,6 +39,19 @@ const char *kill_category_names[] =
     "others",
 };
 
+KillMaster::KillMaster()
+{
+}
+
+KillMaster::KillMaster(const KillMaster &other)
+{
+    *this = other;
+}
+
+KillMaster::~KillMaster()
+{
+}
+
 const char *KillMaster::category_name(kill_category kc) const
 {
     if (kc >= KC_YOU && kc < KC_NCATEGORIES)
@@ -81,10 +95,10 @@ void KillMaster::load(FILE *file)
 
 void KillMaster::record_kill(const monsters *mon, int killer, bool ispet)
 {
-    kill_category kc = 
-        (killer == KILL_YOU || killer == KILL_YOU_MISSILE)? KC_YOU :
-        (ispet)?                                            KC_FRIENDLY :
-                                                            KC_OTHER;
+    const kill_category kc = 
+        YOU_KILL(killer)? KC_YOU :
+        ispet?            KC_FRIENDLY :
+                          KC_OTHER;
     categorized_kills[kc].record_kill(mon);
 }
 
@@ -136,6 +150,7 @@ std::string KillMaster::kill_info() const
     }
 
 #ifdef CLUA_BINDINGS
+    unwind_var<int> lthrottle(clua.throttle_unit_lines, 500000);
     // Call the kill dump Lua function with null a, to tell it we're done.
     if (!clua.callfn("c_kill_list", "ss", NULL, grandt.c_str()))
 #endif
@@ -173,7 +188,8 @@ void KillMaster::add_kill_info(std::string &killtext,
         lua_pushnil(clua);
 
     lua_pushboolean(clua, separator);
-    
+
+    unwind_var<int> lthrottle(clua.throttle_unit_lines, 500000);    
     if (!clua.callfn("c_kill_list", 3, 0))
 #endif
     {
@@ -329,24 +345,6 @@ kill_def::kill_def(const struct monsters *mon) : kills(0), exp(0)
     add_kill(mon, get_packed_place());
 }
 
-static bool ends_with(const std::string &s, const char *suffix)
-{
-    std::string other = suffix;
-    if (s.length() < other.length()) return false;
-    return (s.substr(s.length() - other.length()) == other);
-}
-
-static bool ends_with(const std::string &s, const char *suffixes[])
-{
-    if (!suffixes) return false;
-    for ( ; *suffixes; suffixes++)
-    {
-        if (ends_with(s, *suffixes))
-            return true;
-    }
-    return false;
-}
-
 std::string apostrophise(const std::string &name)
 {
     if (name.empty())
@@ -370,72 +368,6 @@ static const char *modifier_suffixes[] =
     "zombie", "skeleton", "simulacrum", NULL,
 };
 
-// Pluralises a monster name. This'll need to be updated for correctness
-// whenever new monsters are added.
-std::string pluralise(const std::string &name, 
-                      const char *no_of[])
-{
-    std::string::size_type pos;
-
-    // Pluralise first word of names like 'eye of draining', but only if the
-    // whole name is not suffixed by a modifier, such as 'zombie' or 'skeleton'
-    if ( (pos = name.find(" of ")) != std::string::npos 
-            && !ends_with(name, no_of) )
-        return pluralise(name.substr(0, pos)) + name.substr(pos);
-    else if (ends_with(name, "us"))
-        // Fungus, ufetubus, for instance.
-        return name.substr(0, name.length() - 2) + "i";
-    else if (ends_with(name, "larva") || ends_with(name, "amoeba"))
-        // Giant amoebae sounds a little weird, to tell the truth.
-        return name + "e";
-    else if (ends_with(name, "ex"))
-        // Vortex; vortexes is legal, but the classic plural is cooler.
-        return name.substr(0, name.length() - 2) + "ices";
-    else if (ends_with(name, "cyclops"))
-        return name.substr(0, name.length() - 1) + "es";
-    else if (ends_with(name, "y"))
-        return name.substr(0, name.length() - 1) + "ies";
-    else if (ends_with(name, "elf") || ends_with(name, "olf"))
-        // Elf, wolf. Dwarfs can stay dwarfs, if there were dwarfs.
-        return name.substr(0, name.length() - 1) + "ves";
-    else if (ends_with(name, "mage"))
-        // mage -> magi
-        return name.substr(0, name.length() - 1) + "i";
-    else if ( ends_with(name, "sheep") || ends_with(name, "manes")
-            || ends_with(name, "fish") )
-        // Maybe we should generalise 'manes' to ends_with("es")?
-        return name;
-    else if (ends_with(name, "ch") || ends_with(name, "sh") 
-            || ends_with(name, "x"))
-        // To handle cockroaches, fish and sphinxes. Fish will be netted by
-        // the previous check anyway.
-        return name + "es";
-    else if (ends_with(name, "um"))
-        // simulacrum -> simulacra
-        return name.substr(0, name.length() - 2) + "a";
-    else if (ends_with(name, "efreet"))
-        // efreet -> efreeti. Not sure this is correct.
-        return name + "i";
-
-    return name + "s";
-}
-
-// Naively prefix A/an to a monster name. At the moment, we don't have monster
-// names that demand more sophistication (maybe ynoxinul - don't know how 
-// that's pronounced).
-static std::string article_a(const std::string &name)
-{
-    if (!name.length()) return name;
-    switch (name[0])
-    {
-        case 'a': case 'e': case 'i': case 'o': case 'u':
-        case 'A': case 'E': case 'I': case 'O': case 'U':
-            return "An " + name;
-        default:
-            return "A " + name;
-    }
-}
-
 // For a non-unique monster, prefixes a suitable article if we have only one
 // kill, else prefixes a kill count and pluralises the monster name.
 static std::string n_names(const std::string &name, int n)
@@ -444,10 +376,11 @@ static std::string n_names(const std::string &name, int n)
     {
         char buf[20];
         snprintf(buf, sizeof buf, "%d ", n);
-        return buf + pluralise(name, modifier_suffixes);
+        return buf + pluralise(name, standard_plural_qualifiers,
+                               modifier_suffixes);
     }
     else
-        return article_a(name);
+        return article_a(name, false);
 }
 
 // Returns a string describing the number of times a unique has been killed.
@@ -505,14 +438,12 @@ void kill_def::add_place(unsigned short place, bool force)
 
 std::string kill_def::base_name(const kill_monster_desc &md) const
 {
-    char monnamebuf[ITEMNAME_SIZE];     // Le sigh.
-
+    std::string name;
     if (md.monnum == MONS_PANDEMONIUM_DEMON)
-        strcpy(monnamebuf, "demon lord");
+        name = "demon lord";
     else
-        moname(md.monnum, true, DESC_PLAIN, monnamebuf);
+        name = mons_type_name(md.monnum, DESC_PLAIN);
 
-    std::string name = monnamebuf;
     switch (md.modifier)
     {
       case kill_monster_desc::M_ZOMBIE:
@@ -701,7 +632,6 @@ void kill_monster_desc::load(FILE *file)
     modifier = (name_modifier) readShort(file);
 }
 
-#ifdef CLUA_BINDINGS
 ///////////////////////////////////////////////////////////////////////////
 // Kill Lua interface
 //
@@ -1004,11 +934,12 @@ static const struct luaL_reg kill_lib[] =
     { NULL, NULL }
 };
 
-void lua_open_kills(lua_State *ls)
+void luaopen_kills(lua_State *ls)
 {
     luaL_openlib(ls, "kills", kill_lib, 0);
 }
 
+#ifdef CLUA_BINDINGS
 static void kill_lua_filltable(std::vector<kill_exp> &v)
 {
     lua_State *ls = clua.state();
@@ -1019,5 +950,4 @@ static void kill_lua_filltable(std::vector<kill_exp> &v)
         lua_rawseti(ls, -2, i + 1);
     }
 }
-
 #endif
