@@ -1,17 +1,19 @@
 /*
- *  Modified for Crawl Reference by $Author: dshaligram $ on $Date: 2007-03-20T15:01:18.390504Z $
+ *  Modified for Crawl Reference by $Author: ennewalker $ on $Date: 2007-11-23T02:25:44.246247Z $
  */
 
 #include "AppHdr.h"
 
-#ifdef CLUA_BINDINGS
-
 #include "clua.h"
 
 #include "abl-show.h"
 #include "command.h"
 #include "chardump.h"
+#include "cio.h"
 #include "delay.h"
+#include "dgnevent.h"
+#include "dungeon.h"
+#include "files.h"
 #include "food.h"
 #include "invent.h"
 #include "initfile.h"
@@ -19,31 +21,33 @@
 #include "itemprop.h"
 #include "items.h"
 #include "item_use.h"
+#include "it_use2.h"
 #include "libutil.h"
 #include "macro.h"
+#include "mapdef.h"
 #include "message.h"
 #include "mon-util.h"
 #include "output.h"
 #include "player.h"
 #include "randart.h"
+#include "religion.h"
 #include "skills2.h"
 #include "spl-util.h"
+#include "state.h"
 #include "stuff.h"
+#include "transfor.h"
+#include "travel.h"
+#include "view.h"
 
 #include <cstring>
-
-#ifdef HASH_CONTAINERS
-#   include <hash_map>
-#   define CHMAP HASH_CONTAINER_NS::hash_map
-#else
-#   include <map>
-#   define CHMAP std::map
-#endif
-
+#include <map>
 #include <cctype>
 
+#define BUGGY_PCALL_ERROR  "667: Malformed response to guarded pcall."
+#define BUGGY_SCRIPT_ERROR "666: Killing badly-behaved Lua script."
+
 #define CL_RESETSTACK_RETURN(ls, oldtop, retval) \
-    if (true) \
+    do \
     {\
         if (oldtop != lua_gettop(ls)) \
         { \
@@ -51,16 +55,33 @@
         } \
         return (retval); \
     } \
-    else
+    while (false)
 
-CLua clua;
+static int clua_panic(lua_State *);
+static void clua_throttle_hook(lua_State *, lua_Debug *);
+static void *clua_allocator(void *ud, void *ptr, size_t osize, size_t nsize);
+static int clua_guarded_pcall(lua_State *);
+static int clua_dofile(lua_State *);
+static int clua_loadfile(lua_State *);
 
-CLua::CLua() : _state(NULL), sourced_files(), uniqindex(0L)
+CLua clua(true);
+
+const int CLua::MAX_THROTTLE_SLEEPS;
+
+CLua::CLua(bool managed)
+    : error(), managed_vm(managed), shutting_down(false),
+      throttle_unit_lines(10000),
+      throttle_sleep_ms(0), throttle_sleep_start(2),
+      throttle_sleep_end(800), n_throttle_sleeps(0), mixed_call_depth(0),
+      lua_call_depth(0), max_mixed_call_depth(8),
+      max_lua_call_depth(100), memory_used(0),
+      _state(NULL), sourced_files(), uniqindex(0L)
 {
 }
 
 CLua::~CLua()
 {
+    shutting_down = true;
     if (_state)
         lua_close(_state);
 }
@@ -165,44 +186,101 @@ void CLua::set_error(int err, lua_State *ls)
     error = serr? serr : "<Unknown error>";
 }
 
-int CLua::execstring(const char *s, const char *context)
+void CLua::init_throttle()
 {
-    lua_State *ls = state();
-    int err = luaL_loadbuffer(ls, s, strlen(s), context);
-    if (err)
+    if (!managed_vm)
+        return;
+    
+    if (throttle_unit_lines <= 0)
+        throttle_unit_lines = 500;
+
+    if (throttle_sleep_start < 1)
+        throttle_sleep_start = 1;
+
+    if (throttle_sleep_end < throttle_sleep_start)
+        throttle_sleep_end = throttle_sleep_start;
+    
+    if (!mixed_call_depth)
     {
-        set_error(err, ls);
-        return err;
+        lua_sethook(_state, clua_throttle_hook,
+                    LUA_MASKCOUNT, throttle_unit_lines);
+        throttle_sleep_ms = 0;
+        n_throttle_sleeps = 0;
     }
-    err = lua_pcall(ls, 0, 0, 0);
+}
+
+int CLua::loadbuffer(const char *buf, size_t size, const char *context)
+{
+    const int err = luaL_loadbuffer(state(), buf, size, context);
+    set_error(err, state());
+    return err;
+}
+
+int CLua::loadstring(const char *s, const char *context)
+{
+    return loadbuffer(s, strlen(s), context);
+}
+
+int CLua::execstring(const char *s, const char *context, int nresults)
+{
+    int err = 0;
+    if ((err = loadstring(s, context)))
+        return (err);
+    
+    lua_State *ls = state();
+    lua_call_throttle strangler(this);
+    err = lua_pcall(ls, 0, nresults, 0);
     set_error(err, ls);
     return err;
 }
 
-int CLua::execfile(const char *filename)
+bool CLua::is_path_safe(std::string s, bool trusted)
+{
+    lowercase(s);
+    return (s.find("..") == std::string::npos && shell_safe(s.c_str())
+            && (trusted || s.find("clua") == std::string::npos));
+}
+
+int CLua::loadfile(lua_State *ls, const char *filename, bool trusted,
+                   bool die_on_fail)
+{
+    if (!ls)
+        return (-1);
+
+    if (!is_path_safe(filename, trusted))
+    {
+        lua_pushstring(
+            ls,
+            make_stringf("invalid filename: %s", filename).c_str());
+        return (-1);
+    }
+    
+    std::string file = datafile_path(filename, die_on_fail);
+    if (file.empty())
+    {
+        lua_pushstring(ls,
+                       make_stringf("Can't find \"%s\"", filename).c_str());
+        return (-1);
+    }
+    return (luaL_loadfile(ls, file.c_str()));
+}
+
+int CLua::execfile(const char *filename, bool trusted, bool die_on_fail)
 {
     if (sourced_files.find(filename) != sourced_files.end())
         return 0;
 
     sourced_files.insert(filename);
 
-    FILE *f = fopen(filename, "r");
-    if (f)
-        fclose(f);
-    else
-    {
-        error = std::string("Can't read ") + filename;
-        return -1;
-    }
-
     lua_State *ls = state();
-    if (!ls)
-        return -1;
-    
-    int err = luaL_loadfile(ls, filename);
+    int err = loadfile(ls, filename, trusted || !managed_vm, die_on_fail);
+    lua_call_throttle strangler(this);
     if (!err)
         err = lua_pcall(ls, 0, 0, 0);
     set_error(err);
+    if (die_on_fail && !error.empty())
+        end(1, false, "Lua execfile error (%s): %s",
+            filename, dlua.error.c_str());
     return (err);
 }
 
@@ -252,9 +330,7 @@ void CLua::fnreturns(const char *format, ...)
 
     va_list args;
     va_start(args, format);
-
     vfnreturns(format, args);
-
     va_end(args);
 }
 
@@ -350,6 +426,12 @@ int CLua::push_args(lua_State *ls, const char *format, va_list args,
         case 'b':
             lua_pushboolean(ls, va_arg(args, int));
             break;
+        case 'D':
+            clua_push_dgn_event(ls, va_arg(args, const dgn_event *));
+            break;
+        case 'm':
+            clua_push_map(ls, va_arg(args, map_def *));
+            break;
         case 'M':
             push_monster(ls, va_arg(args, monsters *));
             break;
@@ -399,6 +481,7 @@ bool CLua::calltopfn(lua_State *ls, const char *params, va_list args,
     int argc = push_args(ls, params, args, copyto);
     if (retc == -1)
         retc = return_count(ls, params);
+    lua_call_throttle strangler(this);
     int err = lua_pcall(ls, argc, retc, 0);
     set_error(err, ls);
     return (!err);
@@ -470,17 +553,23 @@ bool CLua::callfn(const char *fn, int nargs, int nret)
     lua_State *ls = state();
     if (!ls)
         return (false);
-    
-    lua_getglobal(ls, fn);
-    if (!lua_isfunction(ls, -1))
+
+    // If a function is not provided on the stack, get the named function.
+    if (fn)
     {
-        lua_settop(ls, -nargs - 2);
-        return (false);
+        lua_getglobal(ls, fn);
+        if (!lua_isfunction(ls, -1))
+        {
+            lua_settop(ls, -nargs - 2);
+            return (false);
+        }
+        
+        // Slide the function in front of its args and call it.
+        if (nargs)
+            lua_insert(ls, -nargs - 1);
     }
 
-    // Slide the function in front of its args and call it.
-    if (nargs)
-        lua_insert(ls, -nargs - 1);
+    lua_call_throttle strangler(this);
     int err = lua_pcall(ls, nargs, nret, 0);
     set_error(err, ls);
     return !err;
@@ -488,44 +577,69 @@ bool CLua::callfn(const char *fn, int nargs, int nret)
 
 // Defined in Kills.cc because the kill bindings refer to Kills.cc local 
 // structs
-extern void lua_open_kills(lua_State *ls);
-
-void lua_open_you(lua_State *ls);
-void lua_open_item(lua_State *ls);
-void lua_open_food(lua_State *ls);
-void lua_open_crawl(lua_State *ls);
-void lua_open_file(lua_State *ls);
-void lua_open_options(lua_State *ls);
-void lua_open_monsters(lua_State *ls);
-void lua_open_globals(lua_State *ls);
+extern void luaopen_kills(lua_State *ls);
 
+void luaopen_you(lua_State *ls);
+void luaopen_item(lua_State *ls);
+void luaopen_food(lua_State *ls);
+void luaopen_crawl(lua_State *ls);
+void luaopen_file(lua_State *ls);
+void luaopen_options(lua_State *ls);
+void luaopen_monsters(lua_State *ls);
+void luaopen_globals(lua_State *ls);
 
 void CLua::init_lua()
 {
     if (_state)
         return;
 
-    _state = lua_open();
+    _state = managed_vm? lua_newstate(clua_allocator, this) : luaL_newstate();
     if (!_state)
         return;
+
+    lua_stack_cleaner clean(_state);
+
+    lua_atpanic(_state, clua_panic);
+
     luaopen_base(_state);
     luaopen_string(_state);
     luaopen_table(_state);
+    luaopen_math(_state);
 
     // Open Crawl bindings
-    lua_open_kills(_state);
-    lua_open_you(_state);
-    lua_open_item(_state);
-    lua_open_food(_state);
-    lua_open_crawl(_state);
-    lua_open_file(_state);
-    lua_open_options(_state);
-    lua_open_monsters(_state);
+    luaopen_kills(_state);
+    luaopen_you(_state);
+    luaopen_item(_state);
+    luaopen_food(_state);
+    luaopen_crawl(_state);
+    luaopen_file(_state);
+    luaopen_options(_state);
+    luaopen_monsters(_state);
 
-    lua_open_globals(_state);
+    luaopen_globals(_state);
 
     load_cmacro();
     load_chooks();
+
+    lua_register(_state, "loadfile", clua_loadfile);
+    lua_register(_state, "dofile", clua_dofile);
+
+    if (managed_vm)
+    {
+        lua_register(_state, "pcall", clua_guarded_pcall);
+        execfile("clua/userbase.lua", true, true);
+    }        
+
+    lua_pushboolean(_state, managed_vm);
+    setregistry("lua_vm_is_managed");
+}
+
+bool CLua::is_managed_vm(lua_State *ls)
+{
+    lua_stack_cleaner clean(ls);
+    lua_pushstring(ls, "lua_vm_is_managed");
+    lua_gettable(ls, LUA_REGISTRYINDEX);
+    return (lua_toboolean(ls, -1));
 }
 
 void CLua::load_chooks()
@@ -539,69 +653,11 @@ void CLua::load_chooks()
 
 void CLua::load_cmacro()
 {
-    static const char *c_macro =
-        "function c_macro(fn)"
-        "    if fn == nil then"
-        "        if c_macro_coroutine ~= nil then"
-        "            local coret, mret"
-        "            coret, mret = coroutine.resume(c_macro_coroutine)"
-        "            if not coret or not mret then"
-        "                c_macro_coroutine = nil"
-        "                c_macro_name      = nil"
-        "            end"
-        "            if not coret and mret then"
-        "                error(mret)"
-        "            end"
-        "            return (coret and mret)"
-        "        end"
-        "        return false"
-        "    end"
-        "    if _G[fn] == nil or type(_G[fn]) ~= 'function' then"
-        "        return false"
-        "    end"
-        "    c_macro_name = fn"
-        "    c_macro_coroutine = coroutine.create(_G[fn]) "
-        "    return c_macro() "
-        "end";
-    execstring(c_macro, "base");
+    execfile("clua/macro.lua", true, true);
 }
 
 /////////////////////////////////////////////////////////////////////
 
-#define LUAWRAP(name, wrapexpr) \
-    static int name(lua_State *ls) \
-    {   \
-        wrapexpr; \
-        return (0); \
-    }
-
-#define LUARET1(name, type, val) \
-    static int name(lua_State *ls) \
-    { \
-        lua_push##type(ls, val); \
-        return (1); \
-    }
-
-#define LUARET2(name, type, val1, val2)  \
-    static int name(lua_State *ls) \
-    { \
-        lua_push##type(ls, val1); \
-        lua_push##type(ls, val2); \
-        return (2); \
-    }
-
-template <class T> T *util_get_userdata(lua_State *ls, int ndx)
-{
-    return (lua_islightuserdata(ls, ndx))?
-            static_cast<T *>( lua_touserdata(ls, ndx) )
-          : NULL;
-}
-
-template <class T> T *clua_get_userdata(lua_State *ls, const char *mt)
-{
-    return static_cast<T*>( luaL_checkudata( ls, 1, mt ) );
-}
-
 static void clua_register_metatable(lua_State *ls, const char *tn, 
                                    const luaL_reg *lr,
                                    int (*gcfn)(lua_State *ls) = NULL)
@@ -625,15 +681,6 @@ static void clua_register_metatable(lua_State *ls, const char *tn,
     lua_settop(ls, top);
 }
 
-template <class T> T *clua_new_userdata(
-        lua_State *ls, const char *mt)
-{
-    void *udata = lua_newuserdata( ls, sizeof(T) );
-    luaL_getmetatable(ls, mt);
-    lua_setmetatable(ls, -2);
-    return static_cast<T*>( udata );
-}
-
 /////////////////////////////////////////////////////////////////////
 // Bindings to get information on the player
 //
@@ -644,6 +691,8 @@ static const char *transform_name()
     {
     case TRAN_SPIDER:
         return "spider";
+    case TRAN_BAT:
+        return "bat";
     case TRAN_BLADE_HANDS:
         return "blade";
     case TRAN_STATUE:
@@ -665,8 +714,10 @@ static const char *transform_name()
 
 LUARET1(you_turn_is_over, boolean, you.turn_is_over)
 LUARET1(you_name, string, you.your_name)
-LUARET1(you_race, string, species_name(you.species, you.experience_level))
+LUARET1(you_race, string,
+        species_name(you.species, you.experience_level).c_str())
 LUARET1(you_class, string, get_class_name(you.char_class))
+LUARET1(you_god, string, god_name(you.religion))
 LUARET2(you_hp, number, you.hp, you.hp_max)
 LUARET2(you_mp, number, you.magic_points, you.max_magic_points)
 LUARET1(you_hunger, string, hunger_level())
@@ -684,12 +735,20 @@ LUARET1(you_res_statdrain, number, player_sust_abil(false))
 LUARET1(you_res_mutation, number, wearing_amulet(AMU_RESIST_MUTATION, false))
 LUARET1(you_res_slowing, number, wearing_amulet(AMU_RESIST_SLOW, false))
 LUARET1(you_gourmand, boolean, wearing_amulet(AMU_THE_GOURMAND, false))
-LUARET1(you_levitating, boolean, 
-        player_is_levitating() && !wearing_amulet(AMU_CONTROLLED_FLIGHT))
-LUARET1(you_flying, boolean,
-        player_is_levitating() && wearing_amulet(AMU_CONTROLLED_FLIGHT))
+LUARET1(you_levitating, boolean, you.flight_mode() == FL_LEVITATE)
+LUARET1(you_flying, boolean, you.flight_mode() == FL_FLY)
 LUARET1(you_transform, string, transform_name())
+LUARET1(you_where, string, level_id::current().describe().c_str())
+LUARET1(you_branch, string, level_id::current().describe(false, false).c_str())
+LUARET1(you_subdepth, number, level_id::current().depth)
+LUARET1(you_absdepth, number, you.your_level + 1)
 LUAWRAP(you_stop_activity, interrupt_activity(AI_FORCE_INTERRUPT))
+LUARET1(you_turns, number, you.num_turns)
+LUARET1(you_see_grid, boolean,
+        see_grid(luaL_checkint(ls, 1), luaL_checkint(ls, 2)))
+// increase by 1 because check happens on old level
+LUARET1(bzr_floor_colour, string,
+        colour_to_str(bazaar_floor_colour(you.your_level + 2)))
 
 void lua_push_floor_items(lua_State *ls);
 static int you_floor_items(lua_State *ls)
@@ -704,7 +763,7 @@ static int l_you_spells(lua_State *ls)
     int index = 0;
     for (int i = 0; i < 52; ++i)
     {
-        const int spell = get_spell_by_letter( index_to_letter(i) );
+        const spell_type spell = get_spell_by_letter( index_to_letter(i) );
         if (spell == SPELL_NO_SPELL)
             continue;
 
@@ -730,19 +789,21 @@ static int l_you_abils(lua_State *ls)
 static const struct luaL_reg you_lib[] =
 {
     { "turn_is_over", you_turn_is_over },
+    { "turns"       , you_turns },
     { "spells"      , l_you_spells },
     { "abilities"   , l_you_abils },
     { "name"        , you_name },
     { "race"        , you_race },
     { "class"       , you_class },
+    { "god"         , you_god },
     { "hp"          , you_hp },
     { "mp"          , you_mp },
     { "hunger"      , you_hunger },
     { "strength"    , you_strength },
     { "intelligence", you_intelligence },
     { "dexterity"   , you_dexterity },
-    { "exp"         , you_exp },
-    { "exp_points"  , you_exp_points },
+    { "xl"          , you_exp },
+    { "exp"         , you_exp_points },
 
     { "res_poison"  , you_res_poison },
     { "res_fire"    , you_res_fire   },
@@ -760,10 +821,19 @@ static const struct luaL_reg you_lib[] =
     { "stop_activity", you_stop_activity },
 
     { "floor_items",  you_floor_items },
+
+    { "where",        you_where },
+    { "branch",       you_branch },
+    { "subdepth",     you_subdepth },
+    { "absdepth",     you_absdepth },
+
+    { "see_grid",     you_see_grid },
+    { "bazaar_floor", bzr_floor_colour },
+    
     { NULL, NULL },
 };
 
-void lua_open_you(lua_State *ls)
+void luaopen_you(lua_State *ls)
 {
     luaL_openlib(ls, "you", you_lib, 0);
 }
@@ -1114,10 +1184,27 @@ static int l_item_subtype(lua_State *ls)
             const char *s = NULL;
             if (item->base_type == OBJ_JEWELLERY)
             {
-                if (item->sub_type < AMU_RAGE)
-                    s = ring_types[item->sub_type];
-                else
+                if (jewellery_is_amulet(*item))
                     s = amulet_types[ item->sub_type - AMU_RAGE ];
+                else
+                    s = ring_types[item->sub_type];
+            }
+            else if (item->base_type == OBJ_POTIONS)
+            {
+                if (item->sub_type == POT_BLOOD)
+                   s = "blood";
+                else if (item->sub_type == POT_WATER)
+                   s = "water";
+                else if (item->sub_type == POT_PORRIDGE)
+                   s = "porridge";
+                else if (item->sub_type == POT_BERSERK_RAGE)
+                   s = "berserk";
+                else if (item->sub_type == POT_GAIN_STRENGTH
+                         || item->sub_type == POT_GAIN_DEXTERITY
+                         || item->sub_type == POT_GAIN_INTELLIGENCE)
+                   s = "gain ability";
+                else if (item->sub_type == POT_CURE_MUTATION)
+                   s = "cure mutation";
             }
             
             if (s)
@@ -1135,6 +1222,67 @@ static int l_item_subtype(lua_State *ls)
     return (2);
 }
 
+static int l_item_potion_type(lua_State *ls)
+{
+    LUA_ITEM(item, 1);
+    int val = 99;
+
+    if (item && item->base_type == OBJ_POTIONS)
+    {
+       if (!item_type_known(*item))
+       {
+          val = 0;
+       }
+       else
+       {
+          switch(item->sub_type)
+          {
+            // good potions:
+            case POT_HEALING:
+            case POT_HEAL_WOUNDS:
+            case POT_SPEED:
+            case POT_MIGHT:
+            case POT_LEVITATION:
+            case POT_INVISIBILITY:
+            case POT_EXPERIENCE:
+            case POT_MAGIC:
+            case POT_RESTORE_ABILITIES:
+            case POT_RESISTANCE:
+                 val = 1;
+                 break;
+
+            // bad potions:
+            case POT_POISON:
+            case POT_STRONG_POISON:
+            case POT_SLOWING:
+            case POT_PARALYSIS:
+            case POT_CONFUSION:
+            case POT_DEGENERATION:
+            case POT_DECAY:
+            case POT_MUTATION:
+                 val = 2;
+                 break;
+
+            // need more refined handling:
+            // for eating habits
+            case POT_BLOOD:
+            case POT_WATER:
+            case POT_PORRIDGE:
+            // for undead
+            case POT_BERSERK_RAGE:
+            case POT_GAIN_STRENGTH:
+            case POT_GAIN_DEXTERITY:
+            case POT_GAIN_INTELLIGENCE:
+            case POT_CURE_MUTATION:
+            default:
+                 val = 3;
+          }
+       }
+    }
+    lua_pushnumber(ls, val);
+    return (1);
+}
+
 static int l_item_cursed(lua_State *ls)
 {
     LUA_ITEM(item, 1);
@@ -1160,46 +1308,18 @@ static int l_item_worn(lua_State *ls)
     return (2);
 }
 
-static int desc_code(const char *desc)
-{
-    if (!desc)
-        return DESC_PLAIN;
-    
-    if (!strcmp("The", desc))
-        return DESC_CAP_THE;
-    else if (!strcmp("the", desc))
-        return DESC_NOCAP_THE;
-    else if (!strcmp("A", desc))
-        return DESC_CAP_A;
-    else if (!strcmp("a", desc))
-        return DESC_NOCAP_A;
-    else if (!strcmp("Your", desc))
-        return DESC_CAP_YOUR;
-    else if (!strcmp("your", desc))
-        return DESC_NOCAP_YOUR;
-    else if (!strcmp("its", desc))
-        return DESC_NOCAP_ITS;
-    else if (!strcmp("worn", desc))
-        return DESC_INVENTORY_EQUIP;
-    else if (!strcmp("inv", desc))
-        return DESC_INVENTORY;
-
-    return DESC_PLAIN;
-}
-
 static int l_item_name(lua_State *ls)
 {
     LUA_ITEM(item, 1);
     if (item)
     {
-        int ndesc = DESC_PLAIN;
+        description_level_type ndesc = DESC_PLAIN;
         if (lua_isstring(ls, 2))
-            ndesc = desc_code(lua_tostring(ls, 2));
+            ndesc = description_type_by_name(lua_tostring(ls, 2));
+        else if (lua_isnumber(ls, 2))
+            ndesc = static_cast<description_level_type>(luaL_checkint(ls, 2));
         bool terse = lua_toboolean(ls, 3);
-        char bufitemname[ITEMNAME_SIZE];
-        item_name(*item, ndesc, bufitemname, terse);
-
-        lua_pushstring(ls, bufitemname);
+        lua_pushstring(ls, item->name(ndesc, terse).c_str());
     }
     else
         lua_pushnil(ls);
@@ -1279,7 +1399,7 @@ static int l_item_weap_skill(lua_State *ls)
         return (0);
 
     int skill = range_skill(*item);
-    if (skill == SK_RANGED_COMBAT)
+    if (skill == SK_THROWING)
         skill = weapon_skill(*item);
     if (skill == SK_FIGHTING)
         return (0);
@@ -1289,7 +1409,7 @@ static int l_item_weap_skill(lua_State *ls)
     return (2);
 }
 
-static int l_item_artifact(lua_State *ls)
+static int l_item_artefact(lua_State *ls)
 {
     LUA_ITEM(item, 1);
     if (!item || !is_valid_item(*item))
@@ -1318,6 +1438,8 @@ static int l_item_branded(lua_State *ls)
     case OBJ_MISSILES:
         branded = get_ammo_brand(*item) != SPMSL_NORMAL;
         break;
+    default:
+        break;
     }
     lua_pushboolean(ls, branded);
     return (1);
@@ -1325,10 +1447,11 @@ static int l_item_branded(lua_State *ls)
 
 static const struct luaL_reg item_lib[] =
 {
-    { "artifact",          l_item_artifact },
+    { "artefact",          l_item_artefact },
     { "branded",           l_item_branded },
     { "class",             l_item_class },
     { "subtype",           l_item_subtype },
+    { "potion_type",       l_item_potion_type },
     { "cursed",            l_item_cursed },
     { "worn",              l_item_worn },
     { "name",              l_item_name },
@@ -1353,7 +1476,7 @@ static const struct luaL_reg item_lib[] =
     { NULL, NULL },
 };
 
-void lua_open_item(lua_State *ls)
+void luaopen_item(lua_State *ls)
 {
     luaL_openlib(ls, "item", item_lib, 0);
 }
@@ -1494,7 +1617,7 @@ static const struct luaL_reg food_lib[] =
     { NULL, NULL },
 };
 
-void lua_open_food(lua_State *ls)
+void luaopen_food(lua_State *ls)
 {
     luaL_openlib(ls, "food", food_lib, 0);
 }
@@ -1505,19 +1628,27 @@ void lua_open_food(lua_State *ls)
 
 static int crawl_mpr(lua_State *ls)
 {
+    if (!crawl_state.io_inited)
+        return (0);
+    
     const char *message = luaL_checkstring(ls, 1);
     if (!message)
         return (0);
 
     int ch = MSGCH_PLAIN;
-    const char *channel = lua_tostring(ls, 2);
-    if (channel)
-        ch = str_to_channel(channel);
-    if (ch == -1)
+    if (lua_isnumber(ls, 2))
+        ch = luaL_checkint(ls, 2);
+    else
+    {
+        const char *channel = lua_tostring(ls, 2);
+        if (channel)
+            ch = str_to_channel(channel);
+    }
+
+    if (ch < 0 || ch >= NUM_MESSAGE_CHANNELS)
         ch = MSGCH_PLAIN;
 
-    mpr(message, ch);
-
+    mpr(message, static_cast<msg_channel_type>(ch));
     return (0);
 }
 
@@ -1704,8 +1835,8 @@ static int crawl_regex_find(lua_State *ls)
 
 static int crawl_regex_gc(lua_State *ls)
 {
-    text_pattern **pattern = 
-            clua_get_userdata< text_pattern* >(ls, REGEX_METATABLE);
+    text_pattern **pattern =
+        static_cast<text_pattern **>( lua_touserdata(ls, 1) );        
     if (pattern)
         delete *pattern;
     return (0);
@@ -1754,8 +1885,8 @@ static int crawl_messf_matches(lua_State *ls)
 
 static int crawl_messf_gc(lua_State *ls)
 {
-    message_filter **pattern = 
-            clua_get_userdata< message_filter* >(ls, REGEX_METATABLE);
+    message_filter **pattern =
+        static_cast<message_filter**>( lua_touserdata(ls, 1) );
     if (pattern)
         delete *pattern;
     return (0);
@@ -1795,10 +1926,58 @@ static int crawl_split(lua_State *ls)
     return (1);
 }
 
+LUARET1(crawl_game_started, boolean, crawl_state.need_save)
+LUARET1(crawl_random2, number, random2( luaL_checkint(ls, 1) ))
+LUARET1(crawl_one_chance_in, boolean, one_chance_in( luaL_checkint(ls, 1) ))
+LUARET1(crawl_random2avg, number,
+        random2avg( luaL_checkint(ls, 1), luaL_checkint(ls, 2) ))
+LUARET1(crawl_random_range, number,
+        random_range( luaL_checkint(ls, 1), luaL_checkint(ls, 2),
+                      lua_isnumber(ls, 3)? luaL_checkint(ls, 3) : 1 ))
+LUARET1(crawl_coinflip, boolean, coinflip())
+
+static int crawl_err_trace(lua_State *ls)
+{
+    const int nargs = lua_gettop(ls);
+    const int err = lua_pcall(ls, nargs - 1, LUA_MULTRET, 0);
+
+    if (err)
+    {
+        // This code from lua.c:traceback() (mostly)
+        const char *errs = lua_tostring(ls, 1);
+        std::string errstr = errs? errs : "";
+        lua_getfield(ls, LUA_GLOBALSINDEX, "debug");
+        if (!lua_istable(ls, -1))
+        {
+            lua_pop(ls, 1);
+            return lua_error(ls);
+        }
+        lua_getfield(ls, -1, "traceback");
+        if (!lua_isfunction(ls, -1))
+        {
+            lua_pop(ls, 2);
+            return lua_error(ls);
+        }
+        lua_pushvalue(ls, 1);
+        lua_pushinteger(ls, 2); // Skip crawl_err_trace and traceback.
+        lua_call(ls, 2, 1);
+
+        // What's on top should be the error.
+        lua_error(ls);
+    }
+
+    return (lua_gettop(ls));
+}
+
 static const struct luaL_reg crawl_lib[] =
 {
     { "mpr",            crawl_mpr },
     { "mesclr",         crawl_mesclr },
+    { "random2",        crawl_random2 },
+    { "one_chance_in",  crawl_one_chance_in },
+    { "random2avg"   ,  crawl_random2avg },
+    { "coinflip",       crawl_coinflip },
+    { "random_range",   crawl_random_range },
     { "redraw_screen",  crawl_redraw_screen },
     { "input_line",     crawl_input_line },
     { "c_input_line",   crawl_c_input_line},
@@ -1817,10 +1996,13 @@ static const struct luaL_reg crawl_lib[] =
     { "message_filter", crawl_message_filter },
     { "trim",           crawl_trim },
     { "split",          crawl_split },
+    { "game_started",   crawl_game_started },
+    { "err_trace",      crawl_err_trace },
+
     { NULL, NULL },
 };
 
-void lua_open_crawl(lua_State *ls)
+void luaopen_crawl(lua_State *ls)
 {
     clua_register_metatable(ls, REGEX_METATABLE, crawl_regex_ops,
                             crawl_regex_gc);
@@ -1833,14 +2015,13 @@ void lua_open_crawl(lua_State *ls)
 ///////////////////////////////////////////////////////////
 // File operations
 
-
 static const struct luaL_reg file_lib[] =
 {
     { "write", CLua::file_write },
     { NULL, NULL },
 };
 
-void lua_open_file(lua_State *ls)
+void luaopen_file(lua_State *ls)
 {
     luaL_openlib(ls, "file", file_lib, 0);
 }
@@ -1877,8 +2058,6 @@ static option_handler handlers[] =
 {
     // Boolean options come first
     { "easy_open", &Options.easy_open, option_hboolean },
-    { "verbose_dump", &Options.verbose_dump, option_hboolean },
-    { "detailed_stat_dump", &Options.detailed_stat_dump, option_hboolean },
     { "colour_map", &Options.colour_map, option_hboolean },
     { "clean_map", &Options.clean_map, option_hboolean },
     { "show_uncursed", &Options.show_uncursed, option_hboolean },
@@ -1961,8 +2140,8 @@ static int option_set(lua_State *ls)
     return (0);
 }
 
-#define OPT_METATABLE "options.optaccess"
-void lua_open_options(lua_State *ls)
+#define OPT_METATABLE "clua_metatable_optaccess"
+void luaopen_options(lua_State *ls)
 {
     int top = lua_gettop(ls);
 
@@ -1985,8 +2164,7 @@ void lua_open_options(lua_State *ls)
 
     luaL_getmetatable(ls, OPT_METATABLE);
     lua_setmetatable(ls, -2);
-
-    clua.setglobal("options");
+    lua_setglobal(ls, "options");
 }
 
 /////////////////////////////////////////////////////////////////////
@@ -2001,9 +2179,7 @@ struct MonsterWrap
 
 static int l_mons_name(lua_State *ls, monsters *mons, const char *attr)
 {
-    char monnamebuf[ITEMNAME_SIZE];     // Le sigh.
-    moname(mons->type, true, DESC_PLAIN, monnamebuf);
-    lua_pushstring(ls, monnamebuf);
+    lua_pushstring(ls, mons_type_name(mons->type, DESC_PLAIN).c_str());
     return (1);
 }
 
@@ -2098,7 +2274,20 @@ static void push_monster(lua_State *ls, monsters *mons)
     mw->mons = mons;
 }
 
-void lua_open_monsters(lua_State *ls)
+void clua_push_map(lua_State *ls, map_def *map)
+{
+    map_def **mapref = clua_new_userdata<map_def *>(ls, MAP_METATABLE);
+    *mapref = map;
+}
+
+void clua_push_dgn_event(lua_State *ls, const dgn_event *devent)
+{
+    const dgn_event **de =
+        clua_new_userdata<const dgn_event *>(ls, DEVENT_METATABLE);
+    *de = devent;
+}
+
+void luaopen_monsters(lua_State *ls)
 {
     luaL_newmetatable(ls, MONS_METATABLE);
     lua_pushstring(ls, "__index");
@@ -2118,7 +2307,7 @@ void lua_open_monsters(lua_State *ls)
 
 #define PATTERN_FLUSH_CEILING 100
 
-typedef CHMAP<std::string, text_pattern> pattern_map;
+typedef std::map<std::string, text_pattern> pattern_map;
 static pattern_map pattern_cache;
 
 static text_pattern &get_text_pattern(const std::string &s, bool checkcase)
@@ -2131,7 +2320,7 @@ static text_pattern &get_text_pattern(const std::string &s, bool checkcase)
         pattern_cache.clear();
 
     pattern_cache[s] = text_pattern(s, !checkcase);
-    return pattern_cache[s];
+    return (pattern_cache[s]);
 }
 
 static int lua_pmatch(lua_State *ls)
@@ -2153,13 +2342,12 @@ static int lua_pmatch(lua_State *ls)
     return (1);
 }
 
-void lua_open_globals(lua_State *ls)
+void luaopen_globals(lua_State *ls)
 {
     lua_pushcfunction(ls, lua_pmatch);
     lua_setglobal(ls, "pmatch");
 }
 
-
 ////////////////////////////////////////////////////////////////////////
 // lua_text_pattern
 
@@ -2326,4 +2514,169 @@ bool lua_text_pattern::translate() const
     return translated;
 }
 
-#endif // CLUA_BINDINGS
+//////////////////////////////////////////////////////////////////////////
+
+lua_call_throttle::lua_clua_map lua_call_throttle::lua_map;
+
+// A panic function for the Lua interpreter, usually called when it
+// runs out of memory when trying to load a file or a chunk of Lua from
+// an unprotected Lua op. The only cases of unprotected Lua loads are
+// loads of Lua code from .crawlrc, which is read at start of game.
+//
+// If there's an inordinately large .crawlrc (we're talking seriously
+// massive here) that wants more memory than we're willing to give
+// Lua, then the game will save and exit until the .crawlrc is fixed.
+//
+// Lua can also run out of memory during protected script execution,
+// such as when running a macro or some other game hook, but in such
+// cases the Lua interpreter will throw an exception instead of
+// panicking.
+//
+static int clua_panic(lua_State *ls)
+{
+    if (crawl_state.need_save && !crawl_state.saving_game
+        && !crawl_state.updating_scores)
+    {
+        save_game(true);
+    }
+    return (0);
+}
+
+static void *clua_allocator(void *ud, void *ptr, size_t osize, size_t nsize)
+{
+    CLua *cl = static_cast<CLua *>( ud );
+    cl->memory_used += nsize - osize;
+
+    if (nsize > osize && cl->memory_used >= CLUA_MAX_MEMORY_USE * 1024
+           && cl->mixed_call_depth)
+        return (NULL);
+    
+    if (!nsize)
+    {
+        free(ptr);
+        return (NULL);
+    }
+    else
+        return (realloc(ptr, nsize));
+}
+
+static void clua_throttle_hook(lua_State *ls, lua_Debug *dbg)
+{
+    CLua *lua = lua_call_throttle::find_clua(ls);
+
+    // Co-routines can create a new Lua state; in such cases, we must
+    // fudge it.
+    if (!lua)
+        lua = &clua;
+    
+    if (lua)
+    {
+        if (!lua->throttle_sleep_ms)
+            lua->throttle_sleep_ms = lua->throttle_sleep_start;
+        else if (lua->throttle_sleep_ms < lua->throttle_sleep_end)
+            lua->throttle_sleep_ms *= 2;
+
+        ++lua->n_throttle_sleeps;
+
+        delay(lua->throttle_sleep_ms);
+
+        // Try to kill the annoying script.
+        if (lua->n_throttle_sleeps > CLua::MAX_THROTTLE_SLEEPS)
+        {
+            lua->n_throttle_sleeps = CLua::MAX_THROTTLE_SLEEPS;
+            luaL_error(ls, BUGGY_SCRIPT_ERROR);
+        }
+    }
+}
+
+lua_call_throttle::lua_call_throttle(CLua *_lua)
+    : lua(_lua)
+{
+    lua->init_throttle();
+    if (!lua->mixed_call_depth++)
+        lua_map[lua->state()] = lua;
+}
+
+lua_call_throttle::~lua_call_throttle()
+{
+    if (!--lua->mixed_call_depth)
+        lua_map.erase(lua->state());
+}
+
+CLua *lua_call_throttle::find_clua(lua_State *ls)
+{
+    lua_clua_map::iterator i = lua_map.find(ls);
+    return (i != lua_map.end()? i->second : NULL);
+}
+
+// This function is a replacement for Lua's in-built pcall function. It behaves
+// like pcall in all respects (as documented in the Lua 5.1 reference manual),
+// but does not allow the Lua chunk/script to catch errors thrown by the
+// Lua-throttling code. This is necessary so that we can interrupt scripts that
+// are hogging CPU.
+//
+// If we did not intercept pcall, the script could do the equivalent
+// of this:
+// 
+//    while true do
+//      pcall(function () while true do end end)
+//    end
+//
+// And there's a good chance we wouldn't be able to interrupt the
+// deadloop because our errors would get caught by the pcall (more
+// levels of nesting would just increase the chance of the script
+// beating our throttling).
+//
+static int clua_guarded_pcall(lua_State *ls)
+{
+    const int nargs = lua_gettop(ls);
+    const int err = lua_pcall(ls, nargs - 1, LUA_MULTRET, 0);
+
+    if (err)
+    {
+        const char *errs = lua_tostring(ls, 1);
+        if (!errs || strstr(errs, BUGGY_SCRIPT_ERROR))
+            luaL_error(ls, errs? errs : BUGGY_PCALL_ERROR);
+    }
+    
+    lua_pushboolean(ls, !err);
+    lua_insert(ls, 1);
+
+    return (lua_gettop(ls));
+}
+
+static int clua_loadfile(lua_State *ls)
+{
+    const char *file = luaL_checkstring(ls, 1);
+    if (!file)
+        return (0);
+
+    const int err = CLua::loadfile(ls, file, !CLua::is_managed_vm(ls));
+    if (err)
+    {
+        const int place = lua_gettop(ls);
+        lua_pushnil(ls);
+        lua_insert(ls, place);
+        return (2);
+    }
+    return (1);
+}
+
+static int clua_dofile(lua_State *ls)
+{
+    const char *file = luaL_checkstring(ls, 1);
+    if (!file)
+        return (0);
+
+    const int err = CLua::loadfile(ls, file, !CLua::is_managed_vm(ls));
+    if (err)
+        return (lua_error(ls));
+
+    lua_call(ls, 0, LUA_MULTRET);
+    return (lua_gettop(ls));
+}
+
+std::string quote_lua_string(const std::string &s)
+{
+    return replace_all_of(replace_all_of(s, "\\", "\\\\"), "\"", "\\\"");
+}
