@@ -2,7 +2,7 @@
  *  File:     delay.cc
  *  Summary:  Functions for handling multi-turn actions.    
  *
- *  Modified for Crawl Reference by $Author: dshaligram $ on $Date: 2007-06-06T19:40:56.965493Z $
+ *  Modified for Crawl Reference by $Author: dshaligram $ on $Date: 2007-12-02T11:35:00.163099Z $
  *
  *  Change History (most recent first):
  *
@@ -35,10 +35,13 @@
 #include "output.h"
 #include "player.h"
 #include "randart.h"
+#include "religion.h"
 #include "spl-util.h"
+#include "stash.h"
 #include "stuff.h"
 #include "travel.h"
 #include "tutorial.h"
+#include "view.h"
 
 extern std::vector<SelItem> items_for_multidrop;
 
@@ -51,7 +54,7 @@ static void finish_delay(const delay_queue_item &delay);
 // other delays can be spawned while this delay is running. If is_parent_delay
 // returns true, new delays will be pushed immediately to the front of the
 // delay in question, rather than at the end of the queue.
-static bool is_parent_delay(int delay)
+static bool is_parent_delay(delay_type delay)
 {
     // Interlevel travel can do upstairs/downstairs delays.
     // Lua macros can in theory perform any of the other delays,
@@ -97,7 +100,7 @@ static void clear_pending_delays()
     }
 }
 
-void start_delay( int type, int turns, int parm1, int parm2 )
+void start_delay( delay_type type, int turns, int parm1, int parm2 )
 /***********************************************************/
 {
     delay_queue_item delay;
@@ -148,6 +151,9 @@ void stop_delay( void )
         delay.type == DELAY_BUTCHER
         && (you.delay_queue.size() >= 2
             && you.delay_queue[1].type == DELAY_WEAPON_SWAP);
+
+    const int butcher_swap_weapon =
+        butcher_swap_warn? you.delay_queue[1].parm1 : -10;
     
     // At the very least we can remove any queued delays, right 
     // now there is no problem with doing this... note that
@@ -156,15 +162,23 @@ void stop_delay( void )
     // until it is done... it merely chains up individual actions
     // into a single action.  -- bwr
     clear_pending_delays();
-
+    
     switch (delay.type)
     {
     case DELAY_BUTCHER:
         // Corpse keeps track of work in plus2 field, see handle_delay() -- bwr
         if (butcher_swap_warn)
-            mpr("You stop butchering the corpse; not switching back to "
-                "primary weapon.",
-                MSGCH_WARN);
+        {
+            std::string weapon;
+            if (butcher_swap_weapon == -1)
+                weapon = "unarmed combat";
+            else
+                weapon = "your " +
+                    you.inv[butcher_swap_weapon].name(DESC_BASENAME);
+            mprf(MSGCH_WARN, "You stop butchering the corpse; not switching "
+                 "back to %s.",
+                 weapon.c_str());
+        }
         else
             mpr( "You stop butchering the corpse." );
         pop_delay();
@@ -251,13 +265,19 @@ void stop_delay( void )
         update_turn_count();
 }
 
+void stop_butcher_delay()
+{
+    if (current_delay_action() == DELAY_BUTCHER)
+        stop_delay();
+}
+
 bool you_are_delayed( void )
 /**************************/
 {
     return (!you.delay_queue.empty());
 }
 
-int current_delay_action( void )
+delay_type current_delay_action( void )
 /******************************/
 {
     return (you_are_delayed() ? you.delay_queue.front().type 
@@ -288,8 +308,6 @@ bool is_being_butchered(const item_def &item)
 void handle_delay( void )
 /***********************/
 {
-    char  str_pass[ ITEMNAME_SIZE ];
-
     if (!you_are_delayed())
         return;
 
@@ -370,10 +388,8 @@ void handle_delay( void )
     if (delay.duration > 0)
     {
 #if DEBUG_DIAGNOSTICS
-        snprintf( info, INFO_SIZE, "Delay type: %d (%s), duration: %d", 
-                  delay.type, delay_name(delay.type), delay.duration ); 
-
-        mpr( info, MSGCH_DIAGNOSTICS );
+        mprf(MSGCH_DIAGNOSTICS, "Delay type: %d (%s), duration: %d", 
+             delay.type, delay_name(delay.type), delay.duration ); 
 #endif
         // delay.duration-- *must* be done before multidrop, because
         // multidrop is now a parent delay, which means other delays
@@ -385,16 +401,12 @@ void handle_delay( void )
         switch ( delay.type )
         {
         case DELAY_ARMOUR_ON:
-            in_name( delay.parm1, DESC_NOCAP_YOUR, str_pass );
-            snprintf( info, INFO_SIZE,
-                      "You continue putting on %s.", str_pass );
-            mpr(info, MSGCH_MULTITURN_ACTION);
+            mprf(MSGCH_MULTITURN_ACTION, "You continue putting on %s.",
+                 you.inv[delay.parm1].name(DESC_NOCAP_YOUR).c_str());
             break;
         case DELAY_ARMOUR_OFF:
-            in_name( delay.parm1, DESC_NOCAP_YOUR, str_pass );
-            snprintf( info, INFO_SIZE,
-                      "You continue taking off %s.", str_pass );
-            mpr(info, MSGCH_MULTITURN_ACTION);
+            mprf(MSGCH_MULTITURN_ACTION, "You continue taking off %s.",
+                 you.inv[delay.parm1].name(DESC_NOCAP_YOUR).c_str());
             break;
         case DELAY_BUTCHER:
             mpr("You continue butchering the corpse.",
@@ -425,7 +437,6 @@ void handle_delay( void )
 
 static void finish_delay(const delay_queue_item &delay)
 {
-    char  str_pass[ ITEMNAME_SIZE ];
     switch (delay.type)
     {
     case DELAY_WEAPON_SWAP:
@@ -442,9 +453,8 @@ static void finish_delay(const delay_queue_item &delay)
 
     case DELAY_ARMOUR_OFF:
     {
-        in_name( delay.parm1, DESC_NOCAP_YOUR, str_pass ); 
-        snprintf( info, INFO_SIZE, "You finish taking off %s.", str_pass );
-        mpr(info);
+        mprf("You finish taking off %s.",
+             you.inv[delay.parm1].name(DESC_NOCAP_YOUR).c_str());
 
         const equipment_type slot = 
             get_armour_slot( you.inv[delay.parm1] );
@@ -489,12 +499,15 @@ static void finish_delay(const delay_queue_item &delay)
 
         unwear_armour( delay.parm1 );
 
-        you.redraw_armour_class = 1;
-        you.redraw_evasion = 1;
+        you.redraw_armour_class = true;
+        you.redraw_evasion = true;
         break;
     }
     case DELAY_EAT:
-        mpr( "You finish eating." );
+        if (you.species == SP_VAMPIRE)
+           mpr( "You finish drinking." );
+        else 
+           mpr( "You finish eating." );
         // For chunks, warn the player if they're not getting much
         // nutrition.
         if (delay.parm1)
@@ -503,7 +516,7 @@ static void finish_delay(const delay_queue_item &delay)
 
     case DELAY_MEMORISE:
         mpr( "You finish memorising." );
-        add_spell_to_memory( delay.parm1 );
+        add_spell_to_memory( static_cast<spell_type>( delay.parm1 ) );
         break; 
 
     case DELAY_PASSWALL:
@@ -524,8 +537,6 @@ static void finish_delay(const delay_queue_item &delay)
             case DNGN_METAL_WALL:
             case DNGN_GREEN_CRYSTAL_WALL:
             case DNGN_WAX_WALL:
-            case DNGN_SILVER_STATUE:
-            case DNGN_ORANGE_CRYSTAL_STATUE:
                 ouch(1 + you.hp, 0, KILLED_BY_PETRIFICATION);
                 break;
 
@@ -562,12 +573,19 @@ static void finish_delay(const delay_queue_item &delay)
         if (is_valid_item(item) && item.base_type == OBJ_CORPSES)
         {
             mprf("You finish %s the corpse into pieces.",
-                 (you.species==SP_TROLL || you.species == SP_GHOUL) ? "ripping"
-                 : "chopping");
+                 (you.has_usable_claws() || you.mutation[MUT_FANGS] == 3) ?
+                 "ripping" : "chopping");
 
+            if (you.species == SP_VAMPIRE &&
+                (!god_likes_butchery(you.religion) ||
+                 !you.duration[DUR_PRAYER]))
+            {
+                 mpr("What a waste.");
+            }                
             turn_corpse_into_chunks( mitm[ delay.parm1 ] );
 
-            if (you.berserker && you.berserk_penalty != NO_BERSERK_PENALTY)
+            if (you.duration[DUR_BERSERKER] &&
+                you.berserk_penalty != NO_BERSERK_PENALTY)
             {
                 mpr("You enjoyed that.");
                 you.berserk_penalty = 0;
@@ -577,6 +595,7 @@ static void finish_delay(const delay_queue_item &delay)
         {
             mpr("You stop butchering the corpse.");
         }
+        stashes.update_stash(); // Stash-track the generated item(s)
         break;
     }
 
@@ -593,8 +612,7 @@ static void finish_delay(const delay_queue_item &delay)
         // so that temporary brands and such are cleared. -- bwr
         if (delay.parm1 == you.equip[EQ_WEAPON])
         {   
-            unwield_item( delay.parm1 );
-            you.equip[EQ_WEAPON] = -1;
+            unwield_item();
             canned_msg( MSG_EMPTY_HANDED );
         }
 
@@ -606,24 +624,18 @@ static void finish_delay(const delay_queue_item &delay)
         }
         else
         {
-            quant_name( you.inv[ delay.parm1 ], delay.parm2, 
-                        DESC_NOCAP_A, str_pass );
-
-            snprintf( info, INFO_SIZE, "You drop %s.", str_pass );
-            mpr(info);
-
+            mprf("You drop %s.", quant_name(you.inv[delay.parm1], delay.parm2,
+                                            DESC_NOCAP_A).c_str());
             dec_inv_item_quantity( delay.parm1, delay.parm2 );
         }
         break;
 
     case DELAY_ASCENDING_STAIRS:
         up_stairs();
-        untag_followers();
         break;
 
     case DELAY_DESCENDING_STAIRS:
         down_stairs( delay.parm1 );
-        untag_followers();
         break;
 
     case DELAY_INTERRUPTIBLE:
@@ -649,7 +661,7 @@ static void armour_wear_effects(const int item_slot)
     item_def &arm = you.inv[item_slot];
 
     set_ident_flags(arm, ISFLAG_EQ_ARMOUR_MASK );
-    mprf("You finish putting on %s.", item_name(arm, DESC_NOCAP_YOUR));
+    mprf("You finish putting on %s.", arm.name(DESC_NOCAP_YOUR).c_str());
 
     const equipment_type eq_slot = get_armour_slot(arm);
 
@@ -700,7 +712,7 @@ static void armour_wear_effects(const int item_slot)
         {
         case SPARM_RUNNING:
             mprf("You feel quick%s.",
-                    (you.species == SP_NAGA || you.species == SP_CENTAUR) 
+                    (you.species == SP_NAGA || you.mutation[MUT_HOOVES])
                     ? "" : " on your feet");
             break;
 
@@ -721,7 +733,7 @@ static void armour_wear_effects(const int item_slot)
             break;
 
         case SPARM_DARKNESS:
-            if (!you.invis)
+            if (!you.duration[DUR_INVIS])
                 mpr("You become transparent for a moment.");
             break;
 
@@ -779,9 +791,11 @@ static void armour_wear_effects(const int item_slot)
     if (is_random_artefact( arm ))
         use_randart( item_slot );
 
-    if (item_cursed( arm )) {
+    if (item_cursed( arm ))
+    {
         mpr( "Oops, that feels deathly cold." );
         learned_something_new(TUT_YOU_CURSED);
+        xom_is_stimulated(128);
     }
 
     if (eq_slot == EQ_SHIELD)
@@ -808,6 +822,11 @@ static command_type get_running_command()
         }
         return CMD_MOVE_NOWHERE;
     }
+    else if (Options.travel_delay > 0)
+    {
+        delay(Options.travel_delay);
+    }
+        
     return direction_to_command( you.running.x, you.running.y );
 }
 
@@ -835,6 +854,8 @@ static void handle_run_delays(const delay_queue_item &delay)
     case DELAY_TRAVEL:
         cmd = travel();
         break;
+    default:
+        break;
     }
 
     if (cmd != CMD_NO_CMD)
@@ -883,7 +904,7 @@ void run_macro(const char *macroname)
     if (!clua.callbooleanfn(false, "c_macro", "s", macroname))
     {
         if (clua.error.length())
-            mpr(clua.error.c_str());
+            mprf(MSGCH_WARN, "Lua error: %s", clua.error.c_str());
 
         stop_delay();
     }
@@ -896,7 +917,7 @@ void run_macro(const char *macroname)
 #endif
 }
 
-// Returns true if the delay should be interrupted, false if the user function
+// Returns 1 if the delay should be interrupted, 0 if the user function
 // had no opinion on the matter, -1 if the delay should not be interrupted.
 static int userdef_interrupt_activity( const delay_queue_item &idelay,
                                         activity_interrupt_type ai, 
@@ -978,11 +999,61 @@ inline static void monster_warning(activity_interrupt_type ai,
     if ( ai == AI_SEE_MONSTER && is_run_delay(atype) )
     {
         const monsters* mon = static_cast<const monsters*>(at.data);
+        if (!mon->visible())
+            return;
 #ifndef DEBUG_DIAGNOSTICS
-        mprf(MSGCH_WARN, "%s comes into view.", ptr_monam(mon, DESC_CAP_A));
+        if (at.context != "uncharm")
+        {
+            // Only say "comes into view" if the monster wasn't in view
+            // during the previous turn.
+            if (testbits(mon->flags, MF_WAS_IN_VIEW))
+            {
+                switch(random2(4))
+                {
+                   case 0:
+                     mprf(MSGCH_WARN, "%s's nearness makes you nervous.",
+                          mon->name(DESC_CAP_THE).c_str());
+                     break;
+                   case 1:
+                     mprf(MSGCH_WARN, "%s is too close now for your liking.",
+                          mon->name(DESC_CAP_THE).c_str());
+                     break;
+                   case 2:
+                     mprf(MSGCH_WARN,
+                          "You feel that %s is too close now for comfort.",
+                          mon->name(DESC_NOCAP_THE).c_str());
+                     break;
+                   default:
+                     mprf(MSGCH_WARN,
+                          "%s's presence makes you stop your activity.",
+                          mon->name(DESC_CAP_THE).c_str());
+                     break;
+                }
+            }
+            else
+            {
+                const std::string mweap =
+                    get_monster_desc(mon, false, DESC_NONE);
+                std::string text = mon->name(DESC_CAP_A) + " comes into view.";
+                if (!mweap.empty())
+                    text += " " + mon->pronoun(PRONOUN_CAP)
+                        + " is" + mweap + ".";
+                print_formatted_paragraph(text,
+                                          get_number_of_cols(),
+                                          MSGCH_WARN);
+            }
+        }
+
+        if (Options.tutorial_left)
+        {
+            // enforce that this message comes first
+            tutorial_first_monster(*mon);
+            if (get_mons_colour(mon) != mon->colour)
+                learned_something_new(TUT_MONSTER_BRAND);
+        }
 #else
         formatted_string fs( channel_to_colour(MSGCH_WARN) );
-        fs.cprintf("%s (", ptr_monam(mon, DESC_PLAIN));
+        fs.cprintf("%s (", mon->name(DESC_PLAIN, true).c_str());
         fs.add_glyph( mon );
         fs.cprintf(") in view: (%d,%d), see_grid: %s",
              mon->x, mon->y,
@@ -998,25 +1069,30 @@ static void paranoid_option_disable( activity_interrupt_type ai,
     if (ai == AI_HIT_MONSTER || ai == AI_MONSTER_ATTACKS)
     {
         const monsters* mon = static_cast<const monsters*>(at.data);
-        if (mon && !player_monster_visible(mon))
+        if (mon && !player_monster_visible(mon) && !mons_is_submerged(mon))
         {
             std::vector<std::string> deactivatees;
+            std::vector<std::string> restart;
             if (Options.autoprayer_on)
             {
                 deactivatees.push_back("autoprayer");
                 Options.autoprayer_on = false;
+                restart.push_back("Ctrl+V");
             }
 
             if (Options.autopickup_on && Options.safe_autopickup)
             {
                 deactivatees.push_back("autopickup");
                 Options.autopickup_on = false;
+                restart.push_back("Ctrl+A");
             }
 
             if (!deactivatees.empty())
-                mprf(MSGCH_WARN, "Deactivating %s.",
+                mprf(MSGCH_WARN, "Deactivating %s; reactivate with %s.",
                       comma_separated_line(deactivatees.begin(),
-                                           deactivatees.end()).c_str());
+                                           deactivatees.end()).c_str(),
+                      comma_separated_line(restart.begin(),
+                                           restart.end()).c_str());
         }
     }
 }
