@@ -3,7 +3,7 @@
  *  Summary:    Misc stuff.
  *  Written by: Linley Henzell
  *
- *  Modified for Crawl Reference by $Author: dshaligram $ on $Date: 2007-06-06T19:40:56.965493Z $
+ *  Modified for Crawl Reference by $Author: ennewalker $ on $Date: 2007-11-23T02:25:44.246247Z $
  *
  *  Change History (most recent first):
  *
@@ -21,11 +21,15 @@
 
 #include "beam.h"
 #include "direct.h"
-#include "dungeon.h"
+#include "food.h"
 #include "hiscores.h"
+#include "it_use2.h"
+#include "item_use.h"
 #include "itemname.h"
 #include "itemprop.h"
 #include "items.h"
+#include "makeitem.h"
+#include "message.h"
 #include "misc.h"
 #include "monplace.h"
 #include "monstuff.h"
@@ -36,12 +40,15 @@
 #include "ouch.h"
 #include "player.h"
 #include "randart.h"
+#include "religion.h"
 #include "skills2.h"
 #include "spells3.h"
 #include "spells4.h"
 #include "spl-book.h"
 #include "spl-util.h"
 #include "stuff.h"
+#include "terrain.h"
+#include "traps.h"
 #include "view.h"
 
 // torment_monsters is called with power 0 because torment is
@@ -86,7 +93,7 @@ int torment_monsters(int x, int y, int pow, int caster)
                     aux = "Symbol of Torment";
                     break;
                 case TORMENT_SPWLD:
-                    // FIXME: If we ever make any other weapon / randart
+                    // XXX: If we ever make any other weapon / randart
                     // eligible to torment, this will be incorrect.
                     aux = "Sceptre of Torment";
                     break;
@@ -135,7 +142,7 @@ static std::string who_banished(const std::string &who)
     return (who.empty()? who : " (" + who + ")");
 }
 
-void banished(int gate_type, const std::string &who)
+void banished(dungeon_feature_type gate_type, const std::string &who)
 {
 #ifdef DGL_MILESTONES
     if (gate_type == DNGN_ENTER_ABYSS)
@@ -152,9 +159,14 @@ void banished(int gate_type, const std::string &who)
         take_note(Note(NOTE_USER_NOTE, 0, 0, what.c_str()), true);
     }
 
+    // no longer held in net
+    clear_trapping_net();
+    
     down_stairs(you.your_level, gate_type);  // heh heh
-    untag_followers(); // safety
-}                               // end banished()
+
+    if (gate_type == DNGN_ENTER_ABYSS || gate_type == DNGN_ENTER_PANDEMONIUM)
+        xom_is_stimulated(255);
+}
 
 bool forget_spell(void)
 {
@@ -189,12 +201,12 @@ bool forget_spell(void)
 bool lose_stat(unsigned char which_stat, unsigned char stat_loss, bool force)
 {
     bool statLowered = false;   // must initialize to false {dlb}
-    char *ptr_stat = 0;         // NULL {dlb}
-    char *ptr_redraw = 0;       // NULL {dlb}
+    char *ptr_stat = NULL;
+    bool *ptr_redraw = NULL;
     char newValue = 0;          // holds new value, for comparison to old {dlb}
 
     // begin outputing message: {dlb}
-    strcpy(info, "You feel ");
+    std::string msg = "You feel ";
 
     // set pointers to appropriate variables: {dlb}
     if (which_stat == STAT_RANDOM)
@@ -203,19 +215,19 @@ bool lose_stat(unsigned char which_stat, unsigned char stat_loss, bool force)
     switch (which_stat)
     {
     case STAT_STRENGTH:
-        strcat(info, "weakened");
+        msg += "weakened";
         ptr_stat = &you.strength;
         ptr_redraw = &you.redraw_strength;
         break;
 
     case STAT_DEXTERITY:
-        strcat(info, "clumsy");
+        msg += "clumsy";
         ptr_stat = &you.dex;
         ptr_redraw = &you.redraw_dexterity;
         break;
 
     case STAT_INTELLIGENCE:
-        strcat(info, "dopey");
+        msg += "dopey";
         ptr_stat = &you.intel;
         ptr_redraw = &you.redraw_intelligence;
         break;
@@ -255,11 +267,11 @@ bool lose_stat(unsigned char which_stat, unsigned char stat_loss, bool force)
 
     // a warning to player that s/he cut it close: {dlb}
     if (!statLowered)
-        strcat(info, " for a moment");
+        msg += " for a moment";
 
     // finish outputting message: {dlb}
-    strcat(info, ".");
-    mpr(info);
+    msg += ".";
+    mpr(msg.c_str());
 
     return (statLowered);
 }                               // end lose_stat()
@@ -297,17 +309,14 @@ void direct_effect(struct bolt &pbolt)
     case DMNBM_BRAIN_FEED:
         // lose_stat() must come last {dlb}
         if (one_chance_in(3) && lose_stat(STAT_INTELLIGENCE, 1))
+        {
             mpr("Something feeds on your intellect!");
+            xom_is_stimulated(50);
+        }
         else
+        {
             mpr("Something tries to feed on your intellect!");
-        break;
-
-    case DMNBM_MUTATION:
-        mpr("Strange energies course through your body.");
-        if (one_chance_in(5))
-            mutate(100);
-        else
-            give_bad_mutation();
+        }
         break;
     }
 
@@ -353,12 +362,13 @@ void mons_direct_effect(struct bolt &pbolt, int i)
         break;
 
     case DMNBM_MUTATION:
-        if (mons_holiness(monster) != MH_NATURAL)
+        if (mons_holiness(monster) != MH_NATURAL ||
+            mons_immune_magic(monster))
             simple_monster_message(monster, " is unaffected.");
         else if (check_mons_resist_magic( monster, pbolt.ench_power ))
             simple_monster_message(monster, " resists.");
         else
-            monster_polymorph(monster, RANDOM_MONSTER, 100);
+            monster_polymorph(monster, RANDOM_MONSTER);
         break;
     }
 
@@ -376,17 +386,13 @@ void mons_direct_effect(struct bolt &pbolt, int i)
 
 void random_uselessness(unsigned char ru, unsigned char sc_read_2)
 {
-    char wc[30];
     int temp_rand = 0;          // probability determination {dlb}
 
     switch (ru)
     {
     case 0:
-        strcpy(info, "The dust glows a ");
-        weird_colours(random2(256), wc);
-        strcat(info, wc);
-        strcat(info, " colour!");
-        mpr(info);
+        msg::stream << "The dust glows a " << weird_glow_colour()
+                    << " colour!" << std::endl;
         break;
 
     case 1:
@@ -397,14 +403,9 @@ void random_uselessness(unsigned char ru, unsigned char sc_read_2)
     case 2:
         if (you.equip[EQ_WEAPON] != -1)
         {
-            char str_pass[ ITEMNAME_SIZE ];
-            in_name(you.equip[EQ_WEAPON], DESC_CAP_YOUR, str_pass);
-            strcpy(info, str_pass);
-            strcat(info, " glows ");
-            weird_colours(random2(256), wc);
-            strcat(info, wc);
-            strcat(info, " for a moment.");
-            mpr(info);
+            msg::stream << you.inv[you.equip[EQ_WEAPON]].name(DESC_CAP_YOUR)
+                        << " glows " << weird_glow_colour()
+                        << " for a moment." << std::endl;
         }
         else
         {
@@ -413,39 +414,31 @@ void random_uselessness(unsigned char ru, unsigned char sc_read_2)
         break;
 
     case 3:
-        strcpy(info, "You hear the distant roaring of an enraged ");
-
         temp_rand = random2(8);
-
-        strcat(info, (temp_rand == 0) ? "frog"          :
-                     (temp_rand == 1) ? "pill bug"      :
-                     (temp_rand == 2) ? "millipede"     :
-                     (temp_rand == 3) ? "eggplant"      :
-                     (temp_rand == 4) ? "albino dragon" :
-                     (temp_rand == 5) ? "dragon"        :
-                     (temp_rand == 6) ? "human"
-                                      : "slug");
-
-        strcat(info, "!");
-        mpr(info);
+        mprf("You hear the distant roaring of an enraged %s!",
+             (temp_rand == 0) ? "frog"          :
+             (temp_rand == 1) ? "pill bug"      :
+             (temp_rand == 2) ? "millipede"     :
+             (temp_rand == 3) ? "eggplant"      :
+             (temp_rand == 4) ? "albino dragon" :
+             (temp_rand == 5) ? "dragon"        :
+             (temp_rand == 6) ? "human"
+                              : "slug");
         break;
 
     case 4:
         if (player_can_smell())
         {
-            strcpy(info, "You smell ");
-
             temp_rand = random2(8);
-
-            strcat(info, (temp_rand == 0) ? "coffee."          :
-                         (temp_rand == 1) ? "salt."            :
-                         (temp_rand == 2) ? "burning hair!"    :
-                         (temp_rand == 3) ? "baking bread."    :
-                         (temp_rand == 4) ? "something weird." :
-                         (temp_rand == 5) ? "wet wool."        :
-                         (temp_rand == 6) ? "sulphur."
-                                          : "fire and brimstone!");
-            mpr(info);
+            mprf("You smell %s",
+                 (temp_rand == 0) ? "coffee."          :
+                 (temp_rand == 1) ? "salt."            :
+                 (temp_rand == 2) ? "burning hair!"    :
+                 (temp_rand == 3) ? "baking bread."    :
+                 (temp_rand == 4) ? "something weird." :
+                 (temp_rand == 5) ? "wet wool."        :
+                 (temp_rand == 6) ? "sulphur."
+                                  : "fire and brimstone!");
         }
         break;
 
@@ -454,14 +447,11 @@ void random_uselessness(unsigned char ru, unsigned char sc_read_2)
         break;
 
     case 6:
-        strcpy(info, "Your ");
-
         temp_rand = random2(3);
-
-        strcat(info, (temp_rand == 0) ? "ears itch."   :
-                     (temp_rand == 1) ? "brain hurts!"
-                                      : "nose twitches suddenly!");
-        mpr(info);
+        mprf("Your %s",
+             (temp_rand == 0) ? "ears itch."   :
+             (temp_rand == 1) ? "brain hurts!"
+                              : "nose twitches suddenly!");
         break;
 
     case 7:
@@ -470,87 +460,44 @@ void random_uselessness(unsigned char ru, unsigned char sc_read_2)
         break;
 
     case 8:
-        strcpy(info, "You hear ");
-
         temp_rand = random2(9);
-
-        strcat(info, (temp_rand == 0) ? "snatches of song"                 :
-                     (temp_rand == 1) ? "a voice call someone else's name" :
-                     (temp_rand == 2) ? "a very strange noise"             :
-                     (temp_rand == 3) ? "roaring flame"                    :
-                     (temp_rand == 4) ? "a very strange noise indeed"      :
-                     (temp_rand == 5) ? "the chiming of a distant gong"    :
-                     (temp_rand == 6) ? "the bellowing of a yak"           :
-                     (temp_rand == 7) ? "a crunching sound"
-                                      : "the tinkle of an enormous bell");
-        strcat(info, ".");
-        mpr(info);
+        mprf("You hear %s.",
+             (temp_rand == 0) ? "snatches of song"                 :
+             (temp_rand == 1) ? "a voice call someone else's name" :
+             (temp_rand == 2) ? "a very strange noise"             :
+             (temp_rand == 3) ? "roaring flame"                    :
+             (temp_rand == 4) ? "a very strange noise indeed"      :
+             (temp_rand == 5) ? "the chiming of a distant gong"    :
+             (temp_rand == 6) ? "the bellowing of a yak"           :
+             (temp_rand == 7) ? "a crunching sound"
+                              : "the tinkle of an enormous bell");
         break;
     }
 
     return;
 }                               // end random_uselessness()
 
-bool acquirement(unsigned char force_class, int agent)
+static int find_acquirement_subtype(object_class_type class_wanted,
+                                    int &unique)
 {
-    int thing_created = 0;
+    ASSERT(class_wanted != OBJ_RANDOM);
+    
+    int type_wanted = OBJ_RANDOM;
     int iteration = 0;
 
-    // Remember lava!
-    unsigned char class_wanted = OBJ_RANDOM;
-    unsigned char type_wanted = OBJ_RANDOM;
-
-    unsigned char unique = 1;
-    unsigned char acqc = 0;
-
     const int max_has_value = 100;
     FixedVector< int, max_has_value > already_has;
 
-    char best_spell = 99;
-    char best_any = 99;
-    unsigned char keyin;
+    skill_type best_spell = SK_NONE;
+    skill_type best_any   = SK_NONE;
 
-    for (acqc = 0; acqc < max_has_value; acqc++)
-        already_has[acqc] = 0;
+    already_has.init(0);
 
     int spell_skills = 0;
     for (int i = SK_SPELLCASTING; i <= SK_POISON_MAGIC; i++)
         spell_skills += you.skills[i];
 
-    if (force_class == OBJ_RANDOM)
-    {
-        mpr("This is a scroll of acquirement!");
-
-      query:
-        mpr( "[a|A] Weapon  [b|B] Armour  [c|C] Jewellery      [d|D] Book" );
-        mpr( "[e|E] Staff   [f|F] Food    [g|G] Miscellaneous  [h|H] Gold" );
-
-        //mpr("[r|R] - Just give me something good.");
-        mpr("What kind of item would you like to acquire? ", MSGCH_PROMPT);
-
-        keyin = tolower( get_ch() );
-
-        if (keyin == 'a')
-            class_wanted = OBJ_WEAPONS;
-        else if (keyin == 'b')
-            class_wanted = OBJ_ARMOUR;
-        else if (keyin == 'c')
-            class_wanted = OBJ_JEWELLERY;
-        else if (keyin == 'd')
-            class_wanted = OBJ_BOOKS;
-        else if (keyin == 'e')
-            class_wanted = OBJ_STAVES;
-        else if (keyin == 'f')
-            class_wanted = OBJ_FOOD;
-        else if (keyin == 'g')
-            class_wanted = OBJ_MISCELLANY;
-        else if (keyin == 'h')
-            class_wanted = OBJ_GOLD;
-    }
-    else
-        class_wanted = force_class;
-
-    for (acqc = 0; acqc < ENDOFPACK; acqc++)
+    for (int acqc = 0; acqc < ENDOFPACK; acqc++)
     {
         if (is_valid_item( you.inv[acqc] )
                 && you.inv[acqc].base_type == class_wanted)
@@ -569,17 +516,25 @@ bool acquirement(unsigned char force_class, int agent)
             type_wanted = one_chance_in(10) ? FOOD_ROYAL_JELLY
                                             : FOOD_CHUNK;
         }
+        else if (you.species == SP_VAMPIRE)
+        {
+            // Vampires really don't want any OBJ_FOOD but OBJ_CORPSES
+            // but it's easier to just give them a potion of blood
+            // class type is set elsewhere
+            type_wanted = POT_BLOOD;
+            unique = 2 + random2(4);
+        }
         else 
         {
             // Meat is better than bread (except for herbivores), and
             // by choosing it as the default we don't have to worry
-            // about special cases for carnivorous races (ie kobold)
+            // about special cases for carnivorous races (e.g. kobolds)
             type_wanted = FOOD_MEAT_RATION;
 
             if (you.mutation[MUT_HERBIVOROUS])
                 type_wanted = FOOD_BREAD_RATION; 
 
-            // If we have some regular rations, then we're probably be more
+            // If we have some regular rations, then we're probably more
             // interested in faster foods (especially royal jelly)... 
             // otherwise the regular rations should be a good enough offer.
             if (already_has[FOOD_MEAT_RATION] 
@@ -636,7 +591,7 @@ bool acquirement(unsigned char force_class, int agent)
                 continue;
 
             int wskill = range_skill(OBJ_WEAPONS, i);
-            if (wskill == SK_RANGED_COMBAT)
+            if (wskill == SK_THROWING)
                 wskill = weapon_skill(OBJ_WEAPONS, i);
 
             if (wskill == skill && random2(count += acqweight) < acqweight)
@@ -646,7 +601,7 @@ bool acquirement(unsigned char force_class, int agent)
     else if (class_wanted == OBJ_MISSILES)
     {
         int count = 0;
-        int skill = SK_RANGED_COMBAT;
+        int skill = SK_THROWING;
 
         for (int i = SK_SLINGS; i <= SK_DARTS; i++)
         {
@@ -823,7 +778,8 @@ bool acquirement(unsigned char force_class, int agent)
                     if (one_chance_in(3))
                         type_wanted = AMU_RAGE + random2(10);
 
-                    if (get_ident_type(OBJ_JEWELLERY, type_wanted) == ID_UNKNOWN_TYPE)
+                    if (get_ident_type(OBJ_JEWELLERY, type_wanted) ==
+                        ID_UNKNOWN_TYPE)
                     {
                         break;
                     }
@@ -840,11 +796,9 @@ bool acquirement(unsigned char force_class, int agent)
 
               which_book:
 #if DEBUG_DIAGNOSTICS
-                snprintf( info, INFO_SIZE, 
-                          "acquirement: iteration = %d, best_spell = %d",
-                          iteration, best_spell );
-
-                mpr( info, MSGCH_DIAGNOSTICS );
+                mprf(MSGCH_DIAGNOSTICS,
+                     "acquirement: iteration = %d, best_spell = %d",
+                     iteration, best_spell );
 #endif //jmf: debugging
 
                 switch (best_spell)
@@ -1100,7 +1054,7 @@ bool acquirement(unsigned char force_class, int agent)
 
                 case SK_EVOCATIONS:
                     if (!one_chance_in(4))
-                        type_wanted = STAFF_SMITING + random2(10);
+                        type_wanted = random_rod_subtype();
                     break;
 
                 default: // invocations and leftover spell schools
@@ -1128,16 +1082,15 @@ bool acquirement(unsigned char force_class, int agent)
                     break;
                 }
 
-                // Increased chance of getting spell staff for new or
+                // Increased chance of getting a rod for new or
                 // non-spellcasters.  -- bwr
                 if (one_chance_in(20)
                     || (spell_skills <= 1               // short on spells
-                        && (type_wanted < STAFF_SMITING // already making one
-                            || type_wanted >= STAFF_AIR)
+                        && type_wanted < STAFF_SMITING
                         && !one_chance_in(4)))
                 {
-                    type_wanted = (coinflip() ? STAFF_STRIKING 
-                                              : STAFF_SMITING + random2(10));
+                    type_wanted = coinflip() ? STAFF_STRIKING :
+                        random_rod_subtype();
                 }
                 break;
 
@@ -1146,14 +1099,11 @@ bool acquirement(unsigned char force_class, int agent)
                     type_wanted = random2(NUM_MISCELLANY);
                 while (type_wanted == MISC_HORN_OF_GERYON
                     || type_wanted == MISC_RUNE_OF_ZOT
-                    || type_wanted == MISC_PORTABLE_ALTAR_OF_NEMELEX
                     || type_wanted == MISC_CRYSTAL_BALL_OF_FIXATION
                     || type_wanted == MISC_EMPTY_EBONY_CASKET);
                 break;
-
             default:
-                mesclr();
-                goto query;
+                break;
             }
 
             ASSERT( type_wanted < max_has_value );
@@ -1161,31 +1111,83 @@ bool acquirement(unsigned char force_class, int agent)
         while (already_has[type_wanted] && !one_chance_in(200));
     }
 
+    return (type_wanted);
+}
+
+bool acquirement(object_class_type class_wanted, int agent)
+{
+    int thing_created = 0;
+    int unique = 1;
+
+    while (class_wanted == OBJ_RANDOM)
+    {
+        mesclr();
+        mpr( "[a] Weapon  [b] Armour  [c] Jewellery      [d] Book" );
+        mpr( "[e] Staff   [f] Food    [g] Miscellaneous  [h] Gold" );
+        mpr("What kind of item would you like to acquire? ", MSGCH_PROMPT);
+
+        const int keyin = tolower( get_ch() );
+        switch ( keyin )
+        {
+        case 'a': class_wanted = OBJ_WEAPONS;    break;
+        case 'b': class_wanted = OBJ_ARMOUR;     break;
+        case 'c': class_wanted = OBJ_JEWELLERY;  break;
+        case 'd': class_wanted = OBJ_BOOKS;      break;
+        case 'e': class_wanted = OBJ_STAVES;     break;
+        case 'f': class_wanted = OBJ_FOOD;       break;
+        case 'g': class_wanted = OBJ_MISCELLANY; break;
+        case 'h': class_wanted = OBJ_GOLD;       break;
+        default: break;
+        }
+    }
+
     if (grid_destroys_items(grd[you.x_pos][you.y_pos]))
     {
         // how sad (and stupid)
-        mprf(MSGCH_SOUND, 
-             grid_item_destruction_message(grd[you.x_pos][you.y_pos]));
+        if (!silenced(you.pos()))
+            mprf(MSGCH_SOUND, 
+                 grid_item_destruction_message(grd[you.x_pos][you.y_pos]));
     }
     else
     {
-        randart_properties_t  proprt;
-
-        for (int item_tries = 0; item_tries < 50; item_tries++)
+        for (int item_tries = 0; item_tries < 40; item_tries++)
         {
+            unique = 1;
+            int type_wanted = find_acquirement_subtype(class_wanted, unique);
+            
+            // clobber class_wanted for vampires
+            if (you.species == SP_VAMPIRE && class_wanted == OBJ_FOOD)
+                class_wanted = OBJ_POTIONS;
+
             // BCR - unique is now used for food quantity.
             thing_created = items( unique, class_wanted, type_wanted, true, 
-                    MAKE_GOOD_ITEM, 250 );
+                                   MAKE_GOOD_ITEM, 250 );
 
             if (thing_created == NON_ITEM)
                 continue;
 
+            const item_def &doodad(mitm[thing_created]);
+            if ((doodad.base_type == OBJ_WEAPONS
+                 && !can_wield(&doodad, false, true))
+                || (doodad.base_type == OBJ_ARMOUR
+                    && !can_wear_armour(doodad, false, true))
+
+                // Trog does not gift the Wrath of Trog.
+                || (agent == GOD_TROG && is_fixed_artefact(doodad)
+                    && doodad.special == SPWPN_WRATH_OF_TROG))
+            {
+                destroy_item(thing_created, true);
+                thing_created = NON_ITEM;
+                continue;
+            }
+
             // MT - Check: god-gifted weapons and armor shouldn't kill you.
             // Except Xom.
             if ((agent == GOD_TROG || agent == GOD_OKAWARU)
-                && is_random_artefact(mitm[thing_created]))
+                && is_random_artefact(doodad))
             {
-                randart_wpn_properties( mitm[thing_created], proprt );
+                randart_properties_t  proprt;
+                randart_wpn_properties( doodad, proprt );
 
                 // check vs stats. positive stats will automatically fall
                 // through.  As will negative stats that won't kill you.
@@ -1204,7 +1206,7 @@ bool acquirement(unsigned char force_class, int agent)
 
         if (thing_created == NON_ITEM)
         {
-            mpr("The demon of the infinite void smiles at you.");
+            mpr("The demon of the infinite void smiles upon you.");
             return (false);
         }
 
@@ -1220,24 +1222,7 @@ bool acquirement(unsigned char force_class, int agent)
 
         if (thing.base_type == OBJ_BOOKS)
         {
-            if (thing.sub_type == BOOK_MINOR_MAGIC_I
-                || thing.sub_type == BOOK_MINOR_MAGIC_II
-                || thing.sub_type == BOOK_MINOR_MAGIC_III)
-            {
-                you.had_book[ BOOK_MINOR_MAGIC_I ] = 1;    
-                you.had_book[ BOOK_MINOR_MAGIC_II ] = 1;    
-                you.had_book[ BOOK_MINOR_MAGIC_III ] = 1;    
-            }
-            else if (thing.sub_type == BOOK_CONJURATIONS_I
-                || thing.sub_type == BOOK_CONJURATIONS_II)
-            {
-                you.had_book[ BOOK_CONJURATIONS_I ] = 1;    
-                you.had_book[ BOOK_CONJURATIONS_II ] = 1;    
-            }
-            else
-            {
-                you.had_book[ thing.sub_type ] = 1;    
-            }
+            mark_had_book(thing.sub_type);
         }
         else if (thing.base_type == OBJ_JEWELLERY)
         {
@@ -1273,7 +1258,8 @@ bool acquirement(unsigned char force_class, int agent)
             }
         }
         else if (thing.base_type == OBJ_WEAPONS
-                    && !is_fixed_artefact( thing ))
+                 && !is_fixed_artefact( thing )
+                 && !is_unrandom_artefact( thing ))
         {
             // HACK: make unwieldable weapons wieldable 
             // Note: messing with fixed artefacts is probably very bad.
@@ -1347,9 +1333,28 @@ bool acquirement(unsigned char force_class, int agent)
             default:
                 break;
             }
+            
+            int plusmod = random2(4);
+            // more damage, less accuracy
+            if (agent == GOD_TROG)
+            {
+                thing.plus  -= plusmod;
+                thing.plus2 += plusmod;
+                if (!is_random_artefact(thing))
+                    thing.plus = std::max(static_cast<int>(thing.plus), 0);
+            }
+            // more accuracy, less damage
+            else if (agent == GOD_OKAWARU)
+            {
+                thing.plus  += plusmod;
+                thing.plus2 -= plusmod;
+                if (!is_random_artefact(thing))
+                    thing.plus2 = std::max(static_cast<int>(thing.plus2), 0);
+            }
         }
         else if (thing.base_type == OBJ_ARMOUR
-                    && !is_fixed_artefact( thing ))
+                 && !is_fixed_artefact( thing )
+                 && !is_unrandom_artefact( thing ))
         {
             // HACK: make unwearable hats and boots wearable
             // Note: messing with fixed artefacts is probably very bad.
@@ -1393,7 +1398,8 @@ bool acquirement(unsigned char force_class, int agent)
                     thing.sub_type = ARM_NAGA_BARDING;
                 else if ( you.species == SP_CENTAUR )
                     thing.sub_type = ARM_CENTAUR_BARDING;
-                else {
+                else
+                {
                     thing.sub_type = ARM_BOOTS;
                     // Fix illegal ego types
                     if (get_armour_ego_type(thing) == SPARM_COLD_RESISTANCE ||
@@ -1424,29 +1430,6 @@ bool acquirement(unsigned char force_class, int agent)
     return (true);
 }                               // end acquirement()
 
-// Remove the "empty" autoinscription, if present.
-// Return true if the inscription was there, false otherwise.
-bool remove_empty_inscription( item_def& item )
-{
-    const char* empty_inscription = "empty";
-    size_t p = item.inscription.find(empty_inscription);
-    if ( p != std::string::npos )
-    {
-        // found it, delete it
-        size_t prespace = 0;
-        // if preceded by a space, delete that too
-        if ( p != 0 && item.inscription[p-1] == ' ' )
-            prespace = 1;
-        item.inscription =
-            item.inscription.substr(0, p - prespace) +
-            item.inscription.substr(p + strlen(empty_inscription),
-                                    std::string::npos);
-        return true;
-    }
-    else
-        return false;
-}
-
 bool recharge_wand(void)
 {
     if (you.equip[EQ_WEAPON] == -1)
@@ -1460,7 +1443,6 @@ bool recharge_wand(void)
     int charge_gain = 0;
     if (wand.base_type == OBJ_WANDS)
     {
-        remove_empty_inscription(wand);
         switch (wand.sub_type)
         {
         case WAND_INVISIBILITY:
@@ -1485,9 +1467,10 @@ bool recharge_wand(void)
             break;
         }
 
-        char str_pass[ ITEMNAME_SIZE ];
-        item_name(wand, DESC_CAP_YOUR, str_pass);
-        mprf("%s glows for a moment.", str_pass);
+        // don't display zap counts any more
+        wand.plus2 = ZAPCOUNT_UNKNOWN;
+
+        mprf("%s glows for a moment.", wand.name(DESC_CAP_YOUR).c_str());
 
         wand.plus += 1 + random2avg( ((charge_gain - 1) * 3) + 1, 3 );
 
@@ -1518,9 +1501,7 @@ bool recharge_wand(void)
         if (!work) 
             return (false);
 
-        char str_pass[ITEMNAME_SIZE];
-        item_name( wand, DESC_CAP_YOUR, str_pass );
-        mprf("%s glows for a moment.", str_pass);
+        mprf("%s glows for a moment.", wand.name(DESC_CAP_YOUR).c_str());
     }
 
     you.wield_change = true;
@@ -1553,18 +1534,22 @@ void yell(void)
 
     mpr("What do you say?", MSGCH_PROMPT);
     mprf(" ! - %s", cap_shout.c_str());
-    mpr(" a - Order allies to attack a monster");
 
-    if (!(you.prev_targ == MHITNOT || you.prev_targ == MHITYOU))
+    if (!you.duration[DUR_BERSERKER])
     {
-        struct monsters *target = &menv[you.prev_targ];
+        mpr(" a - Order allies to attack a monster");
 
-        if (mons_near(target) && player_monster_visible(target))
+        if (!(you.prev_targ == MHITNOT || you.prev_targ == MHITYOU))
         {
-            mpr(" p - Order allies to attack your previous target");
-            targ_prev = true;
+            struct monsters *target = &menv[you.prev_targ];
+
+            if (mons_near(target) && player_monster_visible(target))
+            {
+                mpr(" p - Order allies to attack your previous target");
+                targ_prev = true;
+            }
         }
-    }
+   }
 
     mprf(" Anything else - Stay silent%s",
          one_chance_in(20)? " (and be thought a fool)" : "");
@@ -1580,8 +1565,14 @@ void yell(void)
         return;
 
     case 'a':
+        if (you.duration[DUR_BERSERKER])
+        {
+            canned_msg(MSG_TOO_BERSERK);
+            return;
+        }
+
         mpr("Gang up on whom?", MSGCH_PROMPT);
-        direction( targ, DIR_TARGET, TARG_ENEMY );
+        direction( targ, DIR_TARGET, TARG_ENEMY, false, false );
 
         if (targ.isCancel)
         {
@@ -1589,7 +1580,8 @@ void yell(void)
             return;
         }
 
-        if (!targ.isValid || mgrd[targ.tx][targ.ty] == NON_MONSTER)
+        if (!targ.isValid || mgrd[targ.tx][targ.ty] == NON_MONSTER ||
+            !player_monster_visible(&env.mons[mgrd[targ.tx][targ.ty]]))
         {
             mpr("Yeah, whatever.");
             return;
@@ -1599,6 +1591,12 @@ void yell(void)
         break;
 
     case 'p':
+        if (you.duration[DUR_BERSERKER])
+        {
+            canned_msg(MSG_TOO_BERSERK);
+            return;
+        }
+
         if (targ_prev)
         {
             mons_targd = you.prev_targ;
