@@ -3,7 +3,7 @@
  *  Summary:    Functions for eating and butchering.
  *  Written by: Linley Henzell
  *
- *  Modified for Crawl Reference by $Author: dshaligram $ on $Date: 2007-06-05T16:54:50.325619Z $
+ *  Modified for Crawl Reference by $Author: ennewalker $ on $Date: 2007-11-23T02:25:44.246247Z $
  *
  *  Change History (most recent first):
  *
@@ -29,6 +29,7 @@
 #include "clua.h"
 #include "debug.h"
 #include "delay.h"
+#include "initfile.h"
 #include "invent.h"
 #include "items.h"
 #include "itemname.h"
@@ -36,22 +37,29 @@
 #include "item_use.h"
 #include "it_use2.h"
 #include "macro.h"
+#include "message.h"
 #include "misc.h"
 #include "mon-util.h"
 #include "mutation.h"
+#include "output.h"
 #include "player.h"
 #include "religion.h"
 #include "skills2.h"
 #include "spells2.h"
 #include "stuff.h"
+#include "transfor.h"
 #include "tutorial.h"
 
 static int   determine_chunk_effect(int which_chunk_type, bool rotten_chunk);
 static void  eat_chunk( int chunk_effect );
+static void  ghoul_consume_flesh(int chunk_type);
 static void  eating(unsigned char item_class, int item_type);
-static void  ghoul_eat_flesh( int chunk_effect );
 static void  describe_food_change(int hunger_increment);
 static bool  food_change(bool suppress_message);
+static bool  vampire_consume_corpse(int mons_type, int mass,
+                                    int chunk_type, bool rotten);
+static void  heal_from_food(int hp_amt, int mp_amt, bool unrot,
+                            bool restore_str);
 
 /*
  **************************************************
@@ -125,13 +133,8 @@ void weapon_switch( int targ )
     }
     else
     {
-        char buff[80];
-        in_name( targ, DESC_NOCAP_A, buff );
-
-        char let = index_to_letter( targ );
-
-        snprintf( info, INFO_SIZE, "Switching back to %c - %s.", let, buff );
-        mpr( info );
+        mprf("Switching back to %s.",
+             you.inv[targ].name(DESC_INVENTORY).c_str());
     }
 
     // unwield the old weapon and wield the new.
@@ -141,7 +144,7 @@ void weapon_switch( int targ )
     // code is a mess... this whole function's purpose was to 
     // isolate this hack until there's a proper way to do things. -- bwr
     if (you.equip[EQ_WEAPON] != -1)
-        unwield_item(you.equip[EQ_WEAPON], false);
+        unwield_item(false);
 
     you.equip[EQ_WEAPON] = targ;
 
@@ -150,9 +153,10 @@ void weapon_switch( int targ )
         wield_effects( targ, false );
 }
 
-// look for a butchering implement, prompting user if no obvious
-// options exist. Returns whether a weapon was switched.
-static bool find_butchering_implement()
+// look for a butchering implement. If fallback is true,
+// prompt the user if no obvious options exist.
+// Returns whether a weapon was switched.
+static bool find_butchering_implement( bool fallback )
 {
     // look for a butchering implement in your pack
     for (int i = 0; i < ENDOFPACK; ++i)
@@ -163,6 +167,7 @@ static bool find_butchering_implement()
             && item_known_uncursed(you.inv[i])
             && item_type_known(you.inv[i])
             && get_weapon_brand(you.inv[i]) != SPWPN_DISTORTION
+            && !has_warning_inscription(you.inv[i], OPER_WIELD) // don't even ask
             && can_wield( &you.inv[i] ))
         {
             mpr("Switching to a butchering implement.");
@@ -170,7 +175,10 @@ static bool find_butchering_implement()
             return true;
         }
     }
-    
+
+    if ( !fallback )
+        return false;
+
     // if we didn't swap above, then we still can't cut...let's call
     // wield_weapon() in the "prompt the user" way...
     
@@ -185,24 +193,26 @@ static bool find_butchering_implement()
 
 bool butchery(void)
 {
-    char str_pass[ ITEMNAME_SIZE ];
-
-    bool can_butcher = false;
     bool wpn_switch = false;
     bool new_cursed = false;
     int old_weapon = you.equip[EQ_WEAPON];
+    int old_gloves = you.equip[EQ_GLOVES];
 
-    bool barehand_butcher = (you.equip[ EQ_GLOVES ] == -1)
-      && (you.attribute[ATTR_TRANSFORMATION] == TRAN_BLADE_HANDS ||
-          you.attribute[ATTR_TRANSFORMATION] == TRAN_DRAGON ||
-          (you.attribute[ATTR_TRANSFORMATION] == TRAN_NONE &&
-           (you.species == SP_TROLL ||
-            you.species == SP_GHOUL ||
-            you.mutation[MUT_CLAWS])));
+    const transformation_type transform =
+        static_cast<transformation_type>(you.attribute[ATTR_TRANSFORMATION]);
+ 
+    // Xom probably likes this, occasionally
+    bool teeth_butcher = (you.mutation[MUT_FANGS] == 3);
 
-    can_butcher = barehand_butcher ||
-        (you.equip[EQ_WEAPON] != -1 &&
-         can_cut_meat(you.inv[you.equip[EQ_WEAPON]]));
+    bool barehand_butcher = (transform_can_butcher_barehanded(transform)
+                             || you.has_claws()) && you.equip[EQ_GLOVES] == -1;
+
+    bool gloved_butcher = you.has_claws() && (you.equip[EQ_GLOVES] != -1
+                          && !item_cursed(you.inv[you.equip[EQ_GLOVES]]));
+
+    bool can_butcher = teeth_butcher || barehand_butcher
+                       || you.equip[EQ_WEAPON] != -1
+                          && can_cut_meat(you.inv[you.equip[EQ_WEAPON]]);
     
     if (igrd[you.x_pos][you.y_pos] == NON_ITEM)
     {
@@ -216,7 +226,7 @@ bool butchery(void)
         return (false);
     }
 
-    if (player_is_levitating() && !wearing_amulet(AMU_CONTROLLED_FLIGHT))
+    if (you.flight_mode() == FL_LEVITATE)
     {
         mpr("You can't reach the floor from up here.");
         return (false);
@@ -225,7 +235,7 @@ bool butchery(void)
     // It makes more sense that you first find out if there's anything
     // to butcher, *then* decide to actually butcher it.
     // The old code did it the other way.
-    if ( !can_butcher && you.berserker )
+    if ( !can_butcher && you.duration[DUR_BERSERKER] )
     {
         mpr ("You are too berserk to search for a butchering knife!");
         return (false);
@@ -243,9 +253,9 @@ bool butchery(void)
         found_nonzero_corpses = true;
         
         // offer the possibility of butchering
-        it_name(objl, DESC_NOCAP_A, str_pass);
-        snprintf(info, INFO_SIZE, "Butcher %s?", str_pass);
-        int answer = yesnoquit( info, true, 'n', false );
+        snprintf(info, INFO_SIZE, "Butcher %s?",
+                 mitm[objl].name(DESC_NOCAP_A).c_str());
+        const int answer = yesnoquit( info, true, 'n', false );
         if ( answer == -1 )
         {
             canceled_butcher = true;
@@ -253,11 +263,23 @@ bool butchery(void)
         }
         if ( answer == 0 )
             continue;
+
+        bool removed_gloves = false;
         
         if ( Options.easy_butcher && !can_butcher )
         {
-            // try to find a butchering implement
-            wpn_switch = find_butchering_implement();
+            // Try to find a butchering implement.
+            // If you can butcher by taking off your gloves, don't prompt.
+            wpn_switch = find_butchering_implement(!gloved_butcher);
+            removed_gloves = gloved_butcher && !wpn_switch;
+            if ( removed_gloves )
+            {
+                // Actually take off the gloves; this creates a
+                // delay. We assume later on that gloves have a 1-turn
+                // takeoff delay!
+                takeoff_armour(old_gloves);
+                barehand_butcher = true;
+            }
             const int wpn = you.equip[EQ_WEAPON];
             if ( wpn_switch )
             {
@@ -267,11 +289,11 @@ bool butchery(void)
                     item_cursed( you.inv[wpn]);
             }
             
-            // note that barehanded butchery would not reach this
-            // stage, so if wpn == -1 the user selected '-' when
+            // note that if wpn == -1 the user selected '-' when
             // switching weapons
             
-            if (!wpn_switch || wpn == -1 || !can_cut_meat(you.inv[wpn]))
+            if (!barehand_butcher &&
+                (!wpn_switch || wpn == -1 || !can_cut_meat(you.inv[wpn])))
             {
                 // still can't butcher. Early out
                 if ( wpn == -1 ) {
@@ -296,23 +318,33 @@ bool butchery(void)
         if ( can_butcher )
         {
             // we actually butcher now
-            if ( barehand_butcher )
+            if ( teeth_butcher || barehand_butcher )
                 mpr("You start tearing the corpse apart.");
             else
                 mpr("You start hacking away.");
 
-            if (you.duration[DUR_PRAYER] &&
-                (you.religion == GOD_OKAWARU || you.religion == GOD_MAKHLEB ||
-                you.religion == GOD_TROG))
+            bool rotten = (mitm[objl].special < 100);
+            if (you.duration[DUR_PRAYER] && !rotten &&
+                god_likes_butchery(you.religion))
             {
                 offer_corpse(objl);
                 destroy_item(objl);
             }
             else
             {
+                if (you.duration[DUR_PRAYER] && rotten
+                    && god_likes_butchery(you.religion) )
+                {
+                    if (coinflip())
+                        simple_god_message(" refuses to accept that mouldy "
+                                             "sacrifice!", you.religion);
+                    else
+                        simple_god_message(" demands fresh blood!", you.religion);
+                }
+                
                 // If we didn't switch weapons, we get in one turn of butchery;
                 // otherwise the work has to happen in the delay.
-                if (!wpn_switch)
+                if (!wpn_switch && !removed_gloves)
                     ++mitm[objl].plus2;
                 
                 int work_req = 4 - mitm[objl].plus2;
@@ -320,6 +352,12 @@ bool butchery(void)
                     work_req = 0;
 
                 start_delay(DELAY_BUTCHER, work_req, objl, mitm[objl].special);
+
+                if (you.duration[DUR_PRAYER]
+                    && god_hates_butchery(you.religion))
+                {
+                    did_god_conduct(DID_DEDICATED_BUTCHERY, 10);
+                }
             }
         }
 
@@ -327,6 +365,10 @@ bool butchery(void)
         if (!new_cursed && wpn_switch)
             start_delay( DELAY_WEAPON_SWAP, 1, old_weapon );
 
+        // put on the removed gloves
+        if ( removed_gloves )
+            start_delay( DELAY_ARMOUR_ON, 1, old_gloves );
+
         you.turn_is_over = true;    
         return true;
     }
@@ -345,7 +387,6 @@ bool butchery(void)
     return false;
 }                               // end butchery()
 
-#ifdef CLUA_BINDINGS
 void lua_push_items(lua_State *ls, int link)
 {
     lua_newtable(ls);
@@ -377,7 +418,6 @@ void lua_push_inv_items(lua_State *ls = NULL)
         }
     }
 }
-#endif
 
 static bool userdef_eat_food()
 {
@@ -404,8 +444,8 @@ bool prompt_eat_from_inventory(void)
     int which_inventory_slot = 
             prompt_invent_item(
                     "Eat which item?",
-                    MT_INVSELECT,
-                    OBJ_FOOD,
+                    MT_INVLIST,
+                    you.species == SP_VAMPIRE ? OBJ_CORPSES : OBJ_FOOD,
                     true, true, true, 0, NULL,
                     OPER_EAT );
     if (which_inventory_slot == PROMPT_ABORT)
@@ -416,12 +456,21 @@ bool prompt_eat_from_inventory(void)
 
     // this conditional can later be merged into food::can_ingest() when
     // expanded to handle more than just OBJ_FOOD 16mar200 {dlb}
-    if (you.inv[which_inventory_slot].base_type != OBJ_FOOD)
+    if (you.species != SP_VAMPIRE &&
+        you.inv[which_inventory_slot].base_type != OBJ_FOOD)
     {
         mpr("You can't eat that!");
         return (false);
     }
 
+    if (you.species == SP_VAMPIRE &&
+       (you.inv[which_inventory_slot].base_type != OBJ_CORPSES
+         || you.inv[which_inventory_slot].sub_type != CORPSE_BODY))
+    {
+        mpr("You crave blood!");
+        return (false);
+    }
+       
     if (!can_ingest( you.inv[which_inventory_slot].base_type,
                         you.inv[which_inventory_slot].sub_type, false ))
     {
@@ -459,10 +508,7 @@ bool eat_food(bool run_hook)
     if (igrd[you.x_pos][you.y_pos] != NON_ITEM)
     {
         if (eat_from_floor())
-        {
-            burden_change();    // ghouls regain strength from rotten food
             return (true);
-        }
     }
 
     return (prompt_eat_from_inventory());
@@ -489,15 +535,27 @@ static bool food_change(bool suppress_message)
     if (you.species == SP_GHOUL && you.hunger > 6999) 
         you.hunger = 6999;
 
+    // vampires can never be engorged or starve to death
+    if (you.species == SP_VAMPIRE && you.hunger <= 700)
+        you.hunger = 701;
+    if (you.species == SP_VAMPIRE && you.hunger > 10999)
+        you.hunger = 10999;
+
     // get new hunger state
     if (you.hunger <= 1000)
         newstate = HS_STARVING;
+    else if (you.hunger <= 1533)
+        newstate = HS_NEAR_STARVING;
+    else if (you.hunger <= 2066)
+        newstate = HS_VERY_HUNGRY;
     else if (you.hunger <= 2600)
         newstate = HS_HUNGRY;
     else if (you.hunger < 7000)
         newstate = HS_SATIATED;
-    else if (you.hunger < 11000)
+    else if (you.hunger < 9000)
         newstate = HS_FULL;
+    else if (you.hunger < 11000)
+        newstate = HS_VERY_FULL;
 
     if (newstate != you.hunger_state)
     {
@@ -505,7 +563,6 @@ static bool food_change(bool suppress_message)
         you.hunger_state = newstate;
         set_redraw_status( REDRAW_HUNGER );
 
-        // Stop the travel command, if it's in progress and we just got hungry
         if (newstate < HS_SATIATED)
             interrupt_activity( AI_HUNGRY );
 
@@ -521,6 +578,14 @@ static bool food_change(bool suppress_message)
                 mpr("You are feeling hungry.", MSGCH_FOOD);
                 learned_something_new(TUT_YOU_HUNGRY);
                 break;
+            case HS_VERY_HUNGRY:
+                mpr("You are feeling very hungry.", MSGCH_FOOD);
+                learned_something_new(TUT_YOU_HUNGRY);
+                break;
+            case HS_NEAR_STARVING:
+                mpr("You are near starving.", MSGCH_FOOD);
+                learned_something_new(TUT_YOU_HUNGRY);
+                break;
             default:
                 break;
             }
@@ -535,28 +600,70 @@ static bool food_change(bool suppress_message)
 static void describe_food_change(int food_increment)
 {
     int magnitude = (food_increment > 0)?food_increment:(-food_increment);
+    std::string msg;
 
     if (magnitude == 0)
         return;
 
-    strcpy(info, (magnitude <= 100) ? "You feel slightly " :
-                 (magnitude <= 350) ? "You feel somewhat " :
-                 (magnitude <= 800) ? "You feel a quite a bit "
-                                    : "You feel a lot ");
+    if ( magnitude <= 100 )
+        msg = "You feel slightly ";
+    else if (magnitude <= 350)
+        msg = "You feel somewhat ";
+    else if (magnitude <= 800)
+        msg = "You feel quite a bit ";
+    else
+        msg = "You feel a lot ";
 
     if ((you.hunger_state > HS_SATIATED) ^ (food_increment < 0))
-        strcat(info, "more ");
+        msg += "more ";
     else
-        strcat(info, "less ");
+        msg += "less ";
 
-    strcat(info, (you.hunger_state > HS_SATIATED) ? "full."
-                                                  : "hungry.");
-    mpr(info);
+    msg += ((you.hunger_state > HS_SATIATED) ? "full." : "hungry.");
+    mpr(msg.c_str());
 }                               // end describe_food_change()
 
+static bool prompt_eat_chunk(const item_def &item, bool rotten)
+{
+    if (rotten && you.species != SP_GHOUL && you.species != SP_KOBOLD
+        && you.species != SP_TROLL && you.species != SP_OGRE
+        && you.species != SP_HILL_ORC
+        && !yesno("Are you sure you want to eat this rotten meat?",
+                  false, 'n'))
+    {
+        canned_msg(MSG_OK);
+        return (false);
+    }
+    return (true);
+}
+
 void eat_from_inventory(int which_inventory_slot)
 {
-    if (you.inv[which_inventory_slot].sub_type == FOOD_CHUNK)
+    if (you.inv[which_inventory_slot].base_type == OBJ_CORPSES
+        && you.inv[which_inventory_slot].sub_type == CORPSE_BODY)
+    {
+        const int mons_type = you.inv[ which_inventory_slot ].plus;
+        const int chunk_type = mons_corpse_effect( mons_type );
+        const bool rotten = (you.inv[which_inventory_slot].special < 100);
+        const int mass = item_mass( you.inv[which_inventory_slot] );
+
+        if (!vampire_consume_corpse(mons_type, mass, chunk_type, rotten))
+            return;
+
+        if (!mons_skeleton( mons_type ) || one_chance_in(4))
+        {
+            dec_inv_item_quantity( which_inventory_slot, 1 );
+        }
+        else
+        {
+            you.inv[which_inventory_slot].sub_type = CORPSE_SKELETON;
+            you.inv[which_inventory_slot].special = 90;
+            you.inv[which_inventory_slot].colour = LIGHTGREY;
+        }
+        // dec_inv_item_quantity( which_inventory_slot, 1 );
+        return;
+    }
+    else if (you.inv[which_inventory_slot].sub_type == FOOD_CHUNK)
     {
         // this is a bit easier to read... most compilers should
         // handle this the same -- bwr
@@ -564,6 +671,9 @@ void eat_from_inventory(int which_inventory_slot)
         const int chunk_type = mons_corpse_effect( mons_type );
         const bool rotten = (you.inv[which_inventory_slot].special < 100);
 
+        if (!prompt_eat_chunk(you.inv[which_inventory_slot], rotten))
+            return;
+
         eat_chunk( determine_chunk_effect( chunk_type, rotten ) );
     }
     else
@@ -577,11 +687,38 @@ void eat_from_inventory(int which_inventory_slot)
 
 void eat_floor_item(int item_link)
 {
-    if (mitm[item_link].sub_type == FOOD_CHUNK)
+    if (mitm[item_link].base_type == OBJ_CORPSES
+        && mitm[item_link].sub_type == CORPSE_BODY)
+    {
+        const int mons_type = mitm[item_link].plus;
+        const int chunk_type = mons_corpse_effect( mons_type );
+        const bool rotten = (mitm[item_link].special < 100);
+        const int mass = item_mass( mitm[item_link] );
+
+        if (!vampire_consume_corpse(mons_type, mass, chunk_type, rotten))
+            return;
+
+        if (!mons_skeleton( mons_type ) || one_chance_in(4))
+        {
+            dec_mitm_item_quantity( item_link, 1 );
+        }
+        else
+        {
+            mitm[item_link].sub_type = CORPSE_SKELETON;
+            mitm[item_link].special = 90;
+            mitm[item_link].colour = LIGHTGREY;
+        }
+        // dec_mitm_item_quantity( item_link, 1 );
+
+        you.turn_is_over = 1;
+        return;
+    }
+    else if (mitm[item_link].sub_type == FOOD_CHUNK)
     {
         const int chunk_type = mons_corpse_effect( mitm[item_link].plus );
         const bool rotten = (mitm[item_link].special < 100);
-
+        if (!prompt_eat_chunk(mitm[item_link], rotten))
+            return;
         eat_chunk( determine_chunk_effect( chunk_type, rotten ) );
     }
     else
@@ -596,21 +733,25 @@ void eat_floor_item(int item_link)
 
 bool eat_from_floor(void)
 {
-    char str_pass[ ITEMNAME_SIZE ];
-
-    if (player_is_levitating() && !wearing_amulet(AMU_CONTROLLED_FLIGHT))
+    if (you.flight_mode() == FL_LEVITATE)
         return (false);
 
     bool need_more = false;
     for (int o = igrd[you.x_pos][you.y_pos]; o != NON_ITEM; o = mitm[o].link)
     {
-        if (mitm[o].base_type != OBJ_FOOD)
+        item_def& item = mitm[o];
+
+        if (you.species != SP_VAMPIRE && item.base_type != OBJ_FOOD)
+            continue;
+
+        if (you.species == SP_VAMPIRE &&
+            (item.base_type != OBJ_CORPSES || item.sub_type != CORPSE_BODY))
             continue;
 
-        it_name( o, DESC_NOCAP_A, str_pass );
         mprf( MSGCH_PROMPT,
-                  "Eat %s%s?", (mitm[o].quantity > 1) ? "one of " : "", 
-                  str_pass );
+              "%s %s%s?", you.species == SP_VAMPIRE ? "Drink blood from" : "Eat",
+              (item.quantity > 1) ? "one of " : "",
+              item.name(DESC_NOCAP_A).c_str() );
 
         // If we're prompting now, we don't need a -more- when
         // breaking out, because the prompt serves as a -more-. Of
@@ -630,7 +771,7 @@ bool eat_from_floor(void)
 
         if (keyin == 'y')
         {
-            if (!can_ingest( mitm[o].base_type, mitm[o].sub_type, false ))
+            if (!can_ingest( item.base_type, item.sub_type, false ))
             {
                 need_more = true;
                 continue;
@@ -652,17 +793,20 @@ static const char *chunk_flavour_phrase(bool likes_chunks)
     const char *phrase =
         likes_chunks? "tastes great." : "tastes terrible.";
 
-    const int gourmand = you.duration[DUR_GOURMAND];
-    if (gourmand >= GOURMAND_MAX)
-        phrase = 
-            one_chance_in(8)? "tastes like chicken!"
-                            : "tastes great.";
-    else if (gourmand > GOURMAND_MAX * 75 / 100)
-        phrase = "tastes very good.";
-    else if (gourmand > GOURMAND_MAX * 50 / 100)
-        phrase = "tastes good.";
-    else if (gourmand > GOURMAND_MAX * 25 / 100)
-        phrase = "is not very appetising.";
+    if (!likes_chunks)
+    {
+        const int gourmand = you.duration[DUR_GOURMAND];
+        if (gourmand >= GOURMAND_MAX)
+            phrase = 
+                one_chance_in(1000)? "tastes like chicken!"
+                                   : "tastes great.";
+        else if (gourmand > GOURMAND_MAX * 75 / 100)
+            phrase = "tastes very good.";
+        else if (gourmand > GOURMAND_MAX * 50 / 100)
+            phrase = "tastes good.";
+        else if (gourmand > GOURMAND_MAX * 25 / 100)
+            phrase = "is not very appetising.";
+    }
 
     return (phrase);
 }
@@ -731,7 +875,7 @@ static void eat_chunk( int chunk_effect )
 
     if (you.species == SP_GHOUL)
     {
-        ghoul_eat_flesh( chunk_effect );
+        ghoul_consume_flesh(chunk_effect);
         start_delay( DELAY_EAT, 2 );
         lessen_hunger( CHUNK_BASE_NUTRITION, true );
     }
@@ -741,28 +885,33 @@ static void eat_chunk( int chunk_effect )
         {
         case CE_MUTAGEN_RANDOM:
             mpr("This meat tastes really weird.");
-            mutate(100);
+            mutate(RANDOM_MUTATION);
+            xom_is_stimulated(100);
             break;
 
         case CE_MUTAGEN_BAD:
             mpr("This meat tastes *really* weird.");
             give_bad_mutation();
+            xom_is_stimulated(random2(200));
             break;
 
         case CE_HCL:
             rot_player( 10 + random2(10) );
-            disease_player( 50 + random2(100) );
+            if (disease_player( 50 + random2(100) ))
+                xom_is_stimulated(random2(100));
             break;
 
         case CE_POISONOUS:
             mpr("Yeeuch - this meat is poisonous!");
-            poison_player( 3 + random2(4) );
+            if (poison_player( 3 + random2(4) ))
+                xom_is_stimulated(random2(128));
             break;
 
         case CE_ROTTEN:
         case CE_CONTAMINATED:
             mpr("There is something wrong with this meat.");
-            disease_player( 50 + random2(100) );
+            if (disease_player( 50 + random2(100) ))
+                xom_is_stimulated(random2(100));
             break;
 
         // note that this is the only case that takes time and forces redraw
@@ -780,53 +929,28 @@ static void eat_chunk( int chunk_effect )
     return;
 }                               // end eat_chunk()
 
-static void ghoul_eat_flesh( int chunk_effect )
+static void ghoul_consume_flesh(int chunk_type)
 {
-    bool healed = false;
+    int hp_amt = 1 + random2(5) + random2(1 + you.experience_level);
 
-    if (chunk_effect != CE_ROTTEN && chunk_effect != CE_CONTAMINATED)
+    if (chunk_type != CE_ROTTEN && chunk_type != CE_CONTAMINATED)
     {
         mpr("This raw flesh tastes good.");
 
-        if (!one_chance_in(5))
-            healed = true;
-
-        if (player_rotted() && !one_chance_in(3))
-        {
-            mpr("You feel more resilient.");
-            unrot_hp(1);
-        }
+        heal_from_food((!one_chance_in(5)) ? hp_amt : 0, 0,
+            !one_chance_in(3), false);
     }
     else
     {
-        if (chunk_effect == CE_ROTTEN)
-            mpr( "This rotting flesh tastes delicious!" );
-        else // CE_CONTAMINATED
-            mpr( "This flesh tastes delicious!" );
+        if (chunk_type == CE_ROTTEN)
+            mpr("This rotting flesh tastes delicious!");
+        else // CE_CONTAMINATED or CE_HCL
+            mpr("This flesh tastes delicious!");
 
-        healed = true;
-
-        if (player_rotted() && !one_chance_in(4))
-        {
-            mpr("You feel more resilient.");
-            unrot_hp(1);
-        }
+        heal_from_food(hp_amt, 0,
+            !one_chance_in(4), one_chance_in(5));
     }
-
-    if (you.strength < you.max_strength && one_chance_in(5))
-    {
-        mpr("You feel your strength returning.");
-        you.strength++;
-        you.redraw_strength = 1;
-    }
-
-    if (healed && you.hp < you.hp_max)
-        inc_hp(1 + random2(5) + random2(1 + you.experience_level), false);
-
-    calc_hp();
-
-    return;
-}                               // end ghoul_eat_flesh()
+} // end ghoul_consume_flesh()
 
 static void eating(unsigned char item_class, int item_type)
 {
@@ -891,7 +1015,6 @@ static void eating(unsigned char item_class, int item_type)
         }                       // end base sustenance listing {dlb}
 
         // next, sustenance modifier for carnivores/herbivores {dlb}:
-        // for some reason, sausages do not penalize herbivores {dlb}:
         switch (item_type)
         {
         case FOOD_MEAT_RATION:
@@ -899,6 +1022,7 @@ static void eating(unsigned char item_class, int item_type)
             herbivore_modifier = -1500;
             break;
         case FOOD_BEEF_JERKY:
+        case FOOD_SAUSAGE:
             carnivore_modifier = 200;
             herbivore_modifier = -200;
             break;
@@ -951,11 +1075,12 @@ static void eating(unsigned char item_class, int item_type)
                 mpr("That meat ration really hit the spot!");
                 break;
             case FOOD_BEEF_JERKY:
-                strcpy(info, "That beef jerky was ");
-                strcat(info, (one_chance_in(4)) ? "jerk-a-riffic"
-                                                : "delicious");
-                strcat(info, "!");
-                mpr(info);
+                mprf("That beef jerky was %s!",
+                     one_chance_in(4) ? "jerk-a-riffic"
+                                      : "delicious");
+                break;
+            case FOOD_SAUSAGE:
+                mpr("That sausage was delicious!");
                 break;
             default:
                 break;
@@ -972,12 +1097,11 @@ static void eating(unsigned char item_class, int item_type)
             case FOOD_PEAR:
             case FOOD_APPLE:
             case FOOD_APRICOT:
-                strcpy(info, "Mmmm... Yummy ");
-                strcat(info, (item_type == FOOD_APPLE)   ? "apple." :
-                             (item_type == FOOD_PEAR)    ? "pear." :
-                             (item_type == FOOD_APRICOT) ? "apricot."
-                                                         : "fruit.");
-                mpr(info);
+                mprf("Mmmm... Yummy %s.",
+                     (item_type == FOOD_APPLE)   ? "apple." :
+                     (item_type == FOOD_PEAR)    ? "pear." :
+                     (item_type == FOOD_APRICOT) ? "apricot."
+                                                 : "fruit.");
                 break;
             case FOOD_CHOKO:
                 mpr("That choko was very bland.");
@@ -986,16 +1110,12 @@ static void eating(unsigned char item_class, int item_type)
                 mpr("That snozzcumber tasted truly putrid!");
                 break;
             case FOOD_ORANGE:
-                strcpy(info, "That orange was delicious!");
-                if (one_chance_in(8))
-                    strcat(info, " Even the peel tasted good!");
-                mpr(info);
+                mprf("That orange was delicious!%s",
+                     one_chance_in(8) ? " Even the peel tasted good!" : "");
                 break;
             case FOOD_BANANA:
-                strcpy(info, "That banana was delicious!");
-                if (one_chance_in(8))
-                    strcat(info, " Even the peel tasted good!");
-                mpr(info);
+                mprf("That banana was delicious!%s",
+                     one_chance_in(8) ? " Even the peel tasted good!" : "");
                 break;
             case FOOD_STRAWBERRY:
                 mpr("That strawberry was delicious!");
@@ -1034,39 +1154,36 @@ static void eating(unsigned char item_class, int item_type)
                 mprf("Mmm... %s.", SysEnv.crawl_pizza.c_str());
             else
             {
-                temp_rand = random2(9);
-
-                snprintf(info, INFO_SIZE, "Mmm... %s",
-                             (temp_rand == 0) ? "Ham and pineapple." :
-                             (temp_rand == 2) ? "Vegetable." :
-                             (temp_rand == 3) ? "Pepperoni." :
-                             (temp_rand == 4) ? "Yeuchh - Anchovies!" :
-                             (temp_rand == 5) ? "Cheesy." :
-                             (temp_rand == 6) ? "Supreme." :
-                             (temp_rand == 7) ? "Super Supreme!"
-                                              : "Chicken.");
+                if (how_carnivorous >= 1) // non-vegetable
+                    temp_rand = 5 + random2(4);
+                else if (how_herbivorous >= 1) // non-meaty
+                    temp_rand = random2(6) + 2;
+                else
+                    temp_rand = random2(9);
+                    
+                mprf("Mmm... %s",
+                     (temp_rand == 0) ? "Ham and pineapple." :
+                     (temp_rand == 2) ? "Vegetable." :
+                     (temp_rand == 3) ? "Pepperoni." :
+                     (temp_rand == 4) ? "Yeuchh - Anchovies!" :
+                     (temp_rand == 5) ? "Cheesy." :
+                     (temp_rand == 6) ? "Supreme." :
+                     (temp_rand == 7) ? "Super Supreme!"
+                                      : "Chicken.");
             }
-            mpr(info);
             break;
         case FOOD_CHEESE:
-            strcpy(info, "Mmm... ");
             temp_rand = random2(9);
-
-            strcat(info, (temp_rand == 0) ? "Cheddar" :
-                         (temp_rand == 1) ? "Edam" :
-                         (temp_rand == 2) ? "Wensleydale" :
-                         (temp_rand == 3) ? "Camembert" :
-                         (temp_rand == 4) ? "Goat cheese" :
-                         (temp_rand == 5) ? "Fruit cheese" :
-                         (temp_rand == 6) ? "Mozzarella" :
-                         (temp_rand == 7) ? "Sheep cheese"
-                                          : "Yak cheese");
-
-            strcat(info, ".");
-            mpr(info);
-            break;
-        case FOOD_SAUSAGE:
-            mpr("That sausage was delicious!");
+            mprf("Mmm...%s.",
+                 (temp_rand == 0) ? "Cheddar" :
+                 (temp_rand == 1) ? "Edam" :
+                 (temp_rand == 2) ? "Wensleydale" :
+                 (temp_rand == 3) ? "Camembert" :
+                 (temp_rand == 4) ? "Goat cheese" :
+                 (temp_rand == 5) ? "Fruit cheese" :
+                 (temp_rand == 6) ? "Mozzarella" :
+                 (temp_rand == 7) ? "Sheep cheese"
+                                  : "Yak cheese");
             break;
         default:
             break;
@@ -1117,6 +1234,20 @@ bool can_ingest(int what_isit, int kindof_thing, bool suppress_msg, bool reqid,
         return (false);
     }
 
+    if (you.species == SP_VAMPIRE)
+    {
+        if (what_isit == OBJ_CORPSES && kindof_thing == CORPSE_BODY
+            || what_isit == OBJ_POTIONS && kindof_thing == POT_BLOOD)
+        {
+            return (true);
+        }
+        else
+        {
+            if (!suppress_msg)
+               mpr("Blech - you need blood!");
+        }
+        return (false);
+    }
 
     bool ur_carnivorous = (you.mutation[MUT_CARNIVOROUS] == 3);
 
@@ -1134,6 +1265,13 @@ bool can_ingest(int what_isit, int kindof_thing, bool suppress_msg, bool reqid,
     switch (what_isit)
     {
     case OBJ_FOOD:
+        if (you.species == SP_VAMPIRE)
+        {
+            if (!suppress_msg)
+                mpr("Blech - you need blood!");
+             return false;
+        }
+             
         switch (kindof_thing)
         {
         case FOOD_BREAD_RATION:
@@ -1184,11 +1322,61 @@ bool can_ingest(int what_isit, int kindof_thing, bool suppress_msg, bool reqid,
         }
         break;
 
+    case OBJ_CORPSES:
+        if (you.species == SP_VAMPIRE)
+        {
+            if (kindof_thing == CORPSE_BODY)
+                return true;
+            else
+            {
+                if (!suppress_msg)
+                    mpr("Blech - you need blood!");
+                return false;
+            }
+        }
+        return false;
+
+    case OBJ_POTIONS: // called by lua
+        if (get_ident_type(OBJ_POTIONS, kindof_thing) != ID_KNOWN_TYPE)
+            return true;
+        switch (kindof_thing)
+        {
+            case POT_BLOOD:
+                if (ur_herbivorous)
+                {
+                    if (!suppress_msg)
+                        mpr("Urks, you're a herbivore!");
+                    return false;
+                }
+                return true;
+            case POT_WATER:
+                if (you.species == SP_VAMPIRE)
+                {
+                    if (!suppress_msg)
+                        mpr("Blech - you need blood!");
+                    return false;
+                }
+                return true;
+             case POT_PORRIDGE:
+                if (you.species == SP_VAMPIRE)
+                {
+                    if (!suppress_msg)
+                        mpr("Blech - you need blood!");
+                    return false;
+                }
+                else if (ur_carnivorous)
+                {
+                    if (!suppress_msg)
+                        mpr("Sorry, you're a carnivore.");
+                    return false;
+                }
+             default:
+                return true;
+        }
+
     // other object types are set to return false for now until
     // someone wants to recode the eating code to permit consumption
-    // of things other than just food -- corpses first, then more
-    // exotic stuff later would be good to explore - 13mar2000 {dlb}
-    case OBJ_CORPSES:
+    // of things other than just food
     default:
         return (false);
     }
@@ -1202,7 +1390,7 @@ bool can_ingest(int what_isit, int kindof_thing, bool suppress_msg, bool reqid,
 // understood why liches are hungry and not true undead beings ... {dlb}:
 static int determine_chunk_effect(int which_chunk_type, bool rotten_chunk)
 {
-    int poison_resistance_level = player_res_poison();
+    const int poison_resistance_level = player_res_poison();
     int this_chunk_effect = which_chunk_type;
 
     // determine the initial effect of eating a particular chunk {dlb}:
@@ -1220,7 +1408,7 @@ static int determine_chunk_effect(int which_chunk_type, bool rotten_chunk)
     case CE_POISONOUS:
         if (you.species == SP_GHOUL
                 || you.attribute[ATTR_TRANSFORMATION] == TRAN_LICH
-                || poison_resistance_level > 0)
+                || poison_resistance_level > 0 && you.species != SP_VAMPIRE)
         {
             this_chunk_effect = CE_CLEAN;
         }
@@ -1328,3 +1516,107 @@ static int determine_chunk_effect(int which_chunk_type, bool rotten_chunk)
 
     return (this_chunk_effect);
 }                               // end determine_chunk_effect()
+
+static bool vampire_consume_corpse(int mons_type, int mass,
+                                   int chunk_type, bool rotten)
+{
+    if (chunk_type == CE_HCL)
+    {
+        mpr( "There is no blood in this body!" );
+        return false;
+    }
+
+    int food_value = 0, hp_amt = 0, mp_amt = 0;
+
+    if (!rotten)
+    {
+        hp_amt++;
+
+        switch (mons_type)
+        {
+           case MONS_HUMAN:
+               food_value = mass + random2avg(you.experience_level * 10, 2);
+               mpr( "This warm blood tastes really delicious!" );
+               hp_amt += 1 + random2(1 + you.experience_level);
+               break;
+
+           case MONS_ELF:
+               food_value = mass + random2avg(you.experience_level * 10, 2);
+               mpr( "This warm blood tastes magically delicious!" );
+               mp_amt += 1 + random2(3);
+               break;
+
+           default:
+               switch (chunk_type)
+               {
+                  case CE_CLEAN:
+                      food_value = mass;
+                      mpr( "This warm blood tastes delicious!" );
+                      break;
+                  case CE_CONTAMINATED:
+                      food_value = mass / (random2(3) + 1);
+                      mpr( "Somehow that blood was not very filling!" );
+                      break;
+                  case CE_POISONOUS:
+                      food_value = random2(mass) - mass/2;
+                      mpr( "Blech - that blood tastes nasty!" );
+                      break;
+                  case CE_MUTAGEN_RANDOM:
+                      food_value = random2(mass);
+                      mpr( "That blood tastes really weird!" );
+                      mutate(RANDOM_MUTATION);
+                      xom_is_stimulated(100);
+                      break;
+               }
+        }
+        did_god_conduct(DID_DRINK_BLOOD, 8);
+    }
+    else if (wearing_amulet(AMU_THE_GOURMAND))
+    {
+        food_value = mass/3 + random2(you.experience_level * 5);
+        mpr("Slurps.");
+        did_god_conduct(DID_DRINK_BLOOD, 8);
+    }
+    else
+    {
+        mpr("It's not fresh enough.");
+        return false;
+    }
+
+    heal_from_food(hp_amt, mp_amt,
+        !rotten && one_chance_in(4), one_chance_in(3));
+
+    lessen_hunger( food_value, true );
+    describe_food_change(food_value);
+
+//    start_delay( DELAY_EAT, 3 );
+    start_delay( DELAY_EAT, 1 + mass/300 );
+    return true;
+} // end vampire_consume_corpse()
+
+static void heal_from_food(int hp_amt, int mp_amt, bool unrot,
+                           bool restore_str)
+{
+    if (hp_amt > 0)
+        inc_hp(hp_amt, false);
+
+    if (mp_amt > 0)
+        inc_mp(mp_amt, false);
+
+    if (unrot && player_rotted())
+    {
+        mpr("You feel more resilient.");
+        unrot_hp(1);
+    }
+
+    if (restore_str && you.strength < you.max_strength)
+    {
+        mpr("You feel your strength returning.");
+        you.strength++;
+        you.redraw_strength = true;
+        burden_change();
+    }
+
+    calc_hp();
+    calc_mp();
+} // end heal_from_food()
