@@ -3,7 +3,7 @@
  *  Summary: deal with reading and writing of highscore file
  *  Written by: Gordon Lipford
  *
- *  Modified for Crawl Reference by $Author: dshaligram $ on $Date: 2007-04-09T19:01:57.284384Z $
+ *  Modified for Crawl Reference by $Author: ennewalker $ on $Date: 2007-11-23T02:25:44.246247Z $
  *
  *  Change History (most recent first):
  *
@@ -31,6 +31,9 @@
 #include <stdlib.h>
 #include <stdio.h>
 #include <ctype.h>
+#include <unistd.h>
+#include <algorithm>
+#include <memory>
 
 #include "AppHdr.h"
 #include "externs.h"
@@ -38,15 +41,19 @@
 #include "branch.h"
 #include "files.h"
 #include "hiscores.h"
+#include "initfile.h"
 #include "itemname.h"
 #include "itemprop.h"
 #include "items.h"
 #include "libutil.h"
-#include "misc.h"
+#include "message.h"
 #include "mon-util.h"
+#include "ouch.h"
+#include "place.h"
 #include "player.h"
 #include "religion.h"
 #include "shopping.h"
+#include "state.h"
 #include "stuff.h"
 #include "tags.h"
 #include "version.h"
@@ -63,7 +70,7 @@
 #endif
 
 // enough memory allocated to snarf in the scorefile entries
-static scorefile_entry hs_list[SCORE_FILE_ENTRIES];
+static std::auto_ptr<scorefile_entry> hs_list[SCORE_FILE_ENTRIES];
 
 // hackish: scorefile position of newest entry.  Will be highlit during
 // highscore printing (always -1 when run from command line).
@@ -74,9 +81,6 @@ static void hs_close(FILE *handle, const char *mode,
                      const std::string &filename);
 static bool hs_read(FILE *scores, scorefile_entry &dest);
 static void hs_write(FILE *scores, scorefile_entry &entry);
-static void hs_nextstring(const char *&inbuf, char *dest, size_t bufsize);
-static int hs_nextint(const char *&inbuf);
-static long hs_nextlong(const char *&inbuf);
 static time_t parse_time(const std::string &st);
 
 std::string score_file_name()
@@ -100,20 +104,26 @@ void hiscores_new_entry( const scorefile_entry &ne )
     int i, total_entries;
     bool inserted = false;
 
-    // open highscore file (reading) -- note that NULL is not fatal!
-    scores = hs_open("r", score_file_name());
-
-    for (i = 0; i < SCORE_FILE_ENTRIES; ++i)
-        hs_list[i].reset();
+    // open highscore file (reading) -- NULL is fatal!
+    // 
+    // Opening as a+ instead of r+ to force an exclusive lock (see
+    // hs_open) and to create the file if it's not there already.
+    scores = hs_open("a+", score_file_name());
+    if (scores == NULL)
+        end(1, true, "failed to open score file for writing");
 
+    // we're at the end of the file, seek back to beginning.
+    fseek(scores, 0, SEEK_SET);
+    
     // read highscore file, inserting new entry at appropriate point,
     for (i = 0; i < SCORE_FILE_ENTRIES; i++)
     {
-        if (hs_read(scores, hs_list[i]) == false)
+        hs_list[i].reset(new scorefile_entry);
+        if (hs_read(scores, *hs_list[i]) == false)
             break;
 
         // compare points..
-        if (ne.points >= hs_list[i].points && inserted == false)
+        if (!inserted && ne.points >= hs_list[i]->points)
         {
             newest_entry = i;           // for later printing
             inserted = true;
@@ -122,11 +132,11 @@ void hiscores_new_entry( const scorefile_entry &ne )
             if (i+1 < SCORE_FILE_ENTRIES)
             {
                 hs_list[i + 1] = hs_list[i];
-                hs_list[i] = ne;
+                hs_list[i].reset(new scorefile_entry(ne));
                 i++;
             } else {
                 // copy new entry to current position
-                hs_list[i] = ne;
+                *hs_list[i] = ne;
             }
         }
     }
@@ -137,31 +147,37 @@ void hiscores_new_entry( const scorefile_entry &ne )
         newest_entry = i;
         inserted = true;
         // copy new entry
-        hs_list[i] = ne;
+        hs_list[i].reset(new scorefile_entry(ne));
         i++;
     }
 
+    // If we've still not inserted it, it's not a highscore.
+    if (!inserted)
+    {
+        hs_close(scores, "a+", score_file_name());
+        return;
+    }
+
     total_entries = i;
 
-    // close so we can re-open for writing
-    hs_close(scores,"r", score_file_name());
-
-    // open highscore file (writing) -- NULL *is* fatal here.
-    scores = hs_open("w", score_file_name());
-    if (scores == NULL)
-    {
-        perror("Entry not added - failure opening score file for writing.");
-        return;
-    }
+    // The old code closed and reopened the score file, leading to a
+    // race condition where one Crawl process could overwrite the
+    // other's highscore. Now we truncate and rewrite the file without
+    // closing it.
+    if (ftruncate(fileno(scores), 0))
+        end(1, true, "unable to truncate scorefile");
+    
+    rewind(scores);
 
     // write scorefile entries.
     for (i = 0; i < total_entries; i++)
     {
-        hs_write(scores, hs_list[i]);
+        hs_write(scores, *hs_list[i]);
+        hs_list[i].reset(NULL);
     }
 
     // close scorefile.
-    hs_close(scores, "w", score_file_name());
+    hs_close(scores, "a+", score_file_name());
 }
 
 void logfile_new_entry( const scorefile_entry &ne )
@@ -263,7 +279,8 @@ void hiscores_print_list( int display_count, int format )
     // read highscore file
     for (i = 0; i < SCORE_FILE_ENTRIES; i++)
     {
-        if (hs_read( scores, hs_list[i] ) == false)
+        hs_list[i].reset(new scorefile_entry);
+        if (hs_read( scores, *hs_list[i] ) == false)
             break;
     }
     total_entries = i;
@@ -289,7 +306,7 @@ void hiscores_print_list( int display_count, int format )
         if (i == newest_entry)
             textcolor(YELLOW);
 
-        hiscores_print_entry(hs_list[i], i, format, cprintf);
+        hiscores_print_entry(*hs_list[i], i, format, cprintf);
 
         if (i == newest_entry)
             textcolor(LIGHTGREY);
@@ -387,72 +404,11 @@ bool hs_read( FILE *scores, scorefile_entry &dest )
     return (dest.parse(inbuf));
 }
 
-static std::string hs_nextstring(const char *&inbuf, size_t destsize = 800)
-{
-    char *buf = new char[destsize];
-    if (!buf)
-        return ("");
-    hs_nextstring(inbuf, buf, destsize);
-    const std::string res = buf;
-    delete [] buf;
-    return (res);
-}
-
-static void hs_nextstring(const char *&inbuf, char *dest, size_t destsize)
-{
-    ASSERT(destsize > 0);
-
-    char *p = dest;
-
-    if (*inbuf == 0)
-    {
-        *p = 0;
-        return;
-    }
-
-    // assume we're on a ':'
-    if (*inbuf == ':')
-        inbuf++;
-
-    while (*inbuf && *inbuf != ':' && (p - dest) < (int) destsize - 1)
-        *p++ = *inbuf++;
-
-    // If we ran out of buffer, discard the rest of the field.
-    while (*inbuf && *inbuf != ':')
-        inbuf++;
-
-    *p = 0;
-}
-
-static int hs_nextint(const char *&inbuf)
-{
-    char num[20];
-    hs_nextstring(inbuf, num, sizeof num);
-
-    return (num[0] == 0 ? 0 : atoi(num));
-}
-
-static long hs_nextlong(const char *&inbuf)
-{
-    char num[20];
-    hs_nextstring(inbuf, num, sizeof num);
-
-    return (num[0] == 0 ? 0 : atol(num));
-}
-
 static int val_char( char digit )
 {
     return (digit - '0');
 }
 
-static time_t hs_nextdate(const char *&inbuf)
-{
-    char       buff[20];
-    hs_nextstring(inbuf, buff, sizeof buff);
-
-    return parse_time(buff);
-}
-
 static time_t parse_time(const std::string &st)
 {
     struct tm  date;
@@ -482,10 +438,11 @@ static const char *kill_method_names[] =
 {
     "mon", "pois", "cloud", "beam", "deaths_door", "lava", "water",
     "stupidity", "weakness", "clumsiness", "trap", "leaving", "winning",
-    "quitting", "draining", "starvation", "freezing", "burning", "wild_magic",
-    "xom", "statue", "rotting", "targeting", "spore", "tso_smiting",
-    "petrification", "unknown", "something", "falling_down_stairs", "acid",
-    "curare", "melting", "bleeding",
+    "quitting", "draining", "starvation", "freezing", "burning",
+    "wild_magic", "xom", "statue", "rotting", "targeting", "spore",
+    "tso_smiting", "petrification", "unknown", "something",
+    "falling_down_stairs", "acid", "curare", "melting", "bleeding",
+    "beogh_smiting"
 };
 
 const char *kill_method_name(kill_method_type kmt)
@@ -595,12 +552,15 @@ bool scorefile_entry::parse(const std::string &line)
     //    key=value. Colons are not allowed in key names, must be escaped to
     //    :: in values.
     //
-    // 0.2 only reads entries of type (3) and (4), and only writes entries of
-    // type (4). 0.3 or 0.4 may discontinue read support for (3).
+    // 0.3 only reads and writes entries of type (4).
 
     // Leading colon implies 4.0 style line:
     if (line[0] == ':')
-        return (parse_obsolete_scoreline(line));
+    {
+        end(1, false, "Cannot read 4.0-style scorefiles");
+        // Keep gcc happy:
+        return (false);
+    }
     else
         return (parse_scoreline(line));
 }
@@ -630,49 +590,26 @@ static const char *short_branch_name(int branch)
     return ("");
 }
 
-static int str_to_branch(const std::string &branch)
-{
-    for (int i = 0; i < NUM_BRANCHES; ++i)
-    {
-        if (branches[i].abbrevname && branches[i].abbrevname == branch)
-            return (i);
-    }
-    return (BRANCH_MAIN_DUNGEON);
-}
-
 static const char *level_type_names[] =
 {
-    "D", "Lab", "Abyss", "Pan"
+    "D", "Lab", "Abyss", "Pan", "Port"
 };
 
-static const char *level_area_type_name(int level_type)
+const char *level_area_type_name(int level_type)
 {
     if (level_type >= 0 && level_type < NUM_LEVEL_AREA_TYPES)
         return level_type_names[level_type];
     return ("");
 }
 
-static int str_to_level_area_type(const std::string &s)
+static level_area_type str_to_level_area_type(const std::string &s)
 {
     for (int i = 0; i < NUM_LEVEL_AREA_TYPES; ++i)
         if (s == level_type_names[i])
-            return (i);
+            return (static_cast<level_area_type>(i));
     return (LEVEL_DUNGEON);
 }
 
-static int str_to_god(const std::string &god)
-{
-    if (god.empty())
-        return GOD_NO_GOD;
-    
-    for (int i = GOD_NO_GOD; i < NUM_GODS; ++i)
-    {
-        if (god_name(i) == god)
-            return (i);
-    }
-    return (GOD_NO_GOD);
-}
-
 void scorefile_entry::init_with_fields()
 {
     version = fields->str_field("v");
@@ -690,7 +627,7 @@ void scorefile_entry::init_with_fields()
     death_type = str_to_kill_method(fields->str_field("ktyp"));
     death_source_name = fields->str_field("killer");
     auxkilldata = fields->str_field("kaux");
-    branch = str_to_branch(fields->str_field("br"));
+    branch = str_to_branch(fields->str_field("br"), BRANCH_MAIN_DUNGEON);
     dlvl   = fields->int_field("lvl");
     level_type = str_to_level_area_type(fields->str_field("ltyp"));
 
@@ -723,7 +660,9 @@ void scorefile_entry::set_base_xlog_fields() const
     fields->add_field("lv", SCORE_VERSION);
     fields->add_field("name", "%s", name.c_str());
     fields->add_field("uid", "%d", uid);
-    fields->add_field("race", "%s", species_name(race, lvl));
+    fields->add_field("race", "%s",
+                      species_name(static_cast<species_type>(race),
+                                   lvl).c_str());
     fields->add_field("cls", "%s", get_class_name(cls));
     fields->add_field("char", "%s%s",
                       get_species_abbrev(race),
@@ -731,8 +670,9 @@ void scorefile_entry::set_base_xlog_fields() const
     fields->add_field("xl", "%d", lvl);
     fields->add_field("sk", "%s", skill_name(best_skill));
     fields->add_field("sklev", "%d", best_skill_lvl);
-    fields->add_field("title", "%s", skill_title( best_skill, best_skill_lvl, 
-                                                  race, str, dex, god ) );
+    fields->add_field("title", "%s",
+                      skill_title( best_skill, best_skill_lvl, 
+                                   race, str, dex, god ).c_str() );
 
     fields->add_field("place", "%s",
               place_name(get_packed_place(branch, dlvl, level_type),
@@ -777,7 +717,7 @@ void scorefile_entry::set_score_fields() const
     
     fields->add_field("sc", "%ld", points);
     fields->add_field("ktyp", ::kill_method_name(kill_method_type(death_type)));
-    fields->add_field("killer", death_source_desc());
+    fields->add_field("killer", death_source_desc().c_str());
 
     fields->add_field("kaux", "%s", auxkilldata.c_str());
 
@@ -830,101 +770,6 @@ std::string scorefile_entry::short_kill_message() const
     return (msg);
 }
 
-// Maps a 0.1.x branch id to a 0.2 branch id. Ugh. Fortunately we need this
-// only to read old logfiles/scorefiles.
-int scorefile_entry::kludge_branch(int branch_01) const
-{
-    static int branch_map[] = {
-        BRANCH_MAIN_DUNGEON, BRANCH_DIS, BRANCH_GEHENNA,
-        BRANCH_VESTIBULE_OF_HELL, BRANCH_COCYTUS, BRANCH_TARTARUS,
-        BRANCH_INFERNO, BRANCH_THE_PIT, BRANCH_MAIN_DUNGEON,
-        BRANCH_MAIN_DUNGEON, BRANCH_ORCISH_MINES, BRANCH_HIVE,
-        BRANCH_LAIR, BRANCH_SLIME_PITS, BRANCH_VAULTS, BRANCH_CRYPT,
-        BRANCH_HALL_OF_BLADES, BRANCH_HALL_OF_ZOT, BRANCH_ECUMENICAL_TEMPLE,
-        BRANCH_SNAKE_PIT, BRANCH_ELVEN_HALLS, BRANCH_TOMB, BRANCH_SWAMP,
-        BRANCH_CAVERNS
-    };
-
-    if (branch_01 < 0
-        || branch_01 > (int) (sizeof(branch_map) / sizeof(*branch_map)))
-    {
-        return (BRANCH_MAIN_DUNGEON);
-    }
-    return branch_map[branch_01];
-}
-
-// [ds] This is the 4.0 b26 logfile parser. Old-style logs are now deprecated;
-// support for reading them may be discontinued in the next version.
-bool scorefile_entry::parse_obsolete_scoreline(const std::string &line)
-{
-    const char *inbuf = line.c_str();
-    
-    const int ver = hs_nextint(inbuf);
-    const int rel = hs_nextint(inbuf);
-
-    // this would be a good point to check for version numbers and branch
-    // appropriately
-
-    // acceptable versions are 0 (converted from old hiscore format) and 4
-    if (ver != 4 || rel < 2)
-        return (false);
-
-    points = hs_nextlong(inbuf);
-
-    name = hs_nextstring(inbuf);
-
-    uid = hs_nextlong(inbuf);
-    race = hs_nextint(inbuf);
-    cls = hs_nextint(inbuf);
-
-    race_class_name = hs_nextstring(inbuf, 6);
-
-    lvl = hs_nextint(inbuf);
-    best_skill = hs_nextint(inbuf);
-    best_skill_lvl = hs_nextint(inbuf);
-    death_type = hs_nextint(inbuf);
-    death_source = hs_nextint(inbuf);
-    mon_num = hs_nextint(inbuf);
-
-    death_source_name = hs_nextstring(inbuf);
-
-    // To try and keep the scorefile backwards compatible,
-    // we'll branch on version > 4.0 to read the auxkilldata
-    // text field.  
-    if (ver == 4 && rel >= 1)
-        auxkilldata = hs_nextstring( inbuf, ITEMNAME_SIZE );
-    else
-        auxkilldata[0] = 0;
-
-    dlvl = hs_nextint(inbuf);
-    level_type = hs_nextint(inbuf);
-    branch = kludge_branch( hs_nextint(inbuf) );
-
-    final_hp = hs_nextint(inbuf);
-    final_max_hp = hs_nextint(inbuf);
-    final_max_max_hp = hs_nextint(inbuf);
-    damage = hs_nextint(inbuf);
-    str = hs_nextint(inbuf);
-    intel = hs_nextint(inbuf);
-    dex = hs_nextint(inbuf);
-    god = hs_nextint(inbuf);
-    piety = hs_nextint(inbuf);
-    penance = hs_nextint(inbuf);
-
-    wiz_mode = hs_nextint(inbuf);
-
-    birth_time = hs_nextdate(inbuf);
-    death_time = hs_nextdate(inbuf);
-
-    real_time = hs_nextint(inbuf);
-    num_turns = hs_nextint(inbuf);
-
-    num_diff_runes = hs_nextint(inbuf);
-    num_runes = hs_nextint(inbuf);
-
-    return (true);
-}
-
 void scorefile_entry::init_death_cause(int dam, int dsrc, 
                                        int dtype, const char *aux)
 {
@@ -954,8 +799,6 @@ void scorefile_entry::init_death_cause(int dam, int dsrc,
             // but now we pass it in as a string through the scorefile
             // entry to be appended in hiscores_format_single in long or
             // medium scorefile formats.
-            // It still isn't used in monam for anything but flying weapons
-            // though
             if (death_type == KILLED_BY_MONSTER 
                 && monster->inv[MSLOT_WEAPON] != NON_ITEM)
             {
@@ -983,13 +826,18 @@ void scorefile_entry::init_death_cause(int dam, int dsrc,
                 // Setting this is redundant for dancing weapons, however
                 // we do care about the above indentification. -- bwr
                 if (monster->type != MONS_DANCING_WEAPON) 
-                    auxkilldata = it_name( monster->inv[MSLOT_WEAPON], 
-                                           DESC_NOCAP_A );
+                    auxkilldata = mitm[monster->inv[MSLOT_WEAPON]].name(DESC_NOCAP_A);
             }
 
-            death_source_name =
-                monam( monster, monster->number, monster->type, true,
-                       DESC_NOCAP_A, monster->inv[MSLOT_WEAPON] );
+            death_source_name = monster->name(DESC_NOCAP_A, true);
+            if (monster->has_ench(ENCH_SHAPESHIFTER))
+            {
+                death_source_name += " (shapeshifter)";
+            }
+            else if (monster->has_ench(ENCH_GLOWING_SHAPESHIFTER))
+            {
+                death_source_name += " (glowing shapeshifter)";
+            }
         }
     }
     else
@@ -1019,8 +867,8 @@ void scorefile_entry::reset()
     death_source_name[0] = 0;
     auxkilldata[0] = 0;
     dlvl = 0;
-    level_type = 0;
-    branch = 0;
+    level_type = LEVEL_DUNGEON;
+    branch = BRANCH_MAIN_DUNGEON;
     final_hp = -1;
     final_max_hp = -1;
     final_max_max_hp = -1;
@@ -1028,7 +876,7 @@ void scorefile_entry::reset()
     intel = -1;
     dex = -1;
     damage = -1;
-    god = -1;
+    god = GOD_NO_GOD;
     piety = -1;
     penance = -1;
     wiz_mode = 0;
@@ -1052,7 +900,7 @@ void scorefile_entry::init()
     name    = you.your_name;
 
 #ifdef MULTIUSER
-    uid = (int) getuid();
+    uid = static_cast<int>(getuid());
 #else
     uid = 0;
 #endif
@@ -1064,13 +912,6 @@ void scorefile_entry::init()
     //if (death_type == KILLED_BY_WINNING) points += points / 2;
     //if (death_type == KILLED_BY_LEAVING) points += points / 10;
     // these now handled by giving player the value of their inventory
-    char temp_id[4][50];
-
-    for (int d = 0; d < 4; d++)
-    {
-        for (int e = 0; e < 50; e++)
-            temp_id[d][e] = 1;
-    }
 
     FixedVector< int, NUM_RUNE_TYPES >  rune_array;
 
@@ -1089,7 +930,7 @@ void scorefile_entry::init()
         if (is_valid_item( you.inv[d] ))
         {
             if (calc_item_values)
-                points += item_value( you.inv[d], temp_id, true );
+                points += item_value( you.inv[d], true );
 
             if (you.inv[d].base_type == OBJ_MISCELLANY
                 && you.inv[d].sub_type == MISC_RUNE_OF_ZOT)
@@ -1125,9 +966,9 @@ void scorefile_entry::init()
     final_hp = you.hp;
     final_max_hp = you.hp_max;
     final_max_max_hp = you.hp_max + player_rotted();
-    str = you.strength;
-    intel = you.intel;
-    dex = you.dex;
+    str = std::max(you.strength - stat_modifier(STAT_STRENGTH), 1);
+    intel = std::max(you.intel - stat_modifier(STAT_INTELLIGENCE), 1);
+    dex = std::max(you.dex - stat_modifier(STAT_DEXTERITY), 1);
 
     god = you.religion;
     if (you.religion != GOD_NO_GOD)
@@ -1178,7 +1019,6 @@ std::string scorefile_entry::game_time(death_desc_verbosity verbosity) const
         {
             char username[80] = "The";
             char scratch[INFO_SIZE];
-            char tmp[80];
 
 #ifdef MULTIUSER
             if (uid > 0)
@@ -1192,11 +1032,9 @@ std::string scorefile_entry::game_time(death_desc_verbosity verbosity) const
                 }
             }
 #endif
-
-            make_time_string( real_time, tmp, sizeof(tmp) );
-
             snprintf( scratch, INFO_SIZE, "%s game lasted %s (%ld turns).",
-                      username, tmp, num_turns );
+                      username, make_time_string(real_time).c_str(),
+                      num_turns );
 
             line += scratch;
             line += hiscore_newline_string();
@@ -1216,14 +1054,14 @@ const char *scorefile_entry::damage_verb() const
                               "Annihilated";
 }
 
-const char *scorefile_entry::death_source_desc() const
+std::string scorefile_entry::death_source_desc() const
 {
     if (death_type != KILLED_BY_MONSTER && death_type != KILLED_BY_BEAM)
         return ("");
 
+    // XXX no longer handles mons_num correctly! FIXME
     return (!death_source_name.empty()? 
-            death_source_name.c_str()
-            : monam( NULL, mon_num, death_source, true, DESC_NOCAP_A ) );
+            death_source_name : mons_type_name(death_source, DESC_NOCAP_A));
 }
 
 std::string scorefile_entry::damage_string(bool terse) const
@@ -1359,19 +1197,18 @@ scorefile_entry::character_description(death_desc_verbosity verbosity) const
     // Please excuse the following bit of mess in the name of flavour ;)
     if (verbose)
     {
-        strncpy( scratch, skill_title( best_skill, best_skill_lvl, 
-                                       race, str, dex, god ), 
-                 INFO_SIZE );
-
         snprintf( buf, HIGHSCORE_SIZE, "%8ld %s the %s (level %d",
-                  points, name.c_str(), scratch, lvl );
-
+                  points, name.c_str(),
+                  skill_title( best_skill, best_skill_lvl, 
+                               race, str, dex, god ).c_str(),
+                  lvl );
         desc = buf;
     }
     else 
     {
         snprintf( buf, HIGHSCORE_SIZE, "%8ld %s the %s %s (level %d",
-                  points, name.c_str(), species_name(race, lvl), 
+                  points, name.c_str(),
+                  species_name(static_cast<species_type>(race), lvl).c_str(), 
                   get_class_name(cls), lvl );
         desc = buf;
     }
@@ -1395,10 +1232,11 @@ scorefile_entry::character_description(death_desc_verbosity verbosity) const
 
     if (verbose)
     {
-        const char *const srace = species_name( race, lvl );
-
+        std::string srace = species_name(static_cast<species_type>(race), lvl);
         snprintf( scratch, INFO_SIZE, "Began as a%s %s %s",
-                  is_vowel(*srace) ? "n" : "", srace, get_class_name(cls) );
+                  is_vowel(srace[0]) ? "n" : "",
+                  srace.c_str(),
+                  get_class_name(cls) );
         desc += scratch;
 
         if (birth_time > 0)
@@ -1423,7 +1261,7 @@ scorefile_entry::character_description(death_desc_verbosity verbosity) const
             }
             else if (god != GOD_NO_GOD)
             {
-                // Not exactly the same as the religon screen, but
+                // Not exactly the same as the religion screen, but
                 // good enough to fill this slot for now.
                 snprintf( scratch, INFO_SIZE, "Was %s of %s%s",
                           (piety >  160) ? "the Champion" :
@@ -1518,14 +1356,12 @@ scorefile_entry::death_description(death_desc_verbosity verbosity) const
         if (terse)
             desc += death_source_desc();
         else if (oneline)
-            desc += std::string("slain by ") + death_source_desc();
+            desc += "slain by " + death_source_desc();
         else
         {
-            snprintf( scratch, sizeof scratch, "%s by %s",
-                        damage_verb(),
-                        death_source_desc() );
-
-            desc += scratch;
+            desc += damage_verb();
+            desc += " by ";
+            desc += death_source_desc();
         }
 
         // put the damage on the weapon line if there is one
@@ -1630,7 +1466,8 @@ scorefile_entry::death_description(death_desc_verbosity verbosity) const
         break;
 
     case KILLED_BY_STUPIDITY:
-        desc += terse? "stupidity" : "Forgot to breathe";
+        desc += terse? "stupidity" :
+                (you.is_undead? "Forgot to exist" : "Forgot to breathe");
         break;
 
     case KILLED_BY_WEAKNESS:
@@ -1743,6 +1580,11 @@ scorefile_entry::death_description(death_desc_verbosity verbosity) const
         needs_damage = true;
         break;
 
+    case KILLED_BY_BEOGH_SMITING:
+        desc += terse? "smote by Beogh" : "Smote by Beogh";
+        needs_damage = true;
+        break;
+
     case KILLED_BY_PETRIFICATION:
         desc += terse? "petrified" : "Turned to stone";
         break;
