@@ -8,4 +8,7 @@ void demon_taunt( const monsters *mons );
 const char * generic_insult(void);
 const char * racial_insult(void);
 
+std::string imp_taunt_str();
+std::string demon_taunt_str();
+
 #endif
