@@ -3,7 +3,7 @@
  *  Summary:    Functions for inventory related commands.
  *  Written by: Linley Henzell
  *
- *  Modified for Crawl Reference by $Author: dshaligram $ on $Date: 2007-03-06T05:56:50.150565Z $
+ *  Modified for Crawl Reference by $Author: ennewalker $ on $Date: 2007-11-23T02:25:44.246247Z $
  *
  *  Change History (most recent first):
  *
@@ -20,6 +20,8 @@
 #include <stdlib.h>
 #include <string.h>
 #include <ctype.h>
+#include <sstream>
+#include <iomanip>
 
 #ifdef DOS
 #include <conio.h>
@@ -28,14 +30,19 @@
 #include "externs.h"
 
 #include "clua.h"
-#include "itemname.h"
+#include "describe.h"
+#include "food.h"
+#include "initfile.h"
+#include "itemprop.h"
 #include "items.h"
 #include "macro.h"
+#include "message.h"
 #include "player.h"
 #include "shopping.h"
 #include "stuff.h"
 #include "view.h"
 #include "menu.h"
+#include "randart.h"
 
 ///////////////////////////////////////////////////////////////////////////////
 // Inventory menu shenanigans
@@ -60,53 +67,138 @@ std::string InvTitle::get_text() const
 InvEntry::InvEntry( const item_def &i ) : MenuEntry( "", MEL_ITEM ), item( &i )
 {
     data = const_cast<item_def *>( item );
-        
-    char buf[ITEMNAME_SIZE];
-    if (i.base_type == OBJ_GOLD)
-        snprintf(buf, sizeof buf, "%d gold piece%s", i.quantity, 
-                (i.quantity > 1? "s" : ""));
+
+    if ( in_inventory(i) && i.base_type != OBJ_GOLD )
+    {
+        // We need to do this in order to get the 'wielded' annotation.
+        // We then toss out the first four characters, which look
+        // like this: "a - ". Ow. FIXME.
+        text = i.name(DESC_INVENTORY_EQUIP, false).substr(4);
+    }
     else
-        item_name(i, 
-                in_inventory(i)? 
-                    DESC_INVENTORY_EQUIP : DESC_NOCAP_A, buf, false);    
-    text = buf;
+        text = i.name(DESC_NOCAP_A, false);
 
     if (i.base_type != OBJ_GOLD && in_inventory(i))
-    {
-        // FIXME: This is HORRIBLE! We're skipping the inventory letter prefix 
-        // which looks like this: "a - ".
-        text = text.substr( 4 );
         add_hotkey(index_to_letter( i.link ));
-    }
     else
-    {
-        // Dummy hotkey for gold or non-inventory items.
-        add_hotkey(' ');
-    }
+        add_hotkey(' ');        // dummy hotkey
+
     add_class_hotkeys(i);
 
     quantity = i.quantity;
 }
 
+const std::string &InvEntry::get_basename() const
+{
+    if (basename.empty())
+        basename = item->name(DESC_BASENAME);
+    return (basename);
+}
+
+const std::string &InvEntry::get_qualname() const
+{
+    if (qualname.empty())
+        qualname = item->name(DESC_QUALNAME);
+    return (qualname);
+}
+
+const std::string &InvEntry::get_fullname() const
+{
+    return (text);
+}
+
+const bool InvEntry::is_item_cursed() const
+{
+    return (item_ident(*item, ISFLAG_KNOW_CURSE) && item_cursed(*item));
+}
+
+const bool InvEntry::is_item_glowing() const
+{
+    return (!item_ident(*item, ISFLAG_KNOW_TYPE)
+            && (get_equip_desc(*item)
+                || (is_artefact(*item)
+                    && (item->base_type == OBJ_WEAPONS
+                        || item->base_type == OBJ_MISSILES
+                        || item->base_type == OBJ_ARMOUR))));
+}
+
+const bool InvEntry::is_item_ego() const
+{
+    return (item_ident(*item, ISFLAG_KNOW_TYPE) && !is_artefact(*item)
+            && item->special != 0
+            && (item->base_type == OBJ_WEAPONS
+                || item->base_type == OBJ_MISSILES
+                || item->base_type == OBJ_ARMOUR));
+}
+
+const bool InvEntry::is_item_art() const
+{
+    return (item_ident(*item, ISFLAG_KNOW_TYPE) && is_artefact(*item));
+}
+
+const bool InvEntry::is_item_equipped() const
+{
+    if (item->link == -1 || item->x != -1 || item->y != -1)
+        return(false);
+
+    for (int i = 0; i < NUM_EQUIP; i++)
+        if (item->link == you.equip[i])
+            return (true);
+
+    return (false);
+}
+
+// returns values < 0 for edible chunks (non-rotten except for Saprovores),
+// 0 for non-chunks, and values > 0 for rotten chunks for non-Saprovores
+const int InvEntry::item_freshness() const
+{
+    if (item->base_type != OBJ_FOOD || item->sub_type != FOOD_CHUNK)
+        return 0;
+
+    int freshness = item->special;
+    
+    if (freshness >= 100 || you.species == SP_TROLL || you.species == SP_KOBOLD 
+                         || you.species == SP_GHOUL || you.species == SP_OGRE
+                         || you.species == SP_HILL_ORC)
+    {
+        freshness -= 300;
+    }
+
+    // Ensure that chunk freshness is never zero, since zero means
+    // that the item isn't a chunk.
+    if (freshness >= 0) // possibly rotten chunks
+        freshness++;
+
+    return freshness;
+}
+
 std::string InvEntry::get_text() const
 {
-    char buf[ITEMNAME_SIZE];
-    char suffix[ITEMNAME_SIZE] = "";
+    std::ostringstream tstr;
 
+    tstr << ' ' << static_cast<char>(hotkeys[0]) << ' ';
+    if ( !selected_qty )
+        tstr << '-';
+    else if ( selected_qty < quantity )
+        tstr << '#';
+    else
+        tstr << '+';
+    tstr << ' ' << text;
     if (InvEntry::show_prices)
     {
-        int value = item_value(*item, temp_id, true);
+        const int value = item_value(*item, show_prices);
         if (value > 0)
-            snprintf(suffix, sizeof suffix,
-                " (%d gold)", value);
+            tstr << " (" << value << " gold)";
     }
-    snprintf( buf, sizeof buf,
-            " %c %c %s%s",
-            hotkeys[0],
-            (!selected_qty? '-' : selected_qty < quantity? '#' : '+'),
-            text.c_str(),
-            suffix );
-    return (buf);
+
+    if ( Options.show_inventory_weights )
+    {
+        const int mass = item_mass(*item) * item->quantity;
+        tstr << std::setw(get_number_of_cols() - tstr.str().length() - 1)
+             << std::right
+             << make_stringf("(%d.%d aum)", mass / 10, mass % 10);
+    }
+    return tstr.str();
 }
 
 void InvEntry::add_class_hotkeys(const item_def &i)
@@ -160,14 +252,10 @@ void InvEntry::add_class_hotkeys(const item_def &i)
 }
 
 bool InvEntry::show_prices = false;
-char InvEntry::temp_id[4][50];
 
 void InvEntry::set_show_prices(bool doshow)
 {
-    if ((show_prices = doshow))
-    {
-        memset(temp_id, 1, sizeof temp_id);
-    }
+    show_prices = doshow;
 }
 
 InvShowPrices::InvShowPrices(bool doshow)
@@ -251,6 +339,9 @@ void InvMenu::load_inv_items(int item_selector,
         case OBJ_WEAPONS:
             s = "You aren't carrying any weapons.";
             break;
+        case OSEL_UNIDENT:
+            s = "You don't have any unidentified items.";
+            break;
         default:
             s = "You aren't carrying any such object.";
             break;
@@ -263,9 +354,163 @@ void InvMenu::load_inv_items(int item_selector,
     }
 }
 
-static bool compare_menu_entries(const MenuEntry* a, const MenuEntry* b)
+template <std::string (*proc)(const InvEntry *a)>
+int compare_item_str(const InvEntry *a, const InvEntry *b)
 {
-    return (*a < *b);
+    return (proc(a).compare(proc(b)));
+}
+
+template <typename T, T (*proc)(const InvEntry *a)>
+int compare_item(const InvEntry *a, const InvEntry *b)
+{
+    return (int(proc(a)) - int(proc(b)));
+}
+
+// C++ needs anonymous subs already!
+std::string sort_item_basename(const InvEntry *a)
+{
+    return a->get_basename();
+}
+std::string sort_item_qualname(const InvEntry *a)
+{
+    return a->get_qualname();
+}
+std::string sort_item_fullname(const InvEntry *a)
+{
+    return a->get_fullname();
+}
+int sort_item_qty(const InvEntry *a)
+{
+    return a->quantity;
+}
+int sort_item_slot(const InvEntry *a)
+{
+    return a->item->link;
+}
+
+int sort_item_freshness(const InvEntry *a)
+{
+    return a->item_freshness();
+}
+
+bool sort_item_curse(const InvEntry *a)
+{
+    return a->is_item_cursed();
+}
+
+bool sort_item_glowing(const InvEntry *a)
+{
+    return !a->is_item_glowing();
+}
+
+bool sort_item_ego(const InvEntry *a)
+{
+    return !a->is_item_ego();
+}
+
+bool sort_item_art(const InvEntry *a)
+{
+    return !a->is_item_art();
+}
+
+bool sort_item_equipped(const InvEntry *a)
+{
+    return !a->is_item_equipped();
+}
+
+static bool compare_invmenu_items(const InvEntry *a, const InvEntry *b,
+                                  const item_sort_comparators *cmps)
+{
+    for (item_sort_comparators::const_iterator i = cmps->begin();
+         i != cmps->end();
+         ++i)
+    {
+        const int cmp = i->compare(a, b);
+        if (cmp)
+            return (cmp < 0);
+    }
+    return (false);
+}
+
+struct menu_entry_comparator
+{
+    const menu_sort_condition *cond;
+    
+    menu_entry_comparator(const menu_sort_condition *c)
+        : cond(c)
+    {
+    }
+
+    bool operator () (const MenuEntry* a, const MenuEntry* b) const
+    {
+        const InvEntry *ia = dynamic_cast<const InvEntry *>(a);
+        const InvEntry *ib = dynamic_cast<const InvEntry *>(b);
+        return compare_invmenu_items(ia, ib, &cond->cmp);
+    }
+};
+
+void init_item_sort_comparators(item_sort_comparators &list,
+                                const std::string &set)
+{
+    static struct
+    {
+        const std::string cname;
+        item_sort_fn cmp;
+    } cmp_map[]  =
+      {
+          { "basename",  compare_item_str<sort_item_basename> },
+          { "qualname",  compare_item_str<sort_item_qualname> },
+          { "fullname",  compare_item_str<sort_item_fullname> },
+          { "curse",     compare_item<bool, sort_item_curse> },
+          { "glowing",   compare_item<bool, sort_item_glowing> },
+          { "ego",       compare_item<bool, sort_item_ego> },
+          { "art",       compare_item<bool, sort_item_art> },
+          { "equipped",  compare_item<bool, sort_item_equipped> },
+          { "qty",       compare_item<int, sort_item_qty> },
+          { "slot",      compare_item<int, sort_item_slot> },
+          { "freshness", compare_item<int, sort_item_freshness> }
+      };
+
+    list.clear();
+    std::vector<std::string> cmps = split_string(",", set);
+    for (int i = 0, size = cmps.size(); i < size; ++i)
+    {
+        std::string s = cmps[i];
+        if (s.empty())
+            continue;
+        
+        const bool negated = s[0] == '>';
+        if (s[0] == '<' || s[0] == '>')
+            s = s.substr(1);
+
+        for (unsigned ci = 0; ci < ARRAYSIZE(cmp_map); ++ci)
+            if (cmp_map[ci].cname == s)
+            {
+                list.push_back( item_comparator( cmp_map[ci].cmp, negated ) );
+                break;
+            }
+    }
+
+    if (list.empty())
+        list.push_back(
+            item_comparator(compare_item_str<sort_item_fullname>));
+}
+
+const menu_sort_condition *InvMenu::find_menu_sort_condition() const
+{
+    for (int i = 0, size = Options.sort_menus.size(); i < size; ++i)
+        if (Options.sort_menus[i].matches(type))
+            return &Options.sort_menus[i];
+    return (NULL);
+}
+
+void InvMenu::sort_menu(std::vector<InvEntry*> &invitems,
+                        const menu_sort_condition *cond)
+{
+    if (!cond || cond->sort == -1 || (int) invitems.size() < cond->sort)
+        return;
+    
+    std::sort( invitems.begin(), invitems.end(), menu_entry_comparator(cond) );
 }
 
 void InvMenu::load_items(const std::vector<const item_def*> &mitems,
@@ -278,6 +523,8 @@ void InvMenu::load_items(const std::vector<const item_def*> &mitems,
     menu_letter ckey;
     std::vector<InvEntry*> items_in_class;
 
+    const menu_sort_condition *cond = find_menu_sort_condition();
+
     for (int i = 0; i < NUM_OBJECT_CLASSES; ++i)
     {
         if (!inv_class[i]) continue;
@@ -291,10 +538,7 @@ void InvMenu::load_items(const std::vector<const item_def*> &mitems,
             items_in_class.push_back( new InvEntry(*mitems[j]) );
         }
 
-        if (Options.sort_menus != -1 &&
-            (int)items_in_class.size() >= Options.sort_menus)
-            std::sort( items_in_class.begin(), items_in_class.end(),
-                       compare_menu_entries );
+        sort_menu(items_in_class, cond);
 
         for (unsigned int j = 0; j < items_in_class.size(); ++j)
         {
@@ -381,9 +625,21 @@ bool in_inventory( const item_def &i )
 
 unsigned char get_invent( int invent_type )
 {
-    unsigned char nothing = invent_select(NULL, MT_INVLIST, invent_type);
+    unsigned char select;
+    
+    while (true) {
+       select = invent_select(NULL, MT_INVLIST, invent_type,
+                                         MF_SINGLESELECT);
+       if ( isalpha(select) )
+       {
+           const int invidx = letter_to_index(select);
+           if ( is_valid_item(you.inv[invidx]) )
+               describe_item( you.inv[invidx], true );
+       }
+       else break;
+    }
     redraw_screen();
-    return (nothing);
+    return select;
 }                               // end get_invent()
 
 std::string item_class_name( int type, bool terse )
@@ -400,7 +656,7 @@ std::string item_class_name( int type, bool terse )
         case OBJ_FOOD:       return ("food");
         case OBJ_UNKNOWN_I:  return ("?");
         case OBJ_SCROLLS:    return ("scroll");
-        case OBJ_JEWELLERY:  return ("jewelry");
+        case OBJ_JEWELLERY:  return ("jewellery");
         case OBJ_POTIONS:    return ("potion");
         case OBJ_UNKNOWN_II: return ("?");
         case OBJ_BOOKS:      return ("book");
@@ -436,15 +692,18 @@ std::string item_class_name( int type, bool terse )
 }
 
 std::vector<SelItem> select_items( const std::vector<const item_def*> &items,
-                                   const char *title, bool noselect )
+                                   const char *title, bool noselect,
+                                   menu_type mtype )
 {
     std::vector<SelItem> selected;
     if (!items.empty())
     {
         InvMenu menu;
+        menu.set_type(mtype);
         menu.set_title(title);
         menu.load_items(items);
-        menu.set_flags(noselect ? MF_NOSELECT : MF_MULTISELECT);
+        menu.set_flags(noselect ? MF_NOSELECT :
+                       MF_MULTISELECT | MF_ALLOW_FILTER);
         menu.show();
         selected = menu.get_selitems();
     }
@@ -459,6 +718,8 @@ static bool item_class_selected(const item_def &i, int selector)
 
     switch (selector)
     {
+    case OSEL_UNIDENT:
+        return !fully_identified(i);
     case OBJ_MISSILES:
         return (itype == OBJ_MISSILES || itype == OBJ_WEAPONS);
     case OBJ_WEAPONS:
@@ -467,6 +728,13 @@ static bool item_class_selected(const item_def &i, int selector)
                 || itype == OBJ_MISCELLANY);
     case OBJ_SCROLLS:
         return (itype == OBJ_SCROLLS || itype == OBJ_BOOKS);
+    case OSEL_EQUIP:
+        for (int eq = 0; eq < NUM_EQUIP; eq++)
+        {
+             if (you.equip[eq] == i.link)
+                 return (true);
+        }
+        // fall through
     default:
         return (false);
     }
@@ -632,7 +900,7 @@ std::vector<SelItem> prompt_invent_items(
                 Options.drop_mode == DM_SINGLE 
                     && (!pre_select || pre_select->empty())?
                         MF_SINGLESELECT | MF_EASY_EXIT | MF_ANYPRINTABLE :
-                        MF_MULTISELECT;
+                        MF_MULTISELECT | MF_ALLOW_FILTER;
             // The "view inventory listing" mode.
             int ch = invent_select(
                         prompt,
@@ -713,15 +981,13 @@ std::vector<SelItem> prompt_invent_items(
 
 static int digit_to_index( char digit, operation_types oper ) {
     
-    int i;
-    unsigned int j;
-    char iletter = (char)(oper);
+    const char iletter = static_cast<char>(oper);
 
-    for ( i = 0; i < ENDOFPACK; ++i ) {
+    for ( int i = 0; i < ENDOFPACK; ++i ) {
         if (is_valid_item(you.inv[i])) {
             const std::string& r(you.inv[i].inscription);
             /* note that r.size() is unsigned */
-            for ( j = 0; j + 2 < r.size(); ++j ) {
+            for ( unsigned int j = 0; j + 2 < r.size(); ++j ) {
                 if ( r[j] == '@' &&
                      (r[j+1] == iletter || r[j+1] == '*') &&
                      r[j+2] == digit ) {
@@ -733,35 +999,120 @@ static int digit_to_index( char digit, operation_types oper ) {
     return -1;
 }
 
-static bool has_warning_inscription(const item_def& item,
-                                    operation_types oper)
+bool has_warning_inscription(const item_def& item,
+                             operation_types oper)
 {
-    char iletter = (char)(oper);
-    unsigned int i;
-    char name[ITEMNAME_SIZE];
-    item_name(item, DESC_INVENTORY, name, false);
+    const char iletter = static_cast<char>(oper);
     
     const std::string& r(item.inscription);
-    for ( i = 0; i + 1 < r.size(); ++i )
+    for ( unsigned int i = 0; i + 1 < r.size(); ++i )
         if (r[i] == '!' && (r[i+1] == iletter || r[i+1] == '*'))
             return true;
     return false;
-}    
+}
+
+// checks if current item (to be removed) has a warning inscription
+// and prompts the user for confirmation
+static bool check_old_item_warning( const item_def& item,
+                                    operation_types oper )
+{
+    item_def old_item;
+    std::string prompt = "";
+    if (oper == OPER_WIELD) // can we safely unwield old item?
+    {
+        if (you.equip[EQ_WEAPON] == -1)
+            return (true);
+            
+        old_item = you.inv[you.equip[EQ_WEAPON]];
+        if (!has_warning_inscription(old_item, OPER_WIELD))
+            return (true);
+            
+        prompt += "Really unwield ";
+    }
+    else if (oper == OPER_WEAR) // can we safely take off old item?
+    {
+        if (item.base_type != OBJ_ARMOUR)
+            return (true);
+
+        equipment_type eq_slot = get_armour_slot(item);
+        if (you.equip[eq_slot] == -1)
+            return (true);
+
+        old_item = you.inv[you.equip[eq_slot]];
+
+        if (!has_warning_inscription(old_item, OPER_TAKEOFF))
+            return (true);
+            
+        prompt += "Really take off ";
+    }
+    else if (oper == OPER_PUTON) // can we safely remove old item?
+    {
+        if (item.base_type != OBJ_JEWELLERY)
+            return (true);
+            
+        if (jewellery_is_amulet(item))
+        {
+            if (you.equip[EQ_AMULET] == -1)
+                return (true);
+
+            old_item = you.inv[you.equip[EQ_AMULET]];
+            if (!has_warning_inscription(old_item, OPER_TAKEOFF))
+                return (true);
+                
+            prompt += "Really remove ";
+        }
+        else // rings handled in prompt_ring_to_remove
+            return (true);
+    }
+    else // anything else doesn't have a counterpart
+        return (true);
+
+    // now ask
+    prompt += old_item.name(DESC_INVENTORY);
+    prompt += '?';
+    return yesno(prompt.c_str(), false, 'n');
+}
+
+static std::string operation_verb(operation_types oper)
+{
+    switch (oper)
+    {
+    case OPER_WIELD:          return "wield";
+    case OPER_QUAFF:          return "quaff";
+    case OPER_DROP:           return "drop";
+    case OPER_EAT:            return "eat";
+    case OPER_TAKEOFF:        return "take off";
+    case OPER_WEAR:           return "wear";
+    case OPER_PUTON:          return "put on";
+    case OPER_REMOVE:         return "remove";
+    case OPER_READ:           return "read";
+    case OPER_MEMORISE:       return "memorise from";
+    case OPER_ZAP:            return "zap";
+    case OPER_THROW:          return "throw";
+    case OPER_EXAMINE:        return "examine";
+    case OPER_FIRE:           return "fire";
+    case OPER_PRAY:           return "sacrifice";
+    case OPER_EVOKE:          return "evoke";
+    case OPER_ANY:
+    default:
+        return "choose";
+    }
+}
 
 /* return true if user OK'd it (or no warning), false otherwise */ 
 bool check_warning_inscriptions( const item_def& item,
                                  operation_types oper )
 {
-    char prompt[ITEMNAME_SIZE + 100];
-    char name[ITEMNAME_SIZE];
-    if ( has_warning_inscription(item, oper) )
+    if (is_valid_item( item ) && has_warning_inscription(item, oper) )
     {
-        snprintf(prompt, sizeof prompt, "Really choose %s?",
-                 item_name(item, DESC_INVENTORY, name, false));
-        return yesno(prompt, false, 'n');
+        std::string prompt = "Really " + operation_verb(oper) + " ";
+        prompt += item.name(DESC_INVENTORY);
+        prompt += "? ";
+        return (yesno(prompt.c_str(), false, 'n')
+                && check_old_item_warning(item, oper));
     }
     else
-        return true;
+        return (check_old_item_warning(item, oper));
 }
 
 // This function prompts the user for an item, handles the '?' and '*'
