@@ -3,7 +3,7 @@
  *  Summary:    Functions for using some of the wackier inventory items.
  *  Written by: Linley Henzell
  *
- *  Modified for Crawl Reference by $Author: dshaligram $ on $Date: 2007-04-16T17:11:14.444826Z $
+ *  Modified for Crawl Reference by $Author: dshaligram $ on $Date: 2007-12-02T13:23:51.271663Z $
  *
  *  Change History (most recent first):
  *
@@ -22,15 +22,20 @@
 #include "externs.h"
 
 #include "beam.h"
+#include "cloud.h"
+#include "database.h"
 #include "decks.h"
 #include "direct.h"
 #include "effects.h"
 #include "fight.h"
 #include "food.h"
+#include "invent.h"
 #include "items.h"
 #include "it_use2.h"
 #include "itemname.h"
 #include "itemprop.h"
+#include "mapmark.h"
+#include "message.h"
 #include "misc.h"
 #include "monplace.h"
 #include "monstuff.h"
@@ -55,71 +60,111 @@ static bool box_of_beasts(void);
 static bool disc_of_storms(void);
 static bool efreet_flask(void);
 
-void special_wielded(void)
+void special_wielded()
 {
     const int wpn = you.equip[EQ_WEAPON];
     const int old_plus = you.inv[wpn].plus;
     const int old_plus2 = you.inv[wpn].plus2;
     const char old_colour = you.inv[wpn].colour;
-
-    char str_pass[ ITEMNAME_SIZE ];
-    int temp_rand = 0;          // for probability determination {dlb}
-    bool makes_noise = (one_chance_in(20) && !silenced(you.x_pos, you.y_pos));
+    bool makes_noise = false;
 
     switch (you.special_wield)
     {
     case SPWLD_SING:
+    case SPWLD_NOISE:
+    {
+        makes_noise = (one_chance_in(20) && !silenced(you.x_pos, you.y_pos));
+          
         if (makes_noise)
         {
-            strcpy(info, "The Singing Sword ");
-            temp_rand = random2(32);
-            strcat(info,
-                     (temp_rand ==  0) ? "hums a little tune." :
-                     (temp_rand ==  1) ? "breaks into glorious song!" :
-                     (temp_rand ==  2) ? "sings." :
-                     (temp_rand ==  3) ? "sings loudly." :
-                     (temp_rand ==  4) ? "chimes melodiously." :
-                     (temp_rand ==  5) ? "makes a horrible noise." :
-                     (temp_rand ==  6) ? "sings off-key." :
-                     (temp_rand ==  7) ? "sings 'tra-la-la'." :
-                     (temp_rand ==  8) ? "burbles away merrily." :
-                     (temp_rand ==  9) ? "gurgles." :
-                     (temp_rand == 10) ? "suddenly shrieks!" :
-                     (temp_rand == 11) ? "cackles." :
-                     (temp_rand == 12) ? "warbles." :
-                     (temp_rand == 13) ? "chimes harmoniously." :
-                     (temp_rand == 14) ? "makes beautiful music." :
-                     (temp_rand == 15) ? "produces a loud orchestral chord." :
-                     (temp_rand == 16) ? "whines plaintively." :
-                     (temp_rand == 17) ? "tinkles." :
-                     (temp_rand == 18) ? "rings like a bell." :
-                     (temp_rand == 19) ? "wails mournfully." :
-                     (temp_rand == 20) ? "practices its scales." :
-                     (temp_rand == 21) ? "lilts tunefully." :
-                     (temp_rand == 22) ? "hums tunelessly." :
-                     (temp_rand == 23) ? "sighs." :
-                     (temp_rand == 24) ? "makes a deep moaning sound." :
-                     (temp_rand == 25) ? "makes a popping sound." :
-                     (temp_rand == 26) ? "sings a sudden staccato note." :
-                     (temp_rand == 27) ? "says 'Hi! I'm the Singing Sword!'." :
-                     (temp_rand == 28) ? "whispers something." :
-                     (temp_rand == 29) ? "speaks gibberish." :
-                     (temp_rand == 30) ? "raves incoherently."
-                                       : "yells in some weird language.");
-            mpr(info, MSGCH_SOUND);
-        }
+            std::string msg;
+
+            if (you.special_wield == SPWLD_SING)
+            {
+                msg = getSpeakString("Singing Sword");
+                if (msg != "")
+                {
+                    // "Your Singing Sword" sounds disrespectful
+                    // (as if there could be more than one!)
+                    msg = replace_all(msg, "@Your_weapon@", "@The_weapon@");
+                    msg = replace_all(msg, "@your_weapon@", "@the_weapon@");
+                }
+            }
+            else // SPWLD_NOISE
+            {
+                msg = getSpeakString("noisy weapon");
+                if (msg != "")
+                {
+                    msg = replace_all(msg, "@Your_weapon@", "Your @weapon@");
+                    msg = replace_all(msg, "@your_weapon@", "your @weapon@");
+                }
+            }
+            
+            // set appropriate channel (will usually be TALK)
+            msg_channel_type channel = MSGCH_TALK;
+
+            // disallow anything with VISUAL in it;
+            if (msg != "" && msg.find("VISUAL") != std::string::npos)
+                msg = "";
+                
+            if (msg != "")
+            {
+                std::string param = "";
+                std::string::size_type pos = msg.find(":");
+
+                if (pos != std::string::npos)
+                    param = msg.substr(0, pos);
+
+                if (!param.empty())
+                {
+                    bool match = true;
+
+                    if (param == "DANGER")
+                        channel = MSGCH_DANGER;
+                    else if (param == "WARN")
+                        channel = MSGCH_WARN;
+                    else if (param == "SOUND")
+                        channel = MSGCH_SOUND;
+                    else if (param == "PLAIN")
+                        channel = MSGCH_PLAIN;
+                    else if (param == "SPELL" || param == "ENCHANT")
+                        msg = ""; // disallow these as well, channel stays TALK
+                    else if (param != "TALK")
+                        match = false;
+
+                    if (match && msg != "")
+                        msg = msg.substr(pos + 1);
+                }
+            }
+            
+            if (msg == "") // give default noises
+            {
+                if (you.special_wield == SPWLD_SING)
+                    msg = "@The_weapon@ sings.";
+                else
+                {
+                    channel = MSGCH_SOUND;
+                    msg = "You hear a strange noise.";
+                }
+            }
+            
+            // replace weapon references
+            msg = replace_all(msg, "@The_weapon@", "The @weapon@");
+            msg = replace_all(msg, "@the_weapon@", "the @weapon@");
+            msg = replace_all(msg, "@weapon@", you.inv[wpn].name(DESC_BASENAME));
+
+            mpr(msg.c_str(), channel);
+
+        } // makes_noise
         break;
-
+    } 
+    
     case SPWLD_CURSE:
-        makes_noise = false;
-
         if (one_chance_in(30))
-            curse_an_item(0, 0);
+            curse_an_item(false);
         break;
 
     case SPWLD_VARIABLE:
-        makes_noise = false;
-
         do_uncurse_item( you.inv[wpn] );
 
         if (random2(5) < 2)     // 40% chance {dlb}
@@ -142,8 +187,6 @@ void special_wielded(void)
         break;
 
     case SPWLD_TORMENT:
-        makes_noise = false;
-
         if (one_chance_in(200))
         {
             torment( TORMENT_SPWLD, you.x_pos, you.y_pos );
@@ -152,25 +195,19 @@ void special_wielded(void)
         break;
 
     case SPWLD_ZONGULDROK:
-        makes_noise = false;
-
         if (one_chance_in(5))
         {
-            animate_dead( 1 + random2(3), BEH_HOSTILE, MHITYOU, 1 );
+            animate_dead( &you, 1 + random2(3), BEH_HOSTILE, MHITYOU, 1 );
             did_god_conduct( DID_NECROMANCY, 1 );
         }
         break;
 
     case SPWLD_POWER:
-        makes_noise = false;
-
         you.inv[wpn].plus = stepdown_value( -4 + (you.hp / 5), 4, 4, 4, 20 );
         you.inv[wpn].plus2 = you.inv[wpn].plus;
         break;
 
     case SPWLD_OLGREB:
-        makes_noise = false;
-
         // Giving Olgreb's staff a little lift since staves of poison have
         // been made better. -- bwr
         you.inv[wpn].plus = you.skills[SK_POISON_MAGIC] / 3;
@@ -178,15 +215,11 @@ void special_wielded(void)
         break;
 
     case SPWLD_WUCAD_MU:
-        makes_noise = false;
-
         you.inv[wpn].plus  = ((you.intel > 25) ? 22 : you.intel - 3);
         you.inv[wpn].plus2 = ((you.intel > 25) ? 13 : you.intel / 2);
         break;
 
     case SPWLD_SHADOW:
-        makes_noise = false;
-
         if (random2(8) <= player_spec_death())
         {
             did_god_conduct( DID_NECROMANCY, 1 );
@@ -197,36 +230,6 @@ void special_wielded(void)
         show_green = DARKGREY;
         break;
 
-    case SPWLD_HUM:
-        if (makes_noise)
-        {
-            in_name(wpn, DESC_CAP_YOUR, str_pass);
-            strcpy(info, str_pass);
-            strcat(info, " lets out a weird humming sound.");
-            mpr(info, MSGCH_SOUND);
-        }
-        break;                  // to noisy() call at foot 2apr2000 {dlb}
-
-    case SPWLD_CHIME:
-        if (makes_noise)
-        {
-            in_name(wpn, DESC_CAP_YOUR, str_pass);
-            strcpy(info, str_pass);
-            strcat(info, " chimes like a gong.");
-            mpr(info, MSGCH_SOUND);
-        }
-        break;
-
-    case SPWLD_BECKON:
-        if (makes_noise)
-            mpr("You hear a voice call your name.", MSGCH_SOUND);
-        break;
-
-    case SPWLD_SHOUT:
-        if (makes_noise)
-            mpr("You hear a shout.", MSGCH_SOUND);
-        break;
-
     //case SPWLD_PRUNE:
     default:
         return;
@@ -283,11 +286,8 @@ static void reaching_weapon_attack(void)
         // if we're attacking more than a space away
         if ((x_distance > 1) || (y_distance > 1))
         {
-
-#define MAX(x,y) (((x) > (y)) ? (x) : (y))
             x_middle = MAX(beam.tx, you.x_pos) - (x_distance / 2);
             y_middle = MAX(beam.ty, you.y_pos) - (y_distance / 2);
-#undef MAX
 
             // if either the x or the y is the same, we should check for
             // a monster:
@@ -335,11 +335,9 @@ bool evoke_wielded( void )
     int pract = 0;
     bool did_work = false;  // used for default "nothing happens" message
 
-    char str_pass[ ITEMNAME_SIZE ];
-
     int wield = you.equip[EQ_WEAPON];
 
-    if (you.berserker)
+    if (you.duration[DUR_BERSERKER])
     {
         canned_msg( MSG_TOO_BERSERK );
         return (false);
@@ -350,10 +348,17 @@ bool evoke_wielded( void )
         return (false);
     }
 
-    switch (you.inv[wield].base_type)
+    item_def& wpn = you.inv[wield];
+    bool unevokable = false;
+
+    // Check inscriptions.
+    if ( !check_warning_inscriptions(wpn, OPER_EVOKE) )
+        return false;
+
+    switch (wpn.base_type)
     {
     case OBJ_WEAPONS:
-        if (get_weapon_brand( you.inv[wield] ) == SPWPN_REACHING
+        if (get_weapon_brand( wpn ) == SPWPN_REACHING
             && enough_mp(1, false))
         {
             // needed a cost to prevent evocation training abuse -- bwr
@@ -363,12 +368,12 @@ bool evoke_wielded( void )
             pract = (one_chance_in(5) ? 1 : 0);
             did_work = true;
         }
-        else if (is_fixed_artefact( you.inv[wield] ))
+        else if (is_fixed_artefact( wpn ))
         {
-            switch (you.inv[wield].special)
+            switch (wpn.special)
             {
             case SPWPN_STAFF_OF_DISPATER:
-                if (you.deaths_door || !enough_hp(11, true) 
+                if (you.duration[DUR_DEATHS_DOOR] || !enough_hp(11, true) 
                     || !enough_mp(5, true))
                 {
                     break;
@@ -433,7 +438,8 @@ bool evoke_wielded( void )
                     spell_casted = SPELL_HELLFIRE;          //  12 in 240
 
                 power = you.skills[SK_EVOCATIONS] * 8;
-                your_spells( spell_casted, power, false );
+                your_spells( static_cast<spell_type>(spell_casted),
+                             power, false );
                 did_work = true;
                 break;
 
@@ -479,13 +485,18 @@ bool evoke_wielded( void )
                 break;
 
             default:
+                unevokable = true;
                 break;
             }
         }
+        else
+        {
+            unevokable = true;
+        }
         break;
 
     case OBJ_STAVES:
-        if (item_is_rod( you.inv[wield] ))
+        if (item_is_rod( wpn ))
         {
             pract = staff_spell( wield );
             // [ds] Early exit, no turns are lost.
@@ -494,7 +505,7 @@ bool evoke_wielded( void )
 
             did_work = true;  // staff_spell() will handle messages
         }
-        else if (you.inv[wield].sub_type == STAFF_CHANNELING)
+        else if (wpn.sub_type == STAFF_CHANNELING)
         {
             if (you.magic_points < you.max_magic_points 
                 && you.skills[SK_EVOCATIONS] >= random2(30)) 
@@ -505,28 +516,36 @@ bool evoke_wielded( void )
                 pract = (one_chance_in(5) ? 1 : 0);
                 did_work = true;
 
-                if (!item_type_known( you.inv[you.equip[EQ_WEAPON]] ))
+                if (!item_type_known(wpn))
                 {
-                    set_ident_flags( you.inv[you.equip[EQ_WEAPON]], 
-                                     ISFLAG_KNOW_TYPE );
+                    set_ident_flags( wpn, ISFLAG_KNOW_TYPE );
 
-                    strcpy( info, "You are wielding " );
-                    in_name( you.equip[EQ_WEAPON], DESC_NOCAP_A, str_pass );
-                    strcat( info, str_pass );
-                    strcat( info, "." );
-
-                    mpr( info );
+                    mprf("You are wielding %s.",
+                         wpn.name(DESC_NOCAP_A).c_str());
+                         
                     more();
 
                     you.wield_change = true;
                 }
             }
         }
+        else
+        {
+            unevokable = true;
+        }
         break;
 
     case OBJ_MISCELLANY:
         did_work = true; // easier to do it this way for misc items
-        switch (you.inv[wield].sub_type)
+        
+        if ( is_deck(wpn) )
+        {
+            evoke_deck(wpn);
+            pract = 1;
+            break;
+        }
+
+        switch (wpn.sub_type)
         {
         case MISC_BOTTLED_EFREET:
             if (efreet_flask())
@@ -582,41 +601,16 @@ bool evoke_wielded( void )
                         {
                             opened_gates++;
 
-                            // [dshaligram] New approach to placing Hell
-                            // portals to handle the possibility of a mirrored
-                            // Vestibule.
-                            int surround_grid = DNGN_FLOOR;
+                            map_marker *marker =
+                                env.markers.find(coord_def(count_x, count_y),
+                                                 MAT_FEATURE);
 
-                            for (int y = -1; y <= 1; ++y)
-                                for (int x = -1; x <= 1; ++x)
-                                {
-                                    if (!x && !y)
-                                        continue;
-                                    const int grid = 
-                                        grd[count_x + x][count_y + y];
-
-                                    if (grid != DNGN_FLOOR 
-                                            && grid != DNGN_SECRET_DOOR
-                                            && grid != DNGN_CLOSED_DOOR
-                                            && grid != DNGN_OPEN_DOOR)
-                                        surround_grid = grid;
-                                }
-
-                            // this may generate faulty [][] values {dlb}
-                            switch (surround_grid)
+                            if (marker)
                             {
-                            case DNGN_FLOOR:
-                                grd[count_x][count_y] = DNGN_ENTER_DIS;
-                                break;
-                            case DNGN_LAVA:
-                                grd[count_x][count_y] = DNGN_ENTER_GEHENNA;
-                                break;
-                            case DNGN_ROCK_WALL:
-                                grd[count_x][count_y] = DNGN_ENTER_TARTARUS;
-                                break;
-                            case DNGN_DEEP_WATER:
-                                grd[count_x][count_y] = DNGN_ENTER_COCYTUS;
-                                break;
+                                map_feature_marker *featm =
+                                    dynamic_cast<map_feature_marker*>(marker);
+                                grd[count_x][count_y] = featm->feat;
+                                env.markers.remove(marker);
                             }
                         }
                     }
@@ -641,26 +635,6 @@ bool evoke_wielded( void )
             }
             break;
 
-        case MISC_DECK_OF_WONDERS:
-            deck_of_cards(DECK_OF_WONDERS);
-            pract = 1;
-            break;
-
-        case MISC_DECK_OF_SUMMONINGS:
-            deck_of_cards(DECK_OF_SUMMONING);
-            pract = 1;
-            break;
-
-        case MISC_DECK_OF_TRICKS:
-            deck_of_cards(DECK_OF_TRICKS);
-            pract = 1;
-            break;
-
-        case MISC_DECK_OF_POWER:
-            deck_of_cards(DECK_OF_POWER);
-            pract = 1;
-            break;
-
         case MISC_BOX_OF_BEASTS:
             if (box_of_beasts())
                 pract = 1;
@@ -681,33 +655,15 @@ bool evoke_wielded( void )
                 pract = (coinflip() ? 2 : 1);
             break;
 
-        case MISC_PORTABLE_ALTAR_OF_NEMELEX:
-            if (player_in_branch( BRANCH_ECUMENICAL_TEMPLE ))
-            {
-                mpr( "Don't you think this level already has more than "
-                     "enough altars?" );
-            }
-            else if (grd[you.x_pos][you.y_pos] != DNGN_FLOOR)
-                mpr("You need a clear area to place this item.");
-            else
-            {
-                mpr("You unfold the altar and place it on the floor.");
-                you.last_altar_inscription =
-                    you.inv[you.equip[EQ_WEAPON]].inscription;
-                grd[you.x_pos][you.y_pos] = DNGN_ALTAR_NEMELEX_XOBEH;
-                dec_inv_item_quantity( you.equip[EQ_WEAPON], 1 );
-                seen_notable_thing(
-                    DNGN_ALTAR_NEMELEX_XOBEH, you.x_pos, you.y_pos);
-            }
-            break;
-
         default:
             did_work = false;
+            unevokable = true;
             break;
         }
         break;
 
     default:
+        unevokable = true;
         break;
     }
 
@@ -716,19 +672,21 @@ bool evoke_wielded( void )
     else if (pract > 0)
         exercise( SK_EVOCATIONS, pract );
 
-    you.turn_is_over = true;
+    if (!unevokable)
+        you.turn_is_over = true;
 
     return (did_work);
 }                               // end evoke_wielded()
 
 static bool efreet_flask(void)
 {
-    const int behaviour = ((you.skills[SK_EVOCATIONS] > random2(20)) 
-                                ? BEH_FRIENDLY : BEH_HOSTILE);
+    const beh_type behaviour =
+        ((you.skills[SK_EVOCATIONS] / 3 + 10 > random2(20)) 
+         ? BEH_FRIENDLY : BEH_HOSTILE);
 
     mpr("You open the flask...");
 
-    const int efreet = create_monster( MONS_EFREET, 5, behaviour, 
+    const int efreet = create_monster( MONS_EFREET, 0, behaviour, 
                                        you.x_pos, you.y_pos, MHITYOU, 250,
                                        false, false, true );
     if (efreet != -1)
@@ -756,7 +714,8 @@ static bool ball_of_seeing(void)
 
     mpr("You gaze into the crystal ball.");
 
-    use = ((!you.conf) ? random2(you.skills[SK_EVOCATIONS] * 6) : random2(5));
+    use = ((!you.duration[DUR_CONF]) ?
+           random2(you.skills[SK_EVOCATIONS] * 6) : random2(5));
 
     if (use < 2)
     {
@@ -767,22 +726,25 @@ static bool ball_of_seeing(void)
         mpr("You feel power drain from you!");
         set_mp(0, false);
     }
-    else if (use < 10)
+    else if (use < 10 || you.level_type == LEVEL_LABYRINTH)
     {
+        if (you.level_type == LEVEL_LABYRINTH)
+            mpr("You see a maze of twisty little passages, all alike.");
         confuse_player( 10 + random2(10), false );
     }
-    else if (use < 15
-             || you.level_type == LEVEL_LABYRINTH
-             || you.level_type == LEVEL_ABYSS || coinflip())
+    else if (use < 15 || coinflip())
     {
         mpr("You see nothing.");
     }
-    else
+    else if (magic_mapping( 15, 50 + random2( you.skills[SK_EVOCATIONS]), true))
     {
         mpr("You see a map of your surroundings!");
-        magic_mapping( 15, 50 + random2( you.skills[SK_EVOCATIONS] ) );
         ret = true;
     }
+    else
+    {
+        mpr("You see nothing.");
+    }
 
     return (ret);
 }                               // end ball_of_seeing()
@@ -791,7 +753,6 @@ static bool disc_of_storms(void)
 {
     int temp_rand = 0;          // probability determination {dlb}
     int disc_count = 0;
-    unsigned char which_zap = 0;
 
     const int fail_rate = (30 - you.skills[SK_EVOCATIONS]);
     bool ret = false;
@@ -814,9 +775,9 @@ static bool disc_of_storms(void)
 
             temp_rand = random2(3);
 
-            which_zap = ((temp_rand > 1) ? ZAP_LIGHTNING :
-                         (temp_rand > 0) ? ZAP_ELECTRICITY
-                                         : ZAP_ORB_OF_ELECTRICITY);
+            zap_type which_zap = ((temp_rand > 1) ? ZAP_LIGHTNING :
+                                  (temp_rand > 0) ? ZAP_ELECTRICITY
+                                                  : ZAP_ORB_OF_ELECTRICITY);
 
             beam.source_x = you.x_pos;
             beam.source_y = you.y_pos;
@@ -841,16 +802,11 @@ void tome_of_power(char sc_read_2)
     int powc = 5 + you.skills[SK_EVOCATIONS] 
                  + roll_dice( 5, you.skills[SK_EVOCATIONS] ); 
 
-    int spell_casted = 0;
+    spell_type spell_casted = SPELL_NO_SPELL;
     struct bolt beam;
 
-    strcpy(info, "The book opens to a page covered in ");
-
-    char wc[30];
-    weird_writing( wc );
-    strcat( info, wc );
-    strcat( info, "." );
-    mpr( info );
+    msg::stream << "The book opens to a page covered in "
+                << weird_writing() << '.' << std::endl;
 
     you.turn_is_over = true;
 
@@ -881,20 +837,23 @@ void tome_of_power(char sc_read_2)
     case 8:
     case 9:
         mpr("A cloud of weird smoke pours from the book's pages!");
-        big_cloud( CLOUD_GREY_SMOKE + random2(3), KC_YOU,
+        big_cloud( random_smoke_type(), KC_YOU,
                    you.x_pos, you.y_pos, 20, 10 + random2(8) );
+        xom_is_stimulated(16);
         return;
     case 1:
     case 14:
         mpr("A cloud of choking fumes pours from the book's pages!");
         big_cloud(CLOUD_POISON, KC_YOU,
                   you.x_pos, you.y_pos, 20, 7 + random2(5));
+        xom_is_stimulated(64);
         return;
 
     case 2:
     case 13:
         mpr("A cloud of freezing gas pours from the book's pages!");
         big_cloud(CLOUD_COLD, KC_YOU, you.x_pos, you.y_pos, 20, 8 + random2(5));
+        xom_is_stimulated(64);
         return;
 
     case 5:
@@ -923,9 +882,9 @@ void tome_of_power(char sc_read_2)
         beam.is_explosion = true;
 
         explosion(beam);
+        xom_is_stimulated(255);
         return;
 
-
     case 10:
         if (create_monster( MONS_ABOMINATION_SMALL, 6, BEH_HOSTILE,
                             you.x_pos, you.y_pos, MHITYOU, 250 ) != -1)
@@ -933,6 +892,7 @@ void tome_of_power(char sc_read_2)
             mpr("A horrible Thing appears!");
             mpr("It doesn't look too friendly.");
         }
+        xom_is_stimulated(255);
         return;
     }
 
@@ -965,28 +925,21 @@ void tome_of_power(char sc_read_2)
 
 void skill_manual(char sc_read_2)
 {
-    char skname[30];
-
     set_ident_flags( you.inv[sc_read_2], ISFLAG_IDENT_MASK );
 
-    strcpy(skname, skill_name(you.inv[sc_read_2].plus));
+    const int skill = you.inv[sc_read_2].plus;
 
-    strcpy(info, "This is a manual of ");
-    strcat(info, skname);
-    strcat(info, "!");
-    mpr(info);
+    const char* skname = skill_name(skill);
+    mprf("This is a manual of %s!", skname);
 
     you.turn_is_over = true;
 
     if (!yesno("Read it?"))
         return;
 
-    strcpy(info, "You read about ");
-    strcat(info, strlwr(skname));
-    strcat(info, ".");
-    mpr(info);
+    mprf("You read about %s.", skname);
 
-    exercise( you.inv[sc_read_2].plus, 500 );
+    exercise(skill, 500);
 
     if (one_chance_in(10))
     {
@@ -997,9 +950,10 @@ void skill_manual(char sc_read_2)
     {
         mpr("The book looks somewhat more worn.");
     }
+    xom_is_stimulated(14);
 }                               // end skill_manual()
 
-static bool box_of_beasts(void)
+static bool box_of_beasts()
 {
     int beasty = MONS_PROGRAM_BUG;      // error trapping {dlb}
     int temp_rand = 0;          // probability determination {dlb}
@@ -1024,13 +978,14 @@ static bool box_of_beasts(void)
                   (temp_rand == 9) ? MONS_BROWN_SNAKE 
                                    : MONS_GIANT_LIZARD);
 
-        int beh = (one_chance_in(you.skills[SK_EVOCATIONS] + 5) ? BEH_HOSTILE
-                                                                : BEH_FRIENDLY);
+        beh_type beh = (one_chance_in(you.skills[SK_EVOCATIONS] + 5)
+                        ? BEH_HOSTILE : BEH_FRIENDLY);
 
         if (create_monster( beasty, 2 + random2(4), beh,
                             you.x_pos, you.y_pos, MHITYOU, 250 ) != -1)
         {
             mpr("...and something leaps out!");
+            xom_is_stimulated(14);
             ret = true;
         }
     }
@@ -1057,7 +1012,7 @@ static bool ball_of_energy(void)
 
     mpr("You gaze into the crystal ball.");
 
-    use = ((!you.conf) ? random2(you.skills[SK_EVOCATIONS] * 6) : random2(6));
+    use = ((!you.duration[DUR_CONF]) ? random2(you.skills[SK_EVOCATIONS] * 6) : random2(6));
 
     if (use < 2 || you.max_magic_points == 0)
     {
@@ -1101,8 +1056,8 @@ static bool ball_of_fixation(void)
     mpr("You gaze into the crystal ball.");
     mpr("You are mesmerised by a rainbow of scintillating colours!");
 
-    you.paralysis = 100;
-    you.slow = 100;
+    you.duration[DUR_PARALYSIS] = 100;
+    you.duration[DUR_SLOW] = 100;
 
     return (true);
 }                               // end ball_of_fixation()
