@@ -3,7 +3,7 @@
  *  Summary:    Functions for unix and curses support
  *  Written by: ?
  *
- *  Modified for Crawl Reference by $Author: dshaligram $ on $Date: 2007-04-16T17:11:14.444826Z $
+ *  Modified for Crawl Reference by $Author: dshaligram $ on $Date: 2007-11-24T17:24:21.713648Z $
  *
  *  Change History (most recent first):
  *
@@ -43,9 +43,18 @@
 #include "libunix.h"
 #include "defines.h"
 
+#include "cio.h"
 #include "enum.h"
 #include "externs.h"
 #include "files.h"
+#include "state.h"
+#include "stuff.h"
+#include "view.h"
+
+#ifdef UNICODE_GLYPHS
+#include <wchar.h>
+#include <locale.h>
+#endif
 
 #include <termios.h>
 
@@ -58,17 +67,21 @@ static struct termios game_term;
 
 // Its best if curses comes at the end (name conflicts with Solaris). -- bwr
 #ifndef CURSES_INCLUDE_FILE
+    #ifndef _XOPEN_SOURCE_EXTENDED
+    #define _XOPEN_SOURCE_EXTENDED
+    #endif
     #include <curses.h>
 #else
     #include CURSES_INCLUDE_FILE
 #endif
 
 // Character set variable
-int Character_Set = CHARACTER_SET;
+static int Character_Set = CHARACTER_SET;
 
 // Globals holding current text/backg. colors
-short FG_COL = WHITE;
-short BG_COL = BLACK;
+static short FG_COL = WHITE;
+static short BG_COL = BLACK;
+static int   Current_Colour = COLOR_PAIR(BG_COL * 8 + FG_COL);
 
 static int curs_fg_attr(int col);
 static int curs_bg_attr(int col);
@@ -162,6 +175,28 @@ static void setup_colour_pairs( void )
     init_pair(63, COLOR_BLACK, Options.background);
 }          // end setup_colour_pairs()
 
+#ifdef UNICODE_GLYPHS
+static std::string unix_glyph2string(unsigned gly)
+{
+    char buf[50];  // Overkill, I know.
+    wchar_t wcbuf[2];
+    wcbuf[0] = gly;
+    wcbuf[1] = 0;
+    if (wcstombs(buf, wcbuf, sizeof buf) != (size_t) -1)
+        return (buf);
+
+    return std::string(1, gly);
+}
+
+static int unix_multibyte_strlen(const std::string &s)
+{
+    const char *cs = s.c_str();
+    size_t len = mbsrtowcs(NULL, &cs, 0, NULL);
+    return (len == (size_t) -1? s.length() : len);
+}
+#endif
+
+static void unix_handle_terminal_resize();
 
 static void termio_init()
 {
@@ -180,12 +215,101 @@ static void termio_init()
 #endif
 
     tcsetattr(0, TCSAFLUSH, &game_term);
+
+    crawl_state.unicode_ok = false;
+#ifdef UNICODE_GLYPHS
+    if (setlocale(LC_ALL, UNICODE_LOCALE)
+        && !strcmp(nl_langinfo(CODESET), "UTF-8"))
+    {
+        crawl_state.unicode_ok       = true;
+        crawl_state.glyph2strfn      = unix_glyph2string;
+        crawl_state.multibyte_strlen = unix_multibyte_strlen;
+    }
+#endif
+
+    crawl_state.terminal_resize_handler = unix_handle_terminal_resize;
 }
 
+void set_mouse_enabled(bool enabled)
+{
+#ifdef NCURSES_MOUSE_VERSION
+    const int mask = enabled? ALL_MOUSE_EVENTS | REPORT_MOUSE_POSITION : 0;
+    mmask_t oldmask = 0;
+    mousemask(mask, &oldmask);
+#endif
+}
 
-int getch_ck() {
-    int c = getch();
-    switch (c) {
+#ifdef NCURSES_MOUSE_VERSION
+static int proc_mouse_event(int c, const MEVENT *me)
+{
+    crawl_view.mousep.x = me->x + 1;
+    crawl_view.mousep.y = me->y + 1;
+
+    if (!crawl_state.mouse_enabled)
+        return (CK_MOUSE_MOVE);
+
+    c_mouse_event cme(crawl_view.mousep);
+    if (me->bstate & BUTTON1_CLICKED)
+        cme.bstate |= c_mouse_event::BUTTON1;
+    else if (me->bstate & BUTTON1_DOUBLE_CLICKED)
+        cme.bstate |= c_mouse_event::BUTTON1_DBL;
+    else if (me->bstate & BUTTON2_CLICKED)
+        cme.bstate |= c_mouse_event::BUTTON2;
+    else if (me->bstate & BUTTON2_DOUBLE_CLICKED)
+        cme.bstate |= c_mouse_event::BUTTON2_DBL;
+    else if (me->bstate & BUTTON3_CLICKED)
+        cme.bstate |= c_mouse_event::BUTTON3;
+    else if (me->bstate & BUTTON3_DOUBLE_CLICKED)
+        cme.bstate |= c_mouse_event::BUTTON3_DBL;
+    else if (me->bstate & BUTTON4_CLICKED)
+        cme.bstate |= c_mouse_event::BUTTON4;
+    else if (me->bstate & BUTTON4_DOUBLE_CLICKED)
+        cme.bstate |= c_mouse_event::BUTTON4_DBL;
+    
+    if (cme)
+    {
+        new_mouse_event(cme);
+        return (CK_MOUSE_CLICK);
+    }
+
+    return (CK_MOUSE_MOVE);
+}
+#endif
+
+static int raw_m_getch()
+{
+    const int c = getch();
+    switch (c)
+    {
+#ifdef NCURSES_MOUSE_VERSION
+    case KEY_MOUSE:
+    {
+        MEVENT me;
+        getmouse(&me);
+        return (proc_mouse_event(c, &me));
+    }
+#endif
+    default:
+        return (c);
+    }
+}
+
+int m_getch()
+{
+    int c;
+    do
+        c = raw_m_getch();
+    while ((c == CK_MOUSE_MOVE || c == CK_MOUSE_CLICK)
+           && !crawl_state.mouse_enabled);
+
+    return (c);
+}
+
+int getch_ck()
+{
+    int c = m_getch();
+    switch (c)
+    {
     // [dshaligram] MacOS ncurses returns 127 for backspace.
     case 127:
     case KEY_BACKSPACE: return CK_BKSP;
@@ -202,8 +326,15 @@ int getch_ck() {
     }
 }
 
-#if defined(USE_UNIX_SIGNALS) && defined(SIGHUP_SAVE)
+#if defined(USE_UNIX_SIGNALS)
 
+static void handle_sigwinch(int)
+{
+    if (crawl_state.waiting_for_command)
+        handle_terminal_resize();
+}
+
+#ifdef SIGHUP_SAVE
 /* [ds] This SIGHUP handling is primitive and far from safe, but it
  * should be better than nothing. Feel free to get rigorous on this.
  */
@@ -228,26 +359,34 @@ static void handle_hangup(int)
         exit(1);
     }
 }
+#endif // SIGHUP_SAVE
 
-#endif // USE_UNIX_SIGNALS && SIGHUP_SAVE
+#endif // USE_UNIX_SIGNALS
 
 static WINDOW *Message_Window;
 static void setup_message_window()
 {
-    extern int get_message_window_height();
-    
-    Message_Window = newwin( get_message_window_height(), get_number_of_cols(),
-                             VIEW_EY, 0 );
+    Message_Window = newwin( crawl_view.msgsz.y, crawl_view.msgsz.x,
+                             crawl_view.msgp.y - 1, crawl_view.msgp.x - 1 );
     if (!Message_Window)
     {
-        fprintf(stderr, "Unable to create message window!");
-        exit(1);
+        if (crawl_state.need_save)
+            save_game(true);
+
+        // Never reaches here unless the game didn't need saving.
+        end(1, false, "Unable to create message window!");
     }
     
     scrollok(Message_Window, true);
     idlok(Message_Window, true);
 }
 
+static void unix_handle_terminal_resize()
+{
+    unixcurses_shutdown();
+    unixcurses_startup();
+}
+
 void clear_message_window()
 {
     wattrset( Message_Window, curs_fg_attr(LIGHTGREY) );
@@ -258,18 +397,16 @@ void clear_message_window()
 void message_out(int which_line, int color, const char *s, int firstcol,
                  bool newline)
 {
-    extern int get_message_window_height();
-
     wattrset( Message_Window, curs_fg_attr(color) );
 
     if (!firstcol)
         firstcol = Options.delay_message_clear? 1 : 0;
     else
         firstcol--;
-    
+
     mvwaddstr(Message_Window, which_line, firstcol, s);
 
-    if (newline && which_line == get_message_window_height() - 1)
+    if (newline && which_line == crawl_view.msgsz.y - 1)
     {
         int x, y;
         getyx(Message_Window, y, x);
@@ -282,7 +419,7 @@ void message_out(int which_line, int color, const char *s, int firstcol,
     {
         int x, y;
         getyx(Message_Window, y, x);
-        move(y + VIEW_EY, x);
+        move(y + crawl_view.msgp.y - 1, crawl_view.msgp.x - 1 + x);
     }
 
     wrefresh(Message_Window);
@@ -304,6 +441,8 @@ void unixcurses_startup( void )
 #ifdef SIGHUP_SAVE
     signal(SIGHUP, handle_hangup);
 #endif
+
+    signal(SIGWINCH, handle_sigwinch);
     
 #endif
 
@@ -327,11 +466,20 @@ void unixcurses_startup( void )
 
     scrollok(stdscr, TRUE);
 
+    crawl_view.init_geometry();
     setup_message_window();
+
+    set_mouse_enabled(false);
 }
 
 void unixcurses_shutdown()
 {
+    if (Message_Window)
+    {
+        delwin(Message_Window);
+        Message_Window = NULL;
+    }
+        
     // resetty();
     endwin();
 
@@ -345,6 +493,12 @@ void unixcurses_shutdown()
 #ifdef SIGINT
     signal(SIGINT, SIG_DFL);
 #endif
+
+#ifdef SIGHUP_SAVE
+    signal(SIGHUP, SIG_DFL);
+#endif
+
+    signal(SIGWINCH, SIG_DFL);
 #endif
 }
 
@@ -388,7 +542,6 @@ int itoa(int value, char *strptr, int radix)
     return (OK);                /* Me? Fail? Nah. */
 }
 
-
 int cprintf(const char *format,...)
 {
     int i;
@@ -413,6 +566,18 @@ int putch(unsigned char chr)
     return (addch(chr));
 }
 
+int putwch(unsigned chr)
+{
+#ifdef UNICODE_GLYPHS
+    if (chr <= 127 || Options.char_set != CSET_UNICODE)
+        return (putch(chr));
+
+    wchar_t c = chr;
+    return (addnwstr(&c, 1));
+#else
+    return (putch(static_cast<unsigned char>(chr)));
+#endif
+}
 
 char getche()
 {
@@ -431,6 +596,44 @@ int window(int x1, int y1, int x2, int y2)
     return (refresh());
 }
 
+// NOTE: This affects libunix.cc draw state; use this just before setting
+// textcolour and drawing a character and call set_altcharset(false)
+// after you're done drawing.
+//
+int cset_adjust(int raw)
+{
+    if (Options.char_set != CSET_ASCII && Options.char_set != CSET_UNICODE)
+    {
+        // switch to alternate char set for 8-bit characters:
+        set_altcharset( raw > 127 );
+
+        // shift the DEC line drawing set:
+        if (Options.char_set == CSET_DEC 
+            && raw >= 0xE0)
+        {
+            raw &= 0x7F;
+        }
+    }
+    return (raw);
+}
+
+void puttext(int x1, int y1, int x2, int y2, const screen_buffer_t *buf)
+{
+    for (int y = y1; y <= y2; ++y)
+    {
+        gotoxy(x1, y);
+        for (int x = x1; x <= x2; ++x)
+        {
+            const screen_buffer_t ch = cset_adjust( *buf );
+            textattr( buf[1] );
+            putwch( ch );
+            buf += 2;
+        }
+    }
+    set_altcharset(false);
+    update_screen();
+}
+
 // These next four are front functions so that we can reduce 
 // the amount of curses special code that occurs outside this
 // this file.  This is good, since there are some issues with 
@@ -500,12 +703,14 @@ bool is_cursor_enabled()
 
 inline unsigned get_brand(int col)
 {
-    return (col & COLFLAG_FRIENDLY_MONSTER)?    Options.friend_brand :
-           (col & COLFLAG_ITEM_HEAP)?           Options.heap_brand :
-           (col & COLFLAG_WILLSTAB)?            Options.stab_brand :
-           (col & COLFLAG_MAYSTAB)?             Options.may_stab_brand :
-           (col & COLFLAG_REVERSE)?             CHATTR_REVERSE :
-                                                CHATTR_NORMAL;    
+    return (col & COLFLAG_FRIENDLY_MONSTER)? Options.friend_brand :
+        (col & COLFLAG_ITEM_HEAP)?           Options.heap_brand :
+        (col & COLFLAG_WILLSTAB)?            Options.stab_brand :
+        (col & COLFLAG_MAYSTAB)?             Options.may_stab_brand :
+        (col & COLFLAG_STAIR_ITEM)?          Options.stair_item_brand :
+        (col & COLFLAG_TRAP_ITEM)?           Options.trap_item_brand :
+        (col & COLFLAG_REVERSE)?             CHATTR_REVERSE :
+                                             CHATTR_NORMAL;
 }
 
 void textattr(int col)
@@ -572,7 +777,7 @@ static int curs_fg_attr(int col)
 
 void textcolor(int col)
 {
-    attrset( curs_fg_attr(col) );
+    attrset( Current_Colour = curs_fg_attr(col) );
 }
 
 static int curs_bg_attr(int col)
@@ -631,7 +836,7 @@ static int curs_bg_attr(int col)
 
 void textbackground(int col)
 {
-    attrset( curs_bg_attr(col) );
+    attrset( Current_Colour = curs_bg_attr(col) );
 }
 
 
@@ -640,29 +845,32 @@ int gotoxy(int x, int y)
     return (move(y - 1, x - 1));
 }
 
-static unsigned oldch, oldmangledch;
-static int faked_x = -1, faked_y;
-
-void fakecursorxy(int x, int y)
+#ifdef UNICODE_GLYPHS
+typedef cchar_t char_info;
+inline bool operator == (const cchar_t &a, const cchar_t &b)
 {
-    if (oldch && faked_x != -1
-        && mvinch(faked_y, faked_x) == oldmangledch)
-    {
-        if (faked_x != x - 1 || faked_y != y - 1)
-            mvaddch(faked_y, faked_x, oldch);
-        else
-            return;
-    }
-    
-    const unsigned c      = mvinch(y - 1, x - 1);
-    const int ch          = c & A_CHARTEXT;
-    const unsigned colour = c & A_COLOR;
+    return (a.attr == b.attr && *a.chars == *b.chars);
+}
+inline char_info character_at(int y, int x)
+{
+    cchar_t c;
+    mvin_wch(y, x, &c);
+    return (c);
+}
+inline bool valid_char(const cchar_t &c)
+{
+    return *c.chars;
+}
+inline void write_char_at(int y, int x, const cchar_t &ch)
+{
+    move(y, x);
+    add_wchnstr(&ch, 1);
+}
+static void flip_colour(cchar_t &ch)
+{
+    const unsigned colour = (ch.attr & A_COLOR);
     const int pair        = PAIR_NUMBER(colour);
 
-    faked_x = x - 1;
-    faked_y = y - 1;
-    oldch   = c;
-
     int fg     = pair & 7;
     int bg     = (pair >> 3) & 7;
 
@@ -673,9 +881,56 @@ void fakecursorxy(int x, int y)
     }
 
     const int newpair = (fg * 8 + bg);
-    
-    mvaddch( y - 1, x - 1,  oldmangledch = ((ch & 127) | COLOR_PAIR(newpair)) );
+    ch.attr = COLOR_PAIR(newpair);
+}
+#else // ! UNICODE_GLYPHS
+typedef unsigned char_info;
+#define character_at(y,x)    mvinch(y,x)
+#define valid_char(x) (x)
+#define write_char_at(y,x,c) mvaddch(y, x, c)
 
+#define char_info_character(c) ((c) & A_CHARTEXT)
+#define char_info_colour(c)    ((c) & A_COLOR)
+
+static void flip_colour(unsigned &ch)
+{
+    const unsigned colour = char_info_colour(ch);
+    const int pair        = PAIR_NUMBER(colour);
+
+    int fg     = pair & 7;
+    int bg     = (pair >> 3) & 7;
+
+    if (pair == 63)
+    {
+        fg    = COLOR_WHITE;
+        bg    = COLOR_BLACK;
+    }
+
+    const int newpair = (fg * 8 + bg);
+    ch = ((ch & 127) | COLOR_PAIR(newpair));
+}
+#endif
+
+static char_info oldch, oldmangledch;
+static int faked_x = -1, faked_y;
+
+void fakecursorxy(int x, int y)
+{
+    if (valid_char(oldch) && faked_x != -1
+        && character_at(faked_y, faked_x) == oldmangledch)
+    {
+        if (faked_x != x - 1 || faked_y != y - 1)
+            write_char_at(faked_y, faked_x, oldch);
+        else
+            return;
+    }
+    
+    char_info c = character_at(y - 1, x - 1);
+    oldch   = c;
+    faked_x = x - 1;
+    faked_y = y - 1;
+    flip_colour(c);
+    write_char_at( y - 1, x - 1, oldmangledch = c);
     move(y - 1, x - 1);
 }
 
