@@ -1,15 +1,15 @@
 /*
-*  Modified for Crawl Reference by $Author: dshaligram $ on $Date: 2007-03-07T06:02:32.835081Z $
+*  Modified for Crawl Reference by $Author: dshaligram $ on $Date: 2007-10-27T16:44:37.046050Z $
 */
 
 #ifndef LIBW32C_H
 #define LIBW32C_H
 
-#define WIN_NUMBER_OF_LINES     25
-
 #include <string>
 #include <stdarg.h>
 
+typedef unsigned short screen_buffer_t;
+
 void init_libw32c(void);
 void deinit_libw32c(void);
 
@@ -30,15 +30,16 @@ void textcolor(int c);
 void textattr(int c);
 void cprintf( const char *format, ... );
 // void cprintf(const char *s);
-void setStringInput(bool value);
-bool setBuffering(bool value);
-int getConsoleString(char *buf, int maxlen);
+void set_string_input(bool value);
+bool set_buffering(bool value);
+int get_console_string(char *buf, int maxlen);
 void print_timings(void);
 
 void window(int x, int y, int lx, int ly);
 int wherex(void);
 int wherey(void);
 void putch(char c);
+void putwch(unsigned c);
 int getch(void);
 int getch_ck(void);
 int key_to_command(int);
@@ -46,12 +47,11 @@ int getche(void);
 int kbhit(void);
 void delay(int ms);
 void textbackground(int c);
+void puttext(int x1, int y1, int x2, int y2, const screen_buffer_t *);
+void update_screen();
 
 void enable_smart_cursor(bool cursor);
 bool is_smart_cursor_enabled();
-
-inline void srandom(unsigned int seed) { srand(seed); }
+void set_mouse_enabled(bool enabled);
 
 #endif
-
-
