@@ -3,7 +3,7 @@
  *  Summary:    Crude macro-capability
  *  Written by: Juho Snellman <jsnell@lyseo.edu.ouka.fi>
  *
- *  Modified for Crawl Reference by $Author: dshaligram $ on $Date: 2007-01-25T10:43:02.575301Z $
+ *  Modified for Crawl Reference by $Author: ennewalker $ on $Date: 2007-11-23T02:25:44.246247Z $
  *
  *  Change History (most recent first):
  *
@@ -25,6 +25,7 @@ enum KeymapContext {
     KC_DEFAULT,         // For no-arg getchm(), must be zero.
     KC_LEVELMAP,        // When in the 'X' level map
     KC_TARGETING,       // Only during 'x' and other targeting modes
+    KC_CONFIRM,         // When being asked y/n/q questions
 
     KC_CONTEXT_COUNT    // Must always be the last
 };
@@ -45,6 +46,7 @@ void macro_userfn(const char *keys, const char *registryname);
 void macro_buf_add(int key);
 
 bool is_userfunction(int key);
+bool is_synthetic_key(int key);
 
 const char *get_userfunction(int key);
 
