@@ -3,12 +3,16 @@
  *  Summary:    Menus and associated malarkey.
  *  Written by: Darshan Shaligram
  *
- *  Modified for Crawl Reference by $Author: dshaligram $ on $Date: 2007-03-20T16:54:00.148466Z $
+ *  Modified for Crawl Reference by $Author: ennewalker $ on $Date: 2007-11-23T02:25:44.246247Z $
  */
 #include <cctype>
 #include "AppHdr.h"
+#include "cio.h"
 #include "menu.h"
 #include "macro.h"
+#include "message.h"
+#include "player.h"
+#include "tutorial.h"
 #include "view.h"
 #include "initfile.h"
 
@@ -29,7 +33,8 @@ Menu::Menu( int _flags )
     highlighter(new MenuHighlighter),
     num(-1),
     lastch(0),
-    alive(false)
+    alive(false),
+    last_selected(-1)
 {
     set_flags(flags);
 }
@@ -110,7 +115,7 @@ std::vector<MenuEntry *> Menu::show(bool reuse_selections)
         pagesize = max_pagesize;
 
     do_menu();
-    
+
     return (sel);
 }
 
@@ -205,6 +210,56 @@ bool Menu::process_key( int keyin )
         }
         break;
     }
+    case CONTROL('F'):
+    {
+        if ( !( flags & MF_ALLOW_FILTER ) )
+            break;
+        char linebuf[80];
+        gotoxy(1,1);
+        clear_to_end_of_line();
+        textcolor(WHITE);
+        cprintf("Select what? (regex) ");
+        textcolor(DARKGREY);
+        bool validline = !cancelable_get_line(linebuf, sizeof linebuf, 80);
+        if ( validline && linebuf[0] )
+        {
+            text_pattern tpat(linebuf);
+            for ( unsigned int i = 0; i < items.size(); ++i )
+            {
+                if ( items[i]->level == MEL_ITEM &&
+                     tpat.matches(items[i]->get_text()) )
+                {
+                    select_index(i);
+                    if ( flags & MF_SINGLESELECT )
+                    {
+                        // Return the first item found.
+                        get_selected(&sel);
+                        return false;
+                    }
+                }
+            }
+            get_selected(&sel);
+        }
+        nav = true;
+        repaint = true;
+        break;
+    }
+    case '.':
+        if (last_selected != -1)
+        {
+            if ((first_entry + pagesize - last_selected) == 1)
+            {
+                page_down();
+                nav = true;
+            }
+
+            select_index(last_selected + 1);
+            get_selected(&sel);
+
+            repaint = true;
+        }
+        break;
+
     default:
         keyin = post_process(keyin);
         lastch = keyin;
@@ -236,6 +291,14 @@ bool Menu::process_key( int keyin )
         break;
     }
 
+    if (last_selected != -1 &&
+        (items.size() == ((unsigned int) last_selected + 1)
+         || items[last_selected + 1] == NULL
+         || items[last_selected + 1]->level != MEL_ITEM))
+    {
+        last_selected = -1;
+    }
+
     if (!isdigit( keyin ))
         num = -1;
 
@@ -331,7 +394,7 @@ void Menu::deselect_all(bool update_view)
 bool Menu::is_hotkey(int i, int key)
 {
     int end = first_entry + pagesize;
-    if (end > (int) items.size()) end = items.size();
+    if (end > static_cast<int>(items.size())) end = items.size();
 
     bool ishotkey = is_set(MF_SINGLESELECT)?
                         items[i]->is_primary_hotkey(key)
@@ -426,6 +489,7 @@ void Menu::select_index( int index, int qty )
                     continue;
                 if (is_hotkey(i, items[i]->hotkeys[0]) && is_selectable(i))
                 {
+                    last_selected = i;
                     items[i]->select( qty );
                     draw_item( i );
                 }
@@ -441,6 +505,7 @@ void Menu::select_index( int index, int qty )
                 continue;
             if (is_hotkey(i, items[i]->hotkeys[0]))
             {
+                last_selected = i;
                 items[i]->select( qty );
                 draw_item( i );
             }
@@ -449,6 +514,7 @@ void Menu::select_index( int index, int qty )
     else if (items[si]->level == MEL_ITEM && 
             (flags & (MF_SINGLESELECT | MF_MULTISELECT))) 
     {
+        last_selected = si;
         items[si]->select( qty );
         draw_item( si );
     }
@@ -471,7 +537,7 @@ void Menu::draw_menu()
     }
     if (end < (int) items.size() || is_set(MF_ALWAYS_SHOW_MORE))
     {
-        gotoxy( 1, y_offset + pagesize );
+        gotoxy( 1, y_offset + pagesize - count_linebreaks(more) );
         more.display();
     }
 }
@@ -483,11 +549,42 @@ void Menu::update_title()
     gotoxy(x, y);
 }
 
+// to highlight species in aptitudes list ('?%')
+static std::string get_species_key()
+{
+    if (player_genus(GENPC_DRACONIAN) && you.experience_level < 7)
+        return "";
+
+    std::string result = "";
+    switch (you.species)
+    {
+        case SP_RED_DRACONIAN:     result = "Red"; break;
+        case SP_WHITE_DRACONIAN:   result = "White"; break;
+        case SP_GREEN_DRACONIAN:   result = "Green"; break;
+        case SP_GOLDEN_DRACONIAN:  result = "Yellow"; break;
+        case SP_GREY_DRACONIAN:    result = "Grey"; break;
+        case SP_BLACK_DRACONIAN:   result = "Black"; break;
+        case SP_PURPLE_DRACONIAN:  result = "Purple"; break;
+        case SP_MOTTLED_DRACONIAN: result = "Mottled"; break;
+        case SP_PALE_DRACONIAN:    result = "Pale"; break;
+        default:
+             result = species_name(you.species, 1);
+    }
+    result += "  ";
+    return (result);
+}
+
 int Menu::item_colour(int, const MenuEntry *entry) const
 {
     int icol = -1;
     if (highlighter)
         icol = highlighter->entry_colour(entry);
+    else
+    {
+        text_pattern tp(get_species_key());
+        if (!tp.empty() && tp.matches(entry->text))
+            icol = WHITE;
+    }
 
     return (icol == -1? entry->colour : icol);
 }
@@ -536,7 +633,15 @@ void Menu::draw_index_item(int index, const MenuEntry *me) const
 void Menu::draw_stock_item(int index, const MenuEntry *me) const
 {
     textattr( item_colour(index, items[index]) );
-    cprintf( "%s", items[index]->get_text().c_str() );
+    if ( flags & MF_ALLOW_FORMATTING )
+        formatted_string::parse_string(items[index]->get_text()).display();
+    else
+    {
+        std::string text = items[index]->get_text();
+        if ((int) text.length() > get_number_of_cols())
+            text = text.substr(0, get_number_of_cols());
+        cprintf( "%s", text.c_str() );
+    }
 }
 
 bool Menu::page_down()
@@ -618,12 +723,12 @@ void slider_menu::set_limits(int y1, int y2)
 void slider_menu::select_search(const std::string &s)
 {
     std::string srch = s;
-    tolower_string(srch);
+    lowercase(srch);
 
     for (int i = 0, size = items.size(); i < size; ++i)
     {
         std::string text = items[i]->get_text();
-        tolower_string(text);
+        lowercase(text);
 
         std::string::size_type found = text.find(srch);
         if (found != std::string::npos 
@@ -660,9 +765,9 @@ bool slider_menu::process_key(int key)
         return (false);
     }
 
-    if (selected == 0 && 
-            (key == CK_UP || key == CK_PGUP || key == '<' || key == ';') &&
-            Menu::is_set(MF_EASY_EXIT))
+    if (Menu::is_set(MF_NOWRAP) &&
+        selected == 0 &&
+        (key == CK_UP || key == CK_PGUP || key == '<' || key == ';'))
     {
         oldselect = selected;
         selected = -1;
@@ -721,14 +826,14 @@ std::vector<MenuEntry *> slider_menu::show()
     fix_entry();
     do_menu();
 
-    if (selected >= 0 && selected <= (int) items.size())
+    if (selected >= 0 && selected < (int) items.size())
         sel.push_back(items[selected]);
     return (sel);
 }
 
 const MenuEntry *slider_menu::selected_entry() const
 {
-    if (selected >= 0 && selected <= (int) items.size())
+    if (selected >= 0 && selected < (int) items.size())
         return (items[selected]);
 
     return (NULL);
@@ -862,9 +967,23 @@ bool slider_menu::fix_entry(int recurse_depth)
 void slider_menu::new_selection(int nsel)
 {
     if (nsel < 0)
-        nsel = 0;
-    if (nsel >= (int) items.size())
-        nsel = items.size() - 1;
+    {
+        if ( !is_set(MF_NOWRAP) )
+            do {
+                nsel += items.size();
+            } while ( nsel < 0 );
+        else
+            nsel = 0;
+    }
+    if (nsel >= static_cast<int>(items.size()))
+    {
+        if ( !is_set(MF_NOWRAP) )
+            do {
+                nsel -= items.size();
+            } while ( nsel >= static_cast<int>(items.size()) );
+        else
+            nsel = items.size() - 1;
+    }
 
     const int old = selected;
     selected = nsel;
@@ -909,12 +1028,14 @@ bool slider_menu::line_up()
 // Menu colouring
 //
 
-int menu_colour(const std::string &text)
+int menu_colour(const std::string &text, const std::string &prefix)
 {
+    std::string tmp_text = prefix + text;
+
     for (int i = 0, size = Options.menu_colour_mappings.size(); i < size; ++i)
     {
         colour_mapping &cm = Options.menu_colour_mappings[i];
-        if (cm.pattern.matches(text))
+        if (cm.pattern.matches(tmp_text))
             return (cm.colour);
     }
     return (-1);
@@ -922,7 +1043,7 @@ int menu_colour(const std::string &text)
 
 int MenuHighlighter::entry_colour(const MenuEntry *entry) const
 {
-    return (::menu_colour(entry->get_text()));
+    return entry->highlight_colour();
 }
 
 ///////////////////////////////////////////////////////////////////////
@@ -1206,9 +1327,17 @@ int linebreak_string2( std::string& s, int maxcol )
     return breakcount;
 }
 
+std::string get_linebreak_string(const std::string& s, int maxcol)
+{
+    std::string r = s;
+    linebreak_string2(r, maxcol );
+    return r;
+}
+
 // takes a (possibly tagged) string, breaks it into lines and 
 // prints it into the given message channel
-void print_formatted_paragraph(std::string &s, int maxcol, int channel) 
+void print_formatted_paragraph(std::string &s, int maxcol,
+                               msg_channel_type channel) 
 {
     linebreak_string2(s,maxcol);
     std::string text;
@@ -1318,6 +1447,8 @@ bool formatted_scroller::process_key( int keyin )
         keyin = (*f_keyfilter)(keyin);
 
     bool repaint = false;
+    // Any key is assumed to be a movement key for now...
+    bool moved = true;
     switch ( keyin )
     {
     case 0:
@@ -1355,5 +1486,37 @@ bool formatted_scroller::process_key( int keyin )
 
     if (repaint)
         draw_menu();
+    else if (moved && is_set(MF_EASY_EXIT))
+        return (false);
+            
     return true;
 }
+
+int ToggleableMenu::pre_process(int key)
+{
+    if ( std::find(toggle_keys.begin(), toggle_keys.end(), key) !=
+         toggle_keys.end() )
+    {
+        // Toggle all menu entries
+        for ( unsigned int i = 0; i < items.size(); ++i )
+        {
+            ToggleableMenuEntry* const p =
+                dynamic_cast<ToggleableMenuEntry*>(items[i]);
+            if ( p )
+                p->toggle();
+        }
+
+        // Toggle title
+        ToggleableMenuEntry* const pt =
+            dynamic_cast<ToggleableMenuEntry*>(title);
+        if ( pt )
+            pt->toggle();
+
+        // Redraw
+        draw_menu();
+
+        // Don't further process the key
+        return 0;
+    }
+    return key;
+}
