@@ -3,7 +3,7 @@
  *  Summary:    Functions used to help determine which monsters should appear.
  *  Written by: Linley Henzell
  *
- *  Modified for Crawl Reference by $Author: dshaligram $ on $Date: 2007-03-02T21:39:37.218440Z $
+ *  Modified for Crawl Reference by $Author: ennewalker $ on $Date: 2007-11-23T02:25:44.246247Z $
  *
  *  Change History (most recent first):
  *
@@ -17,32 +17,33 @@
 #include "externs.h"
 #include "branch.h"
 #include "misc.h"
+#include "place.h"
 
 // NB - When adding new branches or levels above 50, you must
 // change pre-game deletion routine in new_game in newgame.cc
-int mons_level(int mcls)
+int mons_level(int mcls, const level_id &place)
 {
     int monster_level = 0;
 
-    if (you.level_type == LEVEL_ABYSS)
+    if (place.level_type == LEVEL_ABYSS)
         monster_level = ((mons_abyss(mcls)) ? 51 : 0);
-    else if (you.level_type == LEVEL_PANDEMONIUM)
+    else if (place.level_type == LEVEL_PANDEMONIUM)
         monster_level = ((mons_pan(mcls)) ? 52 : 0);
-    else
-        monster_level=branches[(int)you.where_are_you].mons_level_function(mcls);
+    else if (place.level_type == LEVEL_DUNGEON)
+        monster_level = branches[place.branch].mons_level_function(mcls);
 
     return monster_level;
 }
 
 // higher values returned means the monster is "more common"
 // a return value of zero means the monster will never appear {dlb}
-int mons_rarity(int mcls)
+int mons_rarity(int mcls, const level_id &place)
 {
     // now, what about pandemonium ??? {dlb}
-    if (you.level_type == LEVEL_ABYSS)
+    if (place.level_type == LEVEL_ABYSS)
         return mons_rare_abyss(mcls);
     else
-        return branches[(int)you.where_are_you].mons_rarity_function(mcls);
+        return branches[place.branch].mons_rarity_function(mcls);
 }
 
 bool mons_abyss(int mcls)
@@ -371,8 +372,6 @@ bool mons_pan(int mcls)
     // malign beings
     case MONS_EFREET:
     case MONS_RAKSHASA:
-    //case MONS_RAKSHASA_FAKE:    //jmf: FIXME: really create these?
-                                  // I'm guessing not -- bwr
     // golems
     case MONS_CLAY_GOLEM:
     case MONS_CRYSTAL_GOLEM:
@@ -1733,6 +1732,11 @@ int mons_hallelf_level(int mcls)
         mlev += 7;
         break;
 
+    case MONS_DEEP_ELF_BLADEMASTER:
+    case MONS_DEEP_ELF_MASTER_ARCHER:
+        mlev += 10;
+        break;
+
     default:
         mlev += 99;
         break;
@@ -1793,6 +1797,10 @@ int mons_hallelf_rare(int mcls)
     case MONS_ORC_HIGH_PRIEST:
         return 5;
 
+    case MONS_DEEP_ELF_BLADEMASTER:
+    case MONS_DEEP_ELF_MASTER_ARCHER:
+        return 1;
+
     default:
         return 0;
     }
@@ -1874,6 +1882,82 @@ int mons_tomb_rare(int mcls)
     }
 }                               // end mons_tomb_rare()
 
+int mons_shoals_level(int mcls)
+{
+    int mlev = absdungeon_depth(BRANCH_SHOALS, 1);
+    switch (mcls)
+    {
+    case MONS_BUTTERFLY:
+    case MONS_PLANT:
+    case MONS_GIANT_BAT:
+        break;
+
+    case MONS_CENTAUR:
+    case MONS_ETTIN:
+    case MONS_SHEEP:
+    case MONS_HIPPOGRIFF:
+        mlev++;
+        break;
+
+    case MONS_YAKTAUR:
+        mlev += 2;
+        break;
+
+    case MONS_CENTAUR_WARRIOR:
+    case MONS_CYCLOPS:          // will have a sheep band
+        mlev += 3;
+        break;
+
+    case MONS_STONE_GIANT:
+    case MONS_OKLOB_PLANT:
+        mlev += 4;
+        break;
+
+    case MONS_YAKTAUR_CAPTAIN:
+        mlev += 5;
+        break;
+
+    default:
+        mlev += 99;
+    }
+    return mlev;
+}
+
+int mons_shoals_rare(int mcls)
+{
+    switch (mcls)
+    {
+    case MONS_PLANT:
+        return 150;
+
+    case MONS_ETTIN:
+    case MONS_SHEEP:
+        return 50;
+
+    case MONS_HIPPOGRIFF:
+    case MONS_GIANT_BAT:
+    case MONS_BUTTERFLY:
+    case MONS_CENTAUR:
+        return 35;
+
+    case MONS_YAKTAUR:
+        return 25;
+
+    case MONS_CYCLOPS:
+    case MONS_CENTAUR_WARRIOR:
+        return 20;
+
+    case MONS_STONE_GIANT:
+    case MONS_YAKTAUR_CAPTAIN:
+        return 10;
+        
+    case MONS_OKLOB_PLANT:
+        return 5;
+    default:
+        return 0;
+    }
+}
+
 int mons_swamp_level(int mcls)
 {
     int mlev = absdungeon_depth(BRANCH_SWAMP, 1);
