@@ -3,7 +3,7 @@
  *  Summary:    Functions used when Bad Things happen to the player.
  *  Written by: Linley Henzell
  *
- *  Modified for Crawl Reference by $Author: dshaligram $ on $Date: 2007-04-06T10:25:27.345335Z $
+ *  Modified for Crawl Reference by $Author: ennewalker $ on $Date: 2007-11-23T02:25:44.246247Z $
  *
  *  Change History (most recent first):
  *
@@ -28,6 +28,7 @@
 
 #include <string.h>
 #include <string>
+#include <iostream>
 #include <stdio.h>
 #include <stdlib.h>
 #include <ctype.h>
@@ -61,7 +62,10 @@
 #include "itemprop.h"
 #include "items.h"
 #include "macro.h"
+#include "message.h"
+#include "misc.h"
 #include "mon-util.h"
+#include "monstuff.h"
 #include "notes.h"
 #include "player.h"
 #include "randart.h"
@@ -69,13 +73,14 @@
 #include "shopping.h"
 #include "skills2.h"
 #include "spells4.h"
+#include "state.h"
 #include "stuff.h"
 #include "tutorial.h"
 #include "view.h"
 
 
-void end_game( struct scorefile_entry &se );
-void item_corrode( char itco );
+static void end_game( scorefile_entry &se );
+static void item_corrode( int itco );
 
 
 /* NOTE: DOES NOT check for hellfire!!! */
@@ -84,8 +89,7 @@ int check_your_resists(int hurted, int flavour)
     int resist;
 
 #if DEBUG_DIAGNOSTICS
-    snprintf( info, INFO_SIZE, "checking resistance: flavour=%d", flavour );
-    mpr( info, MSGCH_DIAGNOSTICS );
+    mprf(MSGCH_DIAGNOSTICS, "checking resistance: flavour=%d", flavour );
 #endif
 
     if (flavour == BEAM_FIRE || flavour == BEAM_LAVA 
@@ -126,6 +130,7 @@ int check_your_resists(int hurted, int flavour)
         else if (resist < 0)
         {
             mpr("It burns terribly!");
+            xom_is_stimulated(200);
             hurted *= 15;
             hurted /= 10;
         }
@@ -141,6 +146,7 @@ int check_your_resists(int hurted, int flavour)
         else if (resist < 0)
         {
             mpr("You feel a terrible chill!");
+            xom_is_stimulated(200);
             hurted *= 15;
             hurted /= 10;
         }
@@ -213,6 +219,7 @@ int check_your_resists(int hurted, int flavour)
         else if (resist < 0)
         {
             mpr("You feel a painful chill!");
+            xom_is_stimulated(200);
             hurted *= 13;
             hurted /= 10;
         }
@@ -229,6 +236,7 @@ int check_your_resists(int hurted, int flavour)
         else if (resist < 0)
         {
             mpr("It burns terribly!");
+            xom_is_stimulated(200);
             hurted *= 15;
             hurted /= 10;
         }
@@ -308,9 +316,7 @@ void weapon_acid( char acid_strength )
 
     if (hand_thing == -1)
     {
-        snprintf( info, INFO_SIZE, "Your %s burn!", your_hand(true) );
-        mpr( info );
-
+        msg::stream << "Your " << your_hand(true) << " burn!" << std::endl;
         ouch( roll_dice( 1, acid_strength ), 0, KILLED_BY_ACID );  
     }
     else if (random2(20) <= acid_strength)
@@ -319,7 +325,7 @@ void weapon_acid( char acid_strength )
     }
 }                               // end weapon_acid()
 
-void item_corrode( char itco )
+void item_corrode( int itco )
 {
     int chance_corr = 0;        // no idea what its full range is {dlb}
     bool it_resists = false;    // code simplifier {dlb}
@@ -340,17 +346,18 @@ void item_corrode( char itco )
     if (how_rusty < -5)
         return;
 
+    item_def& item = you.inv[itco];
     // determine possibility of resistance by object type {dlb}:
-    switch (you.inv[itco].base_type)
+    switch (item.base_type)
     {
     case OBJ_ARMOUR:
-        if (is_random_artefact( you.inv[itco] ))
+        if (is_random_artefact( item ))
         {
             it_resists = true;
             suppress_msg = true;
         }
-        else if ((you.inv[itco].sub_type == ARM_CRYSTAL_PLATE_MAIL
-                    || get_equip_race(you.inv[itco]) == ISFLAG_DWARVEN)
+        else if ((item.sub_type == ARM_CRYSTAL_PLATE_MAIL
+                    || get_equip_race(item) == ISFLAG_DWARVEN)
                 && !one_chance_in(5))
         {
             it_resists = true;
@@ -359,13 +366,13 @@ void item_corrode( char itco )
         break;
 
     case OBJ_WEAPONS:
-        if (is_fixed_artefact(you.inv[itco]) 
-            || is_random_artefact(you.inv[itco]))
+        if (is_fixed_artefact(item) 
+            || is_random_artefact(item))
         {
             it_resists = true;
             suppress_msg = true;
         }
-        else if (get_equip_race(you.inv[itco]) == ISFLAG_DWARVEN
+        else if (get_equip_race(item) == ISFLAG_DWARVEN
                 && !one_chance_in(5))
         {
             it_resists = true;
@@ -374,7 +381,7 @@ void item_corrode( char itco )
         break;
 
     case OBJ_MISSILES:
-        if (get_equip_race(you.inv[itco]) == ISFLAG_DWARVEN 
+        if (get_equip_race(item) == ISFLAG_DWARVEN 
                 && !one_chance_in(5))
         {
             it_resists = true;
@@ -402,7 +409,7 @@ void item_corrode( char itco )
         if (chance_corr >= 0 && chance_corr <= 4)
         {
             it_resists = (random2(100) <
-                            2 + (2 << (1 + chance_corr)) + (chance_corr * 8));
+                            2 + (4 << chance_corr) + (chance_corr * 8));
         }
         else
             it_resists = true;  // no idea how often this occurs {dlb}
@@ -414,21 +421,19 @@ void item_corrode( char itco )
     // handle message output and item damage {dlb}:
     if (!suppress_msg)
     {
-        char str_pass[ ITEMNAME_SIZE ];
-        in_name(itco, DESC_CAP_YOUR, str_pass);
-        strcpy(info, str_pass);
-        strcat(info, (it_resists) ? " resists." : " is eaten away!");
-        mpr(info);
+        mprf("%s %s", item.name(DESC_CAP_YOUR).c_str(),
+             (it_resists) ? "resists." : "is eaten away!");
     }
 
     if (!it_resists)
     {
         how_rusty--;
+        xom_is_stimulated(64);
 
-        if (you.inv[itco].base_type == OBJ_WEAPONS)
-            you.inv[itco].plus2 = how_rusty;
+        if (item.base_type == OBJ_WEAPONS)
+            item.plus2 = how_rusty;
         else
-            you.inv[itco].plus = how_rusty;
+            item.plus = how_rusty;
 
         you.redraw_armour_class = 1;     // for armour, rings, etc. {dlb}
 
@@ -520,30 +525,30 @@ static void expose_invent_to_element( beam_type flavour, int strength )
         switch (target_class)
         {
         case OBJ_SCROLLS:
-            snprintf( info, INFO_SIZE, "%s you are carrying %s fire!",
-                      (num_dest > 1) ? "Some of the scrolls" : "A scroll",
-                      (num_dest > 1) ? "catch" : "catches" );
+            mprf("%s you are carrying %s fire!",
+                 (num_dest > 1) ? "Some of the scrolls" : "A scroll",
+                 (num_dest > 1) ? "catch" : "catches" );
             break;
 
         case OBJ_POTIONS:
-            snprintf( info, INFO_SIZE, "%s you are carrying %s and %s!",
-                      (num_dest > 1) ? "Some of the potions" : "A potion",
-                      (num_dest > 1) ? "freeze" : "freezes",
-                      (num_dest > 1) ? "shatter" : "shatters" );
+            mprf("%s you are carrying %s and %s!",
+                 (num_dest > 1) ? "Some of the potions" : "A potion",
+                 (num_dest > 1) ? "freeze" : "freezes",
+                 (num_dest > 1) ? "shatter" : "shatters" );
             break;
 
         case OBJ_FOOD:
-            snprintf( info, INFO_SIZE, "Some of your food is covered with spores!" );
+            mprf("Some of your food is covered with spores!");
             break;
 
         default:
-            snprintf( info, INFO_SIZE, "%s you are carrying %s destroyed!",
-                      (num_dest > 1) ? "Some items" : "An item",
-                      (num_dest > 1) ? "were" : "was" );
+            mprf("%s you are carrying %s destroyed!",
+                 (num_dest > 1) ? "Some items" : "An item",
+                 (num_dest > 1) ? "were" : "was" );
             break;
         }
 
-        mpr( info );    // XXX: should this be in a channel?
+        xom_is_stimulated((num_dest > 1) ? 32 : 16);
     }
 }
 
@@ -578,7 +583,7 @@ void expose_player_to_element( beam_type flavour, int strength )
         expose_invent_to_element( flavour, strength );
 }
 
-void lose_level(void)
+void lose_level()
 {
     // because you.experience is unsigned long, if it's going to be -ve
     // must die straightaway.
@@ -588,9 +593,8 @@ void lose_level(void)
     you.experience = exp_needed( you.experience_level + 1 ) - 1;
     you.experience_level--;
 
-    snprintf( info, INFO_SIZE, "You are now a level %d %s!", 
-             you.experience_level, you.class_name );
-    mpr( info, MSGCH_WARN );
+    mprf(MSGCH_WARN,
+         "You are now a level %d %s!", you.experience_level, you.class_name);
 
     // Constant value to avoid grape jelly trick... see level_change() for
     // where these HPs and MPs are given back.  -- bwr
@@ -609,9 +613,10 @@ void lose_level(void)
     take_note(Note(NOTE_XP_LEVEL_CHANGE, you.experience_level, 0, buf));
 
     you.redraw_experience = 1;
+    xom_is_stimulated(255);
 }                               // end lose_level()
 
-void drain_exp(void)
+void drain_exp(bool announce_full)
 {
     int protection = player_prot_life();
 
@@ -625,7 +630,8 @@ void drain_exp(void)
 
     if (protection >= 3 || you.is_undead)
     {
-        mpr("You fully resist.");
+        if ( announce_full )
+            mpr("You fully resist.");
         return;
     }
 
@@ -653,6 +659,7 @@ void drain_exp(void)
     if (exp_drained > 0)
     {
         mpr("You feel drained.");
+        xom_is_stimulated(20);
         you.experience -= exp_drained;
         you.exp_available -= exp_drained;
 
@@ -660,9 +667,7 @@ void drain_exp(void)
             you.exp_available = 0;
 
 #if DEBUG_DIAGNOSTICS
-        snprintf( info, INFO_SIZE, "You lose %ld experience points.",
-                  exp_drained );
-        mpr( info, MSGCH_DIAGNOSTICS );
+        mprf(MSGCH_DIAGNOSTICS, "You lose %ld experience points.",exp_drained);
 #endif
 
         you.redraw_experience = 1;
@@ -672,14 +677,69 @@ void drain_exp(void)
     }
 }                               // end drain_exp()
 
+static void xom_checks_damage(kill_method_type death_type,
+                              int dam, int death_source)
+{
+    //if (you.hp <= dam)
+    //    xom_is_stimulated(32);
+
+    if ((death_type != KILLED_BY_MONSTER && death_type != KILLED_BY_BEAM)
+        || death_source < 0 || death_source >= MAX_MONSTERS)
+    {
+        return ;
+    }
+    
+    int amusementvalue = 1;
+
+    const monsters *monster = &menv[death_source];
+
+    if (!monster->alive())
+        return;
+
+    int leveldif = monster->hit_dice - you.experience_level;
+
+    if (leveldif == 0)
+        leveldif = 1;
+
+    /* Note that Xom is amused when you are significantly hurt
+     * by a creature of higher level than yourself as well as
+     * by a creatured of lower level than yourself. */
+    amusementvalue += leveldif * leveldif * dam;
+    
+    if (!player_monster_visible(monster))
+        amusementvalue += 10;
+    
+    if (monster->speed < (int) player_movement_speed())
+        amusementvalue += 8;
+    
+    if (death_type != KILLED_BY_BEAM)
+    {
+        if (you.skills[SK_THROWING] <= (you.experience_level / 4))
+            amusementvalue += 2;
+    }
+    else
+    {
+        if (you.skills[SK_FIGHTING] <= (you.experience_level / 4))
+            amusementvalue += 2;
+    }
+
+    if (player_in_a_dangerous_place())
+        amusementvalue += 2;
+            
+    amusementvalue /= (you.hp > 0) ? you.hp : 1;
+  
+    xom_is_stimulated(amusementvalue);
+}
+
 // death_source should be set to zero for non-monsters {dlb}
-void ouch( int dam, int death_source, char death_type, const char *aux )
+void ouch( int dam, int death_source, kill_method_type death_type,
+           const char *aux, bool see_source )
 {
     ait_hp_loss hpl(dam, death_type);
     interrupt_activity( AI_HP_LOSS, &hpl );
 
-    if (you.deaths_door && death_type != KILLED_BY_LAVA
-                                    && death_type != KILLED_BY_WATER)
+    if (you.duration[DUR_DEATHS_DOOR] && death_type != KILLED_BY_LAVA
+        && death_type != KILLED_BY_WATER)
     {
         return;
     }
@@ -687,9 +747,8 @@ void ouch( int dam, int death_source, char death_type, const char *aux )
     // assumed bug for high damage amounts
     if (dam > 300)
     {
-        snprintf( info, INFO_SIZE, 
-                  "Potential bug: Unexpectedly high damage = %d", dam );
-        mpr( info, MSGCH_DANGER );
+        mprf(MSGCH_DANGER,
+             "Potential bug: Unexpectedly high damage = %d", dam );
         return;                 
     }
 
@@ -697,21 +756,12 @@ void ouch( int dam, int death_source, char death_type, const char *aux )
     {
         switch (you.religion)
         {
-        case GOD_XOM:
-            if (random2(you.hp_max) > you.hp && dam > random2(you.hp)
-                                                    && one_chance_in(5))
-            {
-                simple_god_message( " protects you from harm!" );
-                return;
-            }
-            break;
-
         case GOD_ZIN:
         case GOD_SHINING_ONE:
         case GOD_ELYVILON:
         case GOD_YREDELEMNUL:
             if (dam >= you.hp && you.duration[DUR_PRAYER]
-                                                && random2(you.piety) >= 30)
+                && random2(you.piety) >= 30)
             {
                 simple_god_message( " protects you from harm!" );
                 return;
@@ -721,30 +771,43 @@ void ouch( int dam, int death_source, char death_type, const char *aux )
             break;
         }
 
-        // Damage applied here:
+
         dec_hp( dam, true );
 
         // Even if we have low HP messages off, we'll still give a
         // big hit warning (in this case, a hit for half our HPs) -- bwr
         if (dam > 0 && you.hp_max <= dam * 2)
-            mpr( "Ouch!  That really hurt!", MSGCH_DANGER );
-
-        if (you.hp > 0 && Options.hp_warning
-            && you.hp <= (you.hp_max * Options.hp_warning) / 100)
-        {
-            mpr( "* * * LOW HITPOINT WARNING * * *", MSGCH_DANGER );
-        }
-        take_note(
-                Note(
-                    NOTE_HP_CHANGE, 
-                    you.hp, 
-                    you.hp_max, 
-                    scorefile_entry(dam, death_source, death_type, aux, true)
-                        .death_description(scorefile_entry::DDV_TERSE)
-                        .c_str()) );
+            mpr( "Ouch! That really hurt!", MSGCH_DANGER );
 
         if (you.hp > 0)
+        {
+            if (Options.hp_warning
+                && you.hp <= (you.hp_max * Options.hp_warning) / 100)
+            {
+                mpr( "* * * LOW HITPOINT WARNING * * *", MSGCH_DANGER );
+            }
+
+            xom_checks_damage(death_type, dam, death_source);
+
+            // for note taking
+            std::string damage_desc = "";
+            if (!see_source)
+            {
+                snprintf(info, INFO_SIZE, "something (%d)", dam);
+                damage_desc = info;
+            }
+            else
+            {
+                damage_desc = scorefile_entry(dam, death_source,
+                                              death_type, aux, true)
+                              .death_description(scorefile_entry::DDV_TERSE);
+            }
+
+            take_note(
+                Note(NOTE_HP_CHANGE, you.hp, you.hp_max, damage_desc.c_str()) );
+                
             return;
+        } // else hp <= 0
     }
 
 #ifdef WIZARD
@@ -757,9 +820,8 @@ void ouch( int dam, int death_source, char death_type, const char *aux )
 #ifdef USE_OPTIONAL_WIZARD_DEATH
 
 #if DEBUG_DIAGNOSTICS
-            snprintf( info, INFO_SIZE, "Damage: %d; Hit points: %d", dam, you.hp );
-            mpr( info, MSGCH_DIAGNOSTICS );
-#endif // DEBUG_DIAGNOSTICS
+            mprf(MSGCH_DIAGNOSTICS, "Damage: %d; Hit points: %d", dam, you.hp);
+#endif
 
             if (!yesno("Die?", false, 'n'))
             {
@@ -782,6 +844,11 @@ void ouch( int dam, int death_source, char death_type, const char *aux )
     crawl_state.need_save       = false;
     crawl_state.updating_scores = true;
 
+    take_note(
+         Note( NOTE_DEATH, you.hp, you.hp_max,
+           scorefile_entry(dam, death_source, death_type, aux)
+           .death_description(scorefile_entry::DDV_NORMAL).c_str()), true );
+
     // prevent bogus notes
     activate_notes(false);
 
@@ -814,13 +881,12 @@ void ouch( int dam, int death_source, char death_type, const char *aux )
     end_game(se);
 }
 
-static std::string morgue_name()
+static std::string morgue_name(time_t when_crawl_got_even)
 {
 #ifdef SHORT_FILE_NAMES
     return "morgue";
 #else  // !SHORT_FILE_NAMES
     std::string name = "morgue-" + std::string(you.your_name);
-    time_t when_crawl_got_even = time(NULL);
 
     if (tm *loc = localtime(&when_crawl_got_even))
     {
@@ -843,7 +909,8 @@ void end_game( struct scorefile_entry &se )
     int i;
     bool dead = true;
 
-    if (se.death_type == KILLED_BY_LEAVING ||
+    if (se.death_type == KILLED_BY_LEAVING  ||
+        se.death_type == KILLED_BY_QUITTING ||
         se.death_type == KILLED_BY_WINNING)
     {
         dead = false;
@@ -856,14 +923,23 @@ void end_game( struct scorefile_entry &se )
         {
             if (tmp_file_pairs[level][dungeon])
             {
-                unlink(make_filename( you.your_name, level, dungeon,
-                                      false, false ).c_str());
+                unlink(
+                    make_filename( you.your_name, level,
+                                   static_cast<branch_type>(dungeon),
+                                   LEVEL_DUNGEON, false ).c_str() );
             }
         }
     }
 
-    // temp level, if any
-    unlink( make_filename( you.your_name, 0, 0, true, false ).c_str() );
+    // temp levels, if any
+    unlink( make_filename( you.your_name, 0, BRANCH_MAIN_DUNGEON,
+                           LEVEL_ABYSS, false ).c_str() );
+    unlink( make_filename( you.your_name, 0, BRANCH_MAIN_DUNGEON,
+                           LEVEL_PANDEMONIUM, false ).c_str() );
+    unlink( make_filename( you.your_name, 0, BRANCH_MAIN_DUNGEON,
+                           LEVEL_LABYRINTH, false ).c_str() );
+    unlink( make_filename( you.your_name, 0, BRANCH_MAIN_DUNGEON,
+                           LEVEL_PORTAL_VAULT, false ).c_str() );
 
     // create base file name
     std::string basename = get_savedir_filename( you.your_name, "", "" );
@@ -889,11 +965,11 @@ void end_game( struct scorefile_entry &se )
     if (dead)
     {
         mpr("You die...");      // insert player name here? {dlb}
-        viewwindow(1, false);   // don't do this for leaving/winning characters
-    }
+        viewwindow(true, false); // don't do for leaving/winning characters
 
-    if (Options.tutorial_left)
-        tutorial_death_screen();
+        if (Options.tutorial_left)
+            tutorial_death_screen();
+    }
 
 #ifdef DGL_WHEREIS
     whereis_record( se.death_type == KILLED_BY_QUITTING? "quit" :
@@ -917,10 +993,10 @@ void end_game( struct scorefile_entry &se )
         }
     }
 
-    invent( -1, dead );
+    invent( -1, true );
     clrscr();
 
-    if (!dump_char( morgue_name(), !dead, true ))
+    if (!dump_char( morgue_name(se.death_time), !dead, true, &se ))
     {
         mpr("Char dump unsuccessful! Sorry about that.");
         if (!crawl_state.seen_hups)
