@@ -3,7 +3,7 @@
  *  Summary:    Functions used to print player related info.
  *  Written by: Linley Henzell
  *
- *  Modified for Crawl Reference by $Author: dshaligram $ on $Date: 2007-04-01T07:20:19.020925Z $
+ *  Modified for Crawl Reference by $Author: ennewalker $ on $Date: 2007-11-23T02:25:44.246247Z $
  *
  *  Change History (most recent first):
  *
@@ -23,16 +23,22 @@
 #include "externs.h"
 
 #include "abl-show.h"
+#include "describe.h"
+#include "direct.h"
 #include "format.h"
 #include "fight.h"
 #include "initfile.h"
 #include "itemname.h"
+#include "itemprop.h"
+#include "items.h"
 #include "menu.h"
+#include "message.h"
 #include "ouch.h"
 #include "player.h"
 #include "religion.h"
 #include "skills2.h"
 #include "stuff.h"
+#include "transfor.h"
 
 static int bad_ench_colour( int lvl, int orange, int red )
 {
@@ -65,7 +71,7 @@ void update_message_status()
 {
     textcolor(LIGHTBLUE);
 
-    gotoxy(75, 2);
+    gotoxy(crawl_view.hudp.x + 35, 2);
     if (SysEnv.have_messages)
         cprintf("(msg)");
     else
@@ -88,7 +94,7 @@ void update_turn_count()
     // FIXME: Create some kind of layout manager class so we can
     // templatise the heads-up display layout and stop hardcoding
     // these coords.
-    gotoxy(61, 10);
+    gotoxy(crawl_view.hudp.x + 21, 10);
     textcolor(LIGHTGREY);
 
     // Show the turn count starting from 1. You can still quit on turn 0.
@@ -99,6 +105,8 @@ void print_stats(void)
 {
     textcolor(LIGHTGREY);
 
+    const int xcol = crawl_view.hudp.x;
+    
     // Displayed evasion is now tied to dex.
     if (you.redraw_dexterity)
         you.redraw_evasion = true;
@@ -122,7 +130,7 @@ void print_stats(void)
             }
         }
 
-        gotoxy(44, 3);
+        gotoxy(xcol + 4, 3);
 
         cprintf( "%d", you.hp );
 
@@ -132,12 +140,7 @@ void print_stats(void)
         if (max_max_hp != you.hp_max)
             cprintf( " (%d)", max_max_hp );
 
-#ifdef UNIX
         clear_to_end_of_line();
-#else
-        cprintf("        ");
-#endif
-
         you.redraw_hit_points = 0;
     }
 
@@ -157,19 +160,14 @@ void print_stats(void)
                 break;
             }
         }
-        gotoxy(47, 4);
+        gotoxy(xcol + 7, 4);
 
         cprintf( "%d", you.magic_points);
 
         textcolor(LIGHTGREY);
         cprintf("/%d", you.max_magic_points );
 
-#ifdef UNIX
         clear_to_end_of_line();
-#else
-        cprintf("        ");
-#endif
-
         you.redraw_magic_points = 0;
     }
 
@@ -183,9 +181,9 @@ void print_stats(void)
         if (you.max_strength > 72)
             you.max_strength = 72;
 
-        gotoxy(45, 7);
+        gotoxy(xcol + 5, 7);
 
-        if (you.might)
+        if (you.duration[DUR_MIGHT])
             textcolor(LIGHTBLUE);  // no end of effect warning
         else if (you.strength < you.max_strength)
             textcolor(YELLOW);
@@ -216,7 +214,7 @@ void print_stats(void)
         if (you.max_intel > 72)
             you.max_intel = 72;
 
-        gotoxy(45, 8);
+        gotoxy(xcol + 5, 8);
 
         if (you.intel < you.max_intel)
             textcolor(YELLOW);
@@ -245,7 +243,7 @@ void print_stats(void)
         if (you.max_dex > 72)
             you.max_dex = 72;
 
-        gotoxy(45, 9);
+        gotoxy(xcol + 5, 9);
 
         if (you.dex < you.max_dex)
             textcolor(YELLOW);
@@ -266,7 +264,7 @@ void print_stats(void)
 
     if (you.redraw_armour_class)
     {
-        gotoxy(44, 5);
+        gotoxy(xcol + 4, 5);
 
         if (you.duration[DUR_STONEMAIL])
             dur_colour( BLUE, (you.duration[DUR_STONEMAIL] <= 6) );
@@ -276,7 +274,7 @@ void print_stats(void)
         cprintf( "%d  ", player_AC() );
         textcolor( LIGHTGREY );
 
-        gotoxy(50, 5);
+        gotoxy(xcol + 10, 5);
 
         if (you.duration[DUR_CONDENSATION_SHIELD])      //jmf: added 24mar2000
             textcolor( LIGHTBLUE );  // no end of effect warning
@@ -289,7 +287,7 @@ void print_stats(void)
 
     if (you.redraw_evasion)
     {
-        gotoxy(44, 6);
+        gotoxy(xcol + 4, 6);
 
         if (you.duration[DUR_FORESCRY])
             textcolor(LIGHTBLUE);  // no end of effect warning
@@ -302,14 +300,14 @@ void print_stats(void)
 
     if (you.redraw_gold)
     {
-        gotoxy(46, 10);
+        gotoxy(xcol + 6, 10);
         cprintf( "%-8d", you.gold );
         you.redraw_gold = 0;
     }
 
     if (you.redraw_experience)
     {
-        gotoxy(52, 11);
+        gotoxy(xcol + 12, 11);
 
 #if DEBUG_DIAGNOSTICS
         cprintf( "%d/%lu  (%d/%d)",
@@ -320,45 +318,34 @@ void print_stats(void)
                  you.experience_level, you.experience, you.exp_available );
 #endif
 
-#ifdef UNIX
         clear_to_end_of_line();
-#else
-        cprintf("   ");
-#endif
         you.redraw_experience = 0;
     }
 
     if (you.wield_change)
     {
-        gotoxy(40, 13);
-#ifdef UNIX
+        gotoxy(xcol, 13);
         clear_to_end_of_line();
-#else
-        cprintf("                                       ");
-#endif
+        gotoxy(xcol, 13);
 
         if (you.equip[EQ_WEAPON] != -1)
         {
-            gotoxy(40, 13);
-            textcolor(you.inv[you.equip[EQ_WEAPON]].colour);
 
-            char str_pass[ ITEMNAME_SIZE ];
-            in_name( you.equip[EQ_WEAPON], DESC_INVENTORY, str_pass, false );
-            int prefcol = menu_colour(str_pass);
+            const item_def& wpn = you.inv[you.equip[EQ_WEAPON]];
+            textcolor(wpn.colour);
+
+            const std::string prefix = menu_colour_item_prefix(wpn);
+            const int prefcol = menu_colour(wpn.name(DESC_INVENTORY), prefix);
             if (prefcol != -1)
                 textcolor(prefcol);
 
-            in_name( you.equip[EQ_WEAPON], DESC_INVENTORY, str_pass,
-                     Options.terse_hand );
-            str_pass[39] = 0;
-
-            cprintf("%s", str_pass);
+            cprintf("%s",
+                    wpn.name(DESC_INVENTORY,
+                             Options.terse_hand).substr(0,38).c_str());
             textcolor(LIGHTGREY);
         }
         else
         {
-            gotoxy(40, 13);
-
             if (you.attribute[ATTR_TRANSFORMATION] == TRAN_BLADE_HANDS)
             {
                 textcolor(RED);
@@ -384,14 +371,9 @@ void print_stats(void)
 
     if (you.redraw_status_flags & REDRAW_LINE_1_MASK)
     {
-        gotoxy(40, 14);
-
-#ifdef UNIX
+        gotoxy(xcol, 14);
         clear_to_end_of_line();
-#else
-        cprintf( "                                       " );
-        gotoxy(40, 14);
-#endif
+        gotoxy(xcol, 14);
 
         switch (you.burden_state)
         {
@@ -416,6 +398,11 @@ void print_stats(void)
             cprintf( "Engorged" );
             break;
 
+        case HS_VERY_FULL:
+            textcolor( GREEN );
+            cprintf( "Very Full" );
+            break;
+
         case HS_FULL:
             textcolor( GREEN );
             cprintf( "Full" );
@@ -429,6 +416,16 @@ void print_stats(void)
             cprintf( "Hungry" );
             break;
 
+        case HS_VERY_HUNGRY:
+            textcolor( YELLOW );
+            cprintf( "Very Hungry" );
+            break;
+
+        case HS_NEAR_STARVING:
+            textcolor( YELLOW );
+            cprintf( "Near Starving" );
+            break;
+
         case HS_STARVING:
             textcolor( RED );
             cprintf( "Starving" );
@@ -445,15 +442,9 @@ void print_stats(void)
 
     if (you.redraw_status_flags & REDRAW_LINE_2_MASK)
     {
-        gotoxy(40, 15);
-
-#ifdef UNIX
+        gotoxy(xcol, 15);
         clear_to_end_of_line();
-#else
-        cprintf( "                                       " );
-        gotoxy(40, 15);
-#endif
-
+        gotoxy(xcol, 15);
         // Max length of this line = 8 * 5 - 1 = 39
 
         if (you.duration[DUR_PRAYER])
@@ -492,7 +483,7 @@ void print_stats(void)
             cprintf( "Ins " );
         }
 
-        if (player_is_levitating())
+        if (player_is_airborne())
         {
             bool perm = (you.species == SP_KENKU && you.experience_level >= 15)
                         || (player_equip_ego_type( EQ_BOOTS, SPARM_LEVITATION ))
@@ -500,44 +491,53 @@ void print_stats(void)
 
             if (wearing_amulet( AMU_CONTROLLED_FLIGHT ))
             {
-                dur_colour( MAGENTA, (you.levitation <= 10 && !perm) );
+                dur_colour( you.light_flight()? BLUE : MAGENTA,
+                            (you.duration[DUR_LEVITATION] <= 10 && !perm) );
                 cprintf( "Fly " );
             }
             else
             {
-                dur_colour( BLUE, (you.levitation <= 10 && !perm) );
+                dur_colour( BLUE, (you.duration[DUR_LEVITATION] <= 10 && !perm) );
                 cprintf( "Lev " );
             }
         }
 
-        if (you.invis)
+        if (you.duration[DUR_INVIS])
         {
-            dur_colour( BLUE, (you.invis <= 6) );
+            dur_colour( BLUE, (you.duration[DUR_INVIS] <= 6) );
             cprintf( "Invis " );
         }
 
+        if (you.duration[DUR_SILENCE])
+        {
+            dur_colour( BLUE, (you.duration[DUR_SILENCE] <= 5) );
+            cprintf( "Sil " );
+        }
+
         // Perhaps this should be reversed to show when it can be used?
         // In that case, it should be probably be GREEN, and we'd have
         // to check to see if the player does have a breath weapon. -- bwr
-        if (you.duration[DUR_BREATH_WEAPON])
+        if (you.duration[DUR_BREATH_WEAPON] &&
+            wherex() < get_number_of_cols() - 4)
         {
             textcolor( YELLOW );  // no warning
             cprintf( "BWpn" );
         }
+        
+        if (you.duration[DUR_BARGAIN] && wherex() < get_number_of_cols() - 4)
+        {
+            dur_colour( BLUE, (you.duration[DUR_BARGAIN] <= 15) );
+            cprintf( "Brgn" );
+        }
 
         textcolor( LIGHTGREY );
     }
 
     if (you.redraw_status_flags & REDRAW_LINE_3_MASK)
     {
-        gotoxy(40, 16);
-
-#ifdef UNIX
+        gotoxy(xcol, 16);
         clear_to_end_of_line();
-#else
-        cprintf( "                                       " );
-        gotoxy(40, 16);
-#endif
+        gotoxy(xcol, 16);
         // Max length of this line = 7 * 5 + 3 - 1 = 37
 
         // Note the usage of bad_ench_colour() correspond to levels that
@@ -545,7 +545,7 @@ void print_stats(void)
         // using the '@' command.  Things like confusion and sticky flame
         // hide their amounts and are thus always the same colour (so
         // we're not really exposing any new information). --bwr
-        if (you.conf)
+        if (you.duration[DUR_CONF])
         {
             textcolor( RED );   // no different levels
             cprintf( "Conf " );
@@ -557,12 +557,12 @@ void print_stats(void)
             cprintf( "Fire " );
         }
 
-        if (you.poisoning)
+        if (you.duration[DUR_POISONING])
         {
             // We skip marking "quite" poisoned and instead mark the
             // levels where the rules for dealing poison damage change
             // significantly.  See acr.cc for that code. -- bwr
-            textcolor( bad_ench_colour( you.poisoning, 5, 10 ) );
+            textcolor( bad_ench_colour( you.duration[DUR_POISONING], 5, 10 ) );
             cprintf( "Pois " );
         }
 
@@ -578,9 +578,18 @@ void print_stats(void)
             cprintf( "Rot " );
         }
 
-        if (you.magic_contamination >= 5)
+        if (you.attribute[ATTR_HELD])
         {
-            textcolor( bad_ench_colour( you.magic_contamination, 15, 25 ) );
+            textcolor( RED );
+            cprintf( "Held " );
+        }
+
+        if (you.backlit())
+        {
+            textcolor(
+                you.magic_contamination >= 5?
+                bad_ench_colour( you.magic_contamination, 15, 25 )
+                : LIGHTBLUE );
             cprintf( "Glow " );
         }
 
@@ -590,14 +599,14 @@ void print_stats(void)
             cprintf( "Swift " );
         }
 
-        if (you.slow && !you.haste)
+        if (you.duration[DUR_SLOW] && !you.duration[DUR_HASTE])
         {
             textcolor( RED );  // no end of effect warning
             cprintf( "Slow" );
         }
-        else if (you.haste && !you.slow)
+        else if (you.duration[DUR_HASTE] && !you.duration[DUR_SLOW])
         {
-            dur_colour( BLUE, (you.haste <= 6) );
+            dur_colour( BLUE, (you.duration[DUR_HASTE] <= 6) );
             cprintf( "Fast" );
         }
 
@@ -608,14 +617,12 @@ void print_stats(void)
 
 #if DEBUG_DIAGNOSTICS
     // debug mode GPS
-    gotoxy(40, 17);
+    gotoxy(xcol, 17);
     cprintf( "Position (%2d,%2d)", you.x_pos, you.y_pos );
 #endif
 
-#ifdef UNIX
     // get curses to redraw screen
     update_screen();
-#endif
 }                               // end print_stats()
 
 const char* itosym1(int stat)
@@ -650,8 +657,13 @@ static const char *s_equip_slot_names[] =
 
 const char *equip_slot_to_name(int equip)
 {
-    if (equip == EQ_RINGS)
+    if (equip == EQ_RINGS || equip == EQ_LEFT_RING || equip == EQ_RIGHT_RING)
         return "Ring";
+
+    if (equip == EQ_BOOTS &&
+           (you.species == SP_CENTAUR || you.species == SP_NAGA))
+        return "Barding";
+
     if (equip < 0 || equip >= NUM_EQUIP)
         return "";
     return s_equip_slot_names[equip];
@@ -684,11 +696,11 @@ static const char* determine_color_string( int level ) {
     }
 }
 
-std::vector<formatted_string> get_full_detail(bool calc_unid)
+std::vector<formatted_string> get_full_detail(bool calc_unid, long sc)
 {
     char buf[1000];
     // 3 columns, splits at columns 32, 52
-    column_composer cols(3, 32, 52);
+    column_composer cols(3, 33, 52);
     char god_colour_tag[20];
     god_colour_tag[0] = 0;
     std::string godpowers(god_name(you.religion));
@@ -710,17 +722,24 @@ std::vector<formatted_string> get_full_detail(bool calc_unid)
                 std::string(prank, '*');
         }
     }
+
+    const std::string score =
+        (sc > -1? make_stringf(" (%ld points)", sc) : "");
+    
     snprintf(buf, sizeof buf,
-             "<yellow>%s the %s</yellow>\n\n"
+             "<yellow>%s the %s</yellow>%s\n\n"
              "Race       : %s\n"
              "Class      : %s\n"
              "Worship    : %s%s\n"
              "Level      : %7d\n"
              "Exp        : %7lu\n",
-             you.your_name, player_title(),
-             species_name(you.species,you.experience_level),
+             you.your_name,
+             player_title().c_str(),
+             score.c_str(),
+             species_name(you.species,you.experience_level).c_str(),
              you.class_name,
-             god_colour_tag, godpowers.c_str(),
+             god_colour_tag,
+             godpowers.c_str(),
              you.experience_level,
              you.experience);
     cols.add_formatted(0, buf, false);
@@ -799,13 +818,10 @@ std::vector<formatted_string> get_full_detail(bool calc_unid)
     if (you.real_time != -1)
     {
         const time_t curr = you.real_time + (time(NULL) - you.start_time);
-        char buff[200];
-        make_time_string( curr, buff, sizeof(buff), true );
-
         snprintf(buf, sizeof buf,
                  "Play time  : %10s\n"
                  "Turns      : %10ld\n",
-                 buff, you.num_turns );
+                 make_time_string(curr, true).c_str(), you.num_turns );
         cols.add_formatted(0, buf, true);
     }
 
@@ -845,7 +861,6 @@ std::vector<formatted_string> get_full_detail(bool calc_unid)
     cols.add_formatted(1, buf, true);
 
     {
-        char str_pass[ITEMNAME_SIZE];
         const int e_order[] =
         {
             EQ_WEAPON, EQ_BODY_ARMOUR, EQ_SHIELD, EQ_HELMET, EQ_CLOAK,
@@ -856,20 +871,14 @@ std::vector<formatted_string> get_full_detail(bool calc_unid)
         {
             int eqslot = e_order[i];
             const char *slot = equip_slot_to_name( eqslot );
-            if (eqslot == EQ_LEFT_RING || eqslot == EQ_RIGHT_RING)
-                slot = "Ring";
-            else if (eqslot == EQ_BOOTS &&
-                     (you.species == SP_CENTAUR || you.species == SP_NAGA))
-                slot = "Barding";
 
             if ( you.equip[ e_order[i] ] != -1)
             {
-                const int inum = you.equip[e_order[i]];
-                in_name( inum, DESC_PLAIN, str_pass, true );
-                str_pass[38] = 0; // truncate
-                const char* colname = colour_to_str(you.inv[inum].colour);
+                const item_def& item = you.inv[you.equip[e_order[i]]];
+                const char* colname = colour_to_str(item.colour);
                 snprintf(buf, sizeof buf, "%-7s: <%s>%s</%s>",
-                         slot, colname, str_pass, colname);
+                         slot, colname,
+                         item.name(DESC_PLAIN).substr(0,38).c_str(), colname);
             }
             else
             {
@@ -957,14 +966,14 @@ std::vector<formatted_string> get_full_detail(bool calc_unid)
                  determine_color_string(-1), itosym1(1));
     else
     {
-        const int rrtel = player_teleport(calc_unid);
+        const int rrtel = !!player_teleport(calc_unid);
         snprintf(buf, sizeof buf, "%sRnd.Telep. : %s",
                  determine_color_string(rrtel), itosym1(rrtel));
     }
     cols.add_formatted(2, buf, false);
 
     const int rctel = player_control_teleport(calc_unid);
-    const int rlevi = player_is_levitating();
+    const int rlevi = player_is_airborne();
     const int rcfli = wearing_amulet(AMU_CONTROLLED_FLIGHT, calc_unid);
     snprintf(buf, sizeof buf,
              "%sCtrl.Telep.: %s\n"
@@ -993,24 +1002,22 @@ void print_overview_screen()
 
     std::string text;
     char title[50];
-    snprintf(title, sizeof title, " the %s ", player_title());
+    snprintf(title, sizeof title, " the %s ", player_title().c_str());
 
     char race_class[50];
     snprintf(race_class, sizeof race_class,
              "(%s %s)",
-             species_name(you.species, you.experience_level), you.class_name);
+             species_name(you.species, you.experience_level).c_str(),
+             you.class_name);
 
     char time_turns[50] = "";
 
     if (you.real_time != -1)
     {
         const time_t curr = you.real_time + (time(NULL) - you.start_time);
-        char buff[200];
-        make_time_string( curr, buff, sizeof(buff), true );
-
         snprintf(time_turns, sizeof time_turns,
                  " Turns: %ld, Time: %s",
-                 you.num_turns, buff );
+                 you.num_turns, make_time_string(curr, true).c_str() );
     }
 
     int linelength = strlen(you.your_name) + strlen(title)
@@ -1036,12 +1043,11 @@ void print_overview_screen()
     }
 
     text = "<yellow>";
-    snprintf(info, INFO_SIZE, "%s%s%s", you.your_name, title, race_class);
-    text += info;
-    int k = get_number_of_cols() - linelength -1;
-    text += std::string(k, ' ');
-    snprintf(info, INFO_SIZE, "%s", time_turns);
-    text += info;
+    text += you.your_name;
+    text += title;
+    text += race_class;
+    text += std::string(get_number_of_cols() - linelength - 1, ' ');
+    text += time_turns;
     text += "</yellow>\n";
     
     cmd_help.add_text(text);
@@ -1106,7 +1112,7 @@ void print_overview_screen()
                      colour_to_str(god_colour(you.religion)));
             // piety rankings
             int prank = piety_rank() - 1;
-            if ( prank < 0 )
+            if ( prank < 0 || you.religion == GOD_XOM)
                 prank = 0;
             // Careful about overflow. We erase some of the god's name
             // if necessary.
@@ -1233,14 +1239,14 @@ void print_overview_screen()
                  determine_color_string(-1), itosym1(1));
     else
     {
-        const int rrtel = player_teleport(calc_unid);
+        const int rrtel = !!player_teleport(calc_unid);
         snprintf(buf, sizeof buf, "\n%sRnd.Telep. : %s",
                  determine_color_string(rrtel), itosym1(rrtel));
     }
     cols.add_formatted(1, buf, false);
 
     const int rctel = player_control_teleport(calc_unid);
-    const int rlevi = player_is_levitating();
+    const int rlevi = player_is_airborne();
     const int rcfli = wearing_amulet(AMU_CONTROLLED_FLIGHT, calc_unid);
     snprintf(buf, sizeof buf,
              "%sCtrl.Telep.: %s\n"
@@ -1252,7 +1258,6 @@ void print_overview_screen()
     cols.add_formatted(1, buf, false);
 
     {
-        char str_pass[ITEMNAME_SIZE];
         const int e_order[] =
         {
             EQ_WEAPON, EQ_BODY_ARMOUR, EQ_SHIELD, EQ_HELMET, EQ_CLOAK,
@@ -1263,20 +1268,14 @@ void print_overview_screen()
         {
             int eqslot = e_order[i];
             const char *slot = equip_slot_to_name( eqslot );
-            if (eqslot == EQ_LEFT_RING || eqslot == EQ_RIGHT_RING)
-                slot = "Ring";
-            else if (eqslot == EQ_BOOTS &&
-                     (you.species == SP_CENTAUR || you.species == SP_NAGA))
-                slot = "Barding";
 
             if ( you.equip[ e_order[i] ] != -1)
             {
-                const int inum = you.equip[e_order[i]];
-                in_name( inum, DESC_PLAIN, str_pass, true );
-                str_pass[38] = 0; // truncate
-                const char* colname = colour_to_str(you.inv[inum].colour);
+                const item_def& item = you.inv[you.equip[e_order[i]]];
+                const char* colname = colour_to_str(item.colour);
                 snprintf(buf, sizeof buf, "%-7s: <%s>%s</%s>",
-                         slot, colname, str_pass, colname);
+                         slot, colname,
+                         item.name(DESC_PLAIN).substr(0,37).c_str(), colname);
             }
             else
             {
@@ -1286,9 +1285,32 @@ void print_overview_screen()
                         snprintf(buf, sizeof buf, "%-7s: Blade Hands", slot);
                     else if (you.skills[SK_UNARMED_COMBAT])
                         snprintf(buf, sizeof buf, "%-7s: Unarmed", slot);
+                    else if (!you_tran_can_wear(EQ_WEAPON))
+                    {
+                        snprintf(buf, sizeof buf, "%-7s: "
+                                 "<darkgray>(currently unavailable)</darkgray>",
+                                 slot);
+                    }
                     else
                         snprintf(buf, sizeof buf, "%-7s:", slot);
                 }
+                else if (!you_can_wear(e_order[i], true))
+                {
+                    snprintf(buf, sizeof buf,
+                             "%-7s: <darkgray>(unavailable)</darkgray>", slot);
+                }
+                else if (!you_tran_can_wear(e_order[i], true))
+                {
+                    snprintf(buf, sizeof buf, "%-7s: "
+                             "<darkgray>(currently unavailable)</darkgray>",
+                             slot);
+                }
+                else if (!you_can_wear(e_order[i]))
+                {
+                    snprintf(buf, sizeof buf,
+                             "%-7s: <lightgray>(restricted)</lightgray>",
+                             slot);
+                }
                 else
                 {
                     snprintf(buf, sizeof buf, "%-7s:", slot);
@@ -1343,8 +1365,15 @@ std::string status_mut_abilities()
     if (you.duration[DUR_PRAYER])
         text += "praying, ";
 
-    if (you.duration[DUR_REGENERATION])
-        text += "regenerating, ";
+    if (you.duration[DUR_REGENERATION]
+        && (you.species != SP_VAMPIRE || you.hunger_state >= HS_HUNGRY)
+        || you.species == SP_VAMPIRE && you.hunger_state >= HS_FULL)
+    {
+        if (you.disease)
+            text += "recuperating, ";
+        else
+            text += "regenerating, ";
+    }
 
 // not used as resistance part already says so
 //    if (you.duration[DUR_INSULATION])
@@ -1376,36 +1405,43 @@ std::string status_mut_abilities()
     if (you.duration[DUR_SILENCE])  
         text += "radiating silence, ";
 
-// resistance part already says so
-//    if (you.duration[DUR_SEE_INVISIBLE])
-//        text += "see invisible, ";
-
-    if (you.invis)
+    if (you.duration[DUR_INVIS])
         text += "invisible, ";
 
-    if (you.conf)
+    if (you.duration[DUR_CONF])
         text += "confused, ";
 
-    if (you.paralysis)
+    if (you.duration[DUR_PARALYSIS])
         text += "paralysed, ";
 
-    if (you.exhausted)
+    if (you.duration[DUR_EXHAUSTED])
         text += "exhausted, ";
 
-    if (you.might)
+    if (you.duration[DUR_MIGHT])
         text += "mighty, ";
 
-    if (you.berserker)
+    if (you.duration[DUR_BERSERKER])
         text += "berserking, ";
 
-    if (player_is_levitating())
+    if (player_is_airborne())
         text += "levitating, ";
 
-    if (you.poisoning)
+    if (you.duration[DUR_BARGAIN])
+        text += "charismatic, ";
+
+    if (you.duration[DUR_SLAYING])
+        text += "deadly, ";
+
+    // DUR_STEALTHY handled in stealth printout
+
+    if (you.duration[DUR_MAGIC_SHIELD])
+        text += "shielded, ";
+
+    if (you.duration[DUR_POISONING])
     {
-        text +=   (you.poisoning > 10) ? "extremely" :
-                  (you.poisoning > 5)  ? "very" :
-                  (you.poisoning > 3)  ? "quite"
+        text +=   (you.duration[DUR_POISONING] > 10) ? "extremely" :
+                  (you.duration[DUR_POISONING] > 5)  ? "very" :
+                  (you.duration[DUR_POISONING] > 3)  ? "quite"
                                        : "mildly";
         text += " poisoned, ";
     }
@@ -1421,14 +1457,14 @@ std::string status_mut_abilities()
     if (you.rotting || you.species == SP_GHOUL)
         text += "rotting, ";
 
-    if (you.confusing_touch)
+    if (you.duration[DUR_CONFUSING_TOUCH])
         text += "confusing touch, ";
 
-    if (you.sure_blade)
+    if (you.duration[DUR_SURE_BLADE])
         text += "bonded with blade, ";
 
     int move_cost = (player_speed() * player_movement_speed()) / 10;
-    if ( you.slow )
+    if ( you.duration[DUR_SLOW] )
         move_cost *= 2;
 
     text +=   (move_cost <   8) ? "very quick, " :
@@ -1436,13 +1472,22 @@ std::string status_mut_abilities()
               (move_cost == 10) ? "" :
               (move_cost <  13) ? "slow, " : "";
 
-    if (you.slow && !you.haste)
+    if (you.duration[DUR_SLOW] && !you.duration[DUR_HASTE])
         text += "slowed, ";
-    else if (you.haste && !you.slow)
+    else if (you.duration[DUR_HASTE] && !you.duration[DUR_SLOW])
         text += "hasted, ";
-    else if (!you.haste && you.duration[DUR_SWIFTNESS])
+    else if (!you.duration[DUR_HASTE] && you.duration[DUR_SWIFTNESS])
         text += "swift, ";
 
+    if (you.disease
+        || you.species == SP_VAMPIRE && you.hunger_state < HS_HUNGRY)
+    {
+        text += "non-regenerating, ";
+    }
+
+    if (you.attribute[ATTR_HELD])
+        text += "held, ";
+
     const int mr = player_res_magic();
     snprintf(info, INFO_SIZE, "%s resistant to magic, ",
              (mr <  10) ? "not" :
@@ -1452,6 +1497,7 @@ std::string status_mut_abilities()
              (mr < 120) ? "very" :
              (mr < 140) ? "extremely" :
              "incredibly");
+             
     text += info;
 
     // character evaluates their ability to sneak around:
@@ -1475,7 +1521,13 @@ std::string status_mut_abilities()
       case TRAN_SPIDER:
           text += "\nYou are in spider-form.";
           break;
-      case TRAN_BLADE_HANDS:
+     case TRAN_BAT:
+          text += "\nYou are in ";
+          if (you.species == SP_VAMPIRE)
+              text += "vampire ";
+          text += "bat-form.";
+          break;
+     case TRAN_BLADE_HANDS:
           text += "\nYou have blades for hands.";
           break;
       case TRAN_STATUE:
@@ -1514,7 +1566,7 @@ std::string status_mut_abilities()
                          : "You feel confident with your ability to fight" );
     text += info;
 
-    if (you.deaths_door)
+    if (you.duration[DUR_DEATHS_DOOR])
         text += "\nYou are standing in death's doorway.";
 
     //----------------------------
@@ -1553,7 +1605,6 @@ std::string status_mut_abilities()
 
       case SP_GHOUL:
           text += "saprovore 3";
-          text += ", deterioriation";
           have_any = true;
           break;
 
@@ -1673,6 +1724,8 @@ std::string status_mut_abilities()
           text += "saprovore 1";
           have_any = true;
           break;
+      default:
+          break;
     }                           //end switch - innate abilities
 
     // a bit more stuff
@@ -1686,7 +1739,15 @@ std::string status_mut_abilities()
         have_any = true;
     }
 
-    for (unsigned i = 0; i < 100; i++)
+    if ( beogh_water_walk() )
+    {
+        if (have_any)
+            text += ", ";
+        text += "water walking";
+        have_any = true;
+    }
+
+    for (unsigned i = 0; i < NUM_MUTATIONS; i++)
     {
         if (!you.mutation[i])
             continue;
@@ -1913,7 +1974,7 @@ std::string status_mut_abilities()
             case MUT_DETERIORATION:
                 if (have_any)
                     text += ", ";
-                snprintf(info, INFO_SIZE, "deteriotation %d", level);
+                snprintf(info, INFO_SIZE, "deterioration %d", level);
                 text += info;
                 have_any = true;
                 break;
@@ -1934,14 +1995,14 @@ std::string status_mut_abilities()
             case MUT_FRAIL:
                 if (have_any)
                     text += ", ";
-                snprintf(info, INFO_SIZE, "-%d hp", level*10);
+                snprintf(info, INFO_SIZE, "-%d%% hp", level*10);
                 text += info;
                 have_any = true;
                 break;
             case MUT_ROBUST:
                 if (have_any)
                     text += ", ";
-                snprintf(info, INFO_SIZE, "+%d hp", level*10);
+                snprintf(info, INFO_SIZE, "+%d hp%%", level*10);
                 text += info;
                 have_any = true;
                 break;
@@ -2049,6 +2110,13 @@ std::string status_mut_abilities()
                 text += "hooves";
                 have_any = true;
                 break;
+            case MUT_FANGS:
+                if (have_any)
+                    text += ", ";
+                snprintf(info, INFO_SIZE, "sharp teeth %d", level);
+                text += info;
+                have_any = true;
+                break;
             case MUT_BREATHE_POISON:
                 if (have_any)
                     text += ", ";
