@@ -3,7 +3,7 @@
  *  Summary:    Implementations of some additional spells.
  *  Written by: Linley Henzell
  *
- *  Modified for Crawl Reference by $Author: dshaligram $ on $Date: 2007-04-05T16:46:35.024555Z $
+ *  Modified for Crawl Reference by $Author: ennewalker $ on $Date: 2007-11-23T02:25:44.246247Z $
  *
  *  Change History (most recent first):
  *
@@ -21,6 +21,7 @@
 #include <string.h>
 #include <stdlib.h>
 #include <stdio.h>
+#include <iostream>
 
 #include "externs.h"
 
@@ -33,32 +34,50 @@
 #include "it_use2.h"
 #include "itemname.h"
 #include "itemprop.h"
+#include "message.h"
 #include "misc.h"
 #include "monplace.h"
 #include "monstuff.h"
 #include "mon-util.h"
 #include "player.h"
+#include "randart.h"
 #include "skills2.h"
 #include "spells3.h"
 #include "spells4.h"
 #include "spl-util.h"
 #include "stuff.h"
+#include "terrain.h"
+#include "transfor.h"
+#include "traps.h"
 #include "view.h"
 
-int blink(void)
+static bool abyss_blocks_teleport(bool cblink)
 {
-    struct dist beam;
+    // Lugonu worshippers get their perks.
+    if (you.religion == GOD_LUGONU)
+        return (false);
+
+    // Controlled Blink (the spell) works quite reliably in the Abyss.
+    return (cblink? one_chance_in(3) : !one_chance_in(3));
+}
+
+int blink(int pow, bool high_level_controlled_blink)
+{
+    dist beam;
 
     // yes, there is a logic to this ordering {dlb}:
     if (scan_randarts(RAP_PREVENT_TELEPORTATION))
         mpr("You feel a weird sense of stasis.");
-    else if (you.level_type == LEVEL_ABYSS && !one_chance_in(3))
+    else if (you.level_type == LEVEL_ABYSS
+             && abyss_blocks_teleport(high_level_controlled_blink))
         mpr("The power of the Abyss keeps you in your place!");
-    else if (you.conf)
+    else if (you.duration[DUR_CONF])
         random_blink(false);
     else if (!allow_control_teleport(true))
     {
         mpr("A powerful magic interferes with your control of the blink.");
+        if (high_level_controlled_blink)
+            return (cast_semi_controlled_blink(pow));
         random_blink(false);
     }
     else
@@ -66,7 +85,7 @@ int blink(void)
         // query for location {dlb}:
         for (;;)
         {
-            direction(beam, DIR_TARGET, TARG_ANY, false, "Blink to where?");
+            direction(beam, DIR_TARGET, TARG_ANY, false, false, "Blink to where?");
 
             if (!beam.isValid || coord_def(beam.tx, beam.ty) == you.pos())
             {
@@ -102,8 +121,10 @@ int blink(void)
         }
         else
         {
-            you.x_pos = beam.tx;
-            you.y_pos = beam.ty;
+            // no longer held in net
+            clear_trapping_net();
+            
+            move_player_to_grid(beam.tx, beam.ty, false, true, true);
 
             // controlling teleport contaminates the player -- bwr
             contaminate_player( 1 );
@@ -138,7 +159,7 @@ void random_blink(bool allow_partial_control, bool override_abyss)
 
 #ifdef USE_SEMI_CONTROLLED_BLINK
     //jmf: add back control, but effect is cast_semi_controlled_blink(pow)
-    else if (player_control_teleport() && !you.conf
+    else if (player_control_teleport() && !you.duration[DUR_CONF]
              && allow_partial_control && allow_control_teleport())
     {
         mpr("You may select the general direction of your translocation.");
@@ -150,10 +171,12 @@ void random_blink(bool allow_partial_control, bool override_abyss)
     else
     {
         mpr("You blink.");
+        
+        // no longer held in net
+        clear_trapping_net();
 
         succ = true;
-        you.x_pos = tx;
-        you.y_pos = ty;
+        you.moveto(tx, ty);
 
         if (you.level_type == LEVEL_ABYSS)
         {
@@ -193,7 +216,7 @@ int cast_fire_storm(int powc, bolt &beam)
     beam.ench_power = powc;     // used for radius
     beam.name = "great blast of fire";
     beam.hit = 20 + powc / 10;
-    beam.damage = calc_dice( 6, 15 + powc );
+    beam.damage = calc_dice( 9, 20 + powc );
 
     explosion( beam );
     mpr("A raging storm of fire appears!");
@@ -361,7 +384,6 @@ void identify(int power)
 {
     int id_used = 1;
     int item_slot;
-    char str_pass[ ITEMNAME_SIZE ];
 
     // scrolls of identify *may* produce "extra" identifications {dlb}:
     if (power == -1 && one_chance_in(5))
@@ -369,23 +391,31 @@ void identify(int power)
 
     do
     {
-        item_slot = prompt_invent_item( "Identify which item?", MT_INVSELECT,
-                                        -1, true, true, false );
+        item_slot = prompt_invent_item( "Identify which item?", MT_INVLIST,
+                                        OSEL_UNIDENT, true, true, false );
         if (item_slot == PROMPT_ABORT)
         {
             canned_msg( MSG_OK );
             return;
         }
 
-        set_ident_type( you.inv[item_slot].base_type, 
-                        you.inv[item_slot].sub_type, ID_KNOWN_TYPE );
+        item_def& item(you.inv[item_slot]);
 
-        set_ident_flags( you.inv[item_slot], ISFLAG_IDENT_MASK );
-        remove_empty_inscription(you.inv[item_slot]);
+        if ( fully_identified(item) )
+        {
+            mpr("Choose an unidentified item, or Esc to abort.");
+            if ( Options.auto_list )
+                more();
+            continue;
+        }
+
+        if ( !is_artefact(item) )
+            set_ident_type( item.base_type, item.sub_type, ID_KNOWN_TYPE );
+
+        set_ident_flags( item, ISFLAG_IDENT_MASK );
         
         // output identified item
-        in_name( item_slot, DESC_INVENTORY_EQUIP, str_pass );
-        mpr( str_pass );
+        mpr(item.name(DESC_INVENTORY_EQUIP).c_str());
         if (item_slot == you.equip[EQ_WEAPON])
             you.wield_change = true;
 
@@ -397,7 +427,8 @@ void identify(int power)
     while (id_used > 0);
 }                               // end identify()
 
-int conjure_flame(int pow)
+// return whether the spell was actually cast
+bool conjure_flame(int pow)
 {
     struct dist spelld;
 
@@ -413,12 +444,12 @@ int conjure_flame(int pow)
             done_first_message = true;
         }
 
-        direction( spelld, DIR_TARGET, TARG_ENEMY );
+        direction( spelld, DIR_TARGET, TARG_ENEMY, false, false );
 
         if (!spelld.isValid)
         {
             canned_msg(MSG_OK);
-            return (-1);
+            return false;
         }
 
         if (!see_grid(spelld.tx, spelld.ty))
@@ -426,16 +457,33 @@ int conjure_flame(int pow)
             mpr("You can't see that place!");
             continue;
         }
+        
+        if (spelld.tx == you.x_pos && spelld.ty == you.y_pos)
+        {
+            mpr("You can't place the cloud here!");
+            continue;
+        }
 
-        if (grid_is_solid(grd[ spelld.tx ][ spelld.ty ])
-            || mgrd[ spelld.tx ][ spelld.ty ] != NON_MONSTER 
-            || env.cgrid[ spelld.tx ][ spelld.ty ] != EMPTY_CLOUD)
+        const int cloud = env.cgrid[spelld.tx][spelld.ty];
+
+        if (grid_is_solid(grd[ spelld.tx ][ spelld.ty ]) ||
+            mgrd[ spelld.tx ][ spelld.ty ] != NON_MONSTER ||
+            (cloud != EMPTY_CLOUD && env.cloud[cloud].type != CLOUD_FIRE))
         {
             mpr( "There's already something there!" );
             continue;
         }
+        else if ( cloud != EMPTY_CLOUD )
+        {
+            // reinforce the cloud - but not too much
+            mpr( "The fire roars with new energy!" );
+            const int extra_dur = 2 + std::min(random2(pow) / 2, 20);
+            env.cloud[cloud].decay += extra_dur * 5;
+            env.cloud[cloud].whose = KC_YOU;
+            return true;
+        }
 
-        break;    
+        break;
     }
 
     int durat = 5 + (random2(pow) / 2) + (random2(pow) / 2);
@@ -444,8 +492,8 @@ int conjure_flame(int pow)
         durat = 23;
 
     place_cloud( CLOUD_FIRE, spelld.tx, spelld.ty, durat, KC_YOU );
-    return (1);
-}                               // end cast_conjure_flame()
+    return true;
+}
 
 int stinking_cloud( int pow, bolt &beem )
 {
@@ -469,18 +517,18 @@ int stinking_cloud( int pow, bolt &beem )
     return (1);
 }                               // end stinking_cloud()
 
-int cast_big_c(int pow, int cty, kill_category whose, bolt &beam)
+int cast_big_c(int pow, cloud_type cty, kill_category whose, bolt &beam)
 {
     big_cloud( cty, whose,
                beam.target_x, beam.target_y, pow, 8 + random2(3) );
     return (1);
 }                               // end cast_big_c()
 
-void big_cloud(int cloud_type, kill_category whose,
+void big_cloud(cloud_type cl_type, kill_category whose,
                int cl_x, int cl_y, int pow, int size)
 {
     apply_area_cloud(make_a_normal_cloud, cl_x, cl_y, pow, size,
-                     cloud_type, whose);
+                     cl_type, whose);
 }                               // end big_cloud()
 
 static int healing_spell( int healed )
@@ -517,15 +565,15 @@ static int healing_spell( int healed )
 
     if (heal_monster(monster, healed, false))
     {
-        strcpy(info, "You heal ");
-        strcat(info, ptr_monam( monster, DESC_NOCAP_THE ));
-        strcat(info, ".");
-        mpr(info);
+        mprf("You heal %s.", monster->name(DESC_NOCAP_THE).c_str());
 
         if (monster->hit_points == monster->max_hit_points)
             simple_monster_message( monster, " is completely healed." );
         else
-            print_wounds(monster);
+        {
+            const monsters *mons = static_cast<const monsters*>(monster);
+            print_wounds(mons);
+        }
     }
     else
     {
@@ -591,7 +639,7 @@ bool cast_revivification(int power)
 
 void cast_cure_poison(int mabil)
 {
-    if (!you.poisoning)
+    if (!you.duration[DUR_POISONING])
         canned_msg(MSG_NOTHING_HAPPENS);
     else
         reduce_poison_player( 2 + random2(mabil) + random2(3) );
@@ -603,12 +651,12 @@ void purification(void)
 {
     mpr("You feel purified!");
 
-    you.poisoning = 0;
+    you.duration[DUR_POISONING] = 0;
     you.rotting = 0;
-    you.conf = 0;
-    you.slow = 0;
+    you.duration[DUR_CONF] = 0;
+    you.duration[DUR_SLOW] = 0;
     you.disease = 0;
-    you.paralysis = 0;          // can't currently happen -- bwr
+    you.duration[DUR_PARALYSIS] = 0;          // can't currently happen -- bwr
 }                               // end purification()
 
 int allowed_deaths_door_hp(void)
@@ -625,7 +673,7 @@ void cast_deaths_door(int pow)
 {
     if (you.is_undead)
         mpr("You're already dead!");
-    else if (you.deaths_door)
+    else if (you.duration[DUR_DEATHS_DOOR])
         mpr("Your appeal for an extension has been denied.");
     else
     {
@@ -635,10 +683,10 @@ void cast_deaths_door(int pow)
         set_hp( allowed_deaths_door_hp(), false );
         deflate_hp( you.hp_max, false );
 
-        you.deaths_door = 10 + random2avg(13, 3) + (random2(pow) / 10);
+        you.duration[DUR_DEATHS_DOOR] = 10 + random2avg(13, 3) + (random2(pow) / 10);
 
-        if (you.deaths_door > 25)
-            you.deaths_door = 23 + random2(5);
+        if (you.duration[DUR_DEATHS_DOOR] > 25)
+            you.duration[DUR_DEATHS_DOOR] = 23 + random2(5);
     }
 
     return;
@@ -651,6 +699,9 @@ void abjuration(int pow)
 
     mpr("Send 'em back where they came from!");
 
+    // Scale power into something comparable to summon lifetime.
+    const int abjdur = pow * 12;
+
     for (int ab = 0; ab < MAX_MONSTERS; ab++)
     {
         monster = &menv[ab];
@@ -663,10 +714,15 @@ void abjuration(int pow)
 
         mon_enchant abj = monster->get_ench(ENCH_ABJ);
         if (abj.ench != ENCH_NONE)
-            monster->lose_ench_levels(abj, 1 + (random2(pow) / 8));
-        
-        if (monster->has_ench(ENCH_ABJ))
-            simple_monster_message(monster, " shudders.");
+        {
+            const int sockage = std::max(fuzz_value(abjdur, 60, 30), 40);
+#ifdef DEBUG_DIAGNOSTICS
+            mprf(MSGCH_DIAGNOSTICS, "%s abj: dur: %d, abj: %d",
+                 monster->name(DESC_PLAIN).c_str(), abj.duration, sockage);
+#endif
+            if (!monster->lose_ench_duration(abj, sockage))
+                simple_monster_message(monster, " shudders.");
+        }
     }
 }                               // end abjuration()
 
@@ -675,83 +731,28 @@ void abjuration(int pow)
 // that's why we reduce levitation to 2, so that the player has a chance
 // to stop insta-death... sure the others could lead to death, but that's
 // not as direct as falling into deep water) -- bwr
-void antimagic( void )
+void antimagic()
 {
-    if (you.haste)
-        you.haste = 1;
+    duration_type dur_list[] = {
+        DUR_INVIS, DUR_CONF, DUR_PARALYSIS, DUR_SLOW, DUR_HASTE,
+        DUR_MIGHT, DUR_FIRE_SHIELD, DUR_ICY_ARMOUR, DUR_REPEL_MISSILES,
+        DUR_REGENERATION, DUR_SWIFTNESS, DUR_STONEMAIL, DUR_CONTROL_TELEPORT,
+        DUR_TRANSFORMATION, DUR_DEATH_CHANNEL, DUR_DEFLECT_MISSILES,
+        DUR_FORESCRY, DUR_SEE_INVISIBLE, DUR_WEAPON_BRAND, DUR_SILENCE,
+        DUR_CONDENSATION_SHIELD, DUR_STONESKIN, DUR_BARGAIN,
+        DUR_INSULATION, DUR_RESIST_POISON, DUR_RESIST_FIRE, DUR_RESIST_COLD,
+        DUR_SLAYING, DUR_STEALTH, DUR_MAGIC_SHIELD
+    };
+        
+    if (you.duration[DUR_LEVITATION] > 2 && you.duration[DUR_LEVITATION] < 100)
+        you.duration[DUR_LEVITATION] = 2;
 
-    if (you.slow)
-        you.slow = 1;
-
-    if (you.paralysis)
-        you.paralysis = 1;
-
-    if (you.conf)
-        you.conf = 1;
-
-    if (you.might)
-        you.might = 1;
-
-    if (you.levitation > 2)
-        you.levitation = 2;
-
-    if (you.invis)
-        you.invis = 1;
-
-    if (you.duration[DUR_WEAPON_BRAND])
-        you.duration[DUR_WEAPON_BRAND] = 1;
-
-    if (you.duration[DUR_ICY_ARMOUR])
-        you.duration[DUR_ICY_ARMOUR] = 1;
-
-    if (you.duration[DUR_REPEL_MISSILES])
-        you.duration[DUR_REPEL_MISSILES] = 1;
-
-    if (you.duration[DUR_REGENERATION])
-        you.duration[DUR_REGENERATION] = 1;
-
-    if (you.duration[DUR_DEFLECT_MISSILES])
-        you.duration[DUR_DEFLECT_MISSILES] = 1;
-
-    if (you.fire_shield)
-        you.fire_shield = 1;
-
-    if (you.duration[DUR_SWIFTNESS])
-        you.duration[DUR_SWIFTNESS] = 1;
-
-    if (you.duration[DUR_INSULATION])
-        you.duration[DUR_INSULATION] = 1;
-
-    if (you.duration[DUR_STONEMAIL])
-        you.duration[DUR_STONEMAIL] = 1;
-
-    if (you.duration[DUR_CONTROLLED_FLIGHT])
+    if (you.duration[DUR_CONTROLLED_FLIGHT] < 100 &&
+        you.duration[DUR_CONTROLLED_FLIGHT] > 1) // don't mess with permaflight
         you.duration[DUR_CONTROLLED_FLIGHT] = 1;
-
-    if (you.duration[DUR_CONTROL_TELEPORT])
-        you.duration[DUR_CONTROL_TELEPORT] = 1;
-
-    if (you.duration[DUR_RESIST_POISON])
-        you.duration[DUR_RESIST_POISON] = 1;
-
-    if (you.duration[DUR_TRANSFORMATION])
-        you.duration[DUR_TRANSFORMATION] = 1;
-
-    //jmf: added following
-    if (you.duration[DUR_STONESKIN])
-        you.duration[DUR_STONESKIN] = 1;
-
-    if (you.duration[DUR_FORESCRY])
-        you.duration[DUR_FORESCRY] = 1;
-
-    if (you.duration[DUR_SEE_INVISIBLE])
-        you.duration[DUR_SEE_INVISIBLE] = 1;
-
-    if (you.duration[DUR_SILENCE])
-        you.duration[DUR_SILENCE] = 1;
-
-    if (you.duration[DUR_CONDENSATION_SHIELD])
-        you.duration[DUR_CONDENSATION_SHIELD] = 1;
+    for ( unsigned int i = 0; i < ARRAYSIZE(dur_list); ++i )
+        if ( you.duration[dur_list[i]] > 1 )
+            you.duration[dur_list[i]] = 1;
 
     contaminate_player( -1 * (1+random2(5)));
 }                               // end antimagic()
@@ -760,33 +761,33 @@ void extension(int pow)
 {
     int contamination = random2(2);
 
-    if (you.haste)
+    if (you.duration[DUR_HASTE])
     {
         potion_effect(POT_SPEED, pow);
         contamination++;
     }
 
-    if (you.slow)
+    if (you.duration[DUR_SLOW])
         potion_effect(POT_SLOWING, pow);
 
 #if 0
-    if (you.paralysis)
+    if (you.duration[DUR_PARALYSIS])
         potion_effect(POT_PARALYSIS, pow);  // how did you cast extension?
 
-    if (you.conf)
+    if (you.duration[DUR_CONF])
         potion_effect(POT_CONFUSION, pow);  // how did you cast extension?
 #endif
 
-    if (you.might)
+    if (you.duration[DUR_MIGHT])
     {
         potion_effect(POT_MIGHT, pow);
         contamination++;  
     }
 
-    if (you.levitation)
+    if (you.duration[DUR_LEVITATION] && !you.duration[DUR_CONTROLLED_FLIGHT])
         potion_effect(POT_LEVITATION, pow);
 
-    if (you.invis)
+    if (you.duration[DUR_INVIS])
     {
         potion_effect(POT_INVISIBILITY, pow);
         contamination++;
@@ -804,12 +805,12 @@ void extension(int pow)
     if (you.duration[DUR_DEFLECT_MISSILES])
         deflection(pow);
 
-    if (you.fire_shield)
+    if (you.duration[DUR_FIRE_SHIELD])
     {
-        you.fire_shield += random2(pow / 20);
+        you.duration[DUR_FIRE_SHIELD] += random2(pow / 20);
 
-        if (you.fire_shield > 50)
-            you.fire_shield = 50;
+        if (you.duration[DUR_FIRE_SHIELD] > 50)
+            you.duration[DUR_FIRE_SHIELD] = 50;
 
         mpr("Your ring of flames roars with new vigour!");
     }
@@ -1005,25 +1006,29 @@ void cast_fly(int power)
 {
     int dur_change = 25 + random2(power) + random2(power);
 
-    if (!player_is_levitating())
-        mpr("You fly up into the air.");
-    else
-        mpr("You feel more buoyant.");
+    const bool was_levitating = player_is_airborne();
 
-    if (you.levitation + dur_change > 100)
-        you.levitation = 100;
+    if (you.duration[DUR_LEVITATION] + dur_change > 100)
+        you.duration[DUR_LEVITATION] = 100;
     else
-        you.levitation += dur_change;
+        you.duration[DUR_LEVITATION] += dur_change;
 
     if (you.duration[DUR_CONTROLLED_FLIGHT] + dur_change > 100)
         you.duration[DUR_CONTROLLED_FLIGHT] = 100;
     else
         you.duration[DUR_CONTROLLED_FLIGHT] += dur_change;
 
-    // duration[DUR_CONTROLLED_FLIGHT] makes the game think player 
-    // wears an amulet of controlled flight
-
     burden_change();
+
+    if (!was_levitating)
+    {
+        if (you.light_flight())
+            mpr("You swoop lightly up into the air.");
+        else
+            mpr("You fly up into the air.");
+    }
+    else
+        mpr("You feel more buoyant.");
 }
 
 void cast_insulation(int power)
@@ -1064,10 +1069,10 @@ void cast_teleport_control(int power)
 
 void cast_ring_of_flames(int power)
 {
-    you.fire_shield += 5 + (power / 10) + (random2(power) / 5);
+    you.duration[DUR_FIRE_SHIELD] += 5 + (power / 10) + (random2(power) / 5);
 
-    if (you.fire_shield > 50)
-        you.fire_shield = 50;
+    if (you.duration[DUR_FIRE_SHIELD] > 50)
+        you.duration[DUR_FIRE_SHIELD] = 50;
 
     mpr("The air around you leaps into flame!");
 
@@ -1076,15 +1081,14 @@ void cast_ring_of_flames(int power)
 
 void cast_confusing_touch(int power)
 {
-    snprintf( info, INFO_SIZE, "Your %s begin to glow %s.",
-              your_hand(true), (you.confusing_touch ? "brighter" : "red") ); 
+    msg::stream << "Your " << your_hand(true) << " begin to glow "
+                << (you.duration[DUR_CONFUSING_TOUCH] ? "brighter" : "red")
+                << "." << std::endl;
 
-    mpr( info ); 
+    you.duration[DUR_CONFUSING_TOUCH] += 5 + (random2(power) / 5);
 
-    you.confusing_touch += 5 + (random2(power) / 5);
-
-    if (you.confusing_touch > 50)
-        you.confusing_touch = 50;
+    if (you.duration[DUR_CONFUSING_TOUCH] > 50)
+        you.duration[DUR_CONFUSING_TOUCH] = 50;
 
 }                               // end cast_confusing_touch()
 
@@ -1101,15 +1105,15 @@ bool cast_sure_blade(int power)
     }
     else
     {
-        if (!you.sure_blade)
+        if (!you.duration[DUR_SURE_BLADE])
             mpr("You become one with your weapon.");
-        else if (you.sure_blade < 25)
+        else if (you.duration[DUR_SURE_BLADE] < 25)
             mpr("Your bond becomes stronger.");
 
-        you.sure_blade += 8 + (random2(power) / 10);
+        you.duration[DUR_SURE_BLADE] += 8 + (random2(power) / 10);
 
-        if (you.sure_blade > 25)
-            you.sure_blade = 25;
+        if (you.duration[DUR_SURE_BLADE] > 25)
+            you.duration[DUR_SURE_BLADE] = 25;
 
         success = true;
     }
@@ -1119,9 +1123,9 @@ bool cast_sure_blade(int power)
 
 void manage_fire_shield(void)
 {
-    you.fire_shield--;
+    you.duration[DUR_FIRE_SHIELD]--;
 
-    if (!you.fire_shield)
+    if (!you.duration[DUR_FIRE_SHIELD])
         return;
 
     char stx = 0, sty = 0;
