@@ -3,7 +3,7 @@
  *  Summary:    Implementations of some additional spells.
  *  Written by: Linley Henzell
  *
- *  Modified for Crawl Reference by $Author: dshaligram $ on $Date: 2007-05-11T15:19:19.299392Z $
+ *  Modified for Crawl Reference by $Author: dshaligram $ on $Date: 2007-12-02T13:23:51.271663Z $
  *
  *  Change History (most recent first):
  *
@@ -23,6 +23,7 @@
 
 #include <stdio.h>
 #include <string.h>
+#include <sstream>
 
 #include "externs.h"
 
@@ -35,6 +36,7 @@
 #include "itemname.h"
 #include "itemprop.h"
 #include "items.h"
+#include "message.h"
 #include "misc.h"
 #include "monplace.h"
 #include "monstuff.h"
@@ -46,10 +48,18 @@
 #include "spells4.h"
 #include "spl-cast.h"
 #include "stuff.h"
+#include "terrain.h"
+#include "traps.h"
 #include "view.h"
 
-int raise_corpse( int corps, int corx, int cory, int corps_beh,
-                  int corps_hit, int actual );
+static int raise_corpse( int corps, int corx, int cory, beh_type corps_beh,
+                         int corps_hit, int actual );
+
+static bool is_animatable_corpse(const item_def& item)
+{
+    return (item.base_type == OBJ_CORPSES &&
+            mons_zombie_size(item.plus) != Z_NOZOMBIE);
+}
 
 unsigned char detect_traps( int pow )
 {
@@ -76,7 +86,7 @@ unsigned char detect_traps( int pow )
                 traps_found++;
 
                 grd[ etx ][ ety ] = trap_category( env.trap[count_x].type );
-                set_envmap_char(etx, ety, get_magicmap_char(grd[etx][ety]));
+                set_envmap_obj(etx, ety, grd[etx][ety]);
                 set_terrain_mapped(etx, ety);
             }
         }
@@ -104,7 +114,7 @@ unsigned char detect_items( int pow )
 
             if (igrd[i][j] != NON_ITEM)
             {
-                set_envmap_char(i, j, get_magicmap_char(DNGN_ITEM_DETECTED));
+                set_envmap_obj(i, j, DNGN_ITEM_DETECTED);
                 set_envmap_detected_item(i, j);
             }
         }
@@ -159,7 +169,7 @@ static void mark_detected_creature(int gridx, int gridy, const monsters *mon,
         }
     }
 
-    set_envmap_char(gridx, gridy, mons_char(mon->type));
+    set_envmap_obj(gridx, gridy, mon->type + DNGN_START_OF_MONSTERS);
     set_envmap_detected_mons(gridx, gridy);
 }
 
@@ -176,7 +186,7 @@ unsigned char detect_creatures( int pow )
 
     // Clear the map so detect creatures is more useful and the detection
     // fuzz is harder to analyse by averaging.
-    clear_map();
+    clear_map(false);
 
     mpr("You detect creatures!");
 
@@ -267,14 +277,12 @@ int corpse_rot(int power)
                         place_cloud(CLOUD_MIASMA, adx, ady,
                                     4 + random2avg(16, 3), KC_YOU);
 
-                        goto out_of_raise;
+                        // Don't look for more corpses here
+                        break;
                     }
                     hrg = mitm[objl].link;
                     objl = hrg;
                 }
-
-              out_of_raise:
-                objl = 1;
             }
         }
     }
@@ -287,17 +295,17 @@ int corpse_rot(int power)
     return 0;
 }                               // end corpse_rot()
 
-int animate_dead( int power, int corps_beh, int corps_hit, int actual )
+int animate_dead( actor *caster, int power, beh_type corps_beh,
+                  int corps_hit, int actual )
 {
     UNUSED( power );
+    static env_show_grid losgrid;
 
-    int adx = 0;
-    int ady = 0;
-
-    int minx = you.x_pos - 6;
-    int maxx = you.x_pos + 7;
-    int miny = you.y_pos - 6;
-    int maxy = you.y_pos + 7;
+    const coord_def c(caster->pos());
+    int minx = c.x - 6;
+    int maxx = c.x + 7;
+    int miny = c.y - 6;
+    int maxy = c.y + 7;
     int xinc = 1;
     int yinc = 1;
 
@@ -305,36 +313,42 @@ int animate_dead( int power, int corps_beh, int corps_hit, int actual )
 
     if (coinflip())
     {
-        minx = you.x_pos + 6;
-        maxx = you.x_pos - 7;
+        minx = c.x + 6;
+        maxx = c.x - 7;
         xinc = -1;
     }
 
     if (coinflip())
     {
-        miny = you.y_pos + 6;
-        maxy = you.y_pos - 7;
+        miny = c.y + 6;
+        maxy = c.y - 7;
         yinc = -1;
     }
 
-    for (adx = minx; adx != maxx; adx += xinc)
+    if (caster != &you)
+        losight(losgrid, grd, c.x, c.y);
+
+    env_show_grid &los(caster == &you? env.show : losgrid);
+
+    coord_def a;
+    for (a.x = minx; a.x != maxx; a.x += xinc)
     {
-        for (ady = miny; ady != maxy; ady += yinc)
+        for (a.y = miny; a.y != maxy; a.y += yinc)
         {
-            if (see_grid(adx, ady))
+            if (in_bounds(a) && see_grid(los, c, a))
             {
-                if (igrd[adx][ady] != NON_ITEM)
+                if (igrd(a) != NON_ITEM)
                 {
-                    int objl = igrd[adx][ady];
+                    int objl = igrd(a);
                     int hrg = 0;
 
                     // This searches all the items on the ground for a corpse
                     while (objl != NON_ITEM)
                     {
-                        if (mitm[objl].base_type == OBJ_CORPSES
+                        if (is_animatable_corpse(mitm[objl])
                             && !is_being_butchered(mitm[objl]))
                         {
-                            number_raised += raise_corpse(objl, adx, ady,
+                            number_raised += raise_corpse(objl, a.x, a.y,
                                                 corps_beh, corps_hit, actual);
                             break;
                         }
@@ -360,35 +374,31 @@ int animate_dead( int power, int corps_beh, int corps_hit, int actual )
     return number_raised;
 }                               // end animate_dead()
 
-int animate_a_corpse( int axps, int ayps, int corps_beh, int corps_hit,
+int animate_a_corpse( int axps, int ayps, beh_type corps_beh, int corps_hit,
                       int class_allowed )
-{
-    if (igrd[axps][ayps] == NON_ITEM)
-        return 0;
-
+{    
+    int rc = 0;
     int objl = igrd[axps][ayps];
     // This searches all the items on the ground for a corpse
     while (objl != NON_ITEM)
     {
-        if (mitm[objl].base_type != OBJ_CORPSES
-             || (class_allowed == CORPSE_SKELETON
-                && mitm[objl].sub_type != CORPSE_SKELETON))
+        const item_def& item = mitm[objl];
+        if (is_animatable_corpse(item) &&
+            (class_allowed == CORPSE_BODY || item.sub_type == CORPSE_SKELETON))
         {
-            objl = mitm[objl].link;
-            continue;
+            rc = raise_corpse(objl, axps, ayps, corps_beh, corps_hit, 1);
+            if ( rc )
+                mpr("The dead are walking!");
+            break;
         }
-
-        if (raise_corpse(objl, axps, ayps,
-                         corps_beh, corps_hit, 1 ) > 0)
-            mpr("The dead are walking!");
-        break;
+        objl = item.link;
     }
 
-    return 0;
+    return rc;
 }                               // end animate_a_corpse()
 
 int raise_corpse( int corps, int corx, int cory, 
-                  int corps_beh, int corps_hit, int actual )
+                  beh_type corps_beh, int corps_hit, int actual )
 {
     int returnVal = 1;
 
@@ -421,7 +431,7 @@ int raise_corpse( int corps, int corx, int cory,
     return returnVal;
 }                               // end raise_corpse()
 
-void cast_twisted(int power, int corps_beh, int corps_hit)
+void cast_twisted(int power, beh_type corps_beh, int corps_hit)
 {
     int total_mass = 0;
     int num_corpses = 0;
@@ -459,8 +469,7 @@ void cast_twisted(int power, int corps_beh, int corps_hit)
     }
 
 #if DEBUG_DIAGNOSTICS
-    snprintf( info, INFO_SIZE, "Mass for abomination: %d", total_mass);
-    mpr( info, MSGCH_DIAGNOSTICS );
+    mprf(MSGCH_DIAGNOSTICS, "Mass for abomination: %d", total_mass);
 #endif
 
     // This is what the old statement pretty much boils down to, 
@@ -471,15 +480,15 @@ void cast_twisted(int power, int corps_beh, int corps_hit)
     total_mass += roll_dice( 20, power );
 
 #if DEBUG_DIAGNOSTICS
-    snprintf( info, INFO_SIZE, "Mass including power bonus: %d", total_mass);
-    mpr( info, MSGCH_DIAGNOSTICS );
+    mprf(MSGCH_DIAGNOSTICS, "Mass including power bonus: %d", total_mass);
 #endif
 
     if (total_mass < 400 + roll_dice( 2, 500 )
         || num_corpses < (coinflip() ? 3 : 2))
     {
         mpr("The spell fails.");
-        mpr("The corpses collapse into a pulpy mess.");
+        mprf("The corpse%s collapse%s into a pulpy mess.",
+             num_corpses > 1 ? "s": "", num_corpses > 1 ? "": "s");
         return;
     }
 
@@ -500,6 +509,9 @@ void cast_twisted(int power, int corps_beh, int corps_hit)
         mpr("The corpses collapse into a pulpy mess.");
     else
     {
+        // This was probably intended, but it's really boring. (jpeg)
+        // Use menv[mon].number instead (set in create_monster)
+//        menv[mon].colour = colour;
         mpr("The heap of corpses melds into an agglomeration of writhing flesh!");
         if (type_resurr == MONS_ABOMINATION_LARGE)
         {
@@ -546,21 +558,19 @@ bool brand_weapon(int which_brand, int power)
         return false;
     }
 
-    char str_pass[ ITEMNAME_SIZE ];
-    in_name( wpn, DESC_CAP_YOUR, str_pass );
-    strcpy( info, str_pass );
+    std::string msg = you.inv[wpn].name(DESC_CAP_YOUR);
 
     const int wpn_type = get_vorpal_type(you.inv[wpn]);
 
     switch (which_brand)        // use SPECIAL_WEAPONS here?
     {
     case SPWPN_FLAMING:
-        strcat(info, " bursts into flame!");
+        msg += " bursts into flame!";
         duration_affected = 7;
         break;
 
     case SPWPN_FREEZING:
-        strcat(info, " glows blue.");
+        msg += " glows blue.";
         duration_affected = 7;
         break;
 
@@ -568,12 +578,12 @@ bool brand_weapon(int which_brand, int power)
         if (wpn_type == DVORP_CRUSHING)
             return false;
 
-        strcat(info, " starts dripping with poison.");
+        msg += " starts dripping with poison.";
         duration_affected = 15;
         break;
 
     case SPWPN_DRAINING:
-        strcat(info, " crackles with unholy energy.");
+        msg += " crackles with unholy energy.";
         duration_affected = 12;
         break;
 
@@ -581,22 +591,22 @@ bool brand_weapon(int which_brand, int power)
         if (wpn_type != DVORP_SLICING)
             return false;
 
-        strcat(info, " glows silver and looks extremely sharp.");
+        msg += " glows silver and looks extremely sharp.";
         duration_affected = 10;
         break;
 
     case SPWPN_DISTORTION:      //jmf: added for Warp Weapon
-        strcat(info, " seems to ");
+        msg += " seems to ";
 
         temp_rand = random2(6);
-        strcat(info, (temp_rand == 0) ? "twist" :
-                     (temp_rand == 1) ? "bend" :
-                     (temp_rand == 2) ? "vibrate" :
-                     (temp_rand == 3) ? "flex" :
-                     (temp_rand == 4) ? "wobble"
-                                      : "twang");
+        msg += ((temp_rand == 0) ? "twist" :
+                (temp_rand == 1) ? "bend" :
+                (temp_rand == 2) ? "vibrate" :
+                (temp_rand == 3) ? "flex" :
+                (temp_rand == 4) ? "wobble"
+                                 : "twang");
 
-        strcat( info, coinflip() ? " oddly." : " strangely." );
+        msg += (coinflip() ? " oddly." : " strangely.");
         duration_affected = 5;
 
         // [dshaligram] Clamping power to 2.
@@ -614,7 +624,7 @@ bool brand_weapon(int which_brand, int power)
     case SPWPN_PAIN:
         // well, in theory, we could be silenced, but then how are
         // we casting the brand spell?
-        strcat(info, " shrieks in agony.");
+        msg += " shrieks in agony.";
         noisy( 15, you.x_pos, you.y_pos );
         duration_affected = 8;
         break;
@@ -624,14 +634,14 @@ bool brand_weapon(int which_brand, int power)
             return false;
 
         which_brand = SPWPN_VORPAL;
-        strcat(info, " glows silver and feels heavier.");
+        msg += " glows silver and feels heavier.";
         duration_affected = 7;
         break;
     }
 
     set_item_ego_type( you.inv[wpn], OBJ_WEAPONS, which_brand );
 
-    mpr(info);
+    mpr(msg.c_str());
     you.wield_change = true;
 
     int dur_change = duration_affected + roll_dice( 2, power );
@@ -661,12 +671,11 @@ bool restore_stat(unsigned char which_stat, bool suppress_msg)
     }
 
     // the real function begins here {dlb}:
-    char *ptr_stat = 0;         // NULL {dlb}
-    char *ptr_stat_max = 0;     // NULL {dlb}
-    char *ptr_redraw = 0;       // NULL {dlb}
+    char *ptr_stat = NULL;
+    char *ptr_stat_max = NULL;
+    bool *ptr_redraw = NULL;
 
-    if (!suppress_msg)
-        strcpy(info, "You feel your ");
+    std::string msg = "You feel your ";
 
     if (which_stat == STAT_RANDOM)
         which_stat = random2(NUM_STATS);
@@ -674,8 +683,7 @@ bool restore_stat(unsigned char which_stat, bool suppress_msg)
     switch (which_stat)
     {
     case STAT_STRENGTH:
-        if (!suppress_msg)
-            strcat(info, "strength");
+        msg += "strength";
 
         ptr_stat = &you.strength;
         ptr_stat_max = &you.max_strength;
@@ -683,8 +691,7 @@ bool restore_stat(unsigned char which_stat, bool suppress_msg)
         break;
 
     case STAT_DEXTERITY:
-        if (!suppress_msg)
-            strcat(info, "dexterity");
+        msg += "dexterity";
 
         ptr_stat = &you.dex;
         ptr_stat_max = &you.max_dex;
@@ -692,8 +699,7 @@ bool restore_stat(unsigned char which_stat, bool suppress_msg)
         break;
 
     case STAT_INTELLIGENCE:
-        if (!suppress_msg)
-            strcat(info, "intelligence");
+        msg += "intelligence";
 
         ptr_stat = &you.intel;
         ptr_stat_max = &you.max_intel;
@@ -703,11 +709,9 @@ bool restore_stat(unsigned char which_stat, bool suppress_msg)
 
     if (*ptr_stat < *ptr_stat_max)
     {
-        if (!suppress_msg)
-        {
-            strcat(info, " returning.");
-            mpr(info);
-        }
+        msg += " returning.";
+        if ( !suppress_msg )
+            mpr(msg.c_str());
 
         *ptr_stat = *ptr_stat_max;
         *ptr_redraw = 1;
@@ -739,7 +743,8 @@ void turn_undead(int pow)
         {
             if (check_mons_resist_magic( monster, pow ))
             {
-                simple_monster_message( monster, " resists." );
+                simple_monster_message( monster, mons_immune_magic(monster) ?
+                                        " is unaffected." : " resists." );
                 continue;
             }
 
@@ -799,6 +804,59 @@ void holy_word(int pow, bool silent)
     }                           // end "for tu"
 }                               // end holy_word()
 
+typedef std::pair<const monsters*,int> counted_monster;
+typedef std::vector<counted_monster> counted_monster_list;
+static void record_monster_by_name(counted_monster_list &list,
+                                   const monsters *mons)
+{
+    const std::string name = mons->name(DESC_PLAIN);
+    for (counted_monster_list::iterator i = list.begin(); i != list.end(); ++i)
+    {
+        if (i->first->name(DESC_PLAIN) == name)
+        {
+            i->second++;
+            return;
+        }
+    }
+    list.push_back( counted_monster(mons, 1) );
+}
+
+static int monster_count(const counted_monster_list &list)
+{
+    int nmons = 0;
+    for (counted_monster_list::const_iterator i = list.begin();
+         i != list.end(); ++i)
+    {
+        nmons += i->second;
+    }
+    return (nmons);
+}
+
+static std::string describe_monsters(const counted_monster_list &list)
+{
+    std::ostringstream out;
+
+    description_level_type desc = DESC_CAP_THE;
+    for (counted_monster_list::const_iterator i = list.begin();
+         i != list.end(); desc = DESC_NOCAP_THE)
+    {
+        const counted_monster &cm(*i);
+        if (i != list.begin())
+        {
+            ++i;
+            out << (i == list.end()? " and " : ", ");
+        }
+        else
+            ++i;
+
+        const std::string name =
+            cm.second > 1? pluralise(cm.first->name(desc))
+            : cm.first->name(desc);
+        out << name;
+    }
+    return (out.str());
+}
+
 // poisonous light passes right through invisible players
 // and monsters, and so, they are unaffected by this spell --
 // assumes only you can cast this spell (or would want to)
@@ -814,7 +872,7 @@ void cast_toxic_radiance(void)
     mesclr();
 
     // determine whether the player is hit by the radiance: {dlb}
-    if (you.invis)
+    if (you.duration[DUR_INVIS])
     {
         mpr("The light passes straight through your body.");
     }
@@ -824,32 +882,52 @@ void cast_toxic_radiance(void)
         poison_player(2);
     }
 
+    counted_monster_list affected_monsters;
     // determine which monsters are hit by the radiance: {dlb}
     for (int toxy = 0; toxy < MAX_MONSTERS; toxy++)
     {
         monster = &menv[toxy];
 
-        if (monster->type != -1 && mons_near(monster))
+        if (monster->type != -1 && mons_near(monster)
+            && !monster->submerged())
         {
+            // Monsters affected by corona are still invisible in that
+            // radiation passes through them without affecting them. Therefore,
+            // this check should not be !monster->invisible().
             if (!monster->has_ench(ENCH_INVIS))
             {
-                poison_monster(monster, KC_YOU);
+                bool affected =
+                    poison_monster(monster, KC_YOU, 1, false, false);
 
-                if (coinflip()) // 50-50 chance for a "double hit" {dlb}
-                    poison_monster(monster, KC_YOU);
+                if (coinflip() && poison_monster(monster, KC_YOU, false, false))
+                    affected = true;
 
+                if (affected)
+                    record_monster_by_name(affected_monsters, monster);
             }
             else if (player_see_invis())
             {
                 // message player re:"miss" where appropriate {dlb}
-                strcpy(info, "The light passes through ");
-                strcat(info, ptr_monam( monster, DESC_NOCAP_THE ));
-                strcat(info, ".");
-                mpr(info);
+                mprf("The light passes through %s.",
+                     monster->name(DESC_NOCAP_THE).c_str());
             }
         }
     }
-}                               // end cast_toxic_radiance()
+
+    if (!affected_monsters.empty())
+    {
+        const std::string message =
+            make_stringf("%s %s poisoned.",
+                         describe_monsters(affected_monsters).c_str(),
+                         monster_count(affected_monsters) == 1? "is" : "are");
+        if (static_cast<int>(message.length()) < get_number_of_cols() - 2)
+            mpr(message.c_str());
+        else
+            // Exclamation mark to suggest that a lot of creatures were
+            // affected.
+            mpr("The monsters around you are poisoned!");
+    }
+}
 
 void cast_refrigeration(int pow)
 {
@@ -894,10 +972,8 @@ void cast_refrigeration(int pow)
 
         if (mons_near(monster))
         {
-            snprintf( info, INFO_SIZE, "You freeze %s.", 
-                      ptr_monam( monster, DESC_NOCAP_THE ));
-
-            mpr(info);
+            mprf("You freeze %s.",
+                 monster->name(DESC_NOCAP_THE).c_str());
 
             hurted = roll_dice( dam_dice );
             hurted = mons_adjust_flavoured( monster, beam, hurted );
@@ -910,7 +986,8 @@ void cast_refrigeration(int pow)
                     monster_die(monster, KILL_YOU, 0);
                 else
                 {
-                    print_wounds(monster);
+                    const monsters *mons = static_cast<const monsters*>(monster);
+                    print_wounds(mons);
 
                     //jmf: "slow snakes" finally available
                     if (mons_class_flag( monster->type, M_COLD_BLOOD ) && coinflip())
@@ -954,10 +1031,8 @@ void drain_life(int pow)
 
         if (mons_near(monster))
         {
-            strcpy(info, "You draw life from ");
-            strcat(info, ptr_monam( monster, DESC_NOCAP_THE ));
-            strcat(info, ".");
-            mpr(info);
+            mprf("You draw life from %s.",
+                 monster->name(DESC_NOCAP_THE).c_str());
 
             hurted = 3 + random2(7) + random2(pow);
 
@@ -1032,10 +1107,8 @@ int vampiric_drain(int pow, const dist &vmove)
 
     hurt_monster(monster, inflicted);
 
-    strcpy(info, "You feel life coursing from ");
-    strcat(info, ptr_monam( monster, DESC_NOCAP_THE ));
-    strcat(info, " into your body!");
-    mpr(info);
+    mprf("You feel life coursing from %s into your body!",
+         monster->name(DESC_NOCAP_THE).c_str());
 
     print_wounds(monster);
 
@@ -1086,21 +1159,17 @@ char burn_freeze(int pow, char flavour)
 
     monster = &menv[mgr];
 
-    strcpy(info, "You ");
-    strcat(info, (flavour == BEAM_FIRE)        ? "burn" :
-                 (flavour == BEAM_COLD)        ? "freeze" :
-                 (flavour == BEAM_MISSILE)     ? "crush" :
-                 (flavour == BEAM_ELECTRICITY) ? "zap"
-                                               : "______");
-
-    strcat(info, " ");
-    strcat(info, ptr_monam( monster, DESC_NOCAP_THE ));
-    strcat(info, ".");
-    mpr(info);
+    mprf("You %s %s.",
+         (flavour == BEAM_FIRE)        ? "burn" :
+         (flavour == BEAM_COLD)        ? "freeze" :
+         (flavour == BEAM_MISSILE)     ? "crush" :
+         (flavour == BEAM_ELECTRICITY) ? "zap"
+                                       : "______",
+         monster->name(DESC_NOCAP_THE).c_str());
 
     int hurted = roll_dice( 1, 3 + pow / 3 ); 
 
-    struct bolt beam;
+    bolt beam;
 
     beam.flavour = flavour;
 
@@ -1110,7 +1179,9 @@ char burn_freeze(int pow, char flavour)
     if (hurted)
     {
         if (mons_friendly( monster ))
-            did_god_conduct( DID_ATTACK_FRIEND, 5 );
+        {
+            did_god_conduct( DID_ATTACK_FRIEND, 5, monster );
+        }
         
         if (mons_holiness( monster ) == MH_HOLY)
             did_god_conduct( DID_ATTACK_HOLY, monster->hit_dice );
@@ -1166,7 +1237,7 @@ int summon_elemental(int pow, int restricted_type,
     int targ_x;
     int targ_y;
 
-    int numsc = cap_int(2 + (random2(pow) / 5), 6);
+    int numsc = std::min(2 + (random2(pow) / 5), 6);
 
     for (;;) 
     {
@@ -1186,7 +1257,15 @@ int summon_elemental(int pow, int restricted_type,
         targ_y = you.y_pos + dir_y;
 
         if (mgrd[ targ_x ][ targ_y ] != NON_MONSTER)
-            mpr("Not there!");
+        {
+            if ( player_monster_visible(&menv[mgrd[targ_x][targ_y]]) )
+                mpr("There's something there already!");
+            else
+            {
+                mpr("Something seems to disrupt your summoning.");
+                return 0;
+            }
+        }
         else if (dir_x == 0 && dir_y == 0)
             mpr("You can't summon an elemental from yourself!");
         else 
@@ -1274,7 +1353,7 @@ int summon_elemental(int pow, int restricted_type,
 //jmf: beefed up higher-level casting of this (formerly lame) spell
 void summon_small_mammals(int pow)
 {
-    int thing_called = MONS_PROGRAM_BUG;        // error trapping{dlb}
+    monster_type thing_called = MONS_PROGRAM_BUG; // error trapping{dlb}
 
     int pow_spent = 0;
     int pow_left = pow + 1;
@@ -1294,33 +1373,16 @@ void summon_small_mammals(int pow)
 
         switch (pow_spent)
         {
-        case 75:
-        case 74:
-        case 38:
+        case 75: case 74: case 38:
             thing_called = MONS_ORANGE_RAT;
             break;
 
-        case 65:
-        case 64:
-        case 63:
-        case 27:
-        case 26:
-        case 25:
+        case 65: case 64: case 63: case 27: case 26: case 25:
             thing_called = MONS_GREEN_RAT;
             break;
 
-        case 57:
-        case 56:
-        case 55:
-        case 54:
-        case 53:
-        case 52:
-        case 20:
-        case 18:
-        case 16:
-        case 14:
-        case 12:
-        case 10:
+        case 57: case 56: case 55: case 54: case 53: case 52:
+        case 20: case 18: case 16: case 14: case 12: case 10:
             thing_called = coinflip() ? MONS_QUOKKA : MONS_GREY_RAT;
             break;
 
@@ -1334,6 +1396,49 @@ void summon_small_mammals(int pow)
     }
 }                               // end summon_small_mammals()
 
+void summon_animals(int pow)
+{
+    // maybe we should just generate a Lair monster instead? (and
+    // guarantee that it is mobile)
+    const monster_type animals[] = {
+        MONS_BUMBLEBEE, MONS_WAR_DOG, MONS_SHEEP, MONS_YAK,
+        MONS_HOG, MONS_SOLDIER_ANT, MONS_WOLF,
+        MONS_GRIZZLY_BEAR, MONS_POLAR_BEAR, MONS_BLACK_BEAR, 
+        MONS_GIANT_SNAIL, MONS_BORING_BEETLE, MONS_GILA_MONSTER,
+        MONS_KOMODO_DRAGON, MONS_SPINY_FROG, MONS_HOUND
+    };
+
+    int num_so_far = 0;
+    int power_left = pow + 1;
+
+    const bool varied = coinflip();
+    monster_type mon_chosen = MONS_PROGRAM_BUG;
+
+    while ( power_left >= 0 && num_so_far < 8 )
+    {
+        // pick a random monster and subtract its cost
+        if ( varied || num_so_far == 0 )
+            mon_chosen = RANDOM_ELEMENT(animals);
+
+        const int power_cost = mons_power(mon_chosen) * 3;
+
+        // Allow a certain degree of overuse, but not too much.
+        // Guarantee at least two summons.
+        if ( power_cost >= power_left * 2 && num_so_far >= 2 )
+            break;
+
+        power_left -= power_cost;
+        num_so_far++;
+
+        if ( random2(pow) < 5 ) // unfriendly
+            create_monster( mon_chosen, 4, BEH_HOSTILE,
+                            you.x_pos, you.y_pos, MHITYOU, 250 );
+        else
+            create_monster( mon_chosen, 4, BEH_FRIENDLY,
+                            you.x_pos, you.y_pos, you.pet_target, 250 );
+    }
+}
+
 void summon_scorpions(int pow)
 {
     int numsc = 1 + random2(pow) / 10 + random2(pow) / 10;
@@ -1364,8 +1469,8 @@ void summon_scorpions(int pow)
 
 void summon_ice_beast_etc(int pow, int ibc, bool divine_gift)
 {
-    int numsc = cap_int(2 + (random2(pow) / 4), 6);
-    int beha = divine_gift? BEH_GOD_GIFT : BEH_FRIENDLY;
+    int numsc = std::min(2 + (random2(pow) / 4), 6);
+    beh_type beha = divine_gift? BEH_GOD_GIFT : BEH_FRIENDLY;
 
     switch (ibc)
     {
@@ -1407,6 +1512,82 @@ void summon_ice_beast_etc(int pow, int ibc, bool divine_gift)
     create_monster( ibc, numsc, beha, you.x_pos, you.y_pos, MHITYOU, 250 );
 }                               // end summon_ice_beast_etc()
 
+// Trog sends some fighting buddies for his followers
+void summon_berserker()
+{
+    beh_type beha = BEH_GOD_GIFT;
+    // randomize a bit to make things more interesting
+    int pow = you.piety + random2(30) - random2(30);
+    int numsc = std::min(2 + (random2(pow) / 4), 6);
+
+    monster_type mon = MONS_TROLL;
+    
+    if (pow <= 120) // 100-120 -> 20
+    {
+        // bears
+        if (coinflip())
+            mon = MONS_BLACK_BEAR;
+        else
+            mon = MONS_GRIZZLY_BEAR;
+    }
+    else if (pow <= 150) // 121-150 -> 30
+    {
+        // ogres
+        if (one_chance_in(3))
+            mon = MONS_TWO_HEADED_OGRE;
+        else
+            mon = MONS_OGRE;
+    }
+    else if (pow <= 180) // 151-180 -> 30
+    {
+        // trolls
+        switch(random2(8))
+        {
+        case 0:  // 12.5%
+            mon = MONS_DEEP_TROLL;
+            break;
+        case 1:
+        case 2:  // 25%
+            mon = MONS_IRON_TROLL;
+            break;
+        case 3:
+        case 4:  // 25%
+            mon = MONS_ROCK_TROLL;
+            break;
+        default: // 37.5%
+            mon = MONS_TROLL;
+            break;
+        }
+    }
+    else // 181-200 -> 20
+    {
+        // giants
+        if (coinflip())
+            mon = MONS_HILL_GIANT;
+        else
+            mon = MONS_STONE_GIANT;
+    }
+
+    int mons = create_monster( mon, numsc, beha, you.x_pos, you.y_pos,
+                               MHITYOU, 250 );
+    
+    if (mons != -1)
+    {
+        monsters *summon = &menv[mons];
+        summon->go_berserk(false);
+        mon_enchant berserk = summon->get_ench(ENCH_BERSERK);
+        mon_enchant abj = summon->get_ench(ENCH_ABJ);
+
+        // Let Trog gifts berserk longer, and set abj timeout ==
+        // berserk timeout.
+        berserk.duration = berserk.duration * 3 / 2;
+        berserk.maxduration = berserk.duration;
+        abj.duration = abj.maxduration = berserk.duration;
+        summon->update_ench(berserk);
+        summon->update_ench(abj);
+    }
+}   // end summon_berserker()
+
 bool summon_swarm( int pow, bool unfriendly, bool god_gift )
 {
     int thing_called = MONS_PROGRAM_BUG;        // error trapping {dlb}
@@ -1466,7 +1647,7 @@ bool summon_swarm( int pow, bool unfriendly, bool god_gift )
             break;
         }                       // end switch
 
-        int behaviour = BEH_HOSTILE;  // default to unfriendly
+        beh_type behaviour = BEH_HOSTILE;  // default to unfriendly
 
         // Note: friendly, non-god_gift means spell.
         if (god_gift)
@@ -1571,10 +1752,8 @@ void summon_things( int pow )
             numsc--;
         }
 
-        snprintf( info, INFO_SIZE, "Some Thing%s answered your call!", 
-                  (numsc + big_things > 1) ? "s" : "" );
-
-        mpr(info);
+        mprf("Some Thing%s answered your call!",
+             (numsc + big_things > 1) ? "s" : "" );
     }
 
     return;
