@@ -3,7 +3,7 @@
  *  Summary:    Implementations of some additional spells.
  *  Written by: Linley Henzell
  *
- *  Modified for Crawl Reference by $Author: dshaligram $ on $Date: 2007-06-06T19:40:56.965493Z $
+ *  Modified for Crawl Reference by $Author: ennewalker $ on $Date: 2007-11-23T02:25:44.246247Z $
  *
  *  Change History (most recent first):
  *
@@ -18,6 +18,7 @@
 #include <ctype.h>
 #include <stdio.h>
 #include <string.h>
+#include <iostream>
 
 #include "externs.h"
 
@@ -28,22 +29,27 @@
 #include "direct.h"
 #include "debug.h"
 #include "delay.h"
+#include "food.h"
 #include "itemname.h"
 #include "itemprop.h"
 #include "items.h"
 #include "it_use2.h"
+#include "message.h"
 #include "misc.h"
 #include "monplace.h"
 #include "mon-pick.h"
 #include "monstuff.h"
 #include "mon-util.h"
+#include "place.h"
 #include "player.h"
 #include "randart.h"
+#include "religion.h"
 #include "spells1.h"
 #include "spells4.h"
 #include "spl-cast.h"
 #include "spl-util.h"
 #include "stuff.h"
+#include "traps.h"
 #include "view.h"
 
 static bool monster_on_level(int monster);
@@ -62,7 +68,7 @@ bool cast_selective_amnesia(bool force)
         {
             mpr( "Forget which spell ([?*] list [ESC] exit)? ", MSGCH_PROMPT );
 
-            keyin = (unsigned char) get_ch();
+            keyin = get_ch();
 
             if (keyin == ESCAPE)
                 return (false);        // early return {dlb}
@@ -82,7 +88,7 @@ bool cast_selective_amnesia(bool force)
         }
 
         // actual handling begins here {dlb}:
-        const int spell = get_spell_by_letter( keyin );
+        const spell_type spell = get_spell_by_letter( keyin );
         const int slot  = get_spell_slot_by_letter( keyin );
 
         if (spell == SPELL_NO_SPELL)
@@ -201,16 +207,20 @@ int cast_smiting(int power, dist &beam)
     {
         monster = &menv[mgrd[beam.tx][beam.ty]];
 
-        mprf("You smite %s!", ptr_monam( monster, DESC_NOCAP_THE ));
+        mprf("You smite %s!", monster->name(DESC_NOCAP_THE).c_str());
 
         // Maxes out at around 40 damage at 27 Invocations, which is plenty
-        // in my book (the old max damage was around 70, which seems excessive.
+        // in my book (the old max damage was around 70, which seems excessive)
         hurt_monster(monster, 7 + (random2(power) * 33 / 191));
+        behaviour_event( monster, ME_ANNOY, MHITYOU );
 
         if (monster->hit_points < 1)
             monster_die(monster, KILL_YOU, 0);
         else
-            print_wounds(monster);
+        {
+            const monsters *mons = static_cast<const monsters*>(monster);
+            print_wounds(mons);
+        }
 
         success = true;
     }
@@ -234,7 +244,7 @@ int airstrike(int power, dist &beam)
         monster = &menv[mgrd[beam.tx][beam.ty]];
 
         mprf("The air twists around and strikes %s!",
-             ptr_monam( monster, DESC_NOCAP_THE ));
+             monster->name(DESC_NOCAP_THE).c_str());
 
         hurted = 8 + random2( random2(4) + (random2(power) / 6)
                               + (random2(power) / 7) );
@@ -302,7 +312,7 @@ void sublimation(int power)
         || you.inv[you.equip[EQ_WEAPON]].base_type != OBJ_FOOD
         || you.inv[you.equip[EQ_WEAPON]].sub_type != FOOD_CHUNK)
     {
-        if (you.deaths_door)
+        if (you.duration[DUR_DEATHS_DOOR])
         {
             mpr( "A conflicting enchantment prevents the spell from "
                  "coming into effect." );
@@ -395,10 +405,9 @@ void simulacrum(int power)
 
         if (summoned)
         {
-            strcpy( info, (summoned == 1) ? "An icy figure forms "
-                                          : "Some icy figures form " );
-            strcat( info, "before you!" );
-            mpr( info );
+            mprf("%s before you!",
+                 (summoned == 1) ? "An icy figure forms "
+                                 : "Some icy figures form ");
         }
         else
             mpr( "You feel cold for a second." );
@@ -408,16 +417,15 @@ void simulacrum(int power)
         mpr( "You need to wield a piece of raw flesh for this spell "
              "to be effective!" );
     }
-}                               // end sublimation()
+}
 
 void dancing_weapon(int pow, bool force_hostile)
 {
-    int numsc = cap_int(2 + (random2(pow) / 5), 6);
-    char str_pass[ ITEMNAME_SIZE ];
+    int numsc = std::min(2 + (random2(pow) / 5), 6);
 
-    int i;
     int summs = 0;
-    char behavi = BEH_FRIENDLY;
+    beh_type behavi = BEH_FRIENDLY;
+    bool failed = false;
 
     const int wpn = you.equip[EQ_WEAPON];
 
@@ -427,31 +435,40 @@ void dancing_weapon(int pow, bool force_hostile)
         || is_range_weapon( you.inv[wpn] )
         || is_fixed_artefact( you.inv[wpn] ))
     {
-        goto failed_spell;
+        failed = true;
     }
 
     // See if we can get an mitm for the dancing weapon:
-    i = get_item_slot();
+    int i = get_item_slot();
     if (i == NON_ITEM)
-        goto failed_spell;
+        failed = true;
 
-    // cursed weapons become hostile
-    if (item_cursed( you.inv[wpn] ) || force_hostile)
-        behavi = BEH_HOSTILE; 
-
-    summs = create_monster( MONS_DANCING_WEAPON, numsc, behavi, 
-                            you.x_pos, you.y_pos, you.pet_target, 1 );
-
-    if (summs < 0)
+    if ( !failed )
     {
-        // must delete the item before failing!
-        mitm[i].base_type = OBJ_UNASSIGNED;
-        mitm[i].quantity = 0;
-        goto failed_spell;
+
+        // cursed weapons become hostile
+        if (item_cursed( you.inv[wpn] ) || force_hostile)
+            behavi = BEH_HOSTILE; 
+        
+        summs = create_monster( MONS_DANCING_WEAPON, numsc, behavi, 
+                                you.x_pos, you.y_pos, you.pet_target, 1 );
+        if ( summs == -1 )
+            failed = true;            
+    }
+
+    if ( failed )
+    {
+        destroy_item(i);
+        if ( wpn != -1 )
+            mpr("Your weapon vibrates crazily for a second.");
+        else
+            msg::stream << "Your " << your_hand(true) << " twitch."
+                        << std::endl;
+        return;
     }
 
     // We are successful:
-    unwield_item( wpn );                        // remove wield effects
+    unwield_item(); // unwield the weapon (including removing wield effects)
 
     // copy item (done here after any wield effects are removed)
     mitm[i] = you.inv[wpn];
@@ -463,24 +480,13 @@ void dancing_weapon(int pow, bool force_hostile)
     // Mark the weapon as thrown so we'll autograb it when the tango's done.
     mitm[i].flags |= ISFLAG_THROWN;
 
-    in_name( wpn, DESC_CAP_YOUR, str_pass );
-    strcpy( info, str_pass );
-    strcat( info, " dances into the air!" );
-    mpr( info );
+    mprf("%s dances into the air!", you.inv[wpn].name(DESC_CAP_YOUR).c_str());
 
     you.inv[ wpn ].quantity = 0;
-    you.equip[EQ_WEAPON] = -1;
 
     menv[summs].inv[MSLOT_WEAPON] = i;
     menv[summs].colour = mitm[i].colour;
-
-    return;
-
-failed_spell:
-    if ( wpn != -1 )
-        mpr("Your weapon vibrates crazily for a second.");
-    else
-        mprf(MSGCH_PLAIN, "Your %s twitch.", your_hand(true));        
+    burden_change();
 }                               // end dancing_weapon()
 
 static bool monster_on_level(int monster)
@@ -494,6 +500,20 @@ static bool monster_on_level(int monster)
     return false;
 }                               // end monster_on_level()
 
+// XXX: Relies on RUNE_xxx == BRANCH_xxx. See rune_type in enum.h.
+static bool player_has_rune(branch_type branch)
+{
+    for (int i = 0; i < ENDOFPACK; i++)
+        if (is_valid_item( you.inv[i] )
+            && you.inv[i].base_type == OBJ_MISCELLANY
+            && you.inv[i].sub_type == MISC_RUNE_OF_ZOT
+            && you.inv[i].plus == branch)
+        {
+            return (true);
+        }
+    return (false);
+}
+
 //
 // This function returns true if the player can use controlled
 // teleport here.
@@ -509,23 +529,17 @@ bool allow_control_teleport( bool silent )
         switch (you.where_are_you)
         {
         case BRANCH_TOMB:
-            // The tomb is a laid out maze, it'd be a shame if the player
-            // just teleports through any of it... so we only allow 
-            // teleport once they have the rune.
-            ret = false;
-            for (int i = 0; i < ENDOFPACK; i++)
-            {
-                if (is_valid_item( you.inv[i] )
-                    && you.inv[i].base_type == OBJ_MISCELLANY
-                    && you.inv[i].sub_type == MISC_RUNE_OF_ZOT
-                    && you.inv[i].plus == BRANCH_TOMB)
-                {
-                    ret = true;
-                    break;
-                }
-            }
+            ret = player_has_rune(you.where_are_you);
             break;
 
+        case BRANCH_COCYTUS:
+        case BRANCH_DIS:
+        case BRANCH_TARTARUS:
+        case BRANCH_GEHENNA:
+            if (player_branch_depth() == branches[you.where_are_you].depth)
+                ret = player_has_rune(you.where_are_you);
+            break;
+            
         case BRANCH_SLIME_PITS:
             // Cannot teleport into the slime pit vaults until
             // royal jelly is gone.
@@ -551,7 +565,7 @@ bool allow_control_teleport( bool silent )
         case BRANCH_HALL_OF_ZOT:
             // Cannot control teleport until the Orb is picked up
             if (player_branch_depth() == branches[BRANCH_HALL_OF_ZOT].depth
-                && you.char_direction != DIR_ASCENDING)
+                && you.char_direction != GDT_ASCENDING)
             {
                 ret = false;
             }
@@ -594,16 +608,16 @@ void you_teleport(void)
     return;
 }                               // end you_teleport()
 
-void you_teleport2( bool allow_control, bool new_abyss_area )
+static bool teleport_player( bool allow_control, bool new_abyss_area )
 {
-    bool is_controlled = (allow_control && !you.conf
+    bool is_controlled = (allow_control && !you.duration[DUR_CONF]
                           && player_control_teleport()
                           && allow_control_teleport());
 
     if (scan_randarts(RAP_PREVENT_TELEPORTATION))
     {
         mpr("You feel a strange sense of stasis.");
-        return;
+        return false;
     }
 
     // after this point, we're guaranteed to teleport. Kill the appropriate
@@ -620,7 +634,7 @@ void you_teleport2( bool allow_control, bool new_abyss_area )
     {
         abyss_teleport( new_abyss_area );
         you.pet_target = MHITNOT;
-        return;
+        return true;
     }
 
     FixedVector < int, 2 > plox;
@@ -639,8 +653,7 @@ void you_teleport2( bool allow_control, bool new_abyss_area )
         redraw_screen();
 
 #if DEBUG_DIAGNOSTICS
-        snprintf( info, INFO_SIZE, "Target square (%d,%d)", plox[0], plox[1] );
-        mpr( info, MSGCH_DIAGNOSTICS );
+        mprf(MSGCH_DIAGNOSTICS, "Target square (%d,%d)", plox[0], plox[1] );
 #endif
 
         plox[0] += random2(3) - 1;
@@ -660,14 +673,17 @@ void you_teleport2( bool allow_control, bool new_abyss_area )
         }
 
 #if DEBUG_DIAGNOSTICS
-        snprintf( info, INFO_SIZE, "Scattered target square (%d,%d)", plox[0], plox[1] );
-        mpr( info, MSGCH_DIAGNOSTICS );
+        mprf(MSGCH_DIAGNOSTICS,
+             "Scattered target square (%d,%d)", plox[0], plox[1] );
 #endif
 
         if (is_controlled)
         {
-            you.x_pos = plox[0];
-            you.y_pos = plox[1];
+            // no longer held in net
+            if (plox[0] != you.x_pos || plox[1] != you.y_pos)
+                clear_trapping_net();
+
+            you.moveto(plox[0], plox[1]);
 
             if ((grd[you.x_pos][you.y_pos] != DNGN_FLOOR
                     && grd[you.x_pos][you.y_pos] != DNGN_SHALLOW_WATER)
@@ -686,47 +702,68 @@ void you_teleport2( bool allow_control, bool new_abyss_area )
 
     if (!is_controlled)
     {
-        mpr("Your surroundings suddenly seem different.");
+        int newx, newy;
 
         do
         {
-            you.x_pos = 5 + random2( GXM - 10 );
-            you.y_pos = 5 + random2( GYM - 10 );
+            newx = 5 + random2( GXM - 10 );
+            newy = 5 + random2( GYM - 10 );
         }
-        while ((grd[you.x_pos][you.y_pos] != DNGN_FLOOR
-                   && grd[you.x_pos][you.y_pos] != DNGN_SHALLOW_WATER)
-               || mgrd[you.x_pos][you.y_pos] != NON_MONSTER
-               || env.cgrid[you.x_pos][you.y_pos] != EMPTY_CLOUD);
+        while ((grd[newx][newy] != DNGN_FLOOR
+                && grd[newx][newy] != DNGN_SHALLOW_WATER)
+               || mgrd[newx][newy] != NON_MONSTER
+               || env.cgrid[newx][newy] != EMPTY_CLOUD);
+               
+        // no longer held in net
+        if (newx != you.x_pos || newy != you.y_pos)
+            clear_trapping_net();
+
+        if ( newx == you.x_pos && newy == you.y_pos )
+            mpr("Your surroundings flicker for a moment.");
+        else if ( see_grid(newx, newy) )
+            mpr("Your surroundings seem slightly different.");
+        else
+            mpr("Your surroundings suddenly seem different.");
+
+        you.x_pos = newx;
+        you.y_pos = newy;
+
+        // Necessary to update the view centre.
+        you.moveto(you.pos());
     }
-}                               // end you_teleport()
 
-bool entomb(void)
+    return !is_controlled;
+}
+
+void you_teleport_now( bool allow_control, bool new_abyss_area )
 {
-    int loopy = 0;              // general purpose loop variable {dlb}
-    bool proceed = false;       // loop management varaiable {dlb}
-    int which_trap = 0;         // used in clearing out certain traps {dlb}
-    char srx = 0, sry = 0;
-    char number_built = 0;
+    const bool randtele = teleport_player(allow_control, new_abyss_area);
+    // Xom is amused by uncontrolled teleports that land you in a
+    // dangerous place.
+    if (randtele && player_in_a_dangerous_place())
+        xom_is_stimulated(255);
+}
 
-    FixedVector < unsigned char, 7 > safe_to_overwrite;
+bool entomb(int powc)
+{
+    int number_built = 0;
 
-    // hack - passing chars through '...' promotes them to ints, which
-    // barfs under gcc in fixvec.h.  So don't.
-    safe_to_overwrite[0] = DNGN_FLOOR;
-    safe_to_overwrite[1] = DNGN_SHALLOW_WATER;
-    safe_to_overwrite[2] = DNGN_OPEN_DOOR;
-    safe_to_overwrite[3] = DNGN_TRAP_MECHANICAL;
-    safe_to_overwrite[4] = DNGN_TRAP_MAGICAL;
-    safe_to_overwrite[5] = DNGN_TRAP_III;
-    safe_to_overwrite[6] = DNGN_UNDISCOVERED_TRAP;
+    const dungeon_feature_type safe_to_overwrite[] = {
+        DNGN_FLOOR, DNGN_SHALLOW_WATER, DNGN_OPEN_DOOR,
+        DNGN_TRAP_MECHANICAL, DNGN_TRAP_MAGICAL, DNGN_TRAP_III,
+        DNGN_UNDISCOVERED_TRAP,
+        DNGN_FLOOR_SPECIAL
+    };
 
+    if ( powc > 95 )
+        powc = 95;
+    if ( powc < 25 )
+        powc = 25;
 
-    for (srx = you.x_pos - 1; srx < you.x_pos + 2; srx++)
+    for (int srx = you.x_pos - 1; srx < you.x_pos + 2; srx++)
     {
-        for (sry = you.y_pos - 1; sry < you.y_pos + 2; sry++)
+        for (int sry = you.y_pos - 1; sry < you.y_pos + 2; sry++)
         {
-            proceed = false;
-
             // tile already occupied by monster or yourself {dlb}:
             if (mgrd[srx][sry] != NON_MONSTER
                     || (srx == you.x_pos && sry == you.y_pos))
@@ -734,10 +771,13 @@ bool entomb(void)
                 continue;
             }
 
-            // the break here affects innermost containing loop {dlb}:
-            for (loopy = 0; loopy < 7; loopy++)
+            if ( random2(100) > powc )
+                continue;
+
+            bool proceed = false;
+            for (unsigned int i = 0; i < ARRAYSIZE(safe_to_overwrite); ++i)
             {
-                if (grd[srx][sry] == safe_to_overwrite[loopy])
+                if (grd[srx][sry] == safe_to_overwrite[i])
                 {
                     proceed = true;
                     break;
@@ -783,6 +823,7 @@ bool entomb(void)
                 delete_cloud( env.cgrid[srx][sry] );
 
             // mechanical traps are destroyed {dlb}:
+            int which_trap;
             if ((which_trap = trap_at_xy(srx, sry)) != -1)
             {
                 if (trap_category(env.trap[which_trap].type)
@@ -811,7 +852,6 @@ bool entomb(void)
 void cast_poison_ammo(void)
 {
     const int ammo = you.equip[EQ_WEAPON];
-    char str_pass[ ITEMNAME_SIZE ];
 
     if (ammo == -1
         || you.inv[ammo].base_type != OBJ_MISSILES
@@ -823,14 +863,11 @@ void cast_poison_ammo(void)
         return;
     }
 
+    const char *old_desc = you.inv[ammo].name(DESC_CAP_YOUR).c_str();
     if (set_item_ego_type( you.inv[ammo], OBJ_MISSILES, SPMSL_POISONED ))
     {
-        in_name(ammo, DESC_CAP_YOUR, str_pass);
-        strcpy(info, str_pass);
-        strcat(info, (you.inv[ammo].quantity == 1) ? " is" : " are");
-        strcat(info, " covered in a thin film of poison.");
-        mpr(info);
-
+        mprf("%s %s covered in a thin film of poison.", old_desc,
+             (you.inv[ammo].quantity == 1) ? "is" : "are");
         you.wield_change = true;
     }
     else
@@ -854,8 +891,7 @@ bool project_noise(void)
     redraw_screen();
 
 #if DEBUG_DIAGNOSTICS
-    snprintf( info, INFO_SIZE, "Target square (%d,%d)", plox[0], plox[1] );
-    mpr( info, MSGCH_DIAGNOSTICS );
+    mprf(MSGCH_DIAGNOSTICS, "Target square (%d,%d)", plox[0], plox[1] );
 #endif
 
     if (!silenced( plox[0], plox[1] ))
@@ -871,14 +907,11 @@ bool project_noise(void)
 
         if (!silenced( you.x_pos, you.y_pos ))
         {
-            if (!success)
-                mpr("You hear a dull thud.", MSGCH_SOUND);
+            if (success)
+                mprf(MSGCH_SOUND, "You hear a %svoice call your name.",
+                     (see_grid( plox[0], plox[1] ) ? "distant " : "") );
             else
-            {
-                snprintf( info, INFO_SIZE, "You hear a %svoice call your name.",
-                          (see_grid( plox[0], plox[1] ) ? "distant " : "") );
-                mpr( info , MSGCH_SOUND );
-            }
+                mprf(MSGCH_SOUND, "You hear a dull thud.");
         }
     }
 
@@ -889,6 +922,7 @@ bool project_noise(void)
    Type recalled:
    0 = anything
    1 = undead only (Kiku religion ability)
+   2 = orcs only (Beogh religion ability)
  */
 bool recall(char type_recalled)
 {
@@ -924,22 +958,23 @@ bool recall(char type_recalled)
         if (!monster_habitable_grid(monster, DNGN_FLOOR))
             continue;
 
-        if (type_recalled == 1)
+        if (type_recalled == 1) // undead
         {
-            /* abomin created by twisted res, although it gets others too */
-            if ( !((monster->type == MONS_ABOMINATION_SMALL
-                            || monster->type == MONS_ABOMINATION_LARGE)
-                        && (monster->colour == BROWN
-                            || monster->colour == RED
-                            || monster->colour == LIGHTRED)) )
+            if (monster->type != MONS_REAPER
+                && mons_holiness(monster) != MH_UNDEAD)
             {
-                if (monster->type != MONS_REAPER
-                        && mons_holiness(monster) != MH_UNDEAD)
-                {
-                    continue;
-                }
+                continue;
             }
         }
+        else if (type_recalled == 2) // Beogh
+        {
+            if (mons_species(monster->type) != MONS_ORC)
+                continue;
+                
+            // does not include charmed orcs
+            if (monster->attitude != ATT_FRIENDLY)
+                continue;
+        }
 
         if (empty_surrounds(you.x_pos, you.y_pos, DNGN_FLOOR, 3, false, empty))
         {
@@ -991,7 +1026,7 @@ int portal(void)
 
         for (;;)
         {
-            keyi = (unsigned char) get_ch();
+            keyi = get_ch();
 
             if (keyi == '<')
             {
@@ -1027,7 +1062,7 @@ int portal(void)
 
         for (;;)
         {
-            keyi = (unsigned char) get_ch();
+            keyi = get_ch();
 
             if (keyi == 'x')
             {
@@ -1058,7 +1093,6 @@ int portal(void)
         you.your_level = target_level - 1;
 
         down_stairs( old_level, DNGN_STONE_STAIRS_DOWN_I );
-        untag_followers();
     }
 
     return (1);
