@@ -4,7 +4,7 @@
  *              other neglected areas of Crawl magic ;^)
  *  Written by: Copyleft Josh Fishman 1999-2000, All Rights Preserved
  *
- *  Modified for Crawl Reference by $Author: dshaligram $ on $Date: 2007-06-05T16:54:50.325619Z $
+ *  Modified for Crawl Reference by $Author: ennewalker $ on $Date: 2007-11-23T02:25:44.246247Z $
  *
  *  Change History (most recent first):
  *
@@ -15,6 +15,7 @@
 #include "AppHdr.h"
 
 #include <string>
+#include <iostream>
 #include <stdio.h>
 
 #include "externs.h"
@@ -29,11 +30,13 @@
 #include "dungeon.h"
 #include "effects.h"
 #include "it_use2.h"
+#include "item_use.h"
 #include "itemname.h"
 #include "itemprop.h"
 #include "items.h"
 #include "invent.h"
-#include "misc.h"
+#include "makeitem.h"
+#include "message.h"
 #include "monplace.h"
 #include "monstuff.h"
 #include "mon-util.h"
@@ -48,6 +51,9 @@
 #include "spl-cast.h"
 #include "spl-util.h"
 #include "stuff.h"
+#include "terrain.h"
+#include "transfor.h"
+#include "traps.h"
 #include "view.h"
 
 enum DEBRIS                 // jmf: add for shatter, dig, and Giants to throw
@@ -61,7 +67,7 @@ enum DEBRIS                 // jmf: add for shatter, dig, and Giants to throw
 };          // jmf: ...and I'll actually implement the items Real Soon Now...
 
 // static int make_a_random_cloud(int x, int y, int pow, int ctype);
-static int make_a_rot_cloud(int x, int y, int pow, int ctype);
+static int make_a_rot_cloud(int x, int y, int pow, cloud_type ctype);
 static int quadrant_blink(int x, int y, int pow, int garbage);
 
 //void cast_animate_golem(int pow); // see actual function for reasoning {dlb}
@@ -71,45 +77,45 @@ void do_monster_rot(int mon);
 
 //jmf: FIXME: put somewhere else (misc.cc?)
 // A feeble attempt at Nethack-like completeness for cute messages.
-const char *your_hand( bool plural )
+std::string your_hand( bool plural )
 {
-    static char hand_buff[80];
-
+    std::string result;
     switch (you.attribute[ATTR_TRANSFORMATION])
     {
     default:
         mpr("ERROR: unknown transformation in your_hand() (spells4.cc)");
     case TRAN_NONE:
     case TRAN_STATUE:
-        if (you.species == SP_TROLL || you.species == SP_GHOUL)
+    case TRAN_LICH:
+        if (you.has_usable_claws())
         {
-            strcpy(hand_buff, "claw");
+            result = "claw";
             break;
         }
         // or fall-through
     case TRAN_ICE_BEAST:
-    case TRAN_LICH:
-        strcpy(hand_buff, "hand");
+        result = "hand";
         break;
     case TRAN_SPIDER:
-        strcpy(hand_buff, "front leg");
+        result = "front leg";
         break;
     case TRAN_SERPENT_OF_HELL:
     case TRAN_DRAGON:
-        strcpy(hand_buff, "foreclaw");
+    case TRAN_BAT:
+        result = "foreclaw";
         break;
     case TRAN_BLADE_HANDS:
-        strcpy(hand_buff, "scythe-like blade");
+        result = "scythe-like blade";
         break;
     case TRAN_AIR:
-        strcpy(hand_buff, "misty tendril");
+        result = "misty tendril";
         break;
     }
 
     if (plural)
-        strcat(hand_buff, "s");
+        result += 's';
 
-    return (hand_buff);
+    return result;
 }
 
 // I need to make some debris for metal, crystal and stone.
@@ -160,7 +166,10 @@ inline bool player_hurt_monster(int monster, int damage)
         hurt_monster( &menv[monster], damage );
 
         if (menv[monster].hit_points > 0)
-            print_wounds( &menv[monster] );
+        {
+            const monsters *mons = static_cast<const monsters*>(&menv[monster]);
+            print_wounds(mons);
+        }
         else
         {
             monster_die( &menv[monster], KILL_YOU, 0 );
@@ -177,7 +186,7 @@ static int shatter_monsters(int x, int y, int pow, int garbage)
 {
     UNUSED( garbage );
 
-    dice_def   dam_dice( 0, 5 + pow / 4 );  // number of dice set below
+    dice_def   dam_dice( 0, 5 + pow / 3 );  // number of dice set below
     const int  monster = mgrd[x][y];
 
     if (monster == NON_MONSTER)
@@ -341,7 +350,6 @@ static int shatter_walls(int x, int y, int pow, int garbage)
         break;
 
     case DNGN_METAL_WALL:
-    case DNGN_SILVER_STATUE:
         stuff = DEBRIS_METAL;
         chance = pow / 10;
         break;
@@ -362,11 +370,6 @@ static int shatter_walls(int x, int y, int pow, int garbage)
         stuff = DEBRIS_ROCK;
         break;
 
-    case DNGN_ORANGE_CRYSTAL_STATUE:
-        chance = pow / 6;
-        stuff = DEBRIS_CRYSTAL;
-        break;
-
     case DNGN_GREEN_CRYSTAL_WALL:
         chance = 50;
         stuff = DEBRIS_CRYSTAL;
@@ -397,8 +400,8 @@ void cast_shatter(int pow)
     if (!sil)
         noisy( 30, you.x_pos, you.y_pos );
 
-    snprintf(info, INFO_SIZE, "The dungeon %s!", (sil ? "shakes" : "rumbles"));
-    mpr(info, (sil? MSGCH_PLAIN : MSGCH_SOUND));
+    mprf((sil ? MSGCH_PLAIN : MSGCH_SOUND),
+         "The dungeon %s!", (sil ? "shakes" : "rumbles"));
 
     switch (you.attribute[ATTR_TRANSFORMATION])
     {
@@ -408,6 +411,7 @@ void cast_shatter(int pow)
     case TRAN_DRAGON:
     case TRAN_AIR:
     case TRAN_SERPENT_OF_HELL:
+    case TRAN_BAT:
         break;
 
     case TRAN_STATUE:           // full damage
@@ -528,9 +532,11 @@ static void cast_detect_magic(int pow)
     while (item != NON_ITEM)
     {
         next = mitm[item].link;
-        if (is_dumpable_artifact
-            (mitm[item].base_type, mitm[item].sub_type, mitm[item].plus,
-             mitm[item].plus2, mitm[item].special, 0, 0))
+        if (is_dumpable_artefact(mitm[item].base_type,
+                                 mitm[item].sub_type,
+                                 mitm[item].plus,
+                                 mitm[item].plus2,
+                                 mitm[item].special, 0, 0))
         {
             strong++;
             //FIXME: do checks for randart properties
@@ -601,9 +607,8 @@ void cast_detect_secret_doors(int pow)
     {
         redraw_screen();
 
-        snprintf( info, INFO_SIZE, "You detect %s secret door%s.", 
-                 (found > 1) ? "some" : "a", (found > 1) ? "s" : "" );
-        mpr( info );
+        mprf("You detect %s secret door%s.", 
+             (found > 1) ? "some" : "a", (found > 1) ? "s" : "" );
     }
 }                               // end cast_detect_secret_doors()
 
@@ -623,8 +628,8 @@ void cast_summon_butterflies(int pow)
 
 void cast_summon_large_mammal(int pow)
 {
-    int mon;
-    int temp_rand = random2(pow);
+    monster_type mon;
+    const int temp_rand = random2(pow);
 
     if (temp_rand < 10)
         mon = MONS_JACKAL;
@@ -660,21 +665,21 @@ void cast_summon_large_mammal(int pow)
 
 void cast_sticks_to_snakes(int pow)
 {
-    int mon, i, behaviour;
+    int mon;
     int how_many = 0;
     int max = 1 + random2( 1 + you.skills[SK_TRANSMIGRATION] ) / 4;
-    int dur = cap_int(3 + random2(pow) / 20, 5);
+    int dur = std::min(3 + random2(pow) / 20, 5);
     const int weapon = you.equip[EQ_WEAPON];
 
     if (weapon == -1)
     {
-        snprintf( info, INFO_SIZE, "Your %s feel slithery!", your_hand(true));
-        mpr(info);
+        msg::stream << "Your " << your_hand(true) << " feel slithery!"
+                    << std::endl;
         return;
     }
 
-    behaviour = item_cursed( you.inv[ weapon ] ) ? BEH_HOSTILE
-                                                 : BEH_FRIENDLY;
+    const beh_type behaviour = item_cursed( you.inv[ weapon ] )
+        ? BEH_HOSTILE : BEH_FRIENDLY;
 
     if ((you.inv[ weapon ].base_type == OBJ_MISSILES
          && (you.inv[ weapon ].sub_type == MI_ARROW)))
@@ -682,7 +687,7 @@ void cast_sticks_to_snakes(int pow)
         if (you.inv[ weapon ].quantity < max)
             max = you.inv[ weapon ].quantity;
 
-        for (i = 0; i <= max; i++)
+        for (int i = 0; i <= max; i++)
         {
             if (pow > 50 || (pow > 25 && one_chance_in(3))
                 || (get_ammo_brand(you.inv[weapon]) == SPMSL_POISONED
@@ -761,42 +766,36 @@ void cast_sticks_to_snakes(int pow)
     {
         dec_inv_item_quantity( you.equip[EQ_WEAPON], how_many );
 
-        snprintf( info, INFO_SIZE, "You create %s snake%s!",
-                how_many > 1 ? "some" : "a", how_many > 1 ? "s" : "");
+        mprf("You create %s snake%s!",
+             how_many > 1 ? "some" : "a", how_many > 1 ? "s" : "");
     }
     else
     {
-        snprintf( info, INFO_SIZE, "Your %s feel slithery!", your_hand(true));
+        msg::stream << "Your " << your_hand(true) << " feel slithery!"
+                    << std::endl;
     }
 
-    mpr(info);
-    return;
 }                               // end cast_sticks_to_snakes()
 
 void cast_summon_dragon(int pow)
 {
-    int happy;
 
     // Removed the chance of multiple dragons... one should be more
     // than enough, and if it isn't, the player can cast again...
     // especially since these aren't on the Abjuration plan... they'll
     // last until they die (maybe that should be changed, but this is
     // a very high level spell so it might be okay).  -- bwr
-    happy = (random2(pow) > 5);
+    const bool happy = (random2(pow) > 5);
 
     if (create_monster( MONS_DRAGON, 3,
                         (happy ? BEH_FRIENDLY : BEH_HOSTILE),
                         you.x_pos, you.y_pos, MHITYOU, 250 ) != -1)
     {
-        strcpy(info, "A dragon appears.");
-
-        if (!happy)
-            strcat(info, " It doesn't look very happy.");
+        mprf("A dragon appears.%s",
+             happy ? "" : " It doesn't look very happy.");
     }
     else
-        strcpy(info, "Nothing happens.");
-
-    mpr(info);
+        mprf("Nothing happens.");
 }                               // end cast_summon_dragon()
 
 void cast_conjure_ball_lightning( int pow )
@@ -880,25 +879,42 @@ void cast_mass_sleep(int pow)
     apply_area_visible(sleep_monsters, pow);
 }                               // end cast_mass_sleep()
 
+// This is a hack until we set an is_beast flag in the monster data
+// (which we might never do, this is sort of minor.)
+// It's a list of monster types which can be affected by beast taming.
+static bool is_domesticated_animal(int type)
+{
+    const monster_type types[] = {
+        MONS_GIANT_BAT, MONS_HOUND, MONS_JACKAL, MONS_RAT,
+        MONS_YAK, MONS_WYVERN, MONS_HIPPOGRIFF, MONS_GRIFFON,
+        MONS_DEATH_YAK, MONS_WAR_DOG, MONS_GREY_RAT,
+        MONS_GREEN_RAT, MONS_ORANGE_RAT, MONS_SHEEP,
+        MONS_HOG, MONS_GIANT_FROG, MONS_GIANT_BROWN_FROG,
+        MONS_SPINY_FROG, MONS_BLINK_FROG, MONS_WOLF, MONS_WARG,
+        MONS_BEAR, MONS_GRIZZLY_BEAR, MONS_POLAR_BEAR, MONS_BLACK_BEAR
+    };
+    for ( unsigned int i = 0; i < ARRAYSIZE(types); ++i )
+        if ( types[i] == type )
+            return true;
+    return false;
+}
+
 static int tame_beast_monsters(int x, int y, int pow, int garbage)
 {
     UNUSED( garbage );
-    int which_mons = mgrd[x][y];
+    const int which_mons = mgrd[x][y];
 
-    if (which_mons == NON_MONSTER)                             return 0;
+    if (which_mons == NON_MONSTER)
+        return 0;
 
-    struct monsters *monster = &menv[which_mons];
+    monsters *monster = &menv[which_mons];
 
-    if (mons_holiness(monster) != MH_NATURAL)            return 0;
-    if (mons_intel_type(monster->type) != I_ANIMAL)            return 0;
-    if (mons_friendly(monster))                                return 0;
+    if (!is_domesticated_animal(monster->type) || mons_friendly(monster))
+        return 0;
 
-    // 50% bonus for dogs, add cats if they get implemented
-    if (monster->type == MONS_HOUND || monster->type == MONS_WAR_DOG
-                 || monster->type == MONS_BLACK_BEAR)
-    {
+    // 50% bonus for dogs
+    if (monster->type == MONS_HOUND || monster->type == MONS_WAR_DOG )
         pow += (pow / 2);
-    }
 
     if (you.species == SP_HILL_ORC && monster->type == MONS_WARG)
         pow += (pow / 2);
@@ -906,15 +922,12 @@ static int tame_beast_monsters(int x, int y, int pow, int garbage)
     if (check_mons_resist_magic(monster, pow))
         return 0;
 
-    // I'd like to make the monsters affected permanently, but that's
-    // pretty powerful. Maybe a small (pow/10) chance of being permanently
-    // tamed, large chance of just being enslaved.
     simple_monster_message(monster, " is tamed!");
 
     if (random2(100) < random2(pow / 10))
-        monster->attitude = ATT_FRIENDLY;       // permanent, right?
+        monster->attitude = ATT_FRIENDLY;       // permanent
     else
-        monster->add_ench(ENCH_CHARM);
+        monster->add_ench(ENCH_CHARM);          // temporary
 
     return 1;
 }                               // end tame_beast_monsters()
@@ -998,7 +1011,7 @@ static int ignite_poison_clouds( int x, int y, int pow, int garbage )
         }
     }
 
-    return ((int) did_anything);
+    return did_anything;
 }                               // end ignite_poison_clouds()
 
 static int ignite_poison_monsters(int x, int y, int pow, int garbage)
@@ -1077,7 +1090,8 @@ void cast_ignite_poison(int pow)
     {
         if (set_item_ego_type( you.inv[wpn], OBJ_WEAPONS, SPWPN_FLAMING ))
         {
-            mprf("%s bursts into flame!", in_name(wpn, DESC_CAP_YOUR));
+            mprf("%s bursts into flame!",
+                 you.inv[wpn].name(DESC_CAP_YOUR).c_str());
 
             you.wield_change = true;
             you.duration[DUR_WEAPON_BRAND] += 1 + you.duration[DUR_WEAPON_BRAND] / 2;
@@ -1141,11 +1155,10 @@ void cast_ignite_poison(int pow)
 
     if (pcount > 0)
     {
-        snprintf( info, INFO_SIZE, "%s potion%s you are carrying explode%s!",
-            pcount > 1 ? "Some" : "A",
-            pcount > 1 ? "s" : "",
-            pcount > 1 ? "" : "s");
-        mpr(info);
+        mprf("%s potion%s you are carrying explode%s!",
+             pcount > 1 ? "Some" : "A",
+             pcount > 1 ? "s" : "",
+             pcount > 1 ? "" : "s");
     }
 
     if (wasWielding == true)
@@ -1172,7 +1185,7 @@ void cast_ignite_poison(int pow)
     }
 
     // player is poisoned
-    damage += roll_dice( you.poisoning, 6 );
+    damage += roll_dice( you.duration[DUR_POISONING], 6 );
 
     if (damage)
     {
@@ -1195,10 +1208,10 @@ void cast_ignite_poison(int pow)
 
         ouch( damage, 0, KILLED_BY_TARGETTING );
 
-        if (you.poisoning > 0)
+        if (you.duration[DUR_POISONING] > 0)
         {
             mpr( "You feel that the poison has left your system." );
-            you.poisoning = 0;
+            you.duration[DUR_POISONING] = 0;
         }
     }
 
@@ -1255,7 +1268,7 @@ static int discharge_monsters( int x, int y, int pow, int garbage )
         mpr( "You are struck by lightning." );
         damage = 3 + random2( 5 + pow / 10 );
         damage = check_your_resists( damage, BEAM_ELECTRICITY );
-        if ( player_is_levitating() )
+        if ( player_is_airborne() )
             damage /= 2;
         ouch( damage, 0, KILLED_BY_WILD_MAGIC );
     } 
@@ -1270,10 +1283,8 @@ static int discharge_monsters( int x, int y, int pow, int garbage )
 
         if (damage)
         {
-            strcpy( info, ptr_monam( &(menv[mon]), DESC_CAP_THE ) );
-            strcat( info, " is struck by lightning." );
-            mpr( info );
-
+            mprf( "%s is struck by lightning.",
+                  menv[mon].name(DESC_CAP_THE).c_str());
             player_hurt_monster( mon, damage );
         }
     }
@@ -1306,8 +1317,7 @@ void cast_discharge( int pow )
                                       true, pow, num_targs );
 
 #if DEBUG_DIAGNOSTICS
-    snprintf( info, INFO_SIZE, "Arcs: %d Damage: %d", num_targs, dam );
-    mpr( info, MSGCH_DIAGNOSTICS );
+    mprf(MSGCH_DIAGNOSTICS, "Arcs: %d Damage: %d", num_targs, dam );
 #endif
 
     if (dam == 0) 
@@ -1316,16 +1326,13 @@ void cast_discharge( int pow )
             mpr("The air around you crackles with electrical energy.");
         else 
         {
-            bool plural = coinflip();
-            snprintf( info, INFO_SIZE, "%s blue arc%s ground%s harmlessly %s you.",
-                plural ? "Some" : "A",
-                plural ? "s" : "",
-                plural ? " themselves" : "s itself",
-                plural ? "around" : (coinflip() ? "beside" :
-                                     coinflip() ? "behind" : "before")
-                );
-
-            mpr(info);
+            const bool plural = coinflip();
+            mprf("%s blue arc%s ground%s harmlessly %s you.",
+                 plural ? "Some" : "A",
+                 plural ? "s" : "",
+                 plural ? " themselves" : "s itself",
+                 plural ? "around" : (coinflip() ? "beside" :
+                                      coinflip() ? "behind" : "before"));
         }
     }
 }                               // end cast_discharge()
@@ -1380,19 +1387,14 @@ static int distortion_monsters(int x, int y, int pow, int message)
     }
     else if (coinflip())
     {
-        strcpy(info, "Space bends around ");
-        strcat(info, ptr_monam(defender, DESC_NOCAP_THE));
-        strcat(info, ".");
-        mpr(info);
+        mprf("Space bends around %s.",
+             defender->name(DESC_NOCAP_THE).c_str());
         specdam += 1 + random2avg( 7, 2 ) + random2(pow) / 40;
     }
     else if (coinflip())
     {
-        strcpy(info, "Space warps horribly around ");
-        strcat(info, ptr_monam( defender, DESC_NOCAP_THE ));
-        strcat(info, "!");
-        mpr(info);
-
+        mprf("Space warps horribly around %s!",
+             defender->name(DESC_NOCAP_THE).c_str());
         specdam += 3 + random2avg( 12, 2 ) + random2(pow) / 25;
     }
     else if (one_chance_in(3))
@@ -1447,6 +1449,7 @@ int disperse_monsters(int x, int y, int pow, int message)
     }
     else if (check_mons_resist_magic(defender, pow))
     {
+        // XXX Note that this might affect magic-immunes!
         if (coinflip())
         {
             simple_monster_message(defender, " partially resists.");
@@ -1489,7 +1492,8 @@ static int spell_swap_func(int x, int y, int pow, int message)
     if (defender->type == MONS_BLINK_FROG
         || check_mons_resist_magic( defender, pow ))
     {
-        simple_monster_message( defender, " resists." );
+        simple_monster_message( defender, mons_immune_magic(defender) ?
+                                " is unaffected." : " resists." );
     }
     else
     {
@@ -1504,10 +1508,7 @@ static int spell_swap_func(int x, int y, int pow, int message)
         int old_y = defender->y;
 
         if (swap_places( defender ))
-        {
-            you.x_pos = old_x;
-            you.y_pos = old_y;
-        }
+            you.moveto(old_x, old_y);
     }
 
     return 1;
@@ -1518,7 +1519,7 @@ void cast_swap(int pow)
     apply_one_neighbouring_square( spell_swap_func, pow );
 }
 
-static int make_a_rot_cloud(int x, int y, int pow, int ctype)
+static int make_a_rot_cloud(int x, int y, int pow, cloud_type ctype)
 {
     int next = 0, obj = mgrd[x][y];
 
@@ -1554,7 +1555,8 @@ static int make_a_rot_cloud(int x, int y, int pow, int ctype)
     return 0;
 }                               // end make_a_rot_cloud()
 
-int make_a_normal_cloud(int x, int y, int pow, int ctype, kill_category whose)
+int make_a_normal_cloud(int x, int y, int pow, cloud_type ctype,
+                        kill_category whose)
 {
     place_cloud( ctype, x, y, 
                  (3 + random2(pow / 4) + random2(pow / 4) + random2(pow / 4)),
@@ -1712,7 +1714,7 @@ static int intoxicate_monsters(int x, int y, int pow, int garbage)
     if (mons_res_poison(&menv[mon]) > 0)
         return 0;
 
-    menv[mon].add_ench(ENCH_CONFUSION);
+    menv[mon].add_ench(mon_enchant(ENCH_CONFUSION, 0, KC_YOU));
     return 1;
 }                               // end intoxicate_monsters()
 
@@ -1778,7 +1780,7 @@ static int glamour_monsters(int x, int y, int pow, int garbage)
         break;
     case 1:
     case 4:
-        menv[mon].add_ench(ENCH_CONFUSION);
+        menv[mon].add_ench(mon_enchant(ENCH_CONFUSION, 0, KC_YOU));
         break;
     case 2:
     case 5:
@@ -1792,28 +1794,29 @@ static int glamour_monsters(int x, int y, int pow, int garbage)
     // why no, there's no message as to which effect happened >:^)
     if (!one_chance_in(4))
     {
-        strcpy(info, ptr_monam( &(menv[mon]), DESC_CAP_THE));
+        std::string msg = menv[mon].name(DESC_CAP_THE);
 
         switch (random2(4))
         {
         case 0:
-            strcat(info, " looks dazed.");
+            msg += " looks dazed.";
             break;
         case 1:
-            strcat(info, " blinks several times.");
+            msg += " blinks several times.";
             break;
         case 2:
-            strcat(info, " rubs its eye");
-            if (menv[mon].type != MONS_CYCLOPS)
-                strcat(info, "s");
-            strcat(info, ".");
+            msg += " rubs its eye";
+            if (menv[mon].type != MONS_CYCLOPS &&
+                menv[mon].type != MONS_POLYPHEMUS)
+                msg += "s";
+            msg += ".";
             break;
-        case 4:
-            strcat(info, " tilts its head.");
+        case 3:
+            msg += " tilts its head.";
             break;
         }
 
-        mpr(info);
+        mpr(msg.c_str());
     }
 
     return (1);
@@ -1836,11 +1839,6 @@ bool backlight_monsters(int x, int y, int pow, int garbage)
 
     switch (menv[mon].type)
     {
-    //case MONS_INSUBSTANTIAL_WISP: //jmf: I'm not sure if these glow or not
-    //case MONS_VAPOUR:
-    case MONS_UNSEEN_HORROR:    // consider making this visible? probably not.
-        return (false);
-
     case MONS_FIRE_VORTEX:
     case MONS_ANGEL:
     case MONS_FIEND:
@@ -1867,7 +1865,21 @@ bool backlight_monsters(int x, int y, int pow, int garbage)
     }
 
     mon_enchant bklt = menv[mon].get_ench(ENCH_BACKLIGHT);
-    int lvl = bklt.degree;
+    const int lvl = bklt.degree;
+
+    // this enchantment overrides invisibility (neat)
+    if (menv[mon].has_ench(ENCH_INVIS))
+    {
+        if (!menv[mon].has_ench(ENCH_BACKLIGHT))
+        {
+            menv[mon].add_ench(
+                mon_enchant(ENCH_BACKLIGHT, 1, KC_OTHER, random_range(30, 50)));
+            simple_monster_message( &menv[mon], " is limned in light." );
+        }
+        return (true);
+    }
+    
+    menv[mon].add_ench(mon_enchant(ENCH_BACKLIGHT, 1));
 
     if (lvl == 0)
         simple_monster_message( &menv[mon], " is outlined in light." );
@@ -1876,10 +1888,6 @@ bool backlight_monsters(int x, int y, int pow, int garbage)
     else 
         simple_monster_message( &menv[mon], " glows brighter." );
 
-    // this enchantment wipes out invisibility (neat)
-    menv[mon].del_ench(ENCH_INVIS);
-    menv[mon].add_ench(mon_enchant(ENCH_BACKLIGHT, 1));
-
     return (true);
 }                               // end backlight_monsters()
 
@@ -1891,11 +1899,12 @@ void cast_evaporate(int pow)
     struct bolt beem;
 
     const int potion =
-        prompt_invent_item( "Throw which potion?", MT_INVSELECT, OBJ_POTIONS );
+        prompt_invent_item( "Throw which potion?", MT_INVLIST, OBJ_POTIONS );
 
     if (potion == -1)
     {
-        mprf( "Wisps of steam play over your %s!",  your_hand(true) );
+        msg::stream << "Wisps of steam play over your " << your_hand(true)
+                    << '.' << std::endl;
         return;
     } 
     else if (you.inv[potion].base_type != OBJ_POTIONS)
@@ -1930,7 +1939,7 @@ void cast_evaporate(int pow)
     beem.is_beam = false;
     beem.is_tracer = false;
 
-    beem.hit = you.dex / 2 + roll_dice( 2, you.skills[SK_RANGED_COMBAT] / 2 + 1 );
+    beem.hit = you.dex / 2 + roll_dice( 2, you.skills[SK_THROWING] / 2 + 1 );
     beem.damage = dice_def( 1, 0 );  // no damage, just producing clouds
     beem.ench_power = pow;           // used for duration only?
 
@@ -2007,7 +2016,7 @@ void cast_evaporate(int pow)
     }
 
     if (coinflip())
-        exercise( SK_RANGED_COMBAT, 1 );
+        exercise( SK_THROWING, 1 );
 
     fire_beam(beem);
 
@@ -2025,8 +2034,6 @@ void cast_evaporate(int pow)
 // using up the corpse might also lead to game balance problems. -- bwr
 void cast_fulsome_distillation( int powc )
 {
-    char str_pass[ ITEMNAME_SIZE ];
-
     if (powc > 50)
         powc = 50;
 
@@ -2041,11 +2048,11 @@ void cast_fulsome_distillation( int powc )
              curr_item != NON_ITEM; 
              curr_item = mitm[curr_item].link) 
     {
-        if (mitm[curr_item].base_type == OBJ_CORPSES
-            && mitm[curr_item].sub_type == CORPSE_BODY)
+        const item_def& item = mitm[curr_item];
+        if (item.base_type == OBJ_CORPSES && item.sub_type == CORPSE_BODY)
         {
-            it_name( curr_item, DESC_NOCAP_THE, str_pass );
-            snprintf( info, INFO_SIZE, "Distill a potion from %s?", str_pass );
+            snprintf( info, INFO_SIZE, "Distill a potion from %s?",
+                      item.name(DESC_NOCAP_THE).c_str() );
 
             if (yesno( info, true, 0, false ))
             {
@@ -2165,10 +2172,8 @@ void cast_fulsome_distillation( int powc )
     mitm[corpse].inscription.clear();
     item_colour( mitm[corpse] );  // sets special as well
 
-    it_name( corpse, DESC_NOCAP_A, str_pass );
-    snprintf( info, INFO_SIZE, "You extract %s from the corpse.",
-              str_pass );
-    mpr( info );
+    mprf("You extract %s from the corpse.",
+         mitm[corpse].name(DESC_NOCAP_A).c_str());
 
     // try to move the potion to the player (for convenience)
     if (move_item_to_player( corpse, 1 ) != 1)
@@ -2315,8 +2320,7 @@ static int snake_charm_monsters(int x, int y, int pow, int message)
     if (check_mons_resist_magic(&menv[mon], pow))       return 0;
 
     menv[mon].attitude = ATT_FRIENDLY;
-    snprintf( info, INFO_SIZE, "%s sways back and forth.", ptr_monam( &(menv[mon]), DESC_CAP_THE ));
-    mpr(info);
+    mprf("%s sways back and forth.", menv[mon].name(DESC_CAP_THE).c_str());
 
     return 1;
 }
@@ -2337,7 +2341,7 @@ void cast_fragmentation(int pow)        // jmf: ripped idea from airstrike
     bool hole = true;
     const char *what = NULL;
 
-    mpr("Fragment what (e.g. a wall)?", MSGCH_PROMPT);
+    mpr("Fragment what (e.g. a wall or monster)?", MSGCH_PROMPT);
     direction( beam, DIR_TARGET, TARG_ENEMY );
 
     if (!beam.isValid)
@@ -2354,6 +2358,8 @@ void cast_fragmentation(int pow)        // jmf: ripped idea from airstrike
     blast.type = '#';
     blast.colour = 0;
     blast.set_target(beam);
+    blast.source_x = you.x_pos;
+    blast.source_y = you.y_pos;
     blast.is_tracer = false;
     blast.flavour = BEAM_FRAG;
     blast.hit = AUTOMATIC_HIT;
@@ -2374,10 +2380,11 @@ void cast_fragmentation(int pow)        // jmf: ripped idea from airstrike
         char explode_msg[80];
 
         snprintf( explode_msg, sizeof( explode_msg ), "%s explodes!",
-                  ptr_monam( &(menv[mon]), DESC_CAP_THE ) );
+                  menv[mon].name(DESC_CAP_THE).c_str() );
 
         switch (menv[mon].type)
         {
+        case MONS_ICE_STATUE:
         case MONS_ICE_BEAST: // blast of ice fragments
         case MONS_SIMULACRUM_SMALL:
         case MONS_SIMULACRUM_LARGE:
@@ -2395,8 +2402,8 @@ void cast_fragmentation(int pow)        // jmf: ripped idea from airstrike
         case MONS_SKELETON_LARGE:       // blast of bone
             explode = true;
 
-            snprintf( info, INFO_SIZE, "The sk%s explodes into sharp fragments of bone!",
-                    (menv[mon].type == MONS_FLYING_SKULL) ? "ull" : "eleton");
+            mprf("The %s explodes into sharp fragments of bone!",
+                 (menv[mon].type == MONS_FLYING_SKULL) ? "skull" : "skeleton");
 
             blast.name = "blast of bone shards";
 
@@ -2466,8 +2473,14 @@ void cast_fragmentation(int pow)        // jmf: ripped idea from airstrike
                 blast.damage.num = 6;
             }
 
-            if (player_hurt_monster(mon, roll_dice( blast.damage )))
-                blast.damage.num += 2;
+            {
+                int statue_damage = roll_dice(blast.damage) * 2;
+                if (pow >= 50 && one_chance_in(10))
+                    statue_damage = menv[mon].hit_points;
+                
+                if (player_hurt_monster(mon, statue_damage))
+                    blast.damage.num += 2;
+            }
             break;
 
         case MONS_CRYSTAL_GOLEM:
@@ -2484,7 +2497,7 @@ void cast_fragmentation(int pow)        // jmf: ripped idea from airstrike
             blast.damage.num = 1;  // to mark that a monster was targetted
 
             // Yes, this spell does lousy damage if the 
-            // monster isn't susceptable. -- bwr 
+            // monster isn't susceptible. -- bwr 
             player_hurt_monster( mon, roll_dice( 1, 5 + pow / 25 ) );
             goto do_terrain;
         }
@@ -2547,14 +2560,6 @@ void cast_fragmentation(int pow)        // jmf: ripped idea from airstrike
     case DNGN_METAL_WALL:       
         what = "metal wall";
         blast.colour = CYAN;
-        // fallthru
-    case DNGN_SILVER_STATUE:
-        if (what == NULL)
-        {
-            what = "silver statue";
-            blast.colour = WHITE;
-        }
-
         explode = true;
         blast.name = "blast of metal fragments";
         blast.damage.num = 4;
@@ -2574,23 +2579,13 @@ void cast_fragmentation(int pow)        // jmf: ripped idea from airstrike
     case DNGN_GREEN_CRYSTAL_WALL:       // crystal -- large & nasty explosion
         what = "crystal wall";
         blast.colour = GREEN;
-        // fallthru
-    case DNGN_ORANGE_CRYSTAL_STATUE:
-        if (what == NULL)
-        {
-            what = "crystal statue";
-            blast.colour = LIGHTRED; //jmf: == orange, right?
-        }
-
         explode = true;
         blast.ex_size = 2;
         blast.name = "blast of crystal shards";
         blast.damage.num = 5;
 
         if (okay_to_dest
-            && ((grid == DNGN_GREEN_CRYSTAL_WALL && coinflip())
-                || (grid == DNGN_ORANGE_CRYSTAL_STATUE 
-                    && pow >= 50 && one_chance_in(10))))
+            && grid == DNGN_GREEN_CRYSTAL_WALL && coinflip())
         {
             blast.ex_size = coinflip() ? 3 : 2;
             grd[beam.tx][beam.ty] = DNGN_FLOOR;
@@ -2654,10 +2649,9 @@ void cast_fragmentation(int pow)        // jmf: ripped idea from airstrike
     case DNGN_PERMAROCK_WALL:
     case DNGN_FLOOR:
         explode = false;
-        snprintf( info, INFO_SIZE, "%s seems to be unnaturally hard.",
-                  (grid == DNGN_PERMAROCK_WALL) ? "That wall" 
-                                                : "The dungeon floor" );
-        explode = false;
+        mprf("%s seems to be unnaturally hard.",
+             (grid == DNGN_PERMAROCK_WALL) ? "That wall" 
+                                           : "The dungeon floor");
         break;
 
     case DNGN_TRAP_III: // What are these? Should they explode? -- bwr
@@ -2670,12 +2664,9 @@ void cast_fragmentation(int pow)        // jmf: ripped idea from airstrike
     if (explode && blast.damage.num > 0)
     {
         if (what != NULL)
-        {
-            snprintf( info, INFO_SIZE, "The %s explodes!", what);
-            mpr(info);
-        }
+            mprf("The %s explodes!", what);
 
-        explosion( blast, hole );
+        explosion( blast, hole, true );
     }
     else if (blast.damage.num == 0)
     {
@@ -2698,7 +2689,7 @@ void cast_twist(int pow)
 
     // Get target, using DIR_TARGET for targetting only,
     // since we don't use fire_beam() for this spell.
-    if (spell_direction(targ, tmp, DIR_TARGET) == -1)
+    if ( !spell_direction(targ, tmp, DIR_TARGET) )
         return;
 
     const int mons = mgrd[ targ.tx ][ targ.ty ];
@@ -2713,7 +2704,8 @@ void cast_twist(int pow)
     // Monster can magically save vs attack.
     if (check_mons_resist_magic( &menv[ mons ], pow * 2 ))
     {
-        simple_monster_message( &menv[ mons ], " resists." );
+        simple_monster_message( &menv[mons], mons_immune_magic(&menv[mons]) ?
+                                " is unaffected." : " resists." );
         return;
     }
 
@@ -2728,6 +2720,28 @@ void cast_twist(int pow)
     return;
 }                               // end cast_twist()
 
+bool cast_portaled_projectile(int pow, bolt& beam)
+{
+    if ( pow > 50 )
+        pow = 50;
+
+    if ( grid_is_solid(beam.target_x, beam.target_y) )
+    {
+        mpr("You can't shoot at gazebos.");
+        return false;
+    }
+
+    const int idx = get_fire_item_index();
+    if ( idx == ENDOFPACK )
+    {
+        mpr("No suitable missiles.");
+        return false;
+    }
+
+    throw_it( beam, idx, true, random2(pow/4) );
+    return true;
+}
+
 //
 // This version of far strike is a bit too creative for level one, in
 // order to make it work we needed to put a lot of restrictions on it
@@ -2750,7 +2764,7 @@ void cast_far_strike(int pow)
 
     // Get target, using DIR_TARGET for targetting only,
     // since we don't use fire_beam() for this spell.
-    if (spell_direction(targ, tmp, DIR_TARGET) == -1)
+    if ( !spell_direction(targ, tmp, DIR_TARGET) )
         return;
 
     // Get the target monster...
@@ -2847,7 +2861,8 @@ void cast_far_strike(int pow)
     // augmented with an EV check).
     if (check_mons_resist_magic( monster, pow * 2 ))
     {
-        simple_monster_message( monster, " resists." );
+        simple_monster_message( monster, mons_immune_magic(monster) ?
+                                " is unaffected." : " resists." );
         return;
     }
 
@@ -2883,7 +2898,7 @@ int cast_apportation(int pow)
     }
 
     // Protect the player from destroying the item
-    const int grid = grd[ you.x_pos ][ you.y_pos ];
+    const dungeon_feature_type grid = grd[ you.x_pos ][ you.y_pos ];
 
     if (grid_destroys_items(grid))
     {
@@ -2917,9 +2932,7 @@ int cast_apportation(int pow)
             mpr( "There are no items there." );
         else if (mons_is_mimic( menv[ mon ].type ))
         {
-            snprintf( info, INFO_SIZE, "%s twitches.",
-                      ptr_monam( &(menv[ mon ]), DESC_CAP_THE ) );
-            mpr( info );
+            mprf("%s twitches.", menv[mon].name(DESC_CAP_THE).c_str());
         }
         else 
             mpr( "This spell does not work on creatures." );
@@ -2961,11 +2974,22 @@ int cast_apportation(int pow)
         else
         {
             mpr( "Yoink!" );
-            snprintf( info, INFO_SIZE, "You pull the item%s to yourself.",
-                                 (mitm[ item ].quantity > 1) ? "s" : "" );
-            mpr( info );
+            mprf("You pull the item%s to yourself.",
+                 (mitm[ item ].quantity > 1) ? "s" : "" );
         }
         done = 1;
+        
+        // if we apport a net, free the monster under it
+        if (mitm[item].base_type == OBJ_MISSILES
+            && mitm[item].sub_type == MI_THROWING_NET
+            && item_is_stationary(mitm[item]))
+        {
+           int mon = mgrd[ beam.tx ][ beam.ty ];
+           remove_item_stationary(mitm[item]);
+           
+           if (mon != NON_MONSTER)
+               (&menv[mon])->del_ench(ENCH_HELD, true);
+        }
     }
     else
         mpr( "The spell fails." );
@@ -3005,12 +3029,15 @@ void cast_sandblast(int pow, bolt &beam)
 
 void cast_condensation_shield(int pow)
 {
-    if (you.equip[EQ_SHIELD] != -1 || you.fire_shield)
+    if (you.equip[EQ_SHIELD] != -1 || you.duration[DUR_FIRE_SHIELD])
         canned_msg(MSG_SPELL_FIZZLES);
     else
     {
         if (you.duration[DUR_CONDENSATION_SHIELD] > 0)
+        {
+            mpr("The disc of vapour around you crackles some more.");
             you.duration[DUR_CONDENSATION_SHIELD] += 5 + roll_dice(2, 3);
+        }
         else
         {
             mpr("A crackling disc of dense vapour forms in the air!");
@@ -3060,8 +3087,7 @@ static int quadrant_blink(int x, int y, int pow, int garbage)
             break;
     }
 
-    you.x_pos = tx;
-    you.y_pos = ty;
+    you.moveto(tx, ty);
     return 1;
 }
 
