@@ -1,5 +1,5 @@
 /*
- *  Modified for Crawl Reference by $Author: dshaligram $ on $Date: 2007-04-05T16:46:35.024555Z $
+ *  Modified for Crawl Reference by $Author: ennewalker $ on $Date: 2007-11-23T02:25:44.246247Z $
  */
 
 /*
@@ -138,10 +138,13 @@
 
 {
     SPELL_IDENTIFY, "Identify",
-     SPTYP_DIVINATION,
-     SPFLAG_NONE,
-     6,
-     NULL
+    SPTYP_DIVINATION,
+    SPFLAG_NONE,
+    6,
+    0,
+    NULL,
+    false,
+    true
 },
 
 {
@@ -149,7 +152,10 @@
      SPTYP_TRANSLOCATION,
      SPFLAG_NONE,
      5,
-     NULL
+     0,
+     NULL,
+     false,
+     false
 },
 
 {
@@ -157,7 +163,10 @@
      SPTYP_ENCHANTMENT,
      SPFLAG_NONE,
      5,
-     NULL
+     200,
+     NULL,
+     false,
+     false
 },
 
 {
@@ -165,7 +174,10 @@
      SPTYP_ENCHANTMENT,
      SPFLAG_NONE,
      1,
-     NULL
+     0,
+     NULL,
+     false,
+     true
 },
 
 {
@@ -173,7 +185,10 @@
      SPTYP_ENCHANTMENT,
      SPFLAG_NONE,
      5,
-     NULL
+     0,
+     NULL,
+     false,
+     true
 },
 
 {
@@ -181,7 +196,10 @@
      SPTYP_CONJURATION,
      SPFLAG_DIR_OR_TARGET,
      1,
-     NULL
+     25,
+     NULL,
+     true,
+     false
 },
 
 {
@@ -189,7 +207,10 @@
      SPTYP_CONJURATION | SPTYP_FIRE,
      SPFLAG_DIR_OR_TARGET,
      6,
-     NULL
+     200,
+     NULL,
+     true,
+     false
 },
 
 {
@@ -197,7 +218,10 @@
     SPTYP_TRANSLOCATION,
     SPFLAG_NONE,
     4,
-     NULL
+    0,
+    NULL,
+    false,
+    false
 },
 
 {
@@ -205,7 +229,10 @@
     SPTYP_TRANSLOCATION,
     SPFLAG_NONE,
     1,
-     NULL
+    1000,
+    NULL,
+    false,
+    false
 },
 
 {
@@ -213,7 +240,9 @@
     SPTYP_TRANSLOCATION,
     SPFLAG_DIR_OR_TARGET,
     1,
-     NULL
+    25,
+    NULL,
+    true
 },
 
 {
@@ -221,7 +250,10 @@
      SPTYP_CONJURATION | SPTYP_FIRE,
      SPFLAG_NONE,
      3,
-     NULL
+     100,
+     NULL,
+     false,
+    false
 },
 
 {
@@ -229,7 +261,10 @@
      SPTYP_TRANSMIGRATION | SPTYP_EARTH,
      SPFLAG_DIR_OR_TARGET | SPFLAG_NOT_SELF,
      4,
-     NULL
+     200,
+     NULL,
+     false,
+     true
 },
 
 {
@@ -237,7 +272,9 @@
      SPTYP_CONJURATION | SPTYP_FIRE,
      SPFLAG_DIR_OR_TARGET,
      5,
-     NULL
+     200,
+     NULL,
+     true
 },
 
 {
@@ -245,7 +282,9 @@
      SPTYP_CONJURATION | SPTYP_ICE,
      SPFLAG_DIR_OR_TARGET,
      5,
-     NULL
+     200,
+     NULL,
+     true
 },
 
 {
@@ -253,7 +292,9 @@
      SPTYP_CONJURATION | SPTYP_AIR,
      SPFLAG_DIR_OR_TARGET,
      6,
-     NULL
+     200,
+     NULL,
+     true
 },
 
 {
@@ -261,7 +302,9 @@
      SPTYP_CONJURATION | SPTYP_FIRE | SPTYP_EARTH,
      SPFLAG_DIR_OR_TARGET,
      5,
-     NULL
+     200,
+     NULL,
+     true
 },
 
 {
@@ -269,7 +312,9 @@
      SPTYP_TRANSMIGRATION,  // removed enchantment, wasn't needed -- bwr
      SPFLAG_DIR_OR_TARGET | SPFLAG_NOT_SELF,
      5,
-     NULL
+     200,
+     NULL,
+     true
 },
 
 {
@@ -277,25 +322,32 @@
      SPTYP_ENCHANTMENT,
      SPFLAG_DIR_OR_TARGET,
      3,
-     NULL
+     200,
+     NULL,
+     true
 },
 
 {
     SPELL_HASTE, "Haste",
      SPTYP_ENCHANTMENT,
      SPFLAG_DIR_OR_TARGET | SPFLAG_HELPFUL,
-     6,  // lowered to 6 from 8, since its easily available from various items
-        // and Swiftness is level 2 (and gives a similar effect).  Its also
+     6,  // lowered to 6 from 8, since it's easily available from various items
+        // and Swiftness is level 2 (and gives a similar effect).  It's also
         // not that much better than Invisibility.  -- bwr
-     NULL
+     200,
+     NULL,
+     false,
+     true
 },
 
 {
-    SPELL_PARALYZE, "Paralyse",
+    SPELL_PARALYSE, "Paralyse",
      SPTYP_ENCHANTMENT,
      SPFLAG_DIR_OR_TARGET,
      4,
-     NULL
+     200,
+     NULL,
+     true
 },
 
 {
@@ -303,7 +355,9 @@
      SPTYP_ENCHANTMENT,
      SPFLAG_DIR_OR_TARGET,
      3,
-     NULL
+     200,
+     NULL,
+     true
 },
 
 {
@@ -311,7 +365,10 @@
      SPTYP_ENCHANTMENT,
      SPFLAG_DIR_OR_TARGET | SPFLAG_HELPFUL,
      6,
-     NULL
+     200,
+     NULL,
+     false,
+     true
 },
 
 {
@@ -319,7 +376,9 @@
      SPTYP_CONJURATION | SPTYP_FIRE,
      SPFLAG_DIR_OR_TARGET,
      2,
-     NULL
+     50,
+     NULL,
+     true
 },
 
 {
@@ -327,15 +386,20 @@
      SPTYP_CONJURATION | SPTYP_ICE,
      SPFLAG_DIR_OR_TARGET,
      2,
-     NULL
+     50,
+     NULL,
+     true
 },
 
 {
     SPELL_CONTROLLED_BLINK, "Controlled Blink",
      SPTYP_TRANSLOCATION,
      SPFLAG_NONE,
-     8,
-     NULL
+     7,
+     0,
+     NULL,
+     false,
+    false
 },
 
 {
@@ -343,7 +407,9 @@
      SPTYP_CONJURATION | SPTYP_ICE | SPTYP_AIR,
      SPFLAG_GRID,
      7,
-     "Where do you want to put it?"
+     200,
+     "Where do you want to put it?",
+     true
 },
 
 {
@@ -351,7 +417,9 @@
      SPTYP_CONJURATION | SPTYP_POISON | SPTYP_AIR,
      SPFLAG_DIR_OR_TARGET,
      3,
-     NULL
+     200,
+     NULL,
+     true
 },
 
 {
@@ -359,7 +427,10 @@
      SPTYP_ENCHANTMENT | SPTYP_FIRE,
      SPFLAG_NONE,
      8,
-     NULL
+     200,
+     NULL,
+     false,
+    false
 },
 
 {
@@ -367,7 +438,10 @@
      SPTYP_HOLY,
      SPFLAG_NONE,
      2,
-     NULL
+     0,
+     NULL,
+     false,
+    false
 },
 
 {
@@ -375,7 +449,10 @@
      SPTYP_HOLY,
      SPFLAG_NONE,
      2,
-     NULL
+     0,
+     NULL,
+     false,
+    false
 },
 
 {
@@ -383,7 +460,10 @@
      SPTYP_HOLY,
      SPFLAG_NONE,
      2,
-     NULL
+     0,
+     NULL,
+     false,
+    false
 },
 
 {
@@ -391,7 +471,9 @@
      SPTYP_CONJURATION | SPTYP_POISON,
      SPFLAG_DIR_OR_TARGET,
      5,
-     NULL
+     200,
+     NULL,
+     true
 },
 
 {
@@ -399,7 +481,9 @@
      SPTYP_POISON,
      SPFLAG_NONE,
      4,
-     NULL
+     0,
+     NULL,
+     true
 },
 
 {
@@ -407,7 +491,9 @@
      SPTYP_TRANSLOCATION,
      SPFLAG_DIR_OR_TARGET | SPFLAG_NOT_SELF,
      4,
-     NULL
+     200,
+     NULL,
+     true
 },
 
 {
@@ -415,7 +501,10 @@
      SPTYP_HOLY,
      SPFLAG_NONE,
      2,
-     NULL
+     0,
+     NULL,
+     false,
+     true
 },
 
 {
@@ -423,7 +512,10 @@
      SPTYP_HOLY,
      SPFLAG_NONE,
      6,
-     NULL
+     0,
+     NULL,
+     false,
+    false
 },
 
 {
@@ -431,7 +523,10 @@
      SPTYP_HOLY,
      SPFLAG_NONE,
      3,
-     NULL
+     200,
+     NULL,
+     false,
+    false
 },
 
 {
@@ -439,7 +534,10 @@
      SPTYP_HOLY,
      SPFLAG_NONE,
      5,
-     NULL
+     0,
+     NULL,
+     false,
+    false
 },
 
 {
@@ -447,15 +545,21 @@
      SPTYP_ENCHANTMENT | SPTYP_NECROMANCY,
      SPFLAG_NONE,
      8,
-     NULL
+     200,
+     NULL,
+     false,
+    false
 },
 
 {
     SPELL_SELECTIVE_AMNESIA, "Selective Amnesia",
      SPTYP_ENCHANTMENT,
      SPFLAG_NONE,
-     3,
-     NULL
+     4,
+     0,
+     NULL,
+     false,
+    false
 },
 
 {
@@ -463,7 +567,10 @@
      SPTYP_ENCHANTMENT,
      SPFLAG_NONE,
      6,
-     NULL
+     200,
+     NULL,
+     false,
+    false
 },
 
 {
@@ -471,7 +578,10 @@
      SPTYP_HOLY,
      SPFLAG_TARGET | SPFLAG_NOT_SELF,
      4,
-     "Smite whom?"
+     200,
+     "Smite whom?",
+     false,
+    false
 },
 
 {
@@ -479,7 +589,10 @@
      SPTYP_HOLY,
      SPFLAG_NONE,
      3,
-     NULL
+     0,
+     NULL,
+     false,
+    false
 },
 
 {
@@ -487,7 +600,10 @@
      SPTYP_HOLY,
      SPFLAG_NONE,
      7,
-     NULL
+     0,
+     NULL,
+     false,
+    false
 },
 
 {
@@ -495,7 +611,10 @@
      SPTYP_DIVINATION,
      SPFLAG_NONE,
      3,
-     NULL
+     0,
+     NULL,
+     false,
+    false
 },
 
 {
@@ -503,7 +622,10 @@
      SPTYP_SUMMONING,
      SPFLAG_NONE,
      1,
-     NULL
+     80,
+     NULL,
+     false,
+    false
 },
 
 {
@@ -511,7 +633,10 @@
      SPTYP_SUMMONING,
      SPFLAG_NONE,
      3,
-     NULL
+     200,
+     NULL,
+     false,
+    false
 },
 
 {
@@ -519,7 +644,10 @@
      SPTYP_SUMMONING | SPTYP_POISON,
      SPFLAG_NONE,
      4,
-     NULL
+     200,
+     NULL,
+     false,
+    false
 },
 
 {
@@ -527,7 +655,10 @@
      SPTYP_ENCHANTMENT | SPTYP_AIR,
      SPFLAG_NONE,
      2,
-     NULL
+     150,
+     NULL,
+     false,
+    false
 },
 
 {
@@ -535,7 +666,9 @@
      SPTYP_CONJURATION | SPTYP_NECROMANCY,
      SPFLAG_DIR_OR_TARGET,
      6,
-     NULL
+     200,
+     NULL,
+     true
 },
 
 {
@@ -543,15 +676,19 @@
      SPTYP_CONJURATION | SPTYP_EARTH,
      SPFLAG_DIR_OR_TARGET,
      8,
-     NULL
+     200,
+     NULL,
+     true
 },
 
 {
     SPELL_BOLT_OF_INACCURACY, "Bolt of Inaccuracy",
      SPTYP_CONJURATION,
      SPFLAG_DIR_OR_TARGET,
-     2,
-     NULL
+     3,
+     1000,
+     NULL,
+     true
 },
 
 {
@@ -559,7 +696,9 @@
      SPTYP_CONJURATION | SPTYP_POISON | SPTYP_AIR,
      SPFLAG_GRID,
      6,
-     "Where do you want to put it?"
+     200,
+     "Where do you want to put it?",
+     true
 }
 ,
 
@@ -568,15 +707,20 @@
      SPTYP_CONJURATION | SPTYP_FIRE,
      SPFLAG_GRID,
      9,
-     "Where?" 
+     200,
+     "Where?",
+     true
 },
 
 {
     SPELL_DETECT_TRAPS, "Detect Traps",
      SPTYP_DIVINATION,
-     SPFLAG_NONE,
+     SPFLAG_MAPPING,
      2,
-     NULL
+     50,
+     NULL,
+     false,
+    false
 },
 
 {
@@ -584,7 +728,10 @@
      SPTYP_TRANSLOCATION,
      SPFLAG_NONE,
      2,
-     NULL
+     0,
+     NULL,
+     false,
+     true
 },
 
 
@@ -595,7 +742,9 @@
      SPTYP_CONJURATION,
      SPFLAG_DIR_OR_TARGET,
      4,
-     NULL
+     100,
+     NULL,
+     true
 },
 
 {
@@ -603,7 +752,10 @@
      SPTYP_SUMMONING,
      SPFLAG_NONE,
      6,
-     NULL
+     200,
+     NULL,
+     false,
+    false
 },
 
 {
@@ -611,7 +763,10 @@
      SPTYP_SUMMONING,
      SPFLAG_UNHOLY,
      8,
-     NULL
+     200,
+     NULL,
+     false,
+    false
 },
 
 {
@@ -619,15 +774,20 @@
      SPTYP_ENCHANTMENT,
      SPFLAG_DIR_OR_TARGET | SPFLAG_NOT_SELF,
      4,
-     NULL
+     200,
+     NULL,
+     true
 },
 
 {
     SPELL_MAGIC_MAPPING, "Magic Mapping",
      SPTYP_DIVINATION | SPTYP_EARTH,
-     SPFLAG_NONE,
+     SPFLAG_MAPPING,
      4,
-     NULL
+     45,
+     NULL,
+     false,
+     false
 },
 
 {
@@ -635,7 +795,10 @@
      SPTYP_HOLY,
      SPFLAG_DIR_OR_TARGET | SPFLAG_HELPFUL | SPFLAG_NOT_SELF,
      3,
-     NULL
+     100,
+     NULL,
+     true,
+     false
 },
 
 {
@@ -643,7 +806,10 @@
      SPTYP_NECROMANCY,
      SPFLAG_NONE,
      4,
-     NULL
+     0,
+     NULL,
+     false,
+     true
 },
 
 {
@@ -651,7 +817,9 @@
      SPTYP_NECROMANCY,
      SPFLAG_DIR_OR_TARGET,
      1,
-     NULL
+     25,
+     NULL,
+     true
 },
 
 {
@@ -659,7 +827,10 @@
      SPTYP_ENCHANTMENT,
      SPFLAG_NONE,
      5,
-     NULL
+     200,
+     NULL,
+     false,
+    false
 },
 
 {
@@ -667,7 +838,9 @@
      SPTYP_ENCHANTMENT | SPTYP_NECROMANCY,
      SPFLAG_NONE,
      6,
-     NULL
+     200,
+     NULL,
+     true
 },
 
 {
@@ -675,7 +848,10 @@
      SPTYP_NECROMANCY,
      SPFLAG_NONE,
      1,
-     NULL
+     0,
+     NULL,
+     false,
+    false
 },
   
 {
@@ -683,7 +859,10 @@
      SPTYP_NECROMANCY,
      SPFLAG_DIR | SPFLAG_NOT_SELF,
      3,
-     NULL
+     200,
+     NULL,
+     false,
+    false
 },
 
 {
@@ -691,15 +870,21 @@
      SPTYP_NECROMANCY | SPTYP_SUMMONING,
      SPFLAG_NONE,
      7,
-     NULL
+     200,
+     NULL,
+     false,
+    false
 },
 
 {
     SPELL_DETECT_ITEMS, "Detect Items",
      SPTYP_DIVINATION,
-     SPFLAG_NONE,
+     SPFLAG_MAPPING,
      2,
-     NULL
+     50,
+     NULL,
+     false,
+    false
 },
 
 {
@@ -707,7 +892,10 @@
      SPTYP_NECROMANCY,
      SPFLAG_NONE,
      5,
-     NULL
+     200,
+     NULL,
+     false,
+    false
 },
 
 {
@@ -715,7 +903,10 @@
      SPTYP_FIRE,
      SPFLAG_NONE,
      1,
-     NULL
+     25,
+     NULL,
+     false,
+    false
 },
 
 {
@@ -723,7 +914,10 @@
      SPTYP_ICE,
      SPFLAG_NONE,
      1,
-     NULL
+     25,
+     NULL,
+     false,
+    false
 },
 
 {
@@ -731,7 +925,10 @@
      SPTYP_SUMMONING,
      SPFLAG_NONE,
      4,
-     NULL
+     200,
+     NULL,
+     false,
+    false
 },
 
 {
@@ -739,7 +936,9 @@
      SPTYP_ICE,
      SPFLAG_NONE,
      5,
-     NULL
+     200,
+     NULL,
+     true
 },
 
 {
@@ -747,7 +946,9 @@
      SPTYP_CONJURATION | SPTYP_FIRE,
      SPFLAG_DIR_OR_TARGET,
      4,
-     NULL
+     100,
+     NULL,
+     true
 },
 
 {
@@ -755,7 +956,10 @@
      SPTYP_ICE | SPTYP_SUMMONING,
      SPFLAG_NONE,
      5,
-     NULL
+     200,
+     NULL,
+     false,
+    false
 },
 
 {
@@ -763,7 +967,10 @@
      SPTYP_ENCHANTMENT | SPTYP_ICE,
      SPFLAG_NONE,
      3,
-     NULL
+     200,
+     NULL,
+     false,
+    false
 },
 
 {
@@ -771,7 +978,10 @@
      SPTYP_SUMMONING,
      SPFLAG_UNHOLY,
      3,
-     NULL
+     200,
+     NULL,
+     false,
+    false
 },
 
 {
@@ -779,7 +989,10 @@
      SPTYP_ENCHANTMENT | SPTYP_AIR,
      SPFLAG_NONE,
      2,
-     NULL
+     200,
+     NULL,
+     false,
+    false
 },
 
 {
@@ -787,7 +1000,10 @@
      SPTYP_ENCHANTMENT,
      SPFLAG_NONE,
      3,
-     NULL
+     0,
+     NULL,
+     false,
+    false
 },
 
 {
@@ -795,7 +1011,9 @@
      SPTYP_NECROMANCY,
      SPFLAG_DIR_OR_TARGET,
      4,
-     NULL
+     100,
+     NULL,
+     true
 },
 
 {
@@ -803,7 +1021,10 @@
      SPTYP_HOLY,
      SPFLAG_NONE,
      7,
-     NULL
+     200,
+     NULL,
+     false,
+    false
 },
 
 {
@@ -811,14 +1032,20 @@
      SPTYP_HOLY,
      SPFLAG_NONE,
      4,
-     NULL
+     200,
+     NULL,
+     false,
+    false
 },
 
 {
     SPELL_THUNDERBOLT, "Thunderbolt",
      SPTYP_HOLY | SPTYP_AIR,
      SPFLAG_DIR_OR_TARGET,
-     6         // why is this the only holy spell with a secondary? {dlb}
+     6,         // why is this the only holy spell with a secondary? {dlb}
+     200,
+     NULL,
+     true
 }
 ,
 
@@ -827,7 +1054,9 @@
      SPTYP_HOLY,
      SPFLAG_DIR_OR_TARGET,
      8,
-     NULL
+     200,
+     NULL,
+     true
 },
 
 {
@@ -835,7 +1064,10 @@
      SPTYP_HOLY,
      SPFLAG_NONE,
      7,
-     NULL
+     200,
+     NULL,
+     false,
+    false
 },
 
 {
@@ -843,7 +1075,10 @@
      SPTYP_HOLY,
      SPFLAG_NONE,
      8,
-     NULL
+     200,
+     NULL,
+     false,
+    false
 },
 
 {
@@ -851,7 +1086,10 @@
      SPTYP_HOLY,
      SPFLAG_NONE,
      4,
-     NULL
+     200,
+     NULL,
+     false,
+    false
 },
 
 {
@@ -859,7 +1097,10 @@
      SPTYP_NECROMANCY,
      SPFLAG_NONE,
      5,
-     NULL
+     200,
+     NULL,
+     false,
+    false
 },
 
 {
@@ -867,7 +1108,10 @@
      SPTYP_ENCHANTMENT | SPTYP_NECROMANCY,
      SPFLAG_NONE,
      3,
-     NULL
+     200,
+     NULL,
+     false,
+    false
 },
 
 {
@@ -875,7 +1119,9 @@
      SPTYP_NECROMANCY,
      SPFLAG_DIR_OR_TARGET,
      3,
-     NULL
+     200,
+     NULL,
+     true
 },
 
 {
@@ -883,7 +1129,9 @@
      SPTYP_TRANSLOCATION,
      SPFLAG_DIR_OR_TARGET | SPFLAG_UNHOLY,
      5,
-     NULL
+     200,
+     NULL,
+     true
 },
 
 {
@@ -891,7 +1139,10 @@
      SPTYP_TRANSMIGRATION | SPTYP_NECROMANCY,
      SPFLAG_DIR_OR_TARGET | SPFLAG_NOT_SELF,
      5,
-     NULL
+     200,
+     NULL,
+     false,
+    false
 },
 
 {
@@ -899,7 +1150,9 @@
      SPTYP_CONJURATION | SPTYP_POISON,
      SPFLAG_DIR_OR_TARGET,
      1,
-     NULL
+     25,
+     NULL,
+     true
 },
 
 {
@@ -907,7 +1160,10 @@
      SPTYP_NECROMANCY,
      SPFLAG_NONE,
      2,
-     NULL
+     200,
+     NULL,
+     false,
+    false
 },
 
 {
@@ -915,7 +1171,10 @@
      SPTYP_ENCHANTMENT,
      SPFLAG_NONE,
      3,
-     NULL
+     200,
+     NULL,
+     false,
+    false
 },
 
 {
@@ -923,7 +1182,9 @@
      SPTYP_CONJURATION | SPTYP_FIRE,
      SPFLAG_DIR_OR_TARGET | SPFLAG_UNHOLY,
      9,
-     NULL
+     200,
+     NULL,
+     true
 },
 
 {
@@ -931,7 +1192,10 @@
      SPTYP_SUMMONING,
      SPFLAG_UNHOLY,
      5,
-     NULL
+     200,
+     NULL,
+     false,
+    false
 },
 
 {
@@ -939,7 +1203,10 @@
      SPTYP_SUMMONING,
      SPFLAG_UNHOLY,
      6,
-     NULL
+     200,
+     NULL,
+     false,
+    false
 },
 
 {
@@ -947,7 +1214,10 @@
      SPTYP_SUMMONING,
      SPFLAG_UNHOLY,
      7,
-     NULL
+     200,
+     NULL,
+     false,
+    false
 },
 
 {
@@ -955,7 +1225,10 @@
      SPTYP_NECROMANCY,
      SPFLAG_NONE,
      2,
-     NULL
+     0,
+     NULL,
+     false,
+    false
 },
 
 {
@@ -963,7 +1236,10 @@
      SPTYP_ENCHANTMENT,
      SPFLAG_NONE,
      2,
-     NULL
+     200,
+     NULL,
+     false,
+    false
 },
 
 {
@@ -971,7 +1247,10 @@
      SPTYP_ENCHANTMENT | SPTYP_FIRE,
      SPFLAG_NONE,
      2,
-     NULL
+     200,
+     NULL,
+     false,
+    false
 },
 
 {
@@ -979,7 +1258,10 @@
      SPTYP_ENCHANTMENT | SPTYP_ICE,
      SPFLAG_NONE,
      2,
-     NULL
+     200,
+     NULL,
+     false,
+    false
 },
 
 {
@@ -987,7 +1269,10 @@
      SPTYP_ENCHANTMENT | SPTYP_NECROMANCY,
      SPFLAG_NONE,
      2,
-     NULL
+     200,
+     NULL,
+     false,
+    false
 },
 
 {
@@ -995,7 +1280,10 @@
      SPTYP_EARTH,
      SPFLAG_DIR | SPFLAG_NOT_SELF,
      1,
-     NULL
+     25,
+     NULL,
+     false,
+    false
 },
 
 {
@@ -1003,7 +1291,9 @@
      SPTYP_CONJURATION | SPTYP_EARTH,
      SPFLAG_DIR_OR_TARGET,
      6,
-     NULL
+     200,
+     NULL,
+     true
 },
 
 {
@@ -1011,14 +1301,20 @@
      SPTYP_CONJURATION | SPTYP_EARTH,
      SPFLAG_DIR_OR_TARGET,
      3,
-     NULL
+     50,
+     NULL,
+     true
 },
 
 {
     SPELL_TOMB_OF_DOROKLOHE, "Tomb of Doroklohe",
      SPTYP_CONJURATION | SPTYP_EARTH, // conj makes more sense than tmig -- bwr
      SPFLAG_NONE,
-     7 
+     7,
+     0,
+     NULL,
+     false,
+    false
 }
 ,
 
@@ -1027,7 +1323,10 @@
      SPTYP_ENCHANTMENT | SPTYP_EARTH,
      SPFLAG_NONE,
      6,
-     NULL
+     200,
+     NULL,
+     false,
+    false
 },
 
 {
@@ -1035,7 +1334,9 @@
      SPTYP_CONJURATION | SPTYP_AIR,
      SPFLAG_DIR_OR_TARGET,
      1,
-     NULL
+     25,
+     NULL,
+     true
 },
 
 {
@@ -1043,7 +1344,10 @@
      SPTYP_ENCHANTMENT | SPTYP_AIR,
      SPFLAG_NONE,
      2,
-     NULL
+     200,
+     NULL,
+     false,
+    false
 },
 
 {
@@ -1051,7 +1355,10 @@
      SPTYP_ENCHANTMENT | SPTYP_AIR,
      SPFLAG_NONE,
      4,
-     NULL
+     200,
+     NULL,
+     false,
+    false
 },
 
 {
@@ -1059,7 +1366,10 @@
      SPTYP_ENCHANTMENT | SPTYP_AIR,
      SPFLAG_NONE,
      4,
-     NULL
+     200,
+     NULL,
+     false,
+    false
 },
 
 {
@@ -1067,22 +1377,31 @@
      SPTYP_CONJURATION | SPTYP_AIR,
      SPFLAG_DIR_OR_TARGET,
      7,
-     NULL
+     200,
+     NULL,
+     true
 },
 
 {
     SPELL_DETECT_CREATURES, "Detect Creatures",
      SPTYP_DIVINATION,
-     SPFLAG_NONE,
+     SPFLAG_MAPPING,
      2,
-     NULL
+     60,                        // not 50, note the fuzz
+     NULL,
+     false,
+    false
 },
 
 {
     SPELL_CURE_POISON_II, "Cure Poison",
      SPTYP_POISON,
      SPFLAG_NONE,
-     2
+     2,
+     200,
+     NULL,
+     false,
+    false
 }
 ,
 
@@ -1090,15 +1409,22 @@
     SPELL_CONTROL_TELEPORT, "Control Teleport",
      SPTYP_ENCHANTMENT | SPTYP_TRANSLOCATION,
      SPFLAG_NONE,
-     5,
-     NULL
+     4,
+     200,
+     NULL,
+     false,
+    false
 },
 
 {
     SPELL_POISON_AMMUNITION, "Poison Ammunition",
      SPTYP_ENCHANTMENT | SPTYP_POISON,
      SPFLAG_NONE,
-     4 // jmf: SPTYP_TRANSMIGRATION vs SPTYP_ENCHANTMENT?
+     4, // jmf: SPTYP_TRANSMIGRATION vs SPTYP_ENCHANTMENT?
+     0,
+     NULL,
+     false,
+    false
 }
 ,
 
@@ -1107,7 +1433,10 @@
      SPTYP_ENCHANTMENT | SPTYP_POISON,
      SPFLAG_NONE,
      4,
-     NULL
+     0,
+     NULL,
+     false,
+    false
 },
 
 {
@@ -1115,7 +1444,10 @@
      SPTYP_ENCHANTMENT | SPTYP_POISON,
      SPFLAG_NONE,
      4,
-     NULL
+     200,
+     NULL,
+     false,
+    false
 },
 
 {
@@ -1123,7 +1455,10 @@
      SPTYP_ENCHANTMENT,
      SPFLAG_NONE,
      2,
-     NULL
+     0,
+     NULL,
+     false,
+    false
 },
 
 {
@@ -1131,7 +1466,10 @@
      SPTYP_TRANSMIGRATION,
      SPFLAG_NONE,
      7,
-     NULL
+     0,
+     NULL,
+     false,
+    false
 },
 
 {
@@ -1139,7 +1477,10 @@
      SPTYP_CONJURATION,
      SPFLAG_DIR_OR_TARGET,
      7,
-     NULL
+     100,
+     NULL,
+     false,
+    false
 },
 
 {
@@ -1147,7 +1488,10 @@
      SPTYP_SUMMONING | SPTYP_TRANSLOCATION,
      SPFLAG_NONE,
      3,
-     NULL
+     0,
+     NULL,
+     false,
+    false
 },
 
 {
@@ -1155,7 +1499,10 @@
      SPTYP_TRANSLOCATION,
      SPFLAG_NONE,
      7,
-     NULL
+     0,
+     NULL,
+     false,
+    false
 },
 
 {
@@ -1163,7 +1510,10 @@
      SPTYP_NECROMANCY,
      SPFLAG_DIR_OR_TARGET | SPFLAG_NOT_SELF,
      5,
-     NULL
+     200,
+     NULL,
+     false,
+    false
 },
 
 {
@@ -1171,7 +1521,10 @@
      SPTYP_TRANSMIGRATION | SPTYP_POISON,
      SPFLAG_NONE,
      3,
-     NULL
+     200,
+     NULL,
+     false,
+    false
 },
 
 {
@@ -1179,7 +1532,10 @@
      SPTYP_TRANSMIGRATION,
      SPFLAG_DIR_OR_TARGET,
      1,
-     NULL
+     25,
+     NULL,
+     false,
+    false
 },
 
 {
@@ -1187,7 +1543,9 @@
      SPTYP_TRANSMIGRATION,
      SPFLAG_DIR_OR_TARGET | SPFLAG_NOT_SELF,
      6,
-     NULL
+     200,
+     NULL,
+     true
 },
 
 {
@@ -1195,7 +1553,10 @@
      SPTYP_TRANSMIGRATION,
      SPFLAG_NONE,
      5,  // only removes weapon, so I raised this from 4 -- bwr
-     NULL
+     200,
+     NULL,
+     false,
+    false
 },
 
 {
@@ -1203,7 +1564,10 @@
      SPTYP_TRANSMIGRATION | SPTYP_EARTH,
      SPFLAG_NONE,
      6,
-     NULL
+     200,
+     NULL,
+     false,
+    false
 },
 
 {
@@ -1211,7 +1575,10 @@
      SPTYP_ICE | SPTYP_TRANSMIGRATION,
      SPFLAG_NONE,
      4, // doesn't allow for equipment, so I lowered this from 5 -- bwr
-     NULL
+     200,
+     NULL,
+     false,
+    false
 },
 
 {
@@ -1219,7 +1586,10 @@
      SPTYP_FIRE | SPTYP_TRANSMIGRATION,
      SPFLAG_NONE,
      8,
-     NULL
+     200,
+     NULL,
+     false,
+    false
 },
 
 {
@@ -1227,7 +1597,10 @@
      SPTYP_TRANSMIGRATION | SPTYP_NECROMANCY,
      SPFLAG_NONE,
      8,
-     NULL
+     200,
+     NULL,
+     false,
+    false
 },
 
 {
@@ -1235,7 +1608,10 @@
      SPTYP_NECROMANCY,
      SPFLAG_NONE,
      9,
-     NULL
+     200,
+     NULL,
+     false,
+    false
 },
 
 {
@@ -1243,7 +1619,10 @@
      SPTYP_NECROMANCY,
      SPFLAG_NONE,
      6,
-     NULL
+     0,
+     NULL,
+     false,
+    false
 },
 
 {
@@ -1251,7 +1630,10 @@
      SPTYP_ENCHANTMENT | SPTYP_AIR,
      SPFLAG_NONE,
      6,
-     NULL
+     200,
+     NULL,
+     false,
+    false
 },
 
 {
@@ -1259,7 +1641,9 @@
      SPTYP_CONJURATION | SPTYP_EARTH,
      SPFLAG_DIR_OR_TARGET,
      7,
-     NULL
+     200,
+     NULL,
+     true
 },
 
 {
@@ -1267,7 +1651,9 @@
      SPTYP_CONJURATION | SPTYP_ICE,
      SPFLAG_DIR_OR_TARGET,
      4,
-     NULL
+     100,
+     NULL,
+     true
 },
 
 {
@@ -1275,7 +1661,9 @@
      SPTYP_CONJURATION | SPTYP_ICE,
      SPFLAG_DIR_OR_TARGET,
      9,
-     NULL
+     200,
+     NULL,
+     true
 },
 
 {
@@ -1283,7 +1671,10 @@
      SPTYP_AIR,
      SPFLAG_NONE,
      1,
-     NULL
+     25,
+     NULL,
+     false,
+    false
 },
 
 {
@@ -1291,14 +1682,20 @@
      SPTYP_AIR,
      SPFLAG_TARGET | SPFLAG_NOT_SELF,
      4,
-     NULL
+     200,
+     NULL,
+     false,
+    false
 },
 
 {
     SPELL_SHADOW_CREATURES, "Shadow Creatures",
      SPTYP_SUMMONING,  // jmf: or SPTYP_SUMMONING | SPTYP_CONJURATION
      SPFLAG_NONE,
-     5
+     5,
+     0,
+     NULL,
+     false
 }
 ,
 
@@ -1307,7 +1704,10 @@
      SPTYP_ENCHANTMENT,
      SPFLAG_NONE,
      1,
-     NULL
+     200,
+     NULL,
+     false,
+    false
 },
 
 {
@@ -1315,7 +1715,10 @@
      SPTYP_ENCHANTMENT,
      SPFLAG_NONE,
      2,
-     NULL
+     200,
+     NULL,
+     false,
+    false
 },
 
 
@@ -1328,7 +1731,10 @@
      SPTYP_CONJURATION | SPTYP_FIRE,
      SPFLAG_DIR_OR_TARGET | SPFLAG_NOT_SELF,
      1,
-     NULL
+     25,
+     NULL,
+     false,
+    false
 },
 
 {
@@ -1336,7 +1742,10 @@
      SPTYP_TRANSMIGRATION | SPTYP_EARTH,
      SPFLAG_NONE,
      3,
-     NULL
+     200,
+     NULL,
+     false,
+    false
 },
 
 {
@@ -1344,7 +1753,10 @@
      SPTYP_FIRE | SPTYP_TRANSMIGRATION,
      SPFLAG_NONE,
      7,
-     NULL
+     200,
+     NULL,
+     false,
+    false
 },
 
 {
@@ -1352,7 +1764,10 @@
      SPTYP_TRANSMIGRATION | SPTYP_SUMMONING,
      SPFLAG_NONE,
      2,
-     NULL
+     200,
+     NULL,
+     false,
+    false
 },
 
 {
@@ -1360,7 +1775,10 @@
      SPTYP_SUMMONING,
      SPFLAG_NONE,
      3,
-     NULL
+     200,
+     NULL,
+     false,
+    false
 },
 
 {
@@ -1368,7 +1786,10 @@
      SPTYP_FIRE | SPTYP_SUMMONING,
      SPFLAG_NONE,
      9,
-     NULL
+     200,
+     NULL,
+     false,
+    false
 },
 
 {
@@ -1376,7 +1797,10 @@
      SPTYP_ENCHANTMENT,
      SPFLAG_NONE,
      5,
-     NULL
+     200,
+     NULL,
+     false,
+    false
 },
 
 {
@@ -1384,7 +1808,9 @@
      SPTYP_ENCHANTMENT | SPTYP_ICE,
      SPFLAG_DIR_OR_TARGET | SPFLAG_NOT_SELF,
      2,
-     NULL
+     56,
+     NULL,
+     true
 },
 
 {
@@ -1392,7 +1818,10 @@
      SPTYP_ENCHANTMENT | SPTYP_ICE,
      SPFLAG_NONE,
      7,
-     NULL
+     200,
+     NULL,
+     false,
+    false
 },
 
 {
@@ -1400,7 +1829,10 @@
      SPTYP_DIVINATION,
      SPFLAG_NONE,
      1,
-     NULL
+     0,
+     NULL,
+     false,
+    false
 },
 
 {
@@ -1408,7 +1840,10 @@
      SPTYP_DIVINATION,
      SPFLAG_NONE,
      1,
-     NULL
+     200,
+     NULL,
+     false,
+    false
 },
 
 {
@@ -1416,7 +1851,10 @@
      SPTYP_ENCHANTMENT | SPTYP_DIVINATION,
      SPFLAG_NONE,
      4,
-     NULL
+     200,
+     NULL,
+     false,
+    false
 },
 
 {
@@ -1424,7 +1862,10 @@
      SPTYP_DIVINATION,
      SPFLAG_NONE,
      5,
-     NULL
+     200,
+     NULL,
+     false,
+    false
 },
 
 {
@@ -1432,7 +1873,10 @@
      SPTYP_SUMMONING,
      SPFLAG_NONE,
      1,
-     NULL
+     200,
+     NULL,
+     false,
+    false
 },
 
 {
@@ -1440,7 +1884,10 @@
      SPTYP_ENCHANTMENT | SPTYP_NECROMANCY,
      SPFLAG_NONE,
      5,     // fairly high level - potentially one of the best brands
-     NULL
+     200,
+     NULL,
+     false,
+    false
 },
 
 {
@@ -1448,7 +1895,10 @@
      SPTYP_ENCHANTMENT | SPTYP_TRANSLOCATION,
      SPFLAG_NONE,
      7,     // this is high for a reason - Warp brands are very powerful.
-     NULL
+     0,
+     NULL,
+     false,
+    false
 },
 
 {
@@ -1456,7 +1906,10 @@
      SPTYP_ENCHANTMENT | SPTYP_AIR,
      SPFLAG_NONE,
      5,
-     NULL
+     200,
+     NULL,
+     false,
+    false
 },
 
 {
@@ -1464,7 +1917,10 @@
      SPTYP_TRANSMIGRATION | SPTYP_EARTH,
      SPFLAG_NONE,
      9,
-     NULL
+     200,
+     NULL,
+     false,
+    false
 },
 
 {
@@ -1472,7 +1928,10 @@
      SPTYP_TRANSLOCATION,
      SPFLAG_NONE,
      7,
-     NULL
+     200,
+     NULL,
+     false,
+    false
 },
 
 {
@@ -1480,7 +1939,10 @@
      SPTYP_CONJURATION | SPTYP_AIR,
      SPFLAG_NONE,
      4,
-     NULL
+     200,
+     NULL,
+     false,
+    false
 },
 
 {
@@ -1488,7 +1950,10 @@
      SPTYP_TRANSLOCATION,
      SPFLAG_NONE,
      1,
-     NULL
+     100,
+     NULL,
+     false,
+    false
 },
 
 {
@@ -1496,7 +1961,9 @@
      SPTYP_ENCHANTMENT,
      SPFLAG_DIR_OR_TARGET | SPFLAG_NOT_SELF,
      1,
-     NULL
+     200,
+     NULL,
+     true
 },
 
 {
@@ -1504,7 +1971,10 @@
      SPTYP_TRANSMIGRATION | SPTYP_POISON,
      SPFLAG_NONE,
      4,
-     NULL
+     0,
+     NULL,
+     false,
+    false
 },
 
 {
@@ -1512,6 +1982,7 @@
      SPTYP_ENCHANTMENT,
      SPFLAG_NONE,
      5,
+     200,
      NULL
 },
 
@@ -1520,7 +1991,9 @@
      SPTYP_FIRE | SPTYP_TRANSMIGRATION,
      SPFLAG_NONE,
      2,   // XXX: level 2 or 3, what should it be now? -- bwr
-     NULL
+     200,
+     NULL,
+     true
 },
 
 {
@@ -1528,7 +2001,10 @@
      SPTYP_TRANSMIGRATION | SPTYP_EARTH,
      SPFLAG_NONE,
      4,
-     NULL
+     0,
+     NULL,
+     false,
+    false
 },
 
 {
@@ -1536,7 +2012,10 @@
      SPTYP_TRANSMIGRATION | SPTYP_EARTH,
      SPFLAG_NONE,
      5,
-     NULL
+     200,
+     NULL,
+     false,
+    false
 },
 
 {
@@ -1544,7 +2023,10 @@
      SPTYP_TRANSMIGRATION | SPTYP_AIR,
      SPFLAG_NONE,
      9,
-     NULL
+     200,
+     NULL,
+     false,
+    false
 },
 
 {
@@ -1552,7 +2034,9 @@
      SPTYP_TRANSMIGRATION | SPTYP_EARTH,
      SPFLAG_DIR_OR_TARGET | SPFLAG_NOT_SELF,
      1,
-     NULL
+     50,
+     NULL,
+     true
 },
 
 {
@@ -1560,7 +2044,10 @@
      SPTYP_TRANSMIGRATION | SPTYP_NECROMANCY,
      SPFLAG_NONE,
      5,
-     NULL
+     200,
+     NULL,
+     false,
+    false
 },
 
 {
@@ -1568,7 +2055,10 @@
      SPTYP_TRANSMIGRATION | SPTYP_EARTH,
      SPFLAG_NONE,
      2,
-     NULL
+     200,
+     NULL,
+     false,
+    false
 },
 
 {
@@ -1576,7 +2066,10 @@
      SPTYP_ICE | SPTYP_TRANSMIGRATION,
      SPFLAG_NONE,
      4,
-     NULL
+     200,
+     NULL,
+     false,
+    false
 },
 
 {
@@ -1584,7 +2077,10 @@
      SPTYP_TRANSLOCATION,
      SPFLAG_NONE,
      3,
-     NULL
+     100,
+     NULL,
+     false,
+    false
 },
 
 {
@@ -1592,7 +2088,10 @@
     SPTYP_EARTH | SPTYP_TRANSMIGRATION, // was ench -- bwr
     SPFLAG_NONE,
     2,
-     NULL
+    200,
+     NULL,
+     false,
+    false
 },
 
 {
@@ -1600,7 +2099,10 @@
     SPTYP_ICE | SPTYP_NECROMANCY,
     SPFLAG_NONE,
     6,
-     NULL
+    200,
+     NULL,
+     false,
+    false
 },
 
 {
@@ -1608,7 +2110,10 @@
     SPTYP_AIR | SPTYP_CONJURATION,
     SPFLAG_NONE,
     8,
-     NULL
+    200,
+     NULL,
+     false,
+    false
 },
 
 {
@@ -1616,7 +2121,10 @@
     SPTYP_AIR | SPTYP_CONJURATION,
     SPFLAG_NONE,
     8,
-     NULL
+    200,
+     NULL,
+     false,
+    false
 },
 
 {
@@ -1624,7 +2132,10 @@
     SPTYP_FIRE | SPTYP_CONJURATION,
     SPFLAG_NONE,
     7,
-     NULL
+    0,
+     NULL,
+     false,
+    false
 },
 
 {
@@ -1632,7 +2143,10 @@
     SPTYP_TRANSMIGRATION | SPTYP_NECROMANCY,
     SPFLAG_NONE,
     1,
-     NULL
+    50,
+     NULL,
+     false,
+    false
 },
 
 {
@@ -1640,7 +2154,20 @@
     SPTYP_CONJURATION | SPTYP_POISON,
     SPFLAG_DIR_OR_TARGET,
     6,
-     NULL
+    200,
+    NULL,
+    true
+},
+
+{
+    SPELL_PORTALED_PROJECTILE, "Portaled Projectile",
+        SPTYP_TRANSLOCATION,
+        SPFLAG_TARGET,
+        2,
+        50,
+        NULL,
+        false,
+        false
 },
 
 {
@@ -1648,7 +2175,187 @@
     0,
     SPFLAG_DIR_OR_TARGET,
     1,
-     NULL
+    25,
+    NULL,
+    true
+},
+
+{
+    SPELL_HELLFIRE_BURST, "Hellfire Burst",
+     SPTYP_CONJURATION | SPTYP_FIRE,
+     SPFLAG_DIR_OR_TARGET | SPFLAG_UNHOLY,
+     9,
+     200,
+     NULL,
+     false,
+    false 
+},
+
+{
+    SPELL_VAMPIRE_SUMMON, "Vampire Summon",
+    SPTYP_SUMMONING,
+    SPFLAG_UNHOLY,
+    3,
+    0,
+    NULL,
+    false,
+    false
+},
+
+{
+    SPELL_BRAIN_FEED, "Brain Feed",
+    SPTYP_NECROMANCY,
+    SPFLAG_UNHOLY,
+    3,
+    0,
+    NULL,
+    false,
+    false
+},
+
+{
+    SPELL_FAKE_RAKSHASA_SUMMON, "Rakshasa Summon",
+    SPTYP_SUMMONING | SPTYP_NECROMANCY,
+    SPFLAG_UNHOLY,
+    3,
+    0,
+    NULL,
+    false
+},
+
+{
+    SPELL_STEAM_BALL, "Steam Ball",
+    SPTYP_CONJURATION | SPTYP_FIRE,
+    SPFLAG_DIR_OR_TARGET,
+    4,
+    0,
+    NULL,
+    true
+},
+
+{
+    SPELL_SUMMON_UFETUBUS, "Summon Ufetubus",
+    SPTYP_SUMMONING,
+    SPFLAG_UNHOLY,
+    4,
+    0,
+    NULL,
+    false,
+    false
+},
+
+{
+    SPELL_SUMMON_BEAST, "Summon Beast",
+    SPTYP_SUMMONING,
+    SPFLAG_NONE,
+    4,
+    0,
+    NULL,
+    false
+},
+
+{
+    SPELL_ENERGY_BOLT, "Energy Bolt",
+    SPTYP_CONJURATION,
+    SPFLAG_DIR_OR_TARGET,
+    4,
+    0,
+    NULL,
+    true
+},
+
+{
+    SPELL_POISON_SPLASH, "Poison Splash",
+    SPTYP_POISON,
+    SPFLAG_DIR_OR_TARGET,
+    2,
+    0,
+    NULL,
+    true
+},
+
+{
+    SPELL_SUMMON_UNDEAD, "Summon Undead",
+    SPTYP_SUMMONING | SPTYP_NECROMANCY,
+    SPFLAG_NONE,
+    7,
+    0,
+    NULL,
+    false,
+    false,
+},
+
+{
+    SPELL_CANTRIP, "Cantrip",
+    SPTYP_NONE,
+    SPFLAG_NONE,
+    1,
+    0,
+    NULL,
+    false,
+    false
+},
+
+{
+    SPELL_QUICKSILVER_BOLT, "Quicksilver Bolt",
+    SPTYP_CONJURATION,
+    SPFLAG_DIR_OR_TARGET,
+    5,
+    0,
+    NULL,
+    true
+},
+
+{
+    SPELL_METAL_SPLINTERS, "Metal Splinters",
+    SPTYP_CONJURATION,
+    SPFLAG_DIR_OR_TARGET,
+    5,
+    0,
+    NULL,
+    true
+},
+
+{
+    SPELL_MIASMA, "Miasma",
+    SPTYP_CONJURATION | SPTYP_NECROMANCY,
+    SPFLAG_DIR_OR_TARGET,
+    6,
+    0,
+    NULL,
+    true
+},
+
+{
+    SPELL_SUMMON_DRAKES, "Summon Drakes",
+    SPTYP_SUMMONING,
+    SPFLAG_NONE,
+    6,
+    0,
+    NULL,
+    false,
+    false
+},
+
+{
+    SPELL_BLINK_OTHER, "Blink Other",
+    SPTYP_TRANSLOCATION,
+    SPFLAG_NONE,
+    2,
+    0,
+    NULL,
+    true
+},
+
+{
+    SPELL_SUMMON_MUSHROOMS, "Summon Mushrooms",
+    SPTYP_SUMMONING,
+    SPFLAG_NONE,
+    4,
+    0,
+    NULL,
+    false,
+    false
 },
 
 {
@@ -1656,8 +2363,10 @@
      0,
      0,
      0,
-     NULL
+     0,
+     NULL,
+     false,
+     false
 },
 
-
 #endif
