@@ -3,7 +3,7 @@
  *  Summary:    data handlers for player-avilable spell list         *
  *  Written by: don brodale <dbrodale@bigfootinteractive.com>        *
  *                                                                   *
- *  Modified for Crawl Reference by $Author: dshaligram $ on $Date: 2007-04-05T16:46:35.024555Z $
+ *  Modified for Crawl Reference by $Author: ennewalker $ on $Date: 2007-11-23T02:25:44.246247Z $
  *
  *  Changelog(most recent first):                                    *
  *
@@ -23,6 +23,7 @@
 
 #include "externs.h"
 
+#include "beam.h"
 #include "direct.h"
 #include "debug.h"
 #include "stuff.h"
@@ -33,6 +34,9 @@
 #include "notes.h"
 #include "player.h"
 #include "spl-book.h"
+#include "spl-cast.h"
+#include "spl-util.h"
+#include "terrain.h"
 #include "view.h"
 
 
@@ -41,45 +45,62 @@
 #endif
 
 
-static struct playerspell spelldata[] = {
+static struct spell_desc spelldata[] = {
 #include "spl-data.h"
 };
 
-static int plyrspell_list[NUM_SPELLS];
+static int spell_list[NUM_SPELLS];
 
-#define PLYRSPELLDATASIZE (sizeof(spelldata)/sizeof(struct playerspell))
+#define SPELLDATASIZE (sizeof(spelldata)/sizeof(struct spell_desc))
 
-static struct playerspell *seekspell(int spellid);
-static bool cloud_helper( int (*func) (int, int, int, int, kill_category),
-                          int x, int y, 
-                          int pow, int ctype, kill_category );
+static struct spell_desc *seekspell(spell_type spellid);
+static bool cloud_helper(int (*func)(int, int, int, cloud_type, kill_category),
+                         int x, int y, 
+                         int pow, cloud_type ctype, kill_category );
 
 /*
  *             BEGIN PUBLIC FUNCTIONS
  */
 
 // all this does is merely refresh the internal spell list {dlb}:
-void init_playerspells(void)
+void init_spell_descs(void)
 {
-    unsigned int x = 0;
+    for (int i = 0; i < NUM_SPELLS; i++)
+        spell_list[i] = -1;
 
-    for (x = 0; x < NUM_SPELLS; x++)
-        plyrspell_list[x] = -1;
-
-    // can only use up to PLYRSPELLDATASIZE _MINUS ONE_, or the
-    // last entry tries to set plyrspell_list[SPELL_NO_SPELL] 
+    // can only use up to SPELLDATASIZE _MINUS ONE_, or the
+    // last entry tries to set spell_list[SPELL_NO_SPELL] 
     // which corrupts the heap.
-    for (x = 0; x < PLYRSPELLDATASIZE - 1; x++)
-        plyrspell_list[spelldata[x].id] = x;
+    for (unsigned int i = 0; i < SPELLDATASIZE - 1; i++)
+        spell_list[spelldata[i].id] = i;
+}                               // end init_spell_descs()
 
-    for (x = 0; x < NUM_SPELLS; x++)
+spell_type spell_by_name(std::string name, bool partial_match)
+{
+    if (name.empty())
+        return (SPELL_NO_SPELL);
+
+    lowercase(name);
+
+    int spellmatch = -1;
+    for (int i = 0; i < NUM_SPELLS; i++)
     {
-        if (plyrspell_list[x] == -1)
-            plyrspell_list[x] = plyrspell_list[SPELL_NO_SPELL];
+        spell_type type = static_cast<spell_type>(i);
+        const char *sptitle = spell_title(type);
+        if (!sptitle)
+            continue;
+
+        const std::string spell_name = lowercase_string(sptitle);
+        if (name == spell_name)
+            return (type);
+
+        if (partial_match && spell_name.find(name) != std::string::npos)
+            spellmatch = i;
     }
 
-    return;                     // return value should not matter here {dlb}
-}                               // end init_playerspells()
+    return (spellmatch != -1? static_cast<spell_type>(spellmatch)
+            : SPELL_NO_SPELL);
+}
 
 int get_spell_slot_by_letter( char letter )
 {
@@ -93,7 +114,7 @@ int get_spell_slot_by_letter( char letter )
     return (you.spell_letter_table[index]);
 }
 
-int get_spell_by_letter( char letter )
+spell_type get_spell_by_letter( char letter )
 {
     ASSERT( isalpha( letter ) );
 
@@ -102,7 +123,7 @@ int get_spell_by_letter( char letter )
     return ((slot == -1) ? SPELL_NO_SPELL : you.spells[slot]);
 }
 
-bool add_spell_to_memory( int spell )
+bool add_spell_to_memory( spell_type spell )
 {
     int i, j;
 
@@ -150,46 +171,54 @@ bool del_spell_from_memory_by_slot( int slot )
 }
 
 
-int spell_hunger(int which_spell)
+int spell_hunger(spell_type which_spell)
 {
-    int level = seekspell(which_spell)->level;
+    const int level = spell_difficulty(which_spell);
 
-    switch (level)
-    {
-    case  1: return   50;
-    case  2: return   95;
-    case  3: return  160;
-    case  4: return  250;
-    case  5: return  350;
-    case  6: return  550;
-    case  7: return  700;
-    case  8: return  850;
-    case  9: return 1000;
-    case 10: return 1000;
-    case 11: return 1100;
-    case 12: return 1250;
-    case 13: return 1380;
-    case 14: return 1500;
-    case 15: return 1600;
-    default: return 1600 + (20 * level);
-    }
-}                               // end spell_hunger();
+    const int basehunger[] = {
+        50, 95, 160, 250, 350, 550, 700, 850, 1000
+    };
+
+    int hunger;
+
+    if ( level < 10 && level > 0 )
+        hunger = basehunger[level-1];
+    else
+        hunger = (basehunger[0] * level * level) / 4;
+
+    hunger -= you.intel * you.skills[SK_SPELLCASTING];
+
+    if ( hunger < 0 )
+        hunger = 0;
+
+    return hunger;
+}
+
+bool spell_needs_tracer(spell_type spell)
+{
+    return (seekspell(spell)->ms_needs_tracer);
+}
+
+bool spell_needs_foe(spell_type spell)
+{
+    return (!seekspell(spell)->ms_utility);
+}
 
 // applied to spell misfires (more power = worse) and triggers
 // for Xom acting (more power = more likely to grab his attention) {dlb}
-int spell_mana(int which_spell)
+int spell_mana(spell_type which_spell)
 {
     return (seekspell(which_spell)->level);
 }
 
 // applied in naughties (more difficult = higher level knowledge = worse)
 // and triggers for Sif acting (same reasoning as above, just good) {dlb}
-int spell_difficulty(int which_spell)
+int spell_difficulty(spell_type which_spell)
 {
     return (seekspell(which_spell)->level);
 }
 
-int spell_levels_required( int which_spell )
+int spell_levels_required( spell_type which_spell )
 {
     int levels = spell_difficulty( which_spell );
 
@@ -207,23 +236,23 @@ int spell_levels_required( int which_spell )
     return (levels);
 }
 
-unsigned int get_spell_flags( int which_spell )
+unsigned int get_spell_flags( spell_type which_spell )
 {
     return (seekspell(which_spell)->flags);
 }
 
-const char *get_spell_target_prompt( int which_spell )
+const char *get_spell_target_prompt( spell_type which_spell )
 {
     return (seekspell(which_spell)->target_prompt);
 }
 
-bool spell_typematch(int which_spell, unsigned int which_discipline)
+bool spell_typematch(spell_type which_spell, unsigned int which_discipline)
 {
     return (seekspell(which_spell)->disciplines & which_discipline);
 }
 
 //jmf: next two for simple bit handling
-unsigned int spell_type(int spell)
+unsigned int get_spell_disciplines(spell_type spell)
 {
     return (seekspell(spell)->disciplines);
 }
@@ -242,23 +271,10 @@ int count_bits(unsigned int bits)
     return (c);
 }
 
-// this will probably be used often, so rather than use malloc/free
-// (which may lead to memory fragmentation) I'll just use a static
-// array of characters -- if/when the String changeover takes place,
-// this will all shift, no doubt {dlb}
-/*
-  const char *spell_title( int which_spell )
-  {
-  static char this_title[41] = ""; // this is generous, to say the least {dlb}
-  strncpy(this_title, seekspell(which_spell)->title, 41);
-  // truncation better than overrun {dlb}
-  return ( this_title );
-  }         // end spell_title()
-*/
-
-const char *spell_title(int spell)      //jmf: ah the joys of driving ms. data
+const char *spell_title(spell_type spell)
 {
-    return (seekspell(spell)->title);
+    const spell_desc *spd = seekspell(spell);
+    return (spd? spd->title : NULL);
 }
 
 
@@ -369,7 +385,7 @@ int apply_random_around_square( int (*func) (int, int, int, int),
             // Found target
             count++;
 
-            // Slight differece here over the basic algorithm...
+            // Slight difference here over the basic algorithm...
             //
             // For cases where the number of choices <= max_targs it's
             // obvious (all available choices will be selected).
@@ -413,7 +429,7 @@ int apply_random_around_square( int (*func) (int, int, int, int),
             // probability is that the chosen elements have already
             // passed tests which verify that they *don't* belong
             // in slots m+1...m+k, so the only positions an already
-            // chosen element can end up in are it's original
+            // chosen element can end up in are its original
             // position (in one of the chosen slots), or in the
             // new slot.
             //
@@ -527,9 +543,10 @@ int apply_area_within_radius( int (*func) (int, int, int, int),
 // We really need some sort of a queue structure, since ideally I'd like
 // to do a (shallow) breadth-first-search of the dungeon floor.
 // This ought to work okay for small clouds.
-void apply_area_cloud( int (*func) (int, int, int, int, kill_category),
+void apply_area_cloud( int (*func) (int, int, int, cloud_type, kill_category),
                        int x, int y,
-                       int pow, int number, int ctype, kill_category whose )
+                       int pow, int number, cloud_type ctype,
+                       kill_category whose )
 {
     int spread, clouds_left = number;
     int good_squares = 0, neighbours[8] = { 0, 0, 0, 0, 0, 0, 0, 0 };
@@ -681,30 +698,33 @@ void apply_area_cloud( int (*func) (int, int, int, int, kill_category),
     }
 }                               // end apply_area_cloud()
 
-char spell_direction( struct dist &spelld, struct bolt &pbolt, 
-                      targeting_type restrict, int mode,
-                      const char *prompt )
+// Select a spell direction and fill dist and pbolt appropriately.
+// Return false if the user canceled, true otherwise.
+bool spell_direction( dist &spelld, bolt &pbolt, 
+                      targeting_type restrict, targ_mode_type mode,
+                      bool needs_path, const char *prompt )
 {
     if (restrict != DIR_DIR)
         message_current_target();
 
-    direction( spelld, restrict, mode, false, prompt );
+    direction( spelld, restrict, mode, false, needs_path, prompt );
 
     if (!spelld.isValid)
     {
         // check for user cancel
         canned_msg(MSG_OK);
-        return -1;
+        return false;
     }
 
     pbolt.set_target(spelld);
     pbolt.source_x = you.x_pos;
     pbolt.source_y = you.y_pos;
 
-    return 1;
+    return true;
 }                               // end spell_direction()
 
-const char* spelltype_short_name( int which_spelltype ) {
+const char* spelltype_short_name( int which_spelltype )
+{
     switch (which_spelltype)
     {
     case SPTYP_CONJURATION:
@@ -740,8 +760,6 @@ const char* spelltype_short_name( int which_spelltype ) {
 
 const char *spelltype_name(unsigned int which_spelltype)
 {
-    static char bug_string[80];
-
     switch (which_spelltype)
     {
     case SPTYP_CONJURATION:
@@ -771,17 +789,12 @@ const char *spelltype_name(unsigned int which_spelltype)
     case SPTYP_AIR:
         return ("Air");
     default:
-        snprintf( bug_string, sizeof(bug_string), 
-                  "invalid(%d)", which_spelltype );
-
-        return (bug_string);
+        return "Buggy";
     }
 }                               // end spelltype_name()
 
 int spell_type2skill(unsigned int spelltype)
 {
-    char buffer[80];
-
     switch (spelltype)
     {
     case SPTYP_CONJURATION:    return (SK_CONJURATIONS);
@@ -799,10 +812,10 @@ int spell_type2skill(unsigned int spelltype)
 
     default:
     case SPTYP_HOLY:
-        snprintf( buffer, sizeof(buffer), 
-                  "spell_type2skill: called with spelltype %d", spelltype );
-
-        mpr( buffer );
+#ifdef DEBUG_DIAGNOSTICS
+        mprf(MSGCH_DIAGNOSTICS, "spell_type2skill: called with spelltype %u",
+             spelltype );
+#endif
         return (-1);
     }
 }                               // end spell_type2skill()
@@ -815,15 +828,16 @@ int spell_type2skill(unsigned int spelltype)
  **************************************************
  */
 
-//jmf: simplified; moved init code to top function, init_playerspells()
-static struct playerspell *seekspell(int spell)
+//jmf: simplified; moved init code to top function, init_spell_descs()
+static spell_desc *seekspell(spell_type spell)
 {
-    return (&spelldata[plyrspell_list[spell]]);
+    const int index = spell_list[spell];
+    return (index != -1? &spelldata[index] : NULL);
 }
 
-static bool cloud_helper( int (*func) (int, int, int, int, kill_category),
-                          int x, int y, 
-                          int pow, int ctype, kill_category whose )
+static bool cloud_helper(int (*func)(int, int, int, cloud_type, kill_category),
+                         int x, int y, 
+                         int pow, cloud_type ctype, kill_category whose )
 {
     if (!grid_is_solid(grd[x][y]) && env.cgrid[x][y] == EMPTY_CLOUD)
     {
@@ -833,3 +847,8 @@ static bool cloud_helper( int (*func) (int, int, int, int, kill_category),
 
     return false;
 }
+
+int spell_power_cap(spell_type spell)
+{
+    return seekspell(spell)->power_cap;
+}
