@@ -3,13 +3,17 @@
  *  Summary:    Classes tracking player stashes
  *  Written by: Darshan Shaligram
  *
- *  Modified for Crawl Reference by $Author: dshaligram $ on $Date: 2007-03-06T05:56:50.150565Z $
+ *  Modified for Crawl Reference by $Author: ennewalker $ on $Date: 2007-11-23T02:25:44.246247Z $
  */
 
 #include "AppHdr.h"
+#include "branch.h"
 #include "chardump.h"
+#include "cio.h"
 #include "clua.h"
 #include "describe.h"
+#include "direct.h"
+#include "food.h"
 #include "itemname.h"
 #include "itemprop.h"
 #include "files.h"
@@ -18,32 +22,35 @@
 #include "Kills.h"
 #include "libutil.h"
 #include "menu.h"
+#include "message.h"
 #include "misc.h"
 #include "overmap.h"
+#include "place.h"
 #include "shopping.h"
 #include "spl-book.h"
 #include "stash.h"
 #include "stuff.h"
 #include "tags.h"
+#include "terrain.h"
+#include "traps.h"
 #include "travel.h"
 #include "tutorial.h"
+#include "view.h"
 
 #include <cctype>
 #include <cstdio>
 #include <fstream>
+#include <sstream>
 #include <algorithm>
 
 #define ST_MAJOR_VER ((unsigned char) 4)
-#define ST_MINOR_VER ((unsigned char) 6)
-
-#define LUA_SEARCH_ANNOTATE "ch_stash_search_annotate_item"
-#define LUA_DUMP_ANNOTATE   "ch_stash_dump_annotate_item"
-#define LUA_VIEW_ANNOTATE   "ch_stash_view_annotate_item"
+#define ST_MINOR_VER ((unsigned char) 7)
 
 void stash_init_new_level()
 {
     // If there's an existing stash level for Pan, blow it away.
     stashes.remove_level( level_id(LEVEL_PANDEMONIUM) );
+    stashes.remove_level( level_id(LEVEL_PORTAL_VAULT) );
 }
 
 std::string userdef_annotate_item(const char *s, const item_def *item,
@@ -53,11 +60,11 @@ std::string userdef_annotate_item(const char *s, const item_def *item,
     if (exclusive)
         lua_set_exclusive_item(item);
     std::string ann;
-    clua.callfn(s, "u>s", item, &ann);
+    if (!clua.callfn(s, "u>s", item, &ann) && !clua.error.empty())
+        mprf(MSGCH_WARN, "Lua error: %s", clua.error.c_str());
     if (exclusive)
         lua_set_exclusive_item(NULL);
     return (ann);
-    
 #else
     return ("");
 #endif
@@ -68,11 +75,11 @@ std::string stash_annotate_item(const char *s,
                                 bool exclusive = false)
 {
     std::string text = userdef_annotate_item(s, item, exclusive);
-    if ( (item->base_type == OBJ_BOOKS &&
-            item_type_known(*item) &&
-            item->sub_type != BOOK_MANUAL &&
-            item->sub_type != BOOK_DESTRUCTION)
-         || count_staff_spells(*item, true) > 1 )
+    if ((item->base_type == OBJ_BOOKS
+         && item_type_known(*item)
+         && item->sub_type != BOOK_MANUAL
+         && item->sub_type != BOOK_DESTRUCTION)
+        || count_staff_spells(*item, true) > 1)
     {
         formatted_string fs;
         item_def dup = *item;
@@ -108,7 +115,7 @@ void describe_stash(int x, int y)
         {
             std::string desc = "[Stash: " 
                        + s->description() + "]";
-            mpr(desc.c_str());
+            mpr(desc.c_str(), MSGCH_EXAMINE_FILTER);
         }
     }
 }
@@ -143,7 +150,7 @@ static void save_item(FILE *file, const item_def &item)
 
 static void load_item(FILE *file, item_def &item)
 {
-    item.base_type  = readByte(file);
+    item.base_type  = static_cast<object_class_type>(readByte(file));
     item.sub_type   = readByte(file);
     item.plus       = readShort(file);
     item.plus2      = readShort(file);
@@ -212,11 +219,12 @@ void Stash::filter(const std::string &str)
         base = base.substr(0, cpos);
     }
 
-    unsigned char basec = atoi(base.c_str());
+    const object_class_type basec =
+        static_cast<object_class_type>(atoi(base.c_str()));
     filter(basec, subc);
 }
 
-void Stash::filter(unsigned char base, unsigned char sub)
+void Stash::filter(object_class_type base, unsigned char sub)
 {
     item_def item;
     item.base_type = base;
@@ -251,8 +259,35 @@ bool Stash::pickup_eligible() const
     return (false);
 }
 
+bool Stash::is_boring_feature(dungeon_feature_type feat)
+{
+    switch (feat)
+    {
+    // Discard spammy dungeon features.
+    case DNGN_SHALLOW_WATER: case DNGN_DEEP_WATER:
+    case DNGN_LAVA: case DNGN_OPEN_DOOR: case DNGN_STONE_STAIRS_DOWN_I:
+    case DNGN_STONE_STAIRS_DOWN_II: case DNGN_STONE_STAIRS_DOWN_III:
+    case DNGN_STONE_STAIRS_UP_I: case DNGN_STONE_STAIRS_UP_II:
+    case DNGN_STONE_STAIRS_UP_III: case DNGN_ROCK_STAIRS_DOWN:
+    case DNGN_ROCK_STAIRS_UP: case DNGN_ENTER_SHOP:
+    case DNGN_UNDISCOVERED_TRAP:
+        return (true);
+    default:
+        return (grid_is_solid(feat));
+    }
+}
+
 void Stash::update()
 {
+    feat = grd[x][y];
+    trap = NUM_TRAPS;
+
+    if (is_boring_feature(feat))
+        feat = DNGN_FLOOR;
+
+    if (grid_is_trap(feat))
+        trap = trap_type_at_xy(x, y);
+    
     int objl = igrd[x][y];
     // If this is your position, you know what's on this square
     if (x == you.x_pos && y == you.y_pos)
@@ -284,11 +319,7 @@ void Stash::update()
         // There's something on this square. Take a squint at it.
         const item_def &item = mitm[objl];
 
-        // note item when first entering field of vision
-        // (only works with stashes enabled)        
-//        learned_something_new(TUT_SEEN_FIRST_OBJECT, x, y);
-//        learned_something_new(TUT_SEEN_FIRST_OBJECT, item);
-                tutorial_first_item(item);
+        tutorial_first_item(item);
 
         if (item.link == NON_ITEM)
             items.clear();
@@ -353,21 +384,16 @@ void Stash::update()
 // stash-tracking pre/suffixes.
 std::string Stash::stash_item_name(const item_def &item)
 {
-    char buf[ITEMNAME_SIZE];
-    
-    if (item.base_type == OBJ_GOLD)
-        snprintf(buf, sizeof buf, "%d gold piece%s", item.quantity, 
-                    (item.quantity > 1? "s" : ""));
-    else
-        item_name(item, DESC_NOCAP_A, buf, false);
-
-    return buf;
+    return item.name(DESC_NOCAP_A);
 }
 
 class StashMenu : public InvMenu
 {
 public:
-    StashMenu() : InvMenu(MF_SINGLESELECT), can_travel(false) { }
+    StashMenu() : InvMenu(MF_SINGLESELECT), can_travel(false)
+    {
+        set_type(MT_PICKUP);
+    }
     unsigned char getkey() const;
 public:
     bool can_travel;
@@ -440,8 +466,8 @@ bool Stash::show_menu(const std::string &prefix, bool can_travel) const
         if (sel.size() != 1)
             break;
 
-        const item_def *item = 
-                static_cast<const item_def *>( sel[0]->data );
+        item_def *item =
+                static_cast<item_def *>( sel[0]->data );
         describe_item(*item);
     }
     return false;
@@ -460,25 +486,34 @@ std::string Stash::description() const
     {
         char additionals[50];
         snprintf(additionals, sizeof additionals, 
-                " (+%lu)", 
-                (unsigned long) (sz - 1));
+                " (...%ld)", 
+                 (unsigned long) (sz - 1));
         desc += additionals;
     }
     return (desc);
 }
 
+std::string Stash::feature_description() const
+{
+    if (feat == DNGN_FLOOR)
+        return ("");
+
+    return (::feature_description(feat, trap));
+}
+
 bool Stash::matches_search(const std::string &prefix,
                            const base_pattern &search,
                            stash_search_result &res) const
 {
-    if (!enabled || items.empty()) return false;
+    if (!enabled || (items.empty() && feat == DNGN_FLOOR))
+        return false;
 
     for (unsigned i = 0; i < items.size(); ++i)
     {
         const item_def &item = items[i];
         std::string s = stash_item_name(item);
         std::string ann = stash_annotate_item(
-                    LUA_SEARCH_ANNOTATE, &item);
+                    STASH_LUA_SEARCH_ANNOTATE, &item);
         if (search.matches(prefix + " " + ann + s))
         {
             if (!res.count++)
@@ -487,11 +522,11 @@ bool Stash::matches_search(const std::string &prefix,
             continue;
         }
 
-        if (is_dumpable_artifact(item, Options.verbose_dump))
+        if (is_dumpable_artefact(item, false))
         {
-            std::string desc = munge_description(get_item_description(item, 
-                                                          Options.verbose_dump,
-                                                          true ));
+            std::string desc =
+                munge_description(get_item_description(item, false, true));
+            
             if (search.matches(desc))
             {
                 if (!res.count++)
@@ -500,6 +535,17 @@ bool Stash::matches_search(const std::string &prefix,
             }
         }
     }
+
+    if (!res.matches && feat != DNGN_FLOOR)
+    {
+        const std::string fdesc = feature_description();
+        if (!fdesc.empty() && search.matches(fdesc))
+        {
+            res.match = fdesc;
+            res.matches = 1;
+        }
+    }
+    
     if (res.matches)
     {
         res.stash = this;
@@ -535,7 +581,7 @@ void Stash::write(std::ostream &os,
         strncpy(buf, s.c_str(), sizeof buf);
 
         std::string ann = userdef_annotate_item(
-                LUA_DUMP_ANNOTATE, &item);
+                STASH_LUA_DUMP_ANNOTATE, &item);
 
         if (!ann.empty())
         {
@@ -548,11 +594,11 @@ void Stash::write(std::ostream &os,
            << (!verified && (items.size() > 1 || i)? " (still there?)" : "")
            << std::endl;
 
-        if (is_dumpable_artifact(item, Options.verbose_dump))
+        if (is_dumpable_artefact(item, false))
         {
-            std::string desc = munge_description(get_item_description(item, 
-                                                          Options.verbose_dump,
-                                                          true ));
+            std::string desc =
+                munge_description(get_item_description(item, false, true));
+
             // Kill leading and trailing whitespace
             desc.erase(desc.find_last_not_of(" \n\t") + 1);
             desc.erase(0, desc.find_first_not_of(" \n\t"));
@@ -580,6 +626,9 @@ void Stash::save(FILE *file) const
     writeByte(file, x);
     writeByte(file, y);
 
+    writeByte(file, feat);
+    writeByte(file, trap);
+
     // Note: Enabled save value is inverted logic, so that it defaults to true
     writeByte(file, 
         (unsigned char) ((verified? 1 : 0) | (!enabled? 2 : 0)) );
@@ -598,6 +647,14 @@ void Stash::load(FILE *file)
     x = readByte(file);
     y = readByte(file);
 
+    feat =
+        static_cast<dungeon_feature_type>(
+            static_cast<unsigned char>( readByte(file) ));
+    trap =
+        static_cast<trap_type>(
+            static_cast<unsigned char>( readByte(file) ));
+        
+
     unsigned char flags = readByte(file);
     verified = (flags & 1) != 0;
 
@@ -634,17 +691,6 @@ ShopInfo::ShopInfo(int xp, int yp) : x(xp), y(yp), name(), shoptype(-1),
         shoptype = sh->type;
 }
 
-ShopInfo::ShopId::ShopId(int stype)
-    : shopping_hup_protect(), shoptype(stype), id()
-{
-    shop_init_id_type( shoptype, id );
-}
-
-ShopInfo::ShopId::~ShopId()
-{
-    shop_uninit_id_type( shoptype, id );
-}
-
 void ShopInfo::add_item(const item_def &sitem, unsigned price)
 {
     shop_item it;
@@ -656,20 +702,34 @@ void ShopInfo::add_item(const item_def &sitem, unsigned price)
 std::string ShopInfo::shop_item_name(const shop_item &si) const
 {
     char shopitem[ITEMNAME_SIZE * 2];
-    std::string itemname = Stash::stash_item_name(si.item);
+
+    const unsigned long oldflags = si.item.flags;
+
+    if (shoptype_identifies_stock(static_cast<shop_type>(this->shoptype)))
+        const_cast<shop_item&>(si).item.flags |= ISFLAG_IDENT_MASK;
+           
+    const std::string itemname = Stash::stash_item_name(si.item);
     snprintf(shopitem, sizeof shopitem, "%s (%u gold)", 
-            itemname.c_str(), si.price);
-    return shopitem;   
+             itemname.c_str(), si.price);
+
+    if ( oldflags != si.item.flags )
+        const_cast<shop_item&>(si).item.flags = oldflags;
+
+    return shopitem;
 }
 
 std::string ShopInfo::shop_item_desc(const shop_item &si) const
 {
     std::string desc;
-    if (is_dumpable_artifact(si.item, Options.verbose_dump))
+  
+    const unsigned long oldflags = si.item.flags;
+
+    if (shoptype_identifies_stock(static_cast<shop_type>(this->shoptype)))
+        const_cast<shop_item&>(si).item.flags |= ISFLAG_IDENT_MASK;
+
+    if (is_dumpable_artefact(si.item, false))
     {
-        desc = munge_description(get_item_description(si.item, 
-                    Options.verbose_dump,
-                    true));
+        desc = munge_description(get_item_description(si.item, false, true));
         trim_string(desc);
         
         // Walk backwards and prepend indenting spaces to \n characters
@@ -677,18 +737,30 @@ std::string ShopInfo::shop_item_desc(const shop_item &si) const
             if (desc[i] == '\n')
                 desc.insert(i + 1, " ");
     }
+
+    if ( oldflags != si.item.flags )
+        const_cast<shop_item&>(si).item.flags = oldflags;
+
     return desc;
 }
 
 void ShopInfo::describe_shop_item(const shop_item &si) const
 {
-    describe_item( si.item );
+    const unsigned long oldflags = si.item.flags;
+
+    if (shoptype_identifies_stock(static_cast<shop_type>(this->shoptype)))
+        const_cast<shop_item&>(si).item.flags |= ISFLAG_IDENT_MASK;
+    
+    item_def it = static_cast<item_def>(si.item);
+    describe_item( it );
+
+    if ( oldflags != si.item.flags )
+        const_cast<shop_item&>(si).item.flags = oldflags;
 }
 
 bool ShopInfo::show_menu(const std::string &place,
                          bool can_travel) const
 {
-    ShopId id(shoptype);
     StashMenu menu;
 
     MenuEntry *mtitle = new MenuEntry(name + " (" + place, MEL_TITLE);
@@ -746,16 +818,16 @@ bool ShopInfo::matches_search(const std::string &prefix,
                               const base_pattern &search, 
                               stash_search_result &res) const
 {
-    if (items.empty() && visited) return false;
+    if (items.empty() && visited)
+        return false;
 
-    ShopId id(shoptype);
     bool match = false;
 
     for (unsigned i = 0; i < items.size(); ++i)
     {
         std::string sname = shop_item_name(items[i]);
         std::string ann = stash_annotate_item(
-                    LUA_SEARCH_ANNOTATE, &items[i].item, true);
+                    STASH_LUA_SEARCH_ANNOTATE, &items[i].item, true);
         
         bool thismatch = false;
         if (search.matches(prefix + " " + ann + sname))
@@ -799,8 +871,6 @@ bool ShopInfo::matches_search(const std::string &prefix,
 
 void ShopInfo::write(std::ostream &os, bool identify) const
 {
-    ShopId id(shoptype);
-
     os << "[Shop] " << name << std::endl;
     if (items.size() > 0)
     {
@@ -992,9 +1062,9 @@ void LevelStashes::add_stash(int x, int y)
     }
     else
     {
-        Stash newStash(x, y);
-        if (!newStash.empty())
-            stashes[ newStash.abs_pos() ] = newStash;
+        Stash new_stash(x, y);
+        if (!new_stash.empty())
+            stashes[ new_stash.abs_pos() ] = new_stash;
     }
 }
 
@@ -1003,19 +1073,6 @@ bool LevelStashes::is_current() const
     return (place == level_id::current());
 }
 
-bool LevelStashes::in_hell() const
-{
-    return place.branch >= BRANCH_DIS
-            && place.branch <= BRANCH_THE_PIT
-            && place.branch != BRANCH_VESTIBULE_OF_HELL;
-}
-
-bool LevelStashes::in_branch(int branchid) const
-{
-    return place.branch == branchid;
-}
-
-
 std::string LevelStashes::level_name() const
 {
     return place.describe(true, true);
@@ -1277,28 +1334,26 @@ void StashTracker::update_visible_stashes(
         return ;
 
     LevelStashes *lev = find_current_level();
-    for (int cy = 1; cy <= 17; ++cy)
+    for (int cy = crawl_view.glos1.y; cy <= crawl_view.glos2.y; ++cy)
     {
-        for (int cx = 9; cx <= 25; ++cx)
+        for (int cx = crawl_view.glos1.x; cx <= crawl_view.glos2.x; ++cx)
         {
-            int x = you.x_pos + cx - 17, y = you.y_pos + cy - 9;
-            if (x < 0 || x >= GXM || y < 0 || y >= GYM)
+            if (!in_bounds(cx, cy) || !see_grid(cx, cy))
                 continue;
 
-            if (!env.show[cx - 8][cy] && !(cx == 17 && cy == 9))
-                continue;
-
-            if ((!lev || !lev->update_stash(x, y))
-                    && mode == ST_AGGRESSIVE 
-                    && igrd[x][y] != NON_ITEM)
+            const dungeon_feature_type grid = grd[cx][cy];
+            if ((!lev || !lev->update_stash(cx, cy))
+                && mode == ST_AGGRESSIVE 
+                && (igrd[cx][cy] != NON_ITEM
+                    || !Stash::is_boring_feature(grid)))
             {
                 if (!lev)
                     lev = &get_current_level();
-                lev->add_stash(x, y);
+                lev->add_stash(cx, cy);
             }
 
-            if (grd[x][y] == DNGN_ENTER_SHOP)
-                get_shop(x, y);
+            if (grid == DNGN_ENTER_SHOP)
+                get_shop(cx, cy);
         }
     }
 
@@ -1310,21 +1365,32 @@ void StashTracker::update_visible_stashes(
 static std::string lastsearch;
 static input_history search_history(15);
 
+void StashTracker::show_stash_search_prompt()
+{
+    std::vector<std::string> opts;
+    if (!lastsearch.empty())
+        opts.push_back(
+            make_stringf("Enter for \"%s\"", lastsearch.c_str()) );
+    if (level_type_is_stash_trackable(you.level_type)
+        && lastsearch != ".")
+    {
+        opts.push_back(
+            make_stringf("press . for all items on level"));
+    }
+
+    std::string prompt_qual =
+        comma_separated_line(opts.begin(), opts.end(), ", or ", ", or ");
+
+    if (!prompt_qual.empty())
+        prompt_qual = " [" + prompt_qual + "]";
+
+    mprf(MSGCH_PROMPT, "Search for what%s?\n", prompt_qual.c_str());
+}
+
 void StashTracker::search_stashes()
 {
-    char prompt[200];
-    if (lastsearch.length())
-        snprintf(prompt, sizeof prompt,
-                "Search for what [Enter for \"%s\"]?",
-                lastsearch.c_str());
-    else
-        snprintf(prompt, sizeof prompt,
-                "Search for what?");
+    show_stash_search_prompt();
     
-    mpr(prompt, MSGCH_PROMPT);
-    // Push the cursor down to the next line.
-    mpr("", MSGCH_PROMPT);
-
     char buf[400];
     bool validline = 
         !cancelable_get_line(buf, sizeof buf, 80, &search_history);
@@ -1333,6 +1399,27 @@ void StashTracker::search_stashes()
         return;
 
     std::string csearch = *buf? buf : lastsearch;
+    std::string help = lastsearch;
+    lastsearch = csearch;
+
+    if (csearch == ".")
+    {
+        if (!level_type_is_stash_trackable(you.level_type))
+        {
+            mpr("Cannot track items on this level.");
+            return;
+        }
+#if defined(REGEX_PCRE) || defined(REGEX_POSIX)
+#define RE_ESCAPE "\\"
+#else
+#define RE_ESCAPE ""
+#endif
+        
+        csearch = (RE_ESCAPE "{")
+            + level_id::current().describe()
+            + (RE_ESCAPE "}");
+    }
+    
     std::vector<stash_search_result> results;
 
     base_pattern *search = NULL;
@@ -1340,21 +1427,18 @@ void StashTracker::search_stashes()
     text_pattern tpat( csearch, true );
     search = &tpat;
 
-#ifdef CLUA_BINDINGS
     lua_text_pattern ltpat( csearch );
     
     if (lua_text_pattern::is_lua_pattern(csearch))
         search = &ltpat;
-#endif
 
     if (!search->valid())
     {
         mpr("Your search expression is invalid.", MSGCH_PLAIN);
+        lastsearch = help;
         return ;
     }
 
-    lastsearch = csearch;
-    
     get_matching_stashes(*search, results);
 
     if (results.empty())
@@ -1385,8 +1469,6 @@ void StashTracker::get_matching_stashes(
             return;
     }
 
-    get_matching_features(search, results);
-
     level_id curr = level_id::current();
     for (unsigned i = 0; i < results.size(); ++i)
     {
@@ -1470,26 +1552,13 @@ void StashTracker::display_search_results(
     for (unsigned i = 0; i < results.size(); ++i, ++hotkey)
     {
         stash_search_result &res = results[i];
-        char matchtitle[ITEMNAME_SIZE];
-        std::string place = short_place_name(res.pos.id);
+        std::ostringstream matchtitle;
+        matchtitle << "[" << short_place_name(res.pos.id) << "] "
+                   << res.match;
         if (res.matches > 1 && res.count > 1)
-        {
-            snprintf(matchtitle, sizeof matchtitle,
-                    "[%s] %s (%d)",
-                    place.c_str(),
-                    res.match.c_str(),
-                    res.matches);
-        }
-        else
-        {
-            snprintf(matchtitle, sizeof matchtitle,
-                    "[%s] %s",
-                    place.c_str(),
-                    res.match.c_str());
-        }
-        std::string mename = matchtitle;
-        
-        MenuEntry *me = new MenuEntry(mename, MEL_ITEM, 1, hotkey);
+            matchtitle << " (" << res.matches << ")";
+
+        MenuEntry *me = new MenuEntry(matchtitle.str(), MEL_ITEM, 1, hotkey);
         me->data = &res;
         if (res.shop && !res.shop->is_visited())
             me->colour = CYAN;
@@ -1523,7 +1592,7 @@ void StashTracker::display_search_results(
             if (dotravel && can_travel_to(res->pos.id))
             {
                 redraw_screen();
-                const level_pos lp = res->pos;
+                const travel_target lp = res->pos;
                 start_translevel_travel(lp);
                 return ;
             }
