@@ -3,7 +3,7 @@
  *  Summary:    Misc stuff.
  *  Written by: Linley Henzell
  *
- *  Modified for Crawl Reference by $Author: dshaligram $ on $Date: 2007-04-05T16:46:35.024555Z $
+ *  Modified for Crawl Reference by $Author: dshaligram $ on $Date: 2007-11-24T17:24:21.713648Z $
  *
  *  Change History (most recent first):
  *
@@ -14,11 +14,19 @@
  */
 
 #include "AppHdr.h"
+#include "database.h"
 #include "direct.h"
+#include "message.h"
+#include "misc.h"
 #include "monplace.h"
+#include "state.h"
 #include "stuff.h"
 #include "view.h"
 
+#include <cstdarg>
+#include <sstream>
+#include <iomanip>
+
 #include <errno.h>
 #include <stdlib.h>
 #include <stdio.h>
@@ -47,8 +55,10 @@
 #include "libunix.h"
 #endif
 
+#include "branch.h"
 #include "delay.h"
 #include "externs.h"
+#include "itemprop.h"
 #include "items.h"
 #include "macro.h"
 #include "misc.h"
@@ -63,29 +73,28 @@
 #include "view.h"
 
 // Crude, but functional.
-char *const make_time_string( time_t abs_time, char *const buff, int buff_size,
-                              bool terse )
+std::string make_time_string( time_t abs_time, bool terse )
 {
     const int days  = abs_time / 86400;
     const int hours = (abs_time % 86400) / 3600;
     const int mins  = (abs_time % 3600) / 60;
     const int secs  = abs_time % 60;
 
-    char day_buff[32];
+    std::ostringstream buff;
+    buff << std::setfill('0');
 
     if (days > 0)
     {
         if (terse)
-            snprintf(day_buff, sizeof day_buff, "%d, ", days);
+            buff << days << ", ";
         else
-            snprintf( day_buff, sizeof(day_buff), "%d day%s, ", 
-                  days, (days > 1) ? "s" : "" );
+            buff << days << (days > 1 ? " days" : "day");
     }
 
-    snprintf( buff, buff_size, "%s%02d:%02d:%02d", 
-              (days > 0) ? day_buff : "", hours, mins, secs );
-
-    return (buff);
+    buff << std::setw(2) << hours << ':'
+         << std::setw(2) << mins << ':'
+         << std::setw(2) << secs;
+    return buff.str();
 }
 
 void set_redraw_status( unsigned long flags )
@@ -93,66 +102,127 @@ void set_redraw_status( unsigned long flags )
     you.redraw_status_flags |= flags;
 }
 
-void tag_followers( void )
-{   
-    int count_x, count_y;
+static bool tag_follower_at(const coord_def &pos)
+{
+    if (!in_bounds(pos) || pos == you.pos())
+        return (false);
 
-    for (count_x = you.x_pos - 1; count_x <= you.x_pos + 1; count_x++)
-    {   
-        for (count_y = you.y_pos - 1; count_y <= you.y_pos + 1; count_y++)
-        {   
-            if (count_x == you.x_pos && count_y == you.y_pos)
-                continue;
+    if (mgrd(pos) == NON_MONSTER)
+        return (false);
 
-            if (mgrd[count_x][count_y] == NON_MONSTER)
-                continue;
+    monsters *fmenv = &menv[mgrd(pos)];
 
-            struct monsters *fmenv = &menv[mgrd[count_x][count_y]];
+    if (fmenv->type == MONS_PLAYER_GHOST
+        || !fmenv->alive()
+        || mons_is_stationary(fmenv)
+        || fmenv->incapacitated())
+    {
+        return (false);
+    }
 
-            if ((fmenv->type == MONS_PANDEMONIUM_DEMON)
-                || (fmenv->type == MONS_PLAYER_GHOST)  // cdl
-                || (fmenv->type == -1)
-                || mons_is_stationary(fmenv))
-            {   
-                continue;
-            }
+    if (!monster_habitable_grid(fmenv, DNGN_FLOOR))
+        return (false);
 
-            if (!monster_habitable_grid(fmenv, DNGN_FLOOR))
-                continue;
+    if (fmenv->speed_increment < 50)
+        return (false);
 
-            if (fmenv->speed_increment < 50)
-                continue;
+    // only friendly monsters, or those actively seeking the
+    // player, will follow up/down stairs.
+    if (!(mons_friendly(fmenv) ||
+          (fmenv->behaviour == BEH_SEEK && fmenv->foe == MHITYOU)))
+    {
+        return (false);
+    }
 
-            // only friendly monsters, or those actively seeking the
-            // player, will follow up/down stairs.
-            if (!(mons_friendly(fmenv) ||
-                (fmenv->behaviour == BEH_SEEK && fmenv->foe == MHITYOU)))
-            {   
-                continue;
-            }
+    // Monsters that are not directly adjacent are subject to more
+    // stringent checks.
+    if ((pos - you.pos()).abs() > 2)
+    {
+        if (!mons_friendly(fmenv))
+            return (false);
 
-            // monster is chasing player through stairs:
-            fmenv->flags |= MF_TAKING_STAIRS;
+        // Orcs will follow Beogh worshippers.
+        if (!(mons_species(fmenv->type) == MONS_ORC
+              && you.religion == GOD_BEOGH))
+            return (false);
+    }
+
+    // monster is chasing player through stairs:
+    fmenv->flags |= MF_TAKING_STAIRS;
 
 #if DEBUG_DIAGNOSTICS
-            snprintf( info, INFO_SIZE, "%s is marked for following.",
-                      ptr_monam( fmenv, DESC_CAP_THE ) );
-            mpr( info, MSGCH_DIAGNOSTICS );
-#endif  
+    mprf(MSGCH_DIAGNOSTICS, "%s is marked for following.",
+         fmenv->name(DESC_CAP_THE, true).c_str() );
+#endif
+    return (true);
+}
+
+static int follower_tag_radius2()
+{
+    // If only friendlies are adjacent, we set a max radius of 6, otherwise
+    // only adjacent friendlies may follow.
+    coord_def p;
+    for (p.x = you.x_pos - 1; p.x <= you.x_pos + 1; ++p.x)
+        for (p.y = you.y_pos - 1; p.y <= you.y_pos + 1; ++p.y)
+        {
+            if (p == you.pos())
+                continue;
+            if (const monsters *mon = monster_at(p))
+            {
+                if (!mons_friendly(mon))
+                    return (2);
+            }
         }
+    return (6 * 6);
+}
+
+void tag_followers()
+{
+    const int radius2 = follower_tag_radius2();
+    int n_followers = 18;
+
+    std::vector<coord_def> places[2];
+    int place_set = 0;
+
+    places[place_set].push_back(you.pos());
+    memset(travel_point_distance, 0, sizeof(travel_distance_grid_t));
+    while (!places[place_set].empty())
+    {
+        for (int i = 0, size = places[place_set].size(); i < size; ++i)
+        {
+            const coord_def &p = places[place_set][i];
+
+            coord_def fp;
+            for (fp.x = p.x - 1; fp.x <= p.x + 1; ++fp.x)
+                for (fp.y = p.y - 1; fp.y <= p.y + 1; ++fp.y)
+                {
+                    if (fp == p || (fp - you.pos()).abs() > radius2
+                        || !in_bounds(fp) || travel_point_distance[fp.x][fp.y])
+                    {
+                        continue;
+                    }
+                    travel_point_distance[fp.x][fp.y] = 1;
+                    if (tag_follower_at(fp))
+                    {
+                        // If we've run out of our follower allowance, bail.
+                        if (--n_followers <= 0)
+                            return;
+                        places[!place_set].push_back(fp);
+                    }
+                }
+        }
+        places[place_set].clear();
+        place_set = !place_set;
     }
 }
 
-void untag_followers( void )
+void untag_followers()
 {
     for (int m = 0; m < MAX_MONSTERS; m++)
-    {
-        struct monsters *mon = &menv[m];
-        mon->flags &= (~MF_TAKING_STAIRS);
-    }
+        menv[m].flags &= (~MF_TAKING_STAIRS);
 }
 
-unsigned char get_ch(void)
+unsigned char get_ch()
 {
     unsigned char gotched = getch();
 
@@ -191,6 +261,61 @@ int random_range(int low, int high)
     return (low + random2(high - low + 1));
 }
 
+int random_range(int low, int high, int nrolls)
+{
+    ASSERT(nrolls > 0);
+    int sum = 0;
+    for (int i = 0; i < nrolls; ++i)
+        sum += random_range(low, high);
+    return (sum / nrolls);
+}
+
+int random_choose(int first, ...)
+{
+    va_list args;
+    va_start(args, first);
+
+    int chosen = first, count = 1, nargs = 100;
+
+    while (nargs-- > 0)
+    {
+        const int pick = va_arg(args, int);
+        if (pick == -1)
+            break;
+        if (one_chance_in(++count))
+            chosen = pick;
+    }
+
+    ASSERT(nargs > 0);
+    
+    va_end(args);
+    return (chosen);
+}
+
+int random_choose_weighted(int weight, int first, ...)
+{
+    va_list args;
+    va_start(args, first);
+
+    int chosen = first, cweight = weight, nargs = 100;
+
+    while (nargs-- > 0)
+    {
+        const int nweight = va_arg(args, int);
+        if (!nweight)
+            break;
+
+        const int choice = va_arg(args, int);
+        if (random2(cweight += nweight) < nweight)
+            chosen = choice;
+    }
+
+    ASSERT(nargs > 0);
+    
+    va_end(args);
+    return (chosen);
+}
+
 int random2( int max )
 {
     if (max <= 1)
@@ -299,26 +424,10 @@ int random2limit(int max, int limit)
     return sum;
 }                               // end random2limit()
 
-// answers the question: "Is a grid within character's line of sight?"
-bool see_grid(unsigned char grx, unsigned char gry)
-{
-    if (grx > you.x_pos - 9 && grx < you.x_pos + 9
-                        && gry > you.y_pos - 9 && gry < you.y_pos + 9)
-    {
-        if (env.show[grx - you.x_pos + 9][gry - you.y_pos + 9] != 0)
-            return true;
-
-        // rare case: can player see self?  (of course!)
-        if (grx == you.x_pos && gry == you.y_pos)
-            return true;
-    }
-
-    return false;
-}                               // end see_grid()
-
-static bool io_inited = false;
 void cio_init()
 {
+    crawl_state.io_inited = true;
+
 #ifdef UNIX
     unixcurses_startup();
 #endif
@@ -330,13 +439,20 @@ void cio_init()
 #ifdef DOS
     init_libdos();
 #endif
+    
+    crawl_view.init_geometry();
 
-    io_inited = true;
+    if (Options.char_set == CSET_UNICODE && !crawl_state.unicode_ok)
+    {
+        crawl_state.add_startup_error(
+            "Unicode glyphs are not available, falling back to ASCII.");
+        Options.char_set = CSET_ASCII;
+    }
 }
 
 void cio_cleanup()
 {
-    if (!io_inited)
+    if (!crawl_state.io_inited)
         return;
     
 #ifdef UNIX
@@ -347,7 +463,9 @@ void cio_cleanup()
     deinit_libw32c();
 #endif
 
-    io_inited = false;
+    msg::deinitalise_mpr_streams();
+
+    crawl_state.io_inited = false;
 }
 
 void end(int exit_code, bool print_error, const char *format, ...)
@@ -355,7 +473,7 @@ void end(int exit_code, bool print_error, const char *format, ...)
     std::string error = print_error? strerror(errno) : "";
     
     cio_cleanup();
-
+    databaseSystemShutdown();
     if (format)
     {
         va_list arg;
@@ -375,9 +493,10 @@ void end(int exit_code, bool print_error, const char *format, ...)
         if (error[error.length() - 1] != '\n')
             error += "\n";
         fprintf(stderr, "%s", error.c_str());
+        error.clear();
     }
 
-#if defined(WIN32CONSOLE) || defined(DOS)
+#if defined(WIN32CONSOLE) || defined(DOS) || defined(DGL_PAUSE_AFTER_ERROR)
     if (exit_code)
     {
         fprintf(stderr, "Hit Enter to continue...\n");
@@ -392,15 +511,15 @@ void redraw_screen(void)
 {
     draw_border();
 
-    you.redraw_hit_points = 1;
-    you.redraw_magic_points = 1;
-    you.redraw_strength = 1;
-    you.redraw_intelligence = 1;
-    you.redraw_dexterity = 1;
-    you.redraw_armour_class = 1;
-    you.redraw_evasion = 1;
-    you.redraw_gold = 1;
-    you.redraw_experience = 1;
+    you.redraw_hit_points = true;
+    you.redraw_magic_points = true;
+    you.redraw_strength = true;
+    you.redraw_intelligence = true;
+    you.redraw_dexterity = true;
+    you.redraw_armour_class = true;
+    you.redraw_evasion = true;
+    you.redraw_gold = true;
+    you.redraw_experience = true;
     you.wield_change = true;
 
     set_redraw_status(
@@ -514,18 +633,10 @@ int div_rand_round( int num, int den )
     return (num / den + (random2(den) < num % den));
 }
 
-// I got so tired of seeing: ".. && random2(foo) == 0 && .." in the code
-// that I broke down and wrote this little -- very little -- function.
-// anyway, I think improving the readability of the code offsets whatever
-// overhead the additional (intermediary) function call added to Crawl -
-// we'll just make it up by tightening code elsewhere, right guys?
-// [use with == and != only .. never directly with comparisons or math]
-//                                                      -- 14jan2000 {dlb}
 bool one_chance_in(int a_million)
 {
     return (random2(a_million) == 0);
-}                               // end one_chance_in() - that's it? :P {dlb}
-
+}
 
 // simple little function to quickly modify all three stats
 // at once - does check for '0' modifiers to prevent needless
@@ -564,7 +675,7 @@ void canned_msg(canned_message_type which_message)
     {
     case MSG_SOMETHING_APPEARS:
         mprf("Something appears %s!",
-             (you.species == SP_NAGA || you.species == SP_CENTAUR)
+             (you.species == SP_NAGA || you.mutation[MUT_HOOVES])
              ? "before you" : "at your feet");
         break;
 
@@ -577,6 +688,9 @@ void canned_msg(canned_message_type which_message)
     case MSG_TOO_BERSERK:
         mpr("You are too berserk!");
         break;
+    case MSG_PRESENT_FORM:
+        mpr("You can't do that in your present form.");
+        break;
     case MSG_NOTHING_CARRIED:
         mpr("You aren't carrying anything.");
         break;
@@ -608,8 +722,6 @@ void canned_msg(canned_message_type which_message)
 bool yesno( const char *str, bool safe, int safeanswer, bool clear_after,
             bool interrupt_delays, bool noprompt )
 {
-    unsigned char tmp;
-
     if (interrupt_delays)
         interrupt_activity( AI_FORCE_INTERRUPT );
     for (;;)
@@ -617,7 +729,7 @@ bool yesno( const char *str, bool safe, int safeanswer, bool clear_after,
         if ( !noprompt )
             mpr(str, MSGCH_PROMPT);
 
-        tmp = (unsigned char) getch();
+        int tmp = getchm(KC_CONFIRM);
 
         if ((tmp == ' ' || tmp == 27 || tmp == '\r' || tmp == '\n') 
                 && safeanswer)
@@ -645,16 +757,14 @@ bool yesno( const char *str, bool safe, int safeanswer, bool clear_after,
 // like yesno(), but returns 0 for no, 1 for yes, and -1 for quit
 int yesnoquit( const char* str, bool safe, int safeanswer, bool clear_after )
 {
-    unsigned char tmp;
-
     interrupt_activity( AI_FORCE_INTERRUPT );
     while (1)
     {
         mpr(str, MSGCH_PROMPT);
 
-        tmp = (unsigned char) getch();
+        int tmp = getchm(KC_CONFIRM);
 
-        if ( tmp == 27 || tmp == 'q' || tmp == 'Q' )
+        if ( tmp == ESCAPE || tmp == 'q' || tmp == 'Q' )
             return -1;
         
         if ((tmp == ' ' || tmp == '\r' || tmp == '\n') && safeanswer)
@@ -704,7 +814,7 @@ bool adjacent( int x, int y, int x2, int y2 )
     return (abs(x - x2) <= 1 && abs(y - y2) <= 1);
 }
 
-bool silenced(char x, char y)
+bool silenced(int x, int y)
 {
     if (you.duration[DUR_SILENCE] > 0
         && distance(x, y, you.x_pos, you.y_pos) <= 36)  // (6 * 6)
@@ -723,7 +833,7 @@ bool silenced(char x, char y)
     }
 }                               // end silenced()
 
-bool player_can_hear(char x, char y)
+bool player_can_hear(int x, int y)
 {
     return (!silenced(x, y) && !silenced(you.x_pos, you.y_pos));
 }                               // end player_can_hear()
@@ -747,7 +857,8 @@ bool map_bounds( int x, int y )
 // Returns a random location in (x_pos, y_pos)... the grid will be
 // DNGN_FLOOR if clear, and NON_MONSTER if empty.  Exclusive tells
 // if we're using in_bounds() or map_bounds() restriction.
-void random_in_bounds( int &x_pos, int &y_pos, int terr, bool empty, bool excl )
+void random_in_bounds( int &x_pos, int &y_pos, int terr,
+                       bool empty, bool excl )
 {
     bool done = false;
 
@@ -760,7 +871,8 @@ void random_in_bounds( int &x_pos, int &y_pos, int terr, bool empty, bool excl )
             done = true;
         else if (terr == grd[x_pos][y_pos])
             done = true;
-        else if (terr == DNGN_DEEP_WATER && grd[x_pos][y_pos] == DNGN_SHALLOW_WATER)
+        else if (terr == DNGN_DEEP_WATER
+                 && grd[x_pos][y_pos] == DNGN_SHALLOW_WATER)
             done = true;
         else if (empty 
                 && mgrd[x_pos][y_pos] != NON_MONSTER 
@@ -897,6 +1009,10 @@ int element_colour( int element, bool no_random )
                               : LIGHTBLUE;
         break;
 
+    case EC_BEOGH:
+        ret = (tmp_rand < 60) ? LIGHTRED : BROWN;
+        break;
+
     case EC_CRYSTAL:
         ret = (tmp_rand < 40) ? LIGHTGREY :
               (tmp_rand < 80) ? GREEN 
@@ -969,6 +1085,10 @@ int element_colour( int element, bool no_random )
         ret = tmp_rand < 100? CYAN : BLUE;
         break;
 
+    case EC_SHIMMER_BLUE:
+        ret = random_choose_weighted(80, BLUE, 20, LIGHTBLUE, 5, CYAN, 0);
+        break;
+
     case EC_RANDOM:
         ret = 1 + random2(15);              // always random
         break;
@@ -1001,31 +1121,55 @@ int letter_to_index(int the_letter)
     return the_letter;
 }                               // end letter_to_index()
 
+int fuzz_value(int val, int lowfuzz, int highfuzz, int naverage)
+{
+    const int lfuzz = lowfuzz * val / 100,
+        hfuzz = highfuzz * val / 100;
+    return val + random2avg(lfuzz + hfuzz + 1, naverage) - lfuzz;
+}
+
 // returns 0 if the point is not near stairs
 // returns 1 if the point is near unoccupied stairs
 // returns 2 if the point is near player-occupied stairs
 
-int near_stairs(int px, int py, int max_dist, unsigned char &stair_gfx)
+int near_stairs(const coord_def &p, int max_dist,
+                dungeon_char_type &stair_type,
+                branch_type &branch)
 {
-    int i,j;
-
-    for(i=-max_dist; i<=max_dist; i++)
+    coord_def inc;
+    for (inc.x = -max_dist; inc.x <= max_dist; inc.x++)
     {
-        for(j=-max_dist; j<=max_dist; j++)
+        for(inc.y = -max_dist; inc.y <= max_dist; inc.y++)
         {
-            int x = px + i;
-            int y = py + j;
+            const coord_def np(p + inc);
 
-            if (x<0 || x>=GXM || y<0 || y>=GYM)
+            if (!in_bounds(np))
                 continue;
 
-            // very simple check
-            if (grd[x][y] >= DNGN_STONE_STAIRS_DOWN_I
-                && grd[x][y] <= DNGN_RETURN_FROM_SWAMP
-                && grd[x][y] != DNGN_ENTER_SHOP)        // silly
+            const dungeon_feature_type feat = grd(np);
+            if (is_stair(feat))
             {
-                stair_gfx = get_sightmap_char(grd[x][y]);
-                return ((x == you.x_pos && y == you.y_pos) ? 2 : 1);
+                // shouldn't happen for escape hatches
+                if (grid_is_rock_stair(feat))
+                    continue;
+
+                stair_type = get_feature_dchar(feat);
+                
+                // is it a branch stair?
+                for (int i = 0; i < NUM_BRANCHES; ++i)
+                {
+                     if (branches[i].entry_stairs == feat)
+                     {
+                         branch = branches[i].id;
+                         break;
+                     }
+                     else if (branches[i].exit_stairs == feat)
+                     {
+                         branch = branches[i].parent_branch;
+                         break;
+                     }
+                }
+                return (np == you.pos()? 2 : 1);
             }
         }
     }
@@ -1045,9 +1189,9 @@ void zap_los_monsters()
 {
     losight(env.show, grd, you.x_pos, you.y_pos);
 
-    for (int y = LOS_SY; y <= LOS_EY; ++y)
+    for (int y = crawl_view.vlos1.y; y <= crawl_view.vlos2.y; ++y)
     {
-        for (int x = LOS_SX; x <= LOS_EX; ++x)
+        for (int x = crawl_view.vlos1.x; x <= crawl_view.vlos2.x; ++x)
         {
             if (!in_vlos(x, y))
                 continue;
@@ -1084,9 +1228,21 @@ void zap_los_monsters()
             
 #ifdef DEBUG_DIAGNOSTICS
             mprf(MSGCH_DIAGNOSTICS, "Dismissing %s",
-                 ptr_monam(mon, DESC_PLAIN) );
+                 mon->name(DESC_PLAIN, true).c_str() );
 #endif
             monster_die(mon, KILL_DISMISSED, 0);
         }
     }
 }
+
+//////////////////////////////////////////////////////////////////////////
+// coord_def
+int coord_def::distance_from(const coord_def &other) const
+{
+    return (grid_distance(x, y, other.x, other.y));
+}
+
+int random_rod_subtype()
+{
+    return STAFF_SMITING + random2(NUM_STAVES - STAFF_SMITING);
+}
