@@ -3,7 +3,7 @@
  *  Summary:    Auxilary functions to make savefile versioning simpler.
  *  Written by: Gordon Lipford
  *
- *  Modified for Crawl Reference by $Author: dshaligram $ on $Date: 2007-05-02T16:49:28.298370Z $
+ *  Modified for Crawl Reference by $Author: ennewalker $ on $Date: 2007-11-23T02:25:44.246247Z $
  *
  *  Change History (most recent first):
  *
@@ -13,7 +13,7 @@
 
 /* ------------------------- how tags work ----------------------------------
 
-1. Tag types are enumerated in enum.h, from TAG_VERSION (more a placeholder
+1. Tag types are enumerated in tags.h, from TAG_VERSION (more a placeholder
    than anything else, it is not actually saved as a tag) to TAG_XXX. NUM_TAGS
    is equal to the actual number of defined tags.
 
@@ -58,6 +58,7 @@
 #include <cstdlib>
 #include <stdio.h>
 #include <string.h>            // for memcpy
+#include <iterator>
 
 #ifdef UNIX
 #include <sys/types.h>
@@ -70,11 +71,13 @@
 
 #include "abl-show.h"
 #include "branch.h"
+#include "dungeon.h"
 #include "enum.h"
 #include "externs.h"
 #include "files.h"
 #include "itemname.h"
 #include "itemprop.h"
+#include "mapmark.h"
 #include "monstuff.h"
 #include "mon-util.h"
 #include "mtransit.h"
@@ -88,6 +91,8 @@
 // THE BIG IMPORTANT TAG CONSTRUCTION/PARSE BUFFER
 static char *tagBuffer = NULL;
 
+static int tag_minor_version = -1;
+
 // defined in overmap.cc
 extern std::map<branch_type, level_id> stair_level;
 extern std::map<level_pos, shop_type> shops_present;
@@ -127,6 +132,13 @@ static void unmarshall_monster(tagHeader &th, monsters &m);
 static void marshall_item(tagHeader &th, const item_def &item);
 static void unmarshall_item(tagHeader &th, item_def &item);
 
+template<typename T, typename T_iter, typename T_marshal>
+static void marshall_iterator(struct tagHeader &th, T_iter beg, T_iter end,
+                              T_marshal marshal);
+template<typename T>
+static void unmarshall_vector(struct tagHeader& th, std::vector<T>& vec,
+                              T (*T_unmarshall)(struct tagHeader&));
+
 // provide a wrapper for file writing, just in case.
 int write2(FILE * file, const char *buffer, unsigned int count)
 {
@@ -227,6 +239,38 @@ void marshallMap(struct tagHeader &th, const std::map<key,value>& data,
     }
 }
 
+template<typename T_iter, typename T_marshall_t>
+static void marshall_iterator(struct tagHeader &th, T_iter beg, T_iter end,
+                              T_marshall_t T_marshall)
+{
+    marshallLong(th, std::distance(beg, end));
+    while ( beg != end )
+    {
+        T_marshall(th, *beg);
+        ++beg;
+    }
+}
+
+template<typename T>
+static void unmarshall_vector(struct tagHeader& th, std::vector<T>& vec,
+                              T (*T_unmarshall)(struct tagHeader&))
+{
+    vec.clear();
+    const long num_to_read = unmarshallLong(th);
+    for ( long i = 0; i < num_to_read; ++i )
+        vec.push_back( T_unmarshall(th) );
+}
+
+template <typename T_container, typename T_inserter, typename T_unmarshall>
+static void unmarshall_container(tagHeader &th, T_container &container,
+                                 T_inserter inserter, T_unmarshall unmarshal)
+{
+    container.clear();
+    const long num_to_read = unmarshallLong(th);
+    for (long i = 0; i < num_to_read; ++i)
+        (container.*inserter)(unmarshal(th));
+}
+
 void marshall_level_id( tagHeader& th, const level_id& id )
 {
     marshallByte(th, id.branch );
@@ -241,8 +285,8 @@ void marshall_level_pos( struct tagHeader& th, const level_pos& lpos )
     marshall_level_id(th, lpos.id);
 }
 
-template<typename key, typename value>
-void unmarshallMap(struct tagHeader& th, std::map<key,value>& data,
+template<typename key, typename value, typename map>
+void unmarshallMap(struct tagHeader& th, map& data,
                    key   (*key_unmarshall)  (tagHeader&),
                    value (*value_unmarshall)(tagHeader&) )
 {
@@ -251,7 +295,8 @@ void unmarshallMap(struct tagHeader& th, std::map<key,value>& data,
     for ( i = 0; i < len; ++i )
     {
         k = key_unmarshall(th);
-        data[k] = value_unmarshall(th);
+        std::pair<key, value> p(k, value_unmarshall(th));
+        data.insert(p);
     }
 }
 
@@ -264,9 +309,9 @@ T unmarshall_long_as( struct tagHeader& th )
 level_id unmarshall_level_id( tagHeader& th )
 {
     level_id id;
-    id.branch = unmarshallByte(th);
+    id.branch = static_cast<branch_type>(unmarshallByte(th));
     id.depth = unmarshallLong(th);
-    id.level_type = unmarshallByte(th);
+    id.level_type = static_cast<level_area_type>(unmarshallByte(th));
     return (id);
 }
 
@@ -279,6 +324,68 @@ level_pos unmarshall_level_pos( struct tagHeader& th )
     return lpos;
 }
 
+void marshallCoord(tagHeader &th, const coord_def &c)
+{
+    marshallShort(th, c.x);
+    marshallShort(th, c.y);
+}
+
+void unmarshallCoord(tagHeader &th, coord_def &c)
+{
+    c.x = unmarshallShort(th);
+    c.y = unmarshallShort(th);
+}
+
+template <typename marshall, typename grid>
+void run_length_encode(tagHeader &th, marshall m, const grid &g,
+                       int width, int height)
+{
+    int last = 0, nlast = 0;
+    for (int y = 0; y < height; ++y)
+    {
+        for (int x = 0; x < width; ++x)
+        {
+            if (!nlast)
+                last = g[x][y];
+            if (last == g[x][y] && nlast < 255)
+            {
+                nlast++;
+                continue;
+            }
+
+            marshallByte(th, nlast);
+            m(th, last);
+
+            last = g[x][y];
+            nlast = 1;
+        }
+    }
+
+    marshallByte(th, nlast);
+    m(th, last);
+}
+
+template <typename unmarshall, typename grid>
+void run_length_decode(tagHeader &th, unmarshall um, grid &g,
+                       int width, int height)
+{
+    const int end = width * height;
+    int offset = 0;
+    while (offset < end)
+    {
+        const int run = (unsigned char) unmarshallByte(th);
+        const int value = um(th);
+
+        for (int i = 0; i < run; ++i)
+        {
+            const int y = offset / width;
+            const int x = offset % width;
+            g[x][y] = value;
+            ++offset;
+        }
+    }
+}
+
 union float_marshall_kludge
 {
     // [ds] Does ANSI C guarantee that sizeof(float) == sizeof(long)?
@@ -304,22 +411,22 @@ float unmarshallFloat(struct tagHeader &th)
 }
 
 // string -- marshall length & string data
-void marshallString(struct tagHeader &th, const char *data, int maxSize)
+void marshallString(struct tagHeader &th, const std::string &data, int maxSize)
 {
-    // allow for very long strings.
-    short len = strlen(data);
+    // allow for very long strings (well, up to 32K).
+    int len = data.length();
     if (maxSize > 0 && len > maxSize)
         len = maxSize;
     marshallShort(th, len);
 
     // put in the actual string -- we'll null terminate on
     // unmarshall.
-    memcpy(&tagBuffer[th.offset], data, len);
+    memcpy(&tagBuffer[th.offset], data.c_str(), len);
     th.offset += len;
 }
 
 // string -- unmarshall length & string data
-void unmarshallString(struct tagHeader &th, char *data, int maxSize)
+int unmarshallCString(struct tagHeader &th, char *data, int maxSize)
 {
     // get length
     short len = unmarshallShort(th);
@@ -333,6 +440,8 @@ void unmarshallString(struct tagHeader &th, char *data, int maxSize)
     data[copylen] = 0;
 
     th.offset += len;
+
+    return (copylen);
 }
 
 std::string unmarshallString(tagHeader &th, int maxSize)
@@ -343,10 +452,9 @@ std::string unmarshallString(tagHeader &th, int maxSize)
     if (!buffer)
         return ("");    
     *buffer = 0;
-    unmarshallString(th, buffer, maxSize);
-    const std::string res = buffer;
+    const int slen = unmarshallCString(th, buffer, maxSize);
+    const std::string res(buffer, slen);
     delete [] buffer;
-
     return (res);
 }
 
@@ -616,7 +724,7 @@ void tag_set_expected(char tags[], int fileType)
         switch(fileType)
         {
             case TAGTYPE_PLAYER:
-                if ((i >= TAG_YOU && i <=TAG_YOU_DUNGEON)
+                if ((i >= TAG_YOU && i <= TAG_YOU_DUNGEON)
                     || i == TAG_LOST_MONSTERS)
                 {
                     tags[i] = 1;
@@ -657,14 +765,7 @@ static void tag_construct_you(struct tagHeader &th)
 
     marshallByte(th,you.religion);
     marshallByte(th,you.piety);
-    marshallByte(th,you.invis);
-    marshallByte(th,you.conf);
-    marshallByte(th,you.paralysis);
-    marshallByte(th,you.slow);
-    marshallByte(th,you.fire_shield);
     marshallByte(th,you.rotting);
-    marshallByte(th,you.exhausted);
-    marshallByte(th,you.deaths_door);
     marshallByte(th,you.symbol);
     marshallByte(th,you.colour);
     marshallByte(th,you.pet_target);
@@ -675,35 +776,15 @@ static void tag_construct_you(struct tagHeader &th)
     marshallByte(th,you.your_level);
     marshallByte(th,you.is_undead);
     marshallByte(th,you.special_wield);
-    marshallByte(th,you.berserker);
     marshallByte(th,you.berserk_penalty);
     marshallByte(th,you.level_type);
+    marshallString(th, you.level_type_name);
     marshallByte(th,you.synch_time);
     marshallByte(th,you.disease);
     marshallByte(th,you.species);
 
     marshallShort(th, you.hp);
 
-    if (you.haste > 215)
-        you.haste = 215;
-
-    marshallByte(th,you.haste);
-
-    if (you.might > 215)
-        you.might = 215;
-
-    marshallByte(th,you.might);
-
-    if (you.levitation > 215)
-        you.levitation = 215;
-
-    marshallByte(th,you.levitation);
-
-    if (you.poisoning > 215)
-        you.poisoning = 215;
-
-    marshallByte(th,you.poisoning);
-
     marshallShort(th, you.hunger);
 
     // how many you.equip?
@@ -716,8 +797,6 @@ static void tag_construct_you(struct tagHeader &th)
     marshallByte(th,you.strength);
     marshallByte(th,you.intel);
     marshallByte(th,you.dex);
-    marshallByte(th,you.confusing_touch);
-    marshallByte(th,you.sure_blade);
     marshallByte(th,you.hit_points_regeneration);
     marshallByte(th,you.magic_points_regeneration);
 
@@ -772,16 +851,21 @@ static void tag_construct_you(struct tagHeader &th)
     // how many durations?
     marshallByte(th, NUM_DURATIONS);
     for (j = 0; j < NUM_DURATIONS; ++j)
-        marshallByte(th,you.duration[j]);
+        marshallLong(th,you.duration[j]);
 
     // how many attributes?
-    marshallByte(th, 30);
-    for (j = 0; j < 30; ++j)
+    marshallByte(th, NUM_ATTRIBUTES);
+    for (j = 0; j < NUM_ATTRIBUTES; ++j)
         marshallByte(th,you.attribute[j]);
 
+    // sacrifice values
+    marshallByte(th, NUM_OBJECT_CLASSES);
+    for (j = 0; j < NUM_OBJECT_CLASSES; ++j)
+        marshallLong(th, you.sacrifice_value[j]);
+
     // how many mutations/demon powers?
-    marshallShort(th, 100);
-    for (j = 0; j < 100; ++j)
+    marshallShort(th, NUM_MUTATIONS);
+    for (j = 0; j < NUM_MUTATIONS; ++j)
     {
         marshallByte(th,you.mutation[j]);
         marshallByte(th,you.demon_pow[j]);
@@ -829,9 +913,10 @@ static void tag_construct_you(struct tagHeader &th)
     marshallLong( th, you.real_time );
     marshallLong( th, you.num_turns );
 
-    // you.magic_contamination 05/03/05
     marshallShort(th, you.magic_contamination);
-    marshallString(th, you.last_altar_inscription.c_str(), 80);
+
+    marshallShort(th, you.transit_stair);
+    marshallByte(th, you.entering_level);
 }
 
 static void tag_construct_you_items(struct tagHeader &th)
@@ -855,6 +940,8 @@ static void tag_construct_you_items(struct tagHeader &th)
         marshallString(th, you.inv[i].inscription.c_str(), 80);
     }
 
+    marshallByte(th, you.quiver);
+
     // item descrip for each type & subtype
     // how many types?
     marshallByte(th, 5);
@@ -867,30 +954,24 @@ static void tag_construct_you_items(struct tagHeader &th)
     }
 
     // identification status
+    const id_arr& identy(get_typeid_array());
     // how many types?
-    marshallByte(th, 4);
+    marshallByte(th, static_cast<char>(identy.width()));
     // how many subtypes?
-    marshallByte(th, 50);
+    marshallByte(th, static_cast<char>(identy.height()));
 
-    // this is really dumb. We copy the id[] array from itemname
-    // to the stack, for no good reason that I can see.
-    id_arr identy;
-
-    save_id(identy, true);
-
-    for (i = 0; i < 4; ++i)
-    {
-        for (j = 0; j < 50; ++j)
-            marshallByte(th, identy[i][j]);
-    }
+    for (i = 0; i < identy.width(); ++i)
+        for (j = 0; j < identy.height(); ++j)
+            marshallByte(th, static_cast<char>(identy[i][j]));
 
     // how many unique items?
     marshallByte(th, 50);
     for (j = 0; j < 50; ++j)
-    {
-        marshallByte(th,you.unique_items[j]);     /* unique items */
+        marshallByte(th,you.unique_items[j]);
+
+    marshallByte(th, NUM_BOOKS);
+    for (j = 0; j < NUM_BOOKS; ++j)
         marshallByte(th,you.had_book[j]);
-    }
 
     // how many unrandarts?
     marshallShort(th, NO_UNRANDARTS);
@@ -899,6 +980,35 @@ static void tag_construct_you_items(struct tagHeader &th)
         marshallBoolean(th, does_unrandart_exist(j));
 }
 
+static void marshallPlaceInfo(struct tagHeader &th, PlaceInfo place_info)
+{
+    marshallLong(th, place_info.level_type);
+    marshallLong(th, place_info.branch);
+
+    marshallLong(th, place_info.num_visits);
+    marshallLong(th, place_info.levels_seen);
+
+    marshallLong(th, place_info.mon_kill_exp);
+    marshallLong(th, place_info.mon_kill_exp_avail);
+
+    for (int i = 0; i < KC_NCATEGORIES; i++)
+        marshallLong(th, place_info.mon_kill_num[i]);
+
+    marshallLong(th, place_info.turns_total);
+    marshallLong(th, place_info.turns_explore);
+    marshallLong(th, place_info.turns_travel);
+    marshallLong(th, place_info.turns_interlevel);
+    marshallLong(th, place_info.turns_resting);
+    marshallLong(th, place_info.turns_other);
+
+    marshallFloat(th, place_info.elapsed_total);
+    marshallFloat(th, place_info.elapsed_explore);
+    marshallFloat(th, place_info.elapsed_travel);
+    marshallFloat(th, place_info.elapsed_interlevel);
+    marshallFloat(th, place_info.elapsed_resting);
+    marshallFloat(th, place_info.elapsed_other);
+}
+
 static void tag_construct_you_dungeon(struct tagHeader &th)
 {
     int i,j;
@@ -926,6 +1036,19 @@ static void tag_construct_you_dungeon(struct tagHeader &th)
                 marshall_level_pos, marshall_as_long<god_type>);
     marshallMap(th, portals_present,
                 marshall_level_pos, marshall_as_long<portal_type>);
+
+    marshallPlaceInfo(th, you.global_info);
+    std::vector<PlaceInfo> list = you.get_all_place_info();
+    // How many different places we have info on?
+    marshallShort(th, list.size());
+
+    for (unsigned int k = 0; k < list.size(); k++)
+        marshallPlaceInfo(th, list[k]);
+
+    marshall_iterator(th, you.uniq_map_tags.begin(), you.uniq_map_tags.end(),
+                      marshallString);
+    marshall_iterator(th, you.uniq_map_names.begin(), you.uniq_map_names.end(),
+                      marshallString);
 }
 
 static void marshall_follower(tagHeader &th, const follower &f)
@@ -982,39 +1105,28 @@ static void tag_read_you(struct tagHeader &th, char minorVersion)
     char count_c;
     short count_s;
 
-    unmarshallString(th, you.your_name, 30);
+    unmarshallCString(th, you.your_name, 30);
 
     you.religion = static_cast<god_type>(unmarshallByte(th));
     you.piety = unmarshallByte(th);
-    you.invis = unmarshallByte(th);
-    you.conf = unmarshallByte(th);
-    you.paralysis = unmarshallByte(th);
-    you.slow = unmarshallByte(th);
-    you.fire_shield = unmarshallByte(th);
     you.rotting = unmarshallByte(th);
-    you.exhausted = unmarshallByte(th);
-    you.deaths_door = unmarshallByte(th);
     you.symbol = unmarshallByte(th);
     you.colour = unmarshallByte(th);
     you.pet_target = unmarshallByte(th);
 
     you.max_level = unmarshallByte(th);
     you.where_are_you = static_cast<branch_type>( unmarshallByte(th) );
-    you.char_direction = unmarshallByte(th);
+    you.char_direction = static_cast<game_direction_type>(unmarshallByte(th));
     you.your_level = unmarshallByte(th);
-    you.is_undead = unmarshallByte(th);
+    you.is_undead = static_cast<undead_state_type>(unmarshallByte(th));
     you.special_wield = unmarshallByte(th);
-    you.berserker = unmarshallByte(th);
     you.berserk_penalty = unmarshallByte(th);
     you.level_type = static_cast<level_area_type>( unmarshallByte(th) );
+    you.level_type_name = unmarshallString(th);
     you.synch_time = unmarshallByte(th);
     you.disease = unmarshallByte(th);
-    you.species = unmarshallByte(th);
+    you.species = static_cast<species_type>(unmarshallByte(th));
     you.hp = unmarshallShort(th);
-    you.haste = unmarshallByte(th);
-    you.might = unmarshallByte(th);
-    you.levitation = unmarshallByte(th);
-    you.poisoning = unmarshallByte(th);
     you.hunger = unmarshallShort(th);
 
     // how many you.equip?
@@ -1027,8 +1139,6 @@ static void tag_read_you(struct tagHeader &th, char minorVersion)
     you.strength = unmarshallByte(th);
     you.intel = unmarshallByte(th);
     you.dex = unmarshallByte(th);
-    you.confusing_touch = unmarshallByte(th);
-    you.sure_blade = unmarshallByte(th);
     you.hit_points_regeneration = unmarshallByte(th);
     you.magic_points_regeneration = unmarshallByte(th);
 
@@ -1036,7 +1146,7 @@ static void tag_read_you(struct tagHeader &th, char minorVersion)
     you.experience = unmarshallLong(th);
     you.gold = unmarshallLong(th);
 
-    you.char_class = unmarshallByte(th);
+    you.char_class = static_cast<job_type>(unmarshallByte(th));
     you.experience_level = unmarshallByte(th);
     you.exp_available = unmarshallLong(th);
 
@@ -1050,10 +1160,11 @@ static void tag_read_you(struct tagHeader &th, char minorVersion)
     you.base_magic_points = unmarshallShort(th);
     you.base_magic_points2 = unmarshallShort(th);
 
-    you.x_pos = unmarshallShort(th);
-    you.y_pos = unmarshallShort(th);
+    const int x = unmarshallShort(th);
+    const int y = unmarshallShort(th);
+    you.moveto(x, y);
 
-    unmarshallString(th, you.class_name, 30);
+    unmarshallCString(th, you.class_name, 30);
 
     you.burden = unmarshallShort(th);
 
@@ -1062,7 +1173,8 @@ static void tag_read_you(struct tagHeader &th, char minorVersion)
     count_c = unmarshallByte(th);
     for (i = 0; i < count_c; ++i)
     {
-        you.spells[i] = unmarshallByte(th);
+        you.spells[i] =
+            static_cast<spell_type>( (unsigned char) unmarshallByte(th) );
         if (you.spells[i] != SPELL_NO_SPELL)
             you.spell_no++;
     }
@@ -1073,7 +1185,8 @@ static void tag_read_you(struct tagHeader &th, char minorVersion)
     
     count_c = unmarshallByte(th);
     for (i = 0; i < count_c; i++)
-        you.ability_letter_table[i] = unmarshallShort(th);
+        you.ability_letter_table[i] =
+            static_cast<ability_type>(unmarshallShort(th));
 
     // how many skills?
     count_c = unmarshallByte(th);
@@ -1082,7 +1195,6 @@ static void tag_read_you(struct tagHeader &th, char minorVersion)
         you.skills[j] = unmarshallByte(th);
         you.practise_skill[j] = unmarshallByte(th);
         you.skill_points[j] = unmarshallLong(th);
-
         you.skill_order[j] = unmarshallByte(th);
     }
 
@@ -1092,13 +1204,17 @@ static void tag_read_you(struct tagHeader &th, char minorVersion)
     // how many durations?
     count_c = unmarshallByte(th);
     for (j = 0; j < count_c; ++j)
-        you.duration[j] = (unsigned char) unmarshallByte(th);
+        you.duration[j] = unmarshallLong(th);
 
     // how many attributes?
     count_c = unmarshallByte(th);
     for (j = 0; j < count_c; ++j)
         you.attribute[j] = unmarshallByte(th);
 
+    count_c = unmarshallByte(th);
+    for (j = 0; j < count_c; ++j)
+        you.sacrifice_value[j] = unmarshallLong(th);
+
     // how many mutations/demon powers?
     count_s = unmarshallShort(th);
     for (j = 0; j < count_s; ++j)
@@ -1131,14 +1247,16 @@ static void tag_read_you(struct tagHeader &th, char minorVersion)
     you.wizard = (bool) unmarshallByte(th);
 
     // time of character creation
-    unmarshallString( th, buff, 20 );
+    unmarshallCString( th, buff, 20 );
     you.birth_time = parse_date_string( buff );
 
     you.real_time = unmarshallLong(th);
     you.num_turns = unmarshallLong(th);
 
     you.magic_contamination = unmarshallShort(th);
-    you.last_altar_inscription = unmarshallString(th, 80);
+
+    you.transit_stair  = static_cast<dungeon_feature_type>(unmarshallShort(th));
+    you.entering_level = unmarshallByte(th);
 }
 
 static void tag_read_you_items(struct tagHeader &th, char minorVersion)
@@ -1151,23 +1269,18 @@ static void tag_read_you_items(struct tagHeader &th, char minorVersion)
     count_c = unmarshallByte(th);
     for (i = 0; i < count_c; ++i)
     {
-        you.inv[i].orig_monnum = you.inv[i].orig_place = 0;
-        you.inv[i].inscription.clear();
-        you.inv[i].base_type = (unsigned char) unmarshallByte(th);
+        you.inv[i].base_type =
+            static_cast<object_class_type>(unmarshallByte(th));
         you.inv[i].sub_type = (unsigned char) unmarshallByte(th);
         you.inv[i].plus = unmarshallShort(th);
         you.inv[i].special = unmarshallLong(th);
         you.inv[i].colour = (unsigned char) unmarshallByte(th);
         you.inv[i].flags = (unsigned long) unmarshallLong(th);
         you.inv[i].quantity = unmarshallShort(th);
-        you.inv[i].plus2 = unmarshallShort(th);
-        
+        you.inv[i].plus2 = unmarshallShort(th);       
         you.inv[i].orig_place  = unmarshallShort(th);
         you.inv[i].orig_monnum = unmarshallShort(th);
-
-        char insstring[80];
-        unmarshallString(th, insstring, 80);
-        you.inv[i].inscription = std::string(insstring);
+        you.inv[i].inscription = unmarshallString(th, 80);
 
         // these never need to be saved for items in the inventory -- bwr
         you.inv[i].x = -1;
@@ -1176,6 +1289,8 @@ static void tag_read_you_items(struct tagHeader &th, char minorVersion)
         you.inv[i].slot = index_to_letter(i);
     }
 
+    you.quiver = unmarshallByte(th);
+
     // item descrip for each type & subtype
     // how many types?
     count_c = unmarshallByte(th);
@@ -1196,8 +1311,8 @@ static void tag_read_you_items(struct tagHeader &th, char minorVersion)
     {
         for (j = 0; j < count_c2; ++j)
         {
-            char ch;
-            ch = unmarshallByte(th);
+            const item_type_id_state_type ch =
+                static_cast<item_type_id_state_type>(unmarshallByte(th));
 
             switch (i)
             {
@@ -1220,10 +1335,13 @@ static void tag_read_you_items(struct tagHeader &th, char minorVersion)
     // how many unique items?
     count_c = unmarshallByte(th);
     for (j = 0; j < count_c; ++j)
-    {
-        you.unique_items[j] = unmarshallByte(th);
+        you.unique_items[j] =
+            static_cast<unique_item_status_type>(unmarshallByte(th));
+
+    // how many books?
+    count_c = unmarshallByte(th);
+    for (j = 0; j < count_c; ++j)
         you.had_book[j] = unmarshallByte(th);
-    }
 
     // how many unrandarts?
     count_s = unmarshallShort(th);
@@ -1233,15 +1351,50 @@ static void tag_read_you_items(struct tagHeader &th, char minorVersion)
     // # of unrandarts could certainly change.  If it does,
     // the new ones won't exist yet - zero them out.
     for (; j < NO_UNRANDARTS; j++)
-        set_unrandart_exist(j, 0);
+        set_unrandart_exist(j, false);
+}
+
+static PlaceInfo unmarshallPlaceInfo(struct tagHeader &th)
+{
+    PlaceInfo place_info;
+
+    place_info.level_type = (int) unmarshallLong(th);
+    place_info.branch     = (int) unmarshallLong(th);
+
+    place_info.num_visits  = (unsigned long) unmarshallLong(th);
+    place_info.levels_seen = (unsigned long) unmarshallLong(th);
+
+    place_info.mon_kill_exp       = (unsigned long) unmarshallLong(th);
+    place_info.mon_kill_exp_avail = (unsigned long) unmarshallLong(th);
+
+    for (int i = 0; i < KC_NCATEGORIES; i++)
+        place_info.mon_kill_num[i] = (unsigned long) unmarshallLong(th);
+
+    place_info.turns_total      = unmarshallLong(th);
+    place_info.turns_explore    = unmarshallLong(th);
+    place_info.turns_travel     = unmarshallLong(th);
+    place_info.turns_interlevel = unmarshallLong(th);
+    place_info.turns_resting    = unmarshallLong(th);
+    place_info.turns_other      = unmarshallLong(th);
+
+    place_info.elapsed_total      = (double) unmarshallFloat(th);
+    place_info.elapsed_explore    = (double) unmarshallFloat(th);
+    place_info.elapsed_travel     = (double) unmarshallFloat(th);
+    place_info.elapsed_interlevel = (double) unmarshallFloat(th);
+    place_info.elapsed_resting    = (double) unmarshallFloat(th);
+    place_info.elapsed_other      = (double) unmarshallFloat(th);
+
+    return place_info;
 }
 
 static void tag_read_you_dungeon(struct tagHeader &th)
 {
-    int i,j;
-    int count_c;
+    int   i,j;
+    int   count_c;
     short count_s;
 
+    unsigned short count_p;
+
     // how many unique creatures?
     count_c = unmarshallShort(th);
     you.unique_creatures.init(false);
@@ -1274,12 +1427,42 @@ static void tag_read_you_dungeon(struct tagHeader &th)
                   unmarshall_level_pos, unmarshall_long_as<god_type>);
     unmarshallMap(th, portals_present,
                   unmarshall_level_pos, unmarshall_long_as<portal_type>);
+
+    PlaceInfo place_info = unmarshallPlaceInfo(th);
+    ASSERT(place_info.is_global());
+    you.set_place_info(place_info);
+
+    std::vector<PlaceInfo> list = you.get_all_place_info();
+    count_p = (unsigned short) unmarshallShort(th);
+    // Use "<=" so that adding more branches or non-dungeon places
+    // won't break save-file compatibility.
+    ASSERT(count_p <= list.size());
+
+    for (i = 0; i < count_p; i++)
+    {
+        place_info = unmarshallPlaceInfo(th);
+        ASSERT(!place_info.is_global());
+        you.set_place_info(place_info);
+    }
+
+    typedef std::set<std::string> string_set;
+    typedef std::pair<string_set::iterator, bool> ssipair;
+    unmarshall_container(th, you.uniq_map_tags,
+                         (ssipair (string_set::*)(const std::string &))
+                         &string_set::insert,
+                         unmarshallString);
+    unmarshall_container(th, you.uniq_map_names,
+                         (ssipair (string_set::*)(const std::string &))
+                         &string_set::insert,
+                         unmarshallString);
 }
 
 static void tag_read_lost_monsters(tagHeader &th, int minorVersion)
 {
     the_lost_ones.clear();
-    
+
+    unwind_var<int> minor_version(tag_minor_version,
+                                  static_cast<int>(minorVersion));    
     unmarshallMap(th, the_lost_ones,
                   unmarshall_level_id, unmarshall_follower_list);
 }
@@ -1288,9 +1471,6 @@ static void tag_read_lost_monsters(tagHeader &th, int minorVersion)
 
 static void tag_construct_level(struct tagHeader &th)
 {
-    int i;
-    int count_x, count_y;
-
     marshallFloat(th, (float)you.elapsed_time);
 
     // map grids
@@ -1298,23 +1478,28 @@ static void tag_construct_level(struct tagHeader &th)
     marshallShort(th, GXM);
     // how many Y?
     marshallShort(th, GYM);
-    for (count_x = 0; count_x < GXM; count_x++)
+
+    marshallLong(th, env.turns_on_level);
+
+    for (int count_x = 0; count_x < GXM; count_x++)
     {
-        for (count_y = 0; count_y < GYM; count_y++)
+        for (int count_y = 0; count_y < GYM; count_y++)
         {
             marshallByte(th, grd[count_x][count_y]);
-            marshallShort(th, env.map[count_x][count_y]);
-            marshallShort(th, env.map_col[count_x][count_y].colour);
-            marshallShort(th, env.map_col[count_x][count_y].flags);
+            marshallShort(th, env.map[count_x][count_y].object);
+            marshallShort(th, env.map[count_x][count_y].colour);            
+            marshallShort(th, env.map[count_x][count_y].flags);
             marshallByte(th, env.cgrid[count_x][count_y]);
         }
     }
 
+    run_length_encode(th, marshallByte, env.grid_colours, GXM, GYM);
+
     marshallShort(th, env.cloud_no);
 
     // how many clouds?
     marshallShort(th, MAX_CLOUDS);
-    for (i = 0; i < MAX_CLOUDS; i++)
+    for (int i = 0; i < MAX_CLOUDS; i++)
     {
         marshallByte(th, env.cloud[i].x);
         marshallByte(th, env.cloud[i].y);
@@ -1324,8 +1509,8 @@ static void tag_construct_level(struct tagHeader &th)
     }
 
     // how many shops?
-    marshallByte(th, 5);
-    for (i = 0; i < 5; i++)
+    marshallByte(th, MAX_SHOPS);
+    for (int i = 0; i < MAX_SHOPS; i++)
     {
         marshallByte(th, env.shop[i].keeper_name[0]);
         marshallByte(th, env.shop[i].keeper_name[1]);
@@ -1336,6 +1521,8 @@ static void tag_construct_level(struct tagHeader &th)
         marshallByte(th, env.shop[i].type);
         marshallByte(th, env.shop[i].level);
     }
+
+    env.markers.write(th);
 }
 
 static void marshall_item(tagHeader &th, const item_def &item)
@@ -1364,7 +1551,7 @@ static void marshall_item(tagHeader &th, const item_def &item)
 
 static void unmarshall_item(tagHeader &th, item_def &item)
 {
-    item.base_type = (unsigned char) unmarshallByte(th);
+    item.base_type = static_cast<object_class_type>(unmarshallByte(th));
     item.sub_type = (unsigned char) unmarshallByte(th);
     item.plus = unmarshallShort(th);
     item.plus2 = unmarshallShort(th);
@@ -1384,23 +1571,17 @@ static void unmarshall_item(tagHeader &th, item_def &item)
     unmarshallShort(th);  // igrd[item.x][item.y] -- unused
 
     item.slot = unmarshallByte(th);
-    item.inscription.clear();
 
     item.orig_place  = unmarshallShort(th);
-    item.orig_monnum = unmarshallShort(th);
-        
-    char insstring[80];
-    unmarshallString(th, insstring, 80);
-    item.inscription = std::string(insstring);
+    item.orig_monnum = unmarshallShort(th);        
+    item.inscription = unmarshallString(th, 80);
 }
 
 static void tag_construct_level_items(struct tagHeader &th)
 {
-    int i;
-
     // how many traps?
     marshallShort(th, MAX_TRAPS);
-    for (i = 0; i < MAX_TRAPS; ++i)
+    for (int i = 0; i < MAX_TRAPS; ++i)
     {
         marshallByte(th, env.trap[i].type);
         marshallByte(th, env.trap[i].x);
@@ -1409,7 +1590,7 @@ static void tag_construct_level_items(struct tagHeader &th)
 
     // how many items?
     marshallShort(th, MAX_ITEMS);
-    for (i = 0; i < MAX_ITEMS; ++i)
+    for (int i = 0; i < MAX_ITEMS; ++i)
         marshall_item(th, mitm[i]);
 }
 
@@ -1418,6 +1599,8 @@ static void marshall_mon_enchant(tagHeader &th, const mon_enchant &me)
     marshallShort(th, me.ench);
     marshallShort(th, me.degree);
     marshallShort(th, me.who);
+    marshallShort(th, me.duration);
+    marshallShort(th, me.maxduration);
 }
 
 static mon_enchant unmarshall_mon_enchant(tagHeader &th)
@@ -1426,6 +1609,8 @@ static mon_enchant unmarshall_mon_enchant(tagHeader &th)
     me.ench   = static_cast<enchant_type>( unmarshallShort(th) );
     me.degree = unmarshallShort(th);
     me.who    = static_cast<kill_category>( unmarshallShort(th) );
+    me.duration = unmarshallShort(th);
+    me.maxduration = unmarshallShort(th);
     return (me);
 }
 
@@ -1442,12 +1627,13 @@ static void marshall_monster(tagHeader &th, const monsters &m)
     marshallByte(th, m.target_x);
     marshallByte(th, m.target_y);
     marshallLong(th, m.flags);
+    marshallLong(th, m.experience);
 
     marshallShort(th, m.enchantments.size());
     for (mon_enchant_list::const_iterator i = m.enchantments.begin();
          i != m.enchantments.end(); ++i)
     {
-        marshall_mon_enchant(th, *i);
+        marshall_mon_enchant(th, i->second);
     }
 
     marshallShort(th, m.type);
@@ -1505,55 +1691,52 @@ void tag_construct_level_attitude(struct tagHeader &th)
 
 static void tag_read_level( struct tagHeader &th, char minorVersion )
 {
-    int i,j;
-    int gx, gy;
-
-    env.elapsed_time = (double)unmarshallFloat(th);
-
+    env.elapsed_time = unmarshallFloat(th);
     // map grids
     // how many X?
-    gx = unmarshallShort(th);
+    const int gx = unmarshallShort(th);
     // how many Y?
-    gy = unmarshallShort(th);
-    for (i = 0; i < gx; i++)
-    {
-        for (j = 0; j < gy; j++)
-        {
-            grd[i][j] = unmarshallByte(th);
-            
-            env.map[i][j] = (unsigned short) unmarshallShort(th);
-            
-            if ((env.map[i][j] & 0xFF) == 201)       // what is this??
-                env.map[i][j] = (env.map[i][j] & 0xFF00U) | 239;
+    const int gy = unmarshallShort(th);
 
-            env.map_col[i][j].colour = unmarshallShort(th);
-            env.map_col[i][j].flags  = unmarshallShort(th);
+    env.turns_on_level = unmarshallLong(th);
+    
+    for (int i = 0; i < gx; i++)
+    {
+        for (int j = 0; j < gy; j++)
+        {
+            grd[i][j] =
+                static_cast<dungeon_feature_type>(
+                    static_cast<unsigned char>(unmarshallByte(th)) );
+
+            env.map[i][j].object = unmarshallShort(th);
+            env.map[i][j].colour = unmarshallShort(th);
+            env.map[i][j].flags  = unmarshallShort(th);
 
             mgrd[i][j] = NON_MONSTER;
             env.cgrid[i][j] = (unsigned char) unmarshallByte(th);
-
-            // Ugh! Backward compatibility. 101 was the old (buggy) EMPTY_CLOUD
-            if (minorVersion < 2 && env.cgrid[i][j] == 101)
-                env.cgrid[i][j] = EMPTY_CLOUD;
         }
     }
 
+    env.grid_colours.init(BLACK);
+    run_length_decode(th, unmarshallByte, env.grid_colours, GXM, GYM);
+
     env.cloud_no = unmarshallShort(th);
 
     // how many clouds?
-    gx = unmarshallShort(th);
-    for (i = 0; i < gx; i++)
+    const int num_clouds = unmarshallShort(th);
+    for (int i = 0; i < num_clouds; i++)
     {
         env.cloud[i].x = unmarshallByte(th);
         env.cloud[i].y = unmarshallByte(th);
-        env.cloud[i].type = unmarshallByte(th);
+        env.cloud[i].type = static_cast<cloud_type>(unmarshallByte(th));
         env.cloud[i].decay = unmarshallShort(th);
         env.cloud[i].whose = static_cast<kill_category>(unmarshallShort(th));
     }
 
     // how many shops?
-    gx = unmarshallByte(th);
-    for (i = 0; i < gx; i++)
+    const int num_shops = unmarshallByte(th);
+    ASSERT(num_shops <= MAX_SHOPS);
+    for (int i = 0; i < num_shops; i++)
     {
         env.shop[i].keeper_name[0] = unmarshallByte(th);
         env.shop[i].keeper_name[1] = unmarshallByte(th);
@@ -1561,28 +1744,29 @@ static void tag_read_level( struct tagHeader &th, char minorVersion )
         env.shop[i].x = unmarshallByte(th);
         env.shop[i].y = unmarshallByte(th);
         env.shop[i].greed = unmarshallByte(th);
-        env.shop[i].type = unmarshallByte(th);
+        env.shop[i].type = static_cast<shop_type>(unmarshallByte(th));
         env.shop[i].level = unmarshallByte(th);
     }
+
+    env.markers.read(th);
 }
 
 static void tag_read_level_items(struct tagHeader &th, char minorVersion)
 {
-    int i;
-    int count;
-
     // how many traps?
-    count = unmarshallShort(th);
-    for (i = 0; i < count; ++i)
+    const int trap_count = unmarshallShort(th);
+    for (int i = 0; i < trap_count; ++i)
     {
-        env.trap[i].type = unmarshallByte(th);
+        env.trap[i].type =
+            static_cast<trap_type>(
+                static_cast<unsigned char>(unmarshallByte(th)) );
         env.trap[i].x = unmarshallByte(th);
         env.trap[i].y = unmarshallByte(th);
     }
 
     // how many items?
-    count = unmarshallShort(th);
-    for (i = 0; i < count; ++i)
+    const int item_count = unmarshallShort(th);
+    for (int i = 0; i < item_count; ++i)
         unmarshall_item(th, mitm[i]);
 }
 
@@ -1594,17 +1778,25 @@ static void unmarshall_monster(tagHeader &th, monsters &m)
     m.speed = unmarshallByte(th);
     // Avoid sign extension when loading files (Elethiomel's hang)
     m.speed_increment = (unsigned char) unmarshallByte(th);
-    m.behaviour = unmarshallByte(th);
+    m.behaviour = static_cast<beh_type>(unmarshallByte(th));
     m.x = unmarshallByte(th);
     m.y = unmarshallByte(th);
     m.target_x = unmarshallByte(th);
     m.target_y = unmarshallByte(th);
     m.flags = unmarshallLong(th);
 
+    if (tag_minor_version >= 2)
+        m.experience = static_cast<unsigned long>(unmarshallLong(th));
+    else
+        m.experience = 0L;
+
     m.enchantments.clear();
     const int nenchs = unmarshallShort(th);
     for (int i = 0; i < nenchs; ++i)
-        m.enchantments.insert( unmarshall_mon_enchant(th) );
+    {
+        mon_enchant me = unmarshall_mon_enchant(th);
+        m.enchantments[me.ench] = me;
+    }
 
     m.type = unmarshallShort(th);
     m.hit_points = unmarshallShort(th);
@@ -1617,7 +1809,7 @@ static void unmarshall_monster(tagHeader &th, monsters &m)
         m.inv[j] = unmarshallShort(th);
 
     for (int j = 0; j < NUM_MONSTER_SPELL_SLOTS; ++j)
-        m.spells[j] = unmarshallShort(th);
+        m.spells[j] = static_cast<spell_type>( unmarshallShort(th) );
 
     m.god = (god_type) unmarshallByte(th);
 
@@ -1642,6 +1834,8 @@ static void tag_read_level_monsters(struct tagHeader &th, char minorVersion)
     // how many monster inventory slots?
     icount = unmarshallByte(th);
 
+    unwind_var<int> minor_version(tag_minor_version,
+                                  static_cast<int>(minorVersion));
     for (i = 0; i < count; i++)
     {
         monsters &m = menv[i];
@@ -1661,7 +1855,7 @@ void tag_read_level_attitude(struct tagHeader &th)
 
     for (i = 0; i < count; i++)
     {
-        menv[i].attitude = unmarshallByte(th);
+        menv[i].attitude = static_cast<mon_attitude_type>(unmarshallByte(th));
         menv[i].foe = unmarshallShort(th);
     }
 }
@@ -1712,7 +1906,7 @@ void tag_missing_level_attitude()
         }
 
         menv[i].attitude = (isFriendly)?ATT_FRIENDLY : ATT_HOSTILE;
-        menv[i].behaviour = new_beh;
+        menv[i].behaviour = static_cast<beh_type>(new_beh);
         menv[i].foe_memory = 0;
     }
 }
