@@ -16,6 +16,38 @@
 #   include <vector>
 #   include <map>
 
+enum run_check_type
+{
+    RCHECK_LEFT,
+    RCHECK_FRONT,
+    RCHECK_RIGHT
+};
+
+enum run_dir_type
+{
+    RDIR_UP = 0,
+    RDIR_UP_RIGHT,
+    RDIR_RIGHT,
+    RDIR_DOWN_RIGHT,
+    RDIR_DOWN,
+    RDIR_DOWN_LEFT,
+    RDIR_LEFT,
+    RDIR_UP_LEFT,
+    RDIR_REST
+};
+
+enum run_mode_type
+{
+    RMODE_INTERLEVEL     = -4, // Interlevel travel (Ctrl+G)
+    RMODE_EXPLORE_GREEDY = -3, // Explore + grab items (Tab/Ctrl+I)
+    RMODE_EXPLORE        = -2, // Exploring (Ctrl+O)
+    RMODE_TRAVEL         = -1, // Classic or Plain Old travel
+    RMODE_NOT_RUNNING    = 0,  // must remain equal to 0
+    RMODE_CONTINUE,
+    RMODE_START,
+    RMODE_REST_DURATION = 100
+};
+
 /* ***********************************************************************
  * Initialises the travel subsystem.
  *
@@ -23,21 +55,23 @@
  * called from: initfile (what's a better place to initialise stuff?)
  * *********************************************************************** */
 void initialise_travel();
+void init_travel_terrain_check(bool check_race_equip = true);
 void stop_running(void);
 void travel_init_new_level();
-void toggle_exclude(int x, int y);
+void cycle_exclude_radius(const coord_def &p);
+void toggle_exclude(const coord_def &p);
+void set_exclude(const coord_def &p, int radius2);
 void clear_excludes();
 unsigned char is_waypoint(int x, int y);
 void update_excludes();
-bool is_stair(unsigned gridc);
-bool is_travelable_stair(unsigned gridc);
-command_type stair_direction(int stair_feat);
-bool is_player_mapped(unsigned char envch);
+bool is_exclude_root(const coord_def &p);
+bool is_stair(dungeon_feature_type gridc);
+bool is_travelable_stair(dungeon_feature_type gridc);
 command_type direction_to_command( char x, char y );
 bool is_resting( void );
 bool can_travel_interlevel();
-
-bool is_player_mapped(int grid_x, int grid_y);
+bool is_traversable(dungeon_feature_type grid);
+void explore_pickup_event(int did_pickup, int tried_pickup);
 
 void find_travel_pos(int you_x, int you_y, char *move_x, char *move_y, 
                      std::vector<coord_def>* coords = NULL);
@@ -59,7 +93,10 @@ bool is_travelsafe_square(int x, int y, bool ignore_hostile = false,
 void start_explore(bool grab_items = false);
 
 struct level_pos;
-void start_translevel_travel(const level_pos &pos);
+struct level_id;
+struct travel_target;
+
+void start_translevel_travel(const travel_target &pos);
 
 void start_translevel_travel(bool prompt_for_destination = true);
 
@@ -73,8 +110,6 @@ void prevent_travel_to(const std::string &dungeon_feature_name);
 
 // Sort dungeon features as appropriate.
 void arrange_features(std::vector<coord_def> &features);
-
-struct level_id;
 int level_distance(level_id first, level_id second);
 
 bool can_travel_to(const level_id &lid);
@@ -94,7 +129,7 @@ enum translevel_prompt_flags
                                                 | TPF_REMEMBER_TARGET
 };
 
-level_pos prompt_translevel_target(int prompt_flags = TPF_DEFAULT_OPTIONS);
+travel_target prompt_translevel_target(int prompt_flags = TPF_DEFAULT_OPTIONS);
 
 // Magic numbers for point_distance:
 
@@ -128,7 +163,8 @@ enum explore_stop_type
     ES_GREEDY_PICKUP      = 0x04,
     ES_STAIR              = 0x08,
     ES_SHOP               = 0x10,
-    ES_ALTAR              = 0x20
+    ES_ALTAR              = 0x20,
+    ES_PORTAL             = 0x40
 };
 
 ////////////////////////////////////////////////////////////////////////////
@@ -138,9 +174,9 @@ enum explore_stop_type
 struct level_id
 {
 public:
-    int branch;     // The branch in which the level is.
-    int depth;      // What depth (in this branch - starting from 1)
-    int level_type;
+    branch_type branch;     // The branch in which the level is.
+    int depth;              // What depth (in this branch - starting from 1)
+    level_area_type level_type;
 
 public:
     // Returns the level_id of the current level.
@@ -150,36 +186,52 @@ public:
     // 'pos' on the current level leads to.
     static level_id get_next_level_id(const coord_def &pos);
 
-    level_id() : branch(0), depth(-1), level_type(LEVEL_DUNGEON) { }
-    level_id(int br, int dep, int ltype = LEVEL_DUNGEON)
+    level_id()
+        : branch(BRANCH_MAIN_DUNGEON), depth(-1),
+          level_type(LEVEL_DUNGEON)
+    {
+    }
+    level_id(branch_type br, int dep, level_area_type ltype = LEVEL_DUNGEON)
         : branch(br), depth(dep), level_type(ltype)
     {
         if (level_type != LEVEL_DUNGEON)
-            branch = depth = -1;
+        {
+            depth = -1;
+            branch = NUM_BRANCHES;
+        }
     }
-    level_id(int ltype) : branch(-1), depth(-1), level_type(ltype) { }
+    level_id(const level_id &ot)
+        : branch(ot.branch), depth(ot.depth), level_type(ot.level_type)
+    {
+    }
+    level_id(level_area_type ltype)
+        : branch(BRANCH_MAIN_DUNGEON), depth(-1), level_type(ltype)
+    {
+    }
+
+    static level_id parse_level_id(const std::string &s) throw (std::string);
 
     unsigned short packed_place() const;
     std::string describe(bool long_name = false, bool with_number = true) const;
 
-    void reset()
+    void clear()
     {
-        branch = 0;
+        branch = BRANCH_MAIN_DUNGEON;
         depth  = -1;
         level_type = LEVEL_DUNGEON;
     }
     
     bool is_valid() const
     {
-        return (branch != -1 && depth != -1) || level_type != LEVEL_DUNGEON;
+        return (branch != NUM_BRANCHES && depth != -1)
+            || level_type != LEVEL_DUNGEON;
     }
 
     bool operator == ( const level_id &id ) const
     {
-        return (level_type == LEVEL_DUNGEON?
-                branch == id.branch && depth == id.depth
-                && level_type == id.level_type
-                : level_type == id.level_type);
+        return (level_type == id.level_type
+                && (level_type != LEVEL_DUNGEON
+                    || (branch == id.branch && depth == id.depth)));
     }
 
     bool operator != ( const level_id &id ) const
@@ -191,6 +243,9 @@ public:
     {
         if (level_type != id.level_type)
             return (level_type < id.level_type);
+
+        if (level_type != LEVEL_DUNGEON)
+            return (false);
         
         return (branch < id.branch) || (branch==id.branch && depth < id.depth);
     }
@@ -241,9 +296,9 @@ struct level_pos
         return id.depth > -1 && pos.x != -1 && pos.y != -1;
     }
 
-    void reset()
+    void clear()
     {
-        id.reset();
+        id.clear();
         pos = coord_def(-1, -1);
     }
 
@@ -251,6 +306,26 @@ struct level_pos
     void load(FILE *);
 };
 
+struct travel_target
+{
+    level_pos p;
+    bool      entrance_only;
+
+    travel_target(const level_pos &_pos, bool entry = false)
+        : p(_pos), entrance_only(entry)
+    {
+    }
+    travel_target()
+        : p(), entrance_only(false)
+    {
+    }
+    void clear()
+    {
+        p.clear();
+        entrance_only = false;
+    }
+};
+
 // Tracks items discovered by explore in this turn.
 class LevelStashes;
 class explore_discoveries
@@ -258,17 +333,12 @@ class explore_discoveries
 public:
     explore_discoveries();
     
-    void found_feature(const coord_def &pos, int grid);
+    void found_feature(const coord_def &pos, dungeon_feature_type grid);
     void found_item(const coord_def &pos, const item_def &item);
 
     // Reports discoveries and prompts the player to stop (if necessary).
     bool prompt_stop() const;
 
-private:
-    template <class C> void say_any(const C &coll, const char *stub) const;
-    std::string cleaned_feature_description(int feature) const;
-    void add_item(const item_def &item);
-    
 private:
     template <class Z> struct named_thing {
         std::string name;
@@ -276,39 +346,84 @@ private:
 
         named_thing(const std::string &n, Z t) : name(n), thing(t) { }
         operator std::string () const { return name; }
+
+        bool operator == (const named_thing<Z> &other) const
+        {
+            return (name == other.name);
+        }
     };
 
     int es_flags;
     const LevelStashes *current_level;
     std::vector< named_thing<item_def> > items;
     std::vector< named_thing<int> > stairs;
+    std::vector< named_thing<int> > portals;
     std::vector< named_thing<int> > shops;
     std::vector< named_thing<int> > altars;
+
+private:
+    template <class C> void say_any(const C &coll, const char *stub) const;
+    template <class citer> bool has_duplicates(citer, citer) const;
+    
+    std::string cleaned_feature_description(dungeon_feature_type feature) const;
+    void add_item(const item_def &item);
+    void add_stair(const named_thing<int> &stair);
+    std::vector<std::string> apply_quantities(
+        const std::vector< named_thing<int> > &v) const;
+    bool merge_feature(
+        std::vector< named_thing<int> > &v,
+        const named_thing<int> &feat) const;    
 };
 
 struct stair_info
 {
+public:
+    enum stair_type
+    {
+        PHYSICAL,
+        PLACEHOLDER
+    };
+    
+public:
     coord_def position;     // Position of stair
-    int       grid;         // Grid feature of the stair.
+    dungeon_feature_type grid; // Grid feature of the stair.
     level_pos destination;  // The level and the position on the level this
                             // stair leads to. This may be a guess.
     int       distance;     // The distance traveled to reach this stair.
     bool      guessed_pos;  // true if we're not sure that 'destination' is
                             // correct.
+    stair_type type;
 
     stair_info()
         : position(-1, -1), grid(DNGN_FLOOR), destination(),
-          distance(-1), guessed_pos(true)
+          distance(-1), guessed_pos(true), type(PHYSICAL)
     {
     }
 
-    void reset_distance()
-    {
-        distance = -1;
-    }
+    void clear_distance() { distance = -1; }
 
     void save(FILE *) const;
     void load(FILE *);
+
+    std::string describe() const;
+
+    bool can_travel() const { return (type == PHYSICAL); }
+};
+
+struct travel_exclude
+{
+    coord_def pos;
+    int       radius;
+
+    travel_exclude(const coord_def &p, int r = LOS_RADIUS)
+        : pos(p), radius(r)
+    {
+    }
+
+    int radius_sq() const
+    {
+        return (radius * radius * 17 / 16);
+    }
 };
 
 // Information on a level that interlevel travel needs.
@@ -328,12 +443,14 @@ struct LevelInfo
 
     stair_info *get_stair(int x, int y);
     stair_info *get_stair(const coord_def &pos);
+    bool empty() const;
+    bool know_stair(const coord_def &pos) const;
     int get_stair_index(const coord_def &pos) const;
 
-    void reset_distances();
+    void clear_distances();
     void set_level_excludes();
 
-    const std::vector<coord_def> &get_excludes() const
+    const std::vector<travel_exclude> &get_excludes() const
     {
         return excludes;
     }
@@ -367,12 +484,16 @@ private:
     std::vector<stair_info> stairs;
 
     // Squares that are not safe to travel to.
-    std::vector<coord_def> excludes;
+    std::vector<travel_exclude> excludes;
 
     std::vector<short> stair_distances;  // Dist between stairs
     level_id id;
 
     friend class TravelCache;
+
+private:
+    void create_placeholder_stair(const coord_def &, const level_pos &);
+    void resize_stair_distances();
 };
 
 const int TRAVEL_WAYPOINT_COUNT = 10;
@@ -380,14 +501,7 @@ const int TRAVEL_WAYPOINT_COUNT = 10;
 class TravelCache
 {
 public:
-    void reset_distances();
-
-    // Get the LevelInfo for the specified level (defaults to the current
-    // level).
-    LevelInfo& get_level_info(unsigned char branch = 0, int depth = -1)
-    {
-        return get_level_info( level_id(branch, depth) );
-    }
+    void clear_distances();
 
     LevelInfo& get_level_info(const level_id &lev)
     {
@@ -402,6 +516,7 @@ public:
         return (i != levels.end()? &i->second : NULL);
     }
 
+    bool know_stair(const coord_def &c) const;
     bool know_level(const level_id &lev) const
     {
         return levels.find(lev) != levels.end();
@@ -490,6 +605,7 @@ public:
 protected:
     bool is_greed_inducing_square(const coord_def &c) const;
     bool path_examine_point(const coord_def &c);
+    virtual bool point_traverse_delay(const coord_def &c);
     virtual bool path_flood(const coord_def &c, const coord_def &dc);
     bool square_slows_movement(const coord_def &c);
     void check_square_greed(const coord_def &c);
@@ -497,7 +613,7 @@ protected:
 
 protected:
     static const int UNFOUND_DIST  = -30000;
-    static const int INFINITE_DIST =  30000;
+    static const int INFINITE_DIST =  INFINITE_DISTANCE;
     
 protected:
     run_mode_type runmode;
