@@ -1,9 +1,9 @@
 /*
  *  File:       unrand.cc
- *  Summary:    Definitions for unrandom artifacts.
+ *  Summary:    Definitions for unrandom artefacts.
  *  Written by: Linley Henzell
  *
- *  Modified for Crawl Reference by $Author: dshaligram $ on $Date: 2006-12-17T13:49:57.236391Z $
+ *  Modified for Crawl Reference by $Author: ennewalker $ on $Date: 2007-11-23T02:25:44.246247Z $
  *
  *  Change History (most recent first): 
  *               <3>     7 Aug 2001      MV             Added many new items
@@ -15,6 +15,7 @@
 #define UNRAND_H
 
 #include "defines.h"
+#include "itemprop.h"
 
 /*
    List of "unrandom" artefacts. Not the same as "fixed" artefacts, which are
@@ -79,8 +80,7 @@
    Ranges for all these: 0 or 1.
 
    make noise: Irritate nearby creatures and disrupts rest. Weapons only.
-   Range: 1 - 4, for different types of noises (see special_wielded() in
-   it_use3.cc); 0 for none.
+   Range: 0 or 1.
    no spells: Prevents any spellcasting (but not scrolls or wands etc)
    Range: 0 or 1.
    teleport: Every now and then randomly teleports you. *Really* annoying.
@@ -134,7 +134,7 @@
 {
     "Dum", "",
 /* class, type, plus (to-hit), plus2 (depends on class), colour */
-        250, 250, 250, 250, 0,
+        OBJ_UNASSIGNED, 250, 250, 250, 0,
 /* Properties, all approx thirty of them: */
     {
 /* brand, +/- to AC, +/- to ev, +/- to str, +/- to int, +/- to dex */
@@ -438,7 +438,7 @@
         OBJ_JEWELLERY, AMU_CLARITY, +0, 0, LIGHTGREEN,
     {
         0, 0, 0, 0, 0, 0,
-        0, 0, 0, 0, 1, 60,      // life prot, magic resistance
+        0, 0, 0, 0, 1, 100,      // life prot, magic resistance
         0, 0, 0, 0, 0, 0,
         0, 0, 0, 0, 0,
         0, 0, 0, 0, 0,
@@ -613,7 +613,7 @@
         0, -50                                  // stealth (TSO hates backstab)
     }
     ,
-        "This sword was The Shining One's gift to one of his paladins." ,
+        "This sword was The Shining One's gift to a worshipper.",
         "A long sword made of one huge piece of crystal.",
         ""
 }
@@ -860,12 +860,12 @@
         SPWPN_VAMPIRICISM, 0, 0, 0, 0, 0,
         0, 0, 0, 1, 1, 0,       // res poison, prot life
         0, 0, 0, 0, 0, 0,
-        0, 3, 0, 0, 0,          // noise
+        0, 1, 0, 0, 0,          // noise
         0, 0, 0, 0, 0,
         0, -30                  // stealth
     }
     ,
-        "It's a really dark and malign artifact and no wise man would even touch it.",
+        "It's a really dark and malign artefact and no wise man would even touch it.",
         "",
         ""
 }
@@ -1134,7 +1134,7 @@
 {
     "Dum", "",
 /* class, type, plus (to-hit), plus2 (depends on class), colour */
-        250, 250, 250, 250, 0,
+        OBJ_UNASSIGNED, 250, 250, 250, 0,
 /* Properties, all approx thirty of them: */
     {
 /* brand, +/- to AC, +/- to ev, +/- to str, +/- to int, +/- to dex */
